package com.ibm.jvm;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2006  All Rights Reserved
 */

/**
 * APIs used to cause RAS dumps. -Xdump must be enabled on the command line.
 */
public class Dump {

	/**
	 * Cause a JavaDump to occur. A JavaDump is in a human-readable format, and
	 * summarizes the state of the JVM.
	 *
	 * @throws RuntimeException if the vm does not contain RAS dump support
	 */
	public static void JavaDump() {
		JavaDumpImpl();
	}

	/**
	 * Cause a HeapDump to occur. The default HeapDump format (phd files) is not
	 * human-readable and must be processed using available tools.
	 *
	 * @throws RuntimeException if the vm does not contain RAS dump support
	 */
	public static void HeapDump() {
		HeapDumpImpl();
	}

	/**
	 * Cause a SystemDump to occur. A SystemDump is a platform-specific
	 * file that contains information about the active processes, threads, and
	 * system memory. System dumps are usually large.
	 *
	 * @throws RuntimeException if the vm does not contain RAS dump support
	 */
	public static void SystemDump() {
		SystemDumpImpl();
	}

	/*
	 * Dump should not be instantiated.
	 */
	private Dump() {
	}

	private static native void JavaDumpImpl();
	private static native void HeapDumpImpl();
	private static native void SystemDumpImpl();
}
