package com.ibm.oti.appletviewer;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.io.*;
import java.net.URL;
import java.net.URLClassLoader;

/**
 *	A subclass of ObjectInputStream which obtains needed class definitions
 *  from a URLClassLoader which searches a particular URL for classes that
 *  cannot be obtained by the parent classloader.
 *
 * @author		IBM
 * @version		initial
 */
public class AppletInputStream extends ObjectInputStream {

	/**
	 *  The classloader used to load classes need for deserialization
	 */
	protected ClassLoader classLoader;

/**
 * Constructs an instance of this class which deserializes objects located
 * at the URL <code>object</code> and obtains needed classes from an URLClassLoader
 * whose search path includes <code>codeURL</code>.
 *
 * @author	IBM
 * @version	initial
 *
 *				@param	object URL
 *							the URL specifying the serialized object to
 *							to be deserialized
 *
 *				@exception IOException
 *							if there is a problem opening a stream
 *							on the object URL.
 *
 */
public AppletInputStream(URLClassLoader urlClassLoader, URL object)
	throws IOException
{
	super(object.openStream());
	classLoader = urlClassLoader;
}

/**
 * Loads the Java class corresponding to the class descriptor <code>osClass</code>(ObjectStreamClass)
 * just read from the receiver.  Loading is done by an URLClassLoader on the code URL
 * specified as an argument to the constructor.  An URLClassLoader asks its parent
 * classloader to load a class and only loads from the URL if this fails.
 *
 * @author		IBM
 * @version		initial
 *
 * @param		desc		An ObjectStreamClass read from the receiver.
 *
 * @return		Class		A Class corresponding to the descriptor loaded.
 *
 * @exception		java.io.IOException			If any IO problem occurred when trying to load the class.
 * @exception		java.lang.ClassNotFoundException	If the corresponding class cannot be found.
 */
protected Class resolveClass(ObjectStreamClass desc)
	throws IOException, ClassNotFoundException
{
	return Class.forName(desc.getName(), true, classLoader);
}
}
