package com.ibm.oti.appletviewer;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.awt.*;
import java.awt.event.*;

import com.ibm.oti.awt.MessageStrings;

/**
 * A window which displays the applet info and parameter info
 * for a particular applet
 *
 * @author      IBM
 * @version		initial
 */
public class InfoFrame extends Frame {

	/**
 	 * Creates an InfoFrame for the applet referenced by the
 	 * specified appletStub.
 	 *
 	 * @author      IBM
 	 * @version		initial
 	 *
 	 * @param		appletStub	the stub whose applet this
 	 *							window should describe
 	 */
	public InfoFrame(ViewerAppletStub appletStub) {
		super();
		setIconImage(AppletViewer.getIconImage());
		new WindowCloser(this, false);

		BorderLayout layout = new BorderLayout();
		setLayout(layout);

		TextArea textArea = new TextArea();
		textArea.setEditable(false);
		String text = appletStub.getApplet().getAppletInfo();
		if(text == null) {
			text = MessageStrings.getString("BBAWTNoAppletInfo"); //$NON-NLS-1$
		}
		text += "\n\n"; //$NON-NLS-1$
		String[][] parmInfo = appletStub.getApplet().getParameterInfo();
		if(parmInfo == null) {
			text += MessageStrings.getString("BBAWTNoParameterInfo"); //$NON-NLS-1$
		} else {
			for(int i = 0; i < parmInfo.length; i++) {
				if(parmInfo[i] != null) {
					for(int j = 0; j < parmInfo[i].length; j++) {
						text += parmInfo[i][j];
						if(j != parmInfo[i].length - 1)
							text += " - "; //$NON-NLS-1$
					}
					text += "\n"; //$NON-NLS-1$
				}
			}
		}
		textArea.setText(text);
		add(textArea);

		Button dismissButton = new Button(MessageStrings.getString("BBAWTDismiss")); //$NON-NLS-1$
		Panel buttonPanel = new Panel();
		buttonPanel.setLayout(new FlowLayout());
		buttonPanel.add(dismissButton);
		dismissButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				EventQueue queue = Toolkit.getDefaultToolkit().getSystemEventQueue();
				queue.postEvent(new WindowEvent(InfoFrame.this, WindowEvent.WINDOW_CLOSING));
				queue.postEvent(new WindowEvent(InfoFrame.this, WindowEvent.WINDOW_CLOSED));
			}
		});
		add(buttonPanel, BorderLayout.SOUTH);
	}
}
