package com.ibm.oti.appletviewer;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

import java.io.IOException;
import java.io.InputStream;

/**
 * Parses an input stream for an HTML document, creating HTMLTags
 * representing the tags in the document
 *
 * @author      IBM
 * @version		initial
 */
public class Parser {

	/**
	 *	The stream being parsed.
	 */
	protected InputStream inStream;

	/**
	 *	The stream is parsed one chunk at a time.
	 */
	protected StringBuffer chunk = new StringBuffer();

/**
 * Constructs a parser for the provided input stream
 *
 * @author      IBM
 * @version		initial
 *
 * @param		inStream	the input stream for the HTML document
 *							to be parsed
 */
public Parser(InputStream inStream) {
	this.inStream = inStream;
}

/**
 * Returns an HTMLTag representing the next tag in the document,
 * or <code>null</code> if the end of the document has
 * been reached
 *
 * @author      IBM
 * @version		initial
 *
 * @return		an HTMLTag representing the next tag in the document,
 *				or <code>null</code> if the end of the document has
 *				been reached
 */
public HTMLTag getNextTag() {
	if(nextTagChunk() == -1)
		return null;
	return new HTMLTag(chunk);
}

/**
 * Sets <code>this.chunk</code> to the chunk of text representing
 * the next tag.
 *
 * @author      IBM
 * @version		initial
 *
 * @return		-1 iff the end of the input has been reached
 */
protected int nextTagChunk() {
	chunk.setLength(0);
	try {
		boolean inComment = false, inQuotes = false, inTag = false;
		int lastTagChar = 0;
		int c;
		while ((c=inStream.read())!=-1) {
			// if it's in a comment, then throw away this character if it isn't in [->]
			// otherwise, build the chunk until it is the sequence "-->" which clears the
			// inComment flag. Anything else out of sequence clears the tag except for
			// a series of '-' characters so the string "--->" is still correctly parsed
			// as the end of comment, rather than throwing it away after getting "---"
			if (inComment) {
				if ((c!='-') && (c!='>')) {
					chunk.setLength(0);
					continue;
				}
				if (c=='-') {
					if (chunk.length()<2)
						chunk.append('-');
					continue;
				}
				// if it contains "--" and we just got a ">"
				if (chunk.length()==2) {
					chunk.setLength(0);
					inComment=false;
					continue;
				}
				// it must have been a ">" or "->" so ignore it
				chunk.setLength(0);
				continue;
			}

			// the spec says to totally ignore LF chars
			if (c==10)
				continue;
			// the spec says to turn CR and TABs into spaces
			if ((c=='\r') || (c=='\t') || (c=='\f') || (c=='\n'))
				c=' ';
			// keep track of what the last char was in a tag
			if (inTag) {
				// Do not remove duplicate spaces, some applets assume
				// duplicate spaces are preserved.
				//if ((c==' ') && (lastTagChar==' '))
				//	continue;
				lastTagChar=c;
			}

			// if we're in a tag and we get a ">" that's not inside a quoted attribute
			// then it's the end of the tag, add it and return
			if (inTag) {
				if ((c=='>') && (!inQuotes)) {
					chunk.append('>');
					return 1;
				}
				// check for the beginning of a comment
				if ((c=='-') && (chunk.toString().equals("<!-"))) { //$NON-NLS-1$
					inTag=false;
					inComment=true;
					chunk.setLength(0);
					lastTagChar = c;
					continue;
				}

				// if we hit a '"' then clear the inQuotes flag if the lastTagChar
				// was not a '\\' and we're in a quote otherwise set the inQuotes flag
				if (c=='"') {
					if (inQuotes) {
						if (lastTagChar!='\\')
							inQuotes=false;
					}
					else
						inQuotes=true;
				}
				chunk.append((char) c);
				lastTagChar = c;
			}
			else {
				if (c=='<') {
					chunk.setLength(0);
					chunk.append('<');
					inTag=true;
				}
				continue;
			}
		}

		//clear chunk in case a tag was begun but not completed
		//before EOF
		chunk.setLength(0);
		return -1;
	}
	catch (IOException e) {
		e.printStackTrace();
		return -1;
	}
}

}
