package com.ibm.oti.appletviewer;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.applet.*;
import java.io.FilePermission;
import java.io.IOException;
import java.net.*;
import java.security.*;
import java.util.*;

/**
 * An implementation of AppletStub which holds information about
 * a particular applet and loads and instantiates the applet.
 *
 * @author      IBM
 * @version		initial
 */
public class ViewerAppletStub implements AppletStub {

/**
 *  The applet with which this stub is associated.
 */
protected Applet applet;

/**
 *  The window which contains this stub's applet
 **/
protected ViewerFrame frame;

/**
 *  The applet parameters (key: String, value: String)
 *  Obtained from PARAM tags.
 */
protected Hashtable parms;

/**
 *  The applet's name attribute
 */
protected String name;

/**
 *  The applet's code attribute
 */
protected String code;

/**
 *  The applet's archive attribute
 */
protected String archive;

/**
 *  The applet's object attribute
 */
protected String object;

/**
 *  Whether or not the applet is active
 */
protected boolean active = false;

/** The applet's document base URL
 *
 */
protected URL documentBase;

/**
 *  The applet's code base URL
 */
protected URL codeBase;

/**
 *  The context in which the applet occurs
 */
protected AppletContext appletContext;

/**
 *  Constructs an instance of this class.
 *
 *  @author      IBM
 *  @version	initial
 */
public ViewerAppletStub() {
	super();
	parms = new Hashtable();
}

/**
 *	Sets the frame containing the applet.
 *
 * @author      IBM
 * @version		initial
 *
 * @param		frame	the frame containing the applet
 */
public void setFrame(ViewerFrame frame) {
	this.frame = frame;
}

/**
 * Returns the frame containing the applet
 *
 * @author      IBM
 * @version		initial
 *
 * @return		the frame containing the applet
 */
public ViewerFrame getFrame() {
	return frame;
}

/**
 *	Sets the applet's archive attribute.
 *
 * @author      IBM
 * @version		initial
 *
 * @param		archive		the applet's archive attribute
 */
public void setArchive(String archive) {
	this.archive = archive;
}

/**
 * Sets the applet's code attribute.
 *
 * @author      IBM
 * @version		initial
 *
 * @param		code	the applet's code attribute
 */
public void setCode(String code) {
	this.code = code;
}

public void setObject(String object) {
	this.object = object;
}

/**
 *	Sets the applet's name attribute.
 *
 * @author      IBM
 * @version		initial
 *
 * @param		name	the applet's name attribute
 */
public void setName(String name) {
	this.name = name;
}

/**
 *	Sets the applet's codebase attribute
 *
 * @author      IBM
 * @version		initial
 *
 * @param		codeBase	the applet's codebase attribute
 */
public void setCodeBase(URL codeBase) {
	this.codeBase = codeBase;
}

/**
 * Returns the applet's name attribute
 *
 * @author      IBM
 * @version		initial
 *
 * @return		the applet's name attribute
 */
public String getName() {
	return name;
}

/**
  * Helper method to turn a URL specifier from an ARCHIVE attribute into
  * an absolute URL specifier.  See the HTML specification from
  * www.w3.org for further description of all HTML tags and attributes.
  *
  * @author  IBM
  * @version initial
  *
  * @param    base codebase to be used for relative specifier
  * @param    archiveToken archive to be normalized.
  * @return   normalized archive URL specifier
  */
protected String normalizeArchiveToken(String base, String archiveToken){
	StringBuffer archive = new StringBuffer();

	/*
	* While HTML specification says that ARCHIVE needs to be
	* relative to the codebase, neither IE nor Navigator prohibits this
	*/

	if (archiveToken.indexOf(':') == -1) { /* relative */
		archive.append(base);
	}

	archive.append(archiveToken);
	return(archive.toString());
}

/**
 * Loads the applet's class, then creates an instance of the
 * class, and attaches the applet to the stub and vice-versa
 *
 * @author      IBM
 * @version		initial
 *
 * @throws		many different types of exceptions, resulting from
 *				problems loading or instantiating the applet, or
 *				an improperly initialized stub
 */
public void loadApplet()
	throws ClassNotFoundException, IllegalAccessException,
		   InstantiationException, MalformedURLException
{
  	//The name of the class we will request from the ClassLoader
  	String className = null;

	//Determine the codeURL and className:
	String kode = code;
	if (object == null) {
		if (kode == null) {
			throw new MalformedURLException();
		}
	} else {
		//class name will not be used:
		kode = "."; //$NON-NLS-1$
	}

	kode = kode.trim();
	int extensionIdx = kode.lastIndexOf(".class"); //$NON-NLS-1$
	// ".class" could be omitted
	if ((extensionIdx != -1) && (extensionIdx == kode.length() - 6)) {
		kode = kode.substring(0, extensionIdx);
	}
	kode = kode.replace('/', '.');

	className = kode;
	//Temp storage for code base urls
	Vector urls = new Vector();

	if (archive != null) {
		StringTokenizer archiveTokenizer = new StringTokenizer(archive, ","); //$NON-NLS-1$
		URL archive;
		String archiveToken;

		while (archiveTokenizer.hasMoreTokens()) {
			archiveToken = archiveTokenizer.nextToken();
			archiveToken =
				normalizeArchiveToken(codeBase.toString(), archiveToken);
			archive = new URL(archiveToken);
			urls.add(archive);
		}
	}

	urls.add(codeBase);

	//The root of the ClassLoader's search path;
	final URL[] codeURL = new URL[urls.size()];
	for (int i = 0; i < urls.size(); i++) {
		codeURL[i] = (URL) urls.elementAt(i);
	}

	// Grant permissions for applet classes to read from their codebase
	Permission perm = null;
	final Permission[] perms;
	try {
		perm = codeBase.openConnection().getPermission();
	} catch (IOException e) {
	}
	if (perm instanceof FilePermission) {
		if (codeBase.getFile().endsWith("/")) { //$NON-NLS-1$
			Permission subs = new FilePermission(perm.getName() + "-", "read"); //$NON-NLS-1$ //$NON-NLS-2$
			perms = new Permission[] { perm, subs };
		} else {
			perms = new Permission[] { perm };
		}
	} else {
		perms = new Permission[0];
	}

	URLClassLoader classLoader = (URLClassLoader)AccessController.doPrivileged(new PrivilegedAction() {
		public Object run() {
			return new AppletClassLoader(codeURL, perms);
		}
	});

	Applet applet = null;
	if (object == null) {
		//Load the applet class:
		Class appletClass = classLoader.loadClass(className);
		//Instantiate the applet:
		applet = (Applet) appletClass.newInstance();
	} else {
		try {
			URL objectURL = new URL(codeBase, object);
			//deserialize the applet:
			AppletInputStream in =
				new AppletInputStream(classLoader, objectURL);
			applet = (Applet) in.readObject();
		} catch (IOException e) {
		}
	}

	this.applet = applet;
	applet.setStub(this);

	String frameName = name;
	if (frameName == null) {
		if (object == null)
			frameName = className;
		else
			frameName = applet.getClass().toString();
	}
	frame.setAppletName(frameName);
}

/**
 * Returns the applet referenced by this stub.
 *
 * @author      IBM
 * @version		initial
 *
 * @return		the applet referenced by this stub
 */
public Applet getApplet() {
	return applet;
}

/**
 *	Adds a key-value pair for a parameter to the applet's list
 *  of parameters
 *
 * @author      IBM
 * @version		initial
 *
 * @param		name	the parameter's name
 *				value	the parameter's value
 */
public void addParm(String name, String value) {
	parms.put(name.toUpperCase(), value);
}

/**
 *	Sets the applet's context.
 *
 * @author      IBM
 * @version		initial
 *
 * @param		appletContext	the applet's context
 */
public void setAppletContext(AppletContext appletContext) {
	this.appletContext = appletContext;
}

/**
 * Resizes the applet.
 *
 * @author      IBM
 * @version		initial
 *
 * @param		width	The intended new width for the applet.
 * @param		height	The intended new height for the applet.
 */
public void appletResize(int width, int height) {
	frame.resizeApplet(width, height);
}

/**
 * Returns this applet's applet context.
 *
 * @author      IBM
 * @version		initial
 *
 * @return		this applet's applet context
 *
 * @see			AppletContext
 */
public AppletContext getAppletContext() {
	return appletContext;
}

/**
 * Returns this applet's code base URL.  Let the 'code URL' be the
 * URL indicating the location of this applet's class file.  Thus, the 'code
 * base URL' is the part of the code URL excluding the applet's class filename.
 *
 * @author      IBM
 * @version		initial
 *
 * @return		the code base URL of this applet
 *
 */
public URL getCodeBase() {
	return codeBase;
}

/**
 * Returns this applets document base URL.  Let the 'document URL' be the
 * URL of the document in which this applet was embedded.  Thus, the 'document
 * base URL' is the part of the document URL excluding the documents name.
 *
 * @author      IBM
 * @version		initial
 *
 * @return		the document base URL of this applet
 *
 */
public URL getDocumentBase() {
	return documentBase;
}

/**
 *	Sets the applet's document base URL.
 *
 * @author      IBM
 * @version		initial
 *
 * @param		documentBase	the document base URL (absolute) for
 *								the applet
 */
public void setDocumentBase(URL documentBase) {
	this.documentBase = documentBase;
}

/**
 * Returns the value of the applet parameter named <code>name</code>.
 * Applet parameters are typically specified in the HTML document containing
 * the applet.  If this is the case, this method will return the <code>value</code>
 * attribute of a <code>param</code> tag (nested within this applet's
 * <code>applet</code> tag) whose <code>name</code> attribute is <code>name</code>.
 *
 * @author      IBM
 * @version		initial
 *
 * @param		name	the name of the parameter whose value is to be returned
 *
 * @return		the value of the parameter with the specified name, or <code>null</code>
 *				if the named parameter does not exist
 */
public String getParameter(String name) {
	return (String) parms.get(name.toUpperCase());
}

/**
 * Sets whether or not the applet is 'active'.
 *
 * @author      IBM
 * @version		initial
 *
 * @param		active	whether or not the applet should be marked
 *				'active'
 */
public void setActive(boolean active) {
	this.active = active;
}

/**
 *	Returns whether this applet is active.  An applet becomes active
 *  just before its <code>start()</code> method is called and ceases
 *  to be active just before its <code>stop()</code>
 *
 * @author      IBM
 * @version		initial
 *
 * @return		whether or not this applet is active
 *
 */
public boolean isActive() {
	return active;
}

public boolean wasDeserialized() {
	return (object != null);
}

public void startApplet() {
	if (!getApplet().isActive()) {
		setActive(true);
		getApplet().setVisible(true);
		getApplet().start();
	}
}

public void stopApplet() {
	if (getApplet().isActive()) {
		setActive(false);
		getApplet().setVisible(false);
		getApplet().stop();
	}
}
}
