package com.ibm.oti.appletviewer;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

import java.awt.Window;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import com.ibm.oti.awt.Util;

/**
 * A WindowListener which deals appropriately with window closing
 * events.
 *
 * @author      IBM
 * @version		initial
 */
public class WindowCloser implements WindowListener {

/**
 *  Whether or not the closing of a window should causing the
 *  VM to be halted.
 */
boolean exit;

/**
 * Constructs a listener which calls System.exit() when the window
 * is closed.
 *
 * @author      IBM
 * @version		initial
 *
 * @param		window	the window on which to listen
 */
public WindowCloser(Window window) {
	this(window, true);
}

/**
 * Constructs an instance of this class and adds it, as a window listener,
 * to the specified window.  If <code>exit</code> is true, System.exit()
 * is called when the window is closed.
 *
 * @author      IBM
 * @version		initial
 *
 * @param		window	the window on which to listen (possibly null)
 *
 *				exit boolean
 *					whether to halt the VM in response to
 *					a window-closed event
 */
public WindowCloser(Window window, boolean exit) {
	this.exit = exit;
	if (window != null)
		window.addWindowListener(this);
}

/**
 * Constructs an instance of this class. If <code>exit</code> is
 * true, System.exit() is called in response to window-closed events.
 *
 * @author      IBM
 * @version		initial
 *
 * 		@param	exit boolean
 *					whether to halt the VM in response to
 *					a window-closed event
 */
public WindowCloser(boolean exit) {
	this(null, exit);
}

public void windowClosing(WindowEvent e) {
	//close the window
	Window window = e.getWindow();
	if (window != null) {
		window.dispose();
	}
}

public void windowClosed(WindowEvent e) {
	//take down the VM if requested
	if (exit) {
		// If all Frames are disposed, all Windows are also disposed.
		if (0 == Util.getActiveFrameCount()) {
			System.exit(0);
		}
	}
}

public void windowOpened(WindowEvent e) {}
public void windowActivated(WindowEvent e) {}
public void windowDeactivated(WindowEvent e) {}
public void windowIconified(WindowEvent e) {}
public void windowDeiconified(WindowEvent e) {}

}
