package com.ibm.oti.awt;

import java.util.Hashtable;

import com.ibm.ugl.eswt.OS;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * Loads system properties based on the active toolkit.
 */
public class SystemPropertyLoader {

	public static final String Java_Specification_Name = "java.specification.name";
	public static final String Java_Specification_Vendor = "java.specification.vendor";
	public static final String Java_Specification_Version = "java.specification.version";
	public static final String Java_Version = "java.version";

	/*
	 * Please note that PPro11 removed most of the system property
	 * restrictions that were in PPro10.  However, the native platforms
	 * can still restrict the functionality, so we will continue to
	 * set the properties.  The same thing is true for PBPro versions.
	 */

	private static final String AlphaComposite_SRC_OVER = "java.awt.AlphaComposite.SRC_OVER.isRestricted";
	private static final String Component_setCursor = "java.awt.Component.setCursor.isRestricted";
	private static final String Frame_setLocation = "java.awt.Frame.setLocation.isRestricted";
	private static final String Frame_setResizable = "java.awt.Frame.setResizable.isRestricted";
	private static final String Frame_setSize = "java.awt.Frame.setSize.isRestricted";
	private static final String Frame_setState = "java.awt.Frame.setState.isRestricted";
	private static final String Frame_setTitle = "java.awt.Frame.setTitle.isRestricted";

	private static final String Dialog_setLocation = "java.awt.Dialog.setLocation.isRestricted";
	private static final String Dialog_setResizable = "java.awt.Dialog.setResizable.isRestricted";
	private static final String Dialog_setSize = "java.awt.Dialog.setSize.isRestricted";
	private static final String Dialog_setTitle = "java.awt.Dialog.setTitle.isRestricted";
	private static final String TextField_setEchoChar = "java.awt.TextField.setEchoChar.isRestricted";

	private static final String Graphics2D_setStroke_BasicStroke = "java.awt.Graphics2D.setStroke.BasicStroke.isRestricted";

	public static void load(Hashtable properties) {
		int r = OS.Platform_GetCapability(OS.CAPABILITY_AWT_RESTRICTIONS);
		if (r != 0) {

			if ((r & OS.RESTRICTION_AlphaComposite_SRC_OVER) == OS.RESTRICTION_AlphaComposite_SRC_OVER) {
				properties.put(AlphaComposite_SRC_OVER,"true");
			}
			if ((r & OS.RESTRICTION_Component_setCursor) == OS.RESTRICTION_Component_setCursor) {
				properties.put(Component_setCursor,"true");
			}
			if ((r & OS.RESTRICTION_Frame_setLocation) == OS.RESTRICTION_Frame_setLocation) {
				properties.put(Frame_setLocation,"true");
			}
			if ((r & OS.RESTRICTION_Frame_setResizable) == OS.RESTRICTION_Frame_setResizable) {
				properties.put(Frame_setResizable,"true");
			}
			if ((r & OS.RESTRICTION_Frame_setSize) == OS.RESTRICTION_Frame_setSize) {
				properties.put(Frame_setSize,"true");
			}
			if ((r & OS.RESTRICTION_Frame_setState) == OS.RESTRICTION_Frame_setState) {
				properties.put(Frame_setState,"true");
			}
			if ((r & OS.RESTRICTION_Frame_setTitle) == OS.RESTRICTION_Frame_setTitle) {
				properties.put(Frame_setTitle,"true");
			}

			if ((r & OS.RESTRICTION_Dialog_setLocation) == OS.RESTRICTION_Dialog_setLocation) {
				properties.put(Dialog_setLocation,"true");
			}
			if ((r & OS.RESTRICTION_Dialog_setResizable) == OS.RESTRICTION_Dialog_setResizable) {
				properties.put(Dialog_setResizable,"true");
			}
			if ((r & OS.RESTRICTION_Dialog_setSize) == OS.RESTRICTION_Dialog_setSize) {
				properties.put(Dialog_setSize,"true");
			}
			if ((r & OS.RESTRICTION_Dialog_setTitle) == OS.RESTRICTION_Dialog_setTitle) {
				properties.put(Dialog_setTitle,"true");
			}
			if ((r & OS.RESTRICTION_TextField_setEchoChar) == OS.RESTRICTION_TextField_setEchoChar) {
				properties.put(TextField_setEchoChar,"true");
			}

			if ((r & OS.RESTRICTION_Graphics2D_setStroke_BasicStroke) == OS.RESTRICTION_Graphics2D_setStroke_BasicStroke) {
				properties.put(Graphics2D_setStroke_BasicStroke,"true");
			}

		}
	}

}
