package com.ibm.oti.awt;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.ArrayList;

import com.ibm.oti.awt.metal.widgets.Display;
import com.ibm.ugl.eswt.OS;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2006  All Rights Reserved
 */

/**
 * This class contains utility methods for the implementation of AWT.
 */
public class Util {
	static private boolean   gShowSecondaryWindows					= true;
	static private boolean   gVerbose								= false;
	static private boolean   gDebugGraphics							= false;
	static private boolean   gDebugGraphicsGUI						= false;
	static private Dimension gDefaultScrollbarSize					= new Dimension(50, 15);
	static private boolean   gGenerateMouseDraggedOnPocketPC		= true;
	static public boolean	 gStartAppletsSynchronously				= true;
	static public boolean	 gFullScreenWindowBehavior				= false;
	static public boolean    gPocketPCOK 							= false;
	static public boolean    gThrowPeerExceptions					= false;
	static public boolean    gAWTEventLoopRunning				    = true;

	static private ArrayList framesWithPeers = new ArrayList();

	//----------------------------------------------------------------------
	static {
		loadProperties();
	}

	//----------------------------------------------------------------------
	static public void reference() {
	}

	//----------------------------------------------------------------------
	static private void loadProperties() {

		setStringProperty("awt.toolkit","java.awt.ToolkitImpl"); //$NON-NLS-1$ //$NON-NLS-2$

		setStringProperty("awt.image.incrementalDraw", "false"); //$NON-NLS-1$ //$NON-NLS-2$

		String protocolHandler = getStringProperty("java.protocol.handler.pkgs", null); //$NON-NLS-1$
		if ((protocolHandler != null) && (protocolHandler.indexOf("com.ibm.oti.awt") == -1)) { //$NON-NLS-1$
			setStringProperty("java.protocol.handler.pkgs", protocolHandler + "|com.ibm.oti.awt"); //$NON-NLS-1$ //$NON-NLS-2$
		}

		// getGUI("com.ibm.oti.awt.GUI"); //$NON-NLS-1$

		gShowSecondaryWindows =
			getBooleanProperty("com.ibm.oti.awt.ShowSecondaryWindows", gShowSecondaryWindows); //$NON-NLS-1$

		gVerbose =
			getBooleanProperty("com.ibm.oti.awt.Verbose", gVerbose); //$NON-NLS-1$

		gDebugGraphics=
			getBooleanProperty("com.ibm.oti.awt.DebugGraphics", gDebugGraphics); //$NON-NLS-1$

		// in most cases, the default setting should be the same as gDebugGraphics
		gDebugGraphicsGUI=
			getBooleanProperty("com.ibm.oti.awt.DebugGraphicsGUI", gDebugGraphicsGUI); //$NON-NLS-1$

//		Moved to initDefaultFontSize()
//		gDefaultFontSize =
//			getIntProperty("com.ibm.oti.awt.DefaultFontSize", gDefaultFontSize); //$NON-NLS-1$

	//	Moved this section to initFontDPI()
	//	if (isMotif() || isGTK() || isQt()) {
	//		gFontDPI = 72;
	//	}
	//
	//	gFontDPI =
	//		getIntProperty("com.ibm.oti.awt.FontDPI", gFontDPI);

		if (!gShowSecondaryWindows) {
			System.out.println("Secondary Windows disabled due to System property com.ibm.oti.awt.ShowSecondaryWindows"); //$NON-NLS-1$
		}

		gStartAppletsSynchronously=
			getBooleanProperty("com.ibm.oti.awt.StartAppletsSynchronously", gStartAppletsSynchronously); //$NON-NLS-1$

		gFullScreenWindowBehavior=
			getBooleanProperty("com.ibm.oti.awt.FullScreenWindowBehavior", gFullScreenWindowBehavior); //$NON-NLS-1$

		gPocketPCOK=
			getBooleanProperty("com.ibm.oti.awt.PocketPCOK", gPocketPCOK); //$NON-NLS-1$

	}

	//----------------------------------------------------------------------
	static public boolean canShowSecondaryWindows() {
		return gShowSecondaryWindows;
	}

	//----------------------------------------------------------------------
	static public Dimension getDefaultScrollbarSize() {
		return gDefaultScrollbarSize;

	}

	/**
	 * Safely ask for an integer based property.  Return passed
	 * in defaultValue if property is not set via cmd line.
	 */
	static public int getIntProperty(final String key, int defaultValue) {
		String string = (String) AccessController.doPrivileged(new PrivilegedAction() {
			public Object run() {
				return System.getProperty(key);
			}
		});

		if (null == string) return defaultValue;
		if (0 == string.length()) return defaultValue;

		int value;
		try {
			value = Integer.parseInt(string);
		}
		catch (NumberFormatException e) {
			value = defaultValue;
		}

		if (value <= 0) value = defaultValue;

		return value;
	}

	//-----------------------------------------------------------------
	static public boolean getBooleanProperty(final String key, boolean defaultValue) {
		String string = (String) AccessController.doPrivileged(new PrivilegedAction() {
			public Object run() {
				return System.getProperty(key);
			}
		});

		if (null == string) return defaultValue;
		if (0 == string.length()) return defaultValue;

		return Boolean.valueOf(string).booleanValue();
	}

	/**
	 * Safely ask for a string based property.  Return passed
	 * in defaultValue if property is not set via cmd line.
	 */
	static public String getStringProperty(final String key, String defaultValue) {
		String string = (String) AccessController.doPrivileged(new PrivilegedAction() {
			public Object run() {
				return System.getProperty(key);
			}
		});

		if (null == string) return defaultValue;
		if (0 == string.length()) return defaultValue;

		return string;
	}

	static public void setStringProperty(final String key, final String value) {
		AccessController.doPrivileged(new PrivilegedAction() {
			public Object run() {
				return System.setProperty(key, value);
			}
		});
	}

	//----------------------------------------------------------------------
	static public boolean isDebugGraphics() {
		return gDebugGraphics;
	}

	//----------------------------------------------------------------------
	static public boolean isDebugGraphicsGUI() {
		return gDebugGraphicsGUI;
	}

	//----------------------------------------------------------------------
	static public boolean isVerbose() {
		return gVerbose;
	}

	//----------------------------------------------------------------------
	static public boolean useFullScreenWindowBehavior() {
		return gFullScreenWindowBehavior;
	}

	//----------------------------------------------------------------------
	static public void togglePocketPCMouseMovementEvent() {
		gGenerateMouseDraggedOnPocketPC = !gGenerateMouseDraggedOnPocketPC;
	}

	//----------------------------------------------------------------------
	static public boolean isGenerateMouseDraggedOnPocketPC() {
		return gGenerateMouseDraggedOnPocketPC;
	}

	/**
	 * Convenient method for marking methods to be implemented,
	 * since the references search will help find all of these.
	 * An alternative to TO-DO flags.
	 */
	static public void notImplementedYet(String className, String methodName) {
		System.out.println("Not implemented - " + className + " " + methodName); //$NON-NLS-1$ //$NON-NLS-2$
	}

	static public void notImplementedYet() {
	}

	public static Display getDisplay() {
		return ((IToolkitImpl) Toolkit.getDefaultToolkit()).getDisplay();
	}

	public static void shutdownAWT() {
		gAWTEventLoopRunning = false;
	}
	public static boolean keepAWTEventLoopRunning() {
		return gAWTEventLoopRunning;
	}

	public static void addFrame(Frame f) {
		synchronized (framesWithPeers) {
			framesWithPeers.add(f);
		}
	}

	public static void removeFrame(Frame f) {
		synchronized (framesWithPeers) {
			framesWithPeers.remove(f);
		}
	}

	public static void disposeAllFrames() {
		Frame f;
		while (true) {
			synchronized (framesWithPeers) {
				if (framesWithPeers.isEmpty()) return;
				f = (Frame)framesWithPeers.get(0);
				// Don't remove the item from the list here, the dispose
				// call will cause the frame to be removed
			}

			// Call dispose outside of the 'framesWithPeers' lock
			// to avoid deadlock.  The deadlock is a lock order difference
			// between the treelock and framesWithPeers when calling
			// this method and Frame.dipose() in different threads.
			f.dispose();
		}
	}

	public static int getActiveFrameCount() {
		return framesWithPeers.size();
	}

	public static int convertToAWTKeyModifiers(int modifiers) {
		int rc = 0;
		if ((modifiers & OS.MODIFIER_MASK_ALT) != 0) {
			rc |= InputEvent.ALT_MASK;
			rc |= InputEvent.ALT_DOWN_MASK;
		}
		if ((modifiers & OS.MODIFIER_MASK_SHIFT) != 0) {
			rc |= InputEvent.SHIFT_MASK;
			rc |= InputEvent.SHIFT_DOWN_MASK;
		}
		if ((modifiers & OS.MODIFIER_MASK_CONTROL) != 0) {
			rc |= InputEvent.CTRL_MASK;
			rc |= InputEvent.CTRL_DOWN_MASK;
		}
		return rc;
	}

	public static int convertToAWTButtonMasks(int modifiers) {
		int rc = 0;
		if ((modifiers & OS.POINTER_MASK_BUTTON1) != 0) {
			rc |= InputEvent.BUTTON1_MASK;
			rc |= InputEvent.BUTTON1_DOWN_MASK;
		}
		if ((modifiers & OS.POINTER_MASK_BUTTON2) != 0) {
			rc |= InputEvent.BUTTON2_MASK;
			rc |= InputEvent.BUTTON2_DOWN_MASK;
		}
		if ((modifiers & OS.POINTER_MASK_BUTTON3) != 0) {
			rc |= InputEvent.BUTTON3_MASK;
			rc |= InputEvent.BUTTON3_DOWN_MASK;
		}
		return rc;
	}

	public static int convertToNativeButtonMasks(int modifiers) {
		int rc = 0;
		if ((modifiers & InputEvent.BUTTON1_MASK) != 0) {
			rc |= OS.POINTER_MASK_BUTTON1;
		}
		if ((modifiers & InputEvent.BUTTON2_MASK) != 0) {
			rc |= OS.POINTER_MASK_BUTTON2;
		}
		if ((modifiers & InputEvent.BUTTON3_MASK) != 0) {
			rc |= OS.POINTER_MASK_BUTTON3;
		}
		if ((modifiers & InputEvent.BUTTON1_DOWN_MASK) != 0) {
			rc |= OS.POINTER_MASK_BUTTON1;
		}
		if ((modifiers & InputEvent.BUTTON2_DOWN_MASK) != 0) {
			rc |= OS.POINTER_MASK_BUTTON2;
		}
		if ((modifiers & InputEvent.BUTTON3_DOWN_MASK) != 0) {
			rc |= OS.POINTER_MASK_BUTTON3;
		}
		return rc;
	}

	/**
	 * Return the a java.awt.event.KeyEvent keycode to the corresponding
	 * native UGL keycode or -1 if it is not supported.
	 */
	public static int convertAWTToNativeKeycode(int keycode) {
	    switch (keycode) {
	    	// First group: basic keycode set for PPro compliance
	    	// See the PPro Javadoc for java.awt.event.KeyEvent
	    	// and the "Basic Key Codes".
			case KeyEvent.VK_0:					return '0';
			case KeyEvent.VK_1:					return '1';
			case KeyEvent.VK_2:					return '2';
			case KeyEvent.VK_3:					return '3';
			case KeyEvent.VK_4:					return '4';
			case KeyEvent.VK_5:					return '5';
			case KeyEvent.VK_6:					return '6';
			case KeyEvent.VK_7:					return '7';
			case KeyEvent.VK_8:					return '8';
			case KeyEvent.VK_9:					return '9';
			case KeyEvent.VK_A:					return 'A';
			case KeyEvent.VK_B:					return 'B';
			case KeyEvent.VK_C:					return 'C';
			case KeyEvent.VK_D:					return 'D';
			case KeyEvent.VK_E:					return 'E';
			case KeyEvent.VK_F:					return 'F';
			case KeyEvent.VK_G:					return 'G';
			case KeyEvent.VK_H:					return 'H';
			case KeyEvent.VK_I:					return 'I';
			case KeyEvent.VK_J:					return 'J';
			case KeyEvent.VK_K:					return 'K';
			case KeyEvent.VK_L:					return 'L';
			case KeyEvent.VK_M:					return 'M';
			case KeyEvent.VK_N:					return 'N';
			case KeyEvent.VK_O:					return 'O';
			case KeyEvent.VK_P:					return 'P';
			case KeyEvent.VK_Q:					return 'Q';
			case KeyEvent.VK_R:					return 'R';
			case KeyEvent.VK_S:					return 'S';
			case KeyEvent.VK_T:					return 'T';
			case KeyEvent.VK_U:					return 'U';
			case KeyEvent.VK_V:					return 'V';
			case KeyEvent.VK_W:					return 'W';
			case KeyEvent.VK_X:					return 'X';
			case KeyEvent.VK_Y:					return 'Y';
			case KeyEvent.VK_Z:					return 'Z';
			case KeyEvent.VK_ENTER:				return OS.KEYCODE_ENTER;
			case KeyEvent.VK_BACK_SPACE:		return OS.KEYCODE_BACKSPACE;
			case KeyEvent.VK_TAB:				return OS.KEYCODE_TAB;
			case KeyEvent.VK_SPACE:				return ' ';
			case KeyEvent.VK_LEFT:				return OS.KEYCODE_ARROW_LEFT;
			case KeyEvent.VK_RIGHT:				return OS.KEYCODE_ARROW_RIGHT;
			case KeyEvent.VK_UP:				return OS.KEYCODE_ARROW_UP;
			case KeyEvent.VK_DOWN:				return OS.KEYCODE_ARROW_DOWN;

			// Second group: other keycodes which UGL supports
			case KeyEvent.VK_F1:				return OS.KEYCODE_F1;
			case KeyEvent.VK_F2:				return OS.KEYCODE_F2;
			case KeyEvent.VK_F3:				return OS.KEYCODE_F3;
			case KeyEvent.VK_F4:				return OS.KEYCODE_F4;
			case KeyEvent.VK_F5:				return OS.KEYCODE_F5;
			case KeyEvent.VK_F6:				return OS.KEYCODE_F6;
			case KeyEvent.VK_F7:				return OS.KEYCODE_F7;
			case KeyEvent.VK_F8:				return OS.KEYCODE_F8;
			case KeyEvent.VK_F9:				return OS.KEYCODE_F9;
			case KeyEvent.VK_F10:				return OS.KEYCODE_F10;
			case KeyEvent.VK_F11:				return OS.KEYCODE_F11;
			case KeyEvent.VK_F12:				return OS.KEYCODE_F12;
			case KeyEvent.VK_F13:				return OS.KEYCODE_F13;
			case KeyEvent.VK_F14:				return OS.KEYCODE_F14;
			case KeyEvent.VK_F15:				return OS.KEYCODE_F15;
			case KeyEvent.VK_F16:				return OS.KEYCODE_F16;
			case KeyEvent.VK_F17:				return OS.KEYCODE_F17;
			case KeyEvent.VK_F18:				return OS.KEYCODE_F18;
			case KeyEvent.VK_F19:				return OS.KEYCODE_F19;
			case KeyEvent.VK_F20:				return OS.KEYCODE_F20;
			case KeyEvent.VK_F21:				return OS.KEYCODE_F21;
			case KeyEvent.VK_F22:				return OS.KEYCODE_F22;
			case KeyEvent.VK_F23:				return OS.KEYCODE_F23;
			case KeyEvent.VK_F24:				return OS.KEYCODE_F24;
			case KeyEvent.VK_ADD:				return OS.KEYCODE_KEYPAD_ADD;
			case KeyEvent.VK_PLUS:				return '+';
			case KeyEvent.VK_MULTIPLY:			return OS.KEYCODE_KEYPAD_MULTIPLY;
			case KeyEvent.VK_ASTERISK:			return '*';
			case KeyEvent.VK_AMPERSAND:			return '&';
			case KeyEvent.VK_AT:				return '@';
			case KeyEvent.VK_COLON:				return ':';
			case KeyEvent.VK_SEMICOLON:			return ';';
			case KeyEvent.VK_COMMA:				return ',';
			case KeyEvent.VK_BRACELEFT:			return '{';
			case KeyEvent.VK_BRACERIGHT:		return '}';
			case KeyEvent.VK_DOLLAR:			return '$';
			case KeyEvent.VK_EXCLAMATION_MARK:	return '!';
			case KeyEvent.VK_EQUALS:			return '=';
			case KeyEvent.VK_LEFT_PARENTHESIS:	return '(';
			case KeyEvent.VK_RIGHT_PARENTHESIS:	return ')';
			case KeyEvent.VK_SUBTRACT:
			case KeyEvent.VK_MINUS:				return '-';
			case KeyEvent.VK_NUMPAD0:			return OS.KEYCODE_KEYPAD_0;
			case KeyEvent.VK_NUMPAD1:			return OS.KEYCODE_KEYPAD_1;
			case KeyEvent.VK_NUMPAD2:			return OS.KEYCODE_KEYPAD_2;
			case KeyEvent.VK_NUMPAD3:			return OS.KEYCODE_KEYPAD_3;
			case KeyEvent.VK_NUMPAD4:			return OS.KEYCODE_KEYPAD_4;
			case KeyEvent.VK_NUMPAD5:			return OS.KEYCODE_KEYPAD_5;
			case KeyEvent.VK_NUMPAD6:			return OS.KEYCODE_KEYPAD_6;
			case KeyEvent.VK_NUMPAD7:			return OS.KEYCODE_KEYPAD_7;
			case KeyEvent.VK_NUMPAD8:			return OS.KEYCODE_KEYPAD_8;
			case KeyEvent.VK_NUMPAD9:			return OS.KEYCODE_KEYPAD_9;
			case KeyEvent.VK_OPEN_BRACKET:		return '[';
			case KeyEvent.VK_CLOSE_BRACKET:		return ']';
			case KeyEvent.VK_UNDERSCORE:		return '_';
			case KeyEvent.VK_DIVIDE:			return OS.KEYCODE_KEYPAD_DIVIDE;
			case KeyEvent.VK_SLASH:				return '/';
			case KeyEvent.VK_BACK_SLASH:		return '\\';
			case KeyEvent.VK_CAPS_LOCK:			return OS.KEYCODE_LOCK_CAPS;
			case KeyEvent.VK_NUM_LOCK:			return OS.KEYCODE_LOCK_NUM;
			case KeyEvent.VK_SCROLL_LOCK:		return OS.KEYCODE_LOCK_SCROLL;
			case KeyEvent.VK_DELETE:			return OS.KEYCODE_DELETE;
			case KeyEvent.VK_END:				return OS.KEYCODE_END;
			case KeyEvent.VK_ESCAPE:			return OS.KEYCODE_ESCAPE;
			case KeyEvent.VK_HOME:				return OS.KEYCODE_HOME;
			case KeyEvent.VK_KP_DOWN:			return OS.KEYCODE_ARROW_DOWN;
			case KeyEvent.VK_KP_LEFT:			return OS.KEYCODE_ARROW_LEFT;
			case KeyEvent.VK_KP_RIGHT:			return OS.KEYCODE_ARROW_RIGHT;
			case KeyEvent.VK_KP_UP:				return OS.KEYCODE_ARROW_UP;
			case KeyEvent.VK_HELP:				return OS.KEYCODE_HELP;
			case KeyEvent.VK_INSERT:			return OS.KEYCODE_INSERT;
			case KeyEvent.VK_PAGE_DOWN:			return OS.KEYCODE_PAGE_DOWN;
			case KeyEvent.VK_PAGE_UP:			return OS.KEYCODE_PAGE_UP;
			case KeyEvent.VK_PAUSE:				return OS.KEYCODE_PAUSE;
			case KeyEvent.VK_PERIOD:			return '.';
			case KeyEvent.VK_QUOTE:				return '\'';
			case KeyEvent.VK_QUOTEDBL:			return '"';
			case KeyEvent.VK_BACK_QUOTE:		return '`';
			case KeyEvent.VK_PRINTSCREEN:		return OS.KEYCODE_PRINTSCREEN;

			case KeyEvent.VK_ALT:				return OS.KEYCODE_ALT;
			case KeyEvent.VK_CIRCUMFLEX:		return '^';
			case KeyEvent.VK_CONTROL:			return OS.KEYCODE_CONTROL;
			case KeyEvent.VK_SHIFT:				return OS.KEYCODE_SHIFT;

			// Third group: unsupported keycodes
			default:
			    return -1;

//			case KeyEvent.VK_STOP:				return OS.KEYCODE_
//			case KeyEvent.VK_PASTE:				return OS.KEYCODE_
//			case KeyEvent.VK_LESS:				return OS.KEYCODE_
//			case KeyEvent.VK_CANCEL:
//			case KeyEvent.VK_ACCEPT = 30;
//			case KeyEvent.VK_AGAIN = 65481;
//			case KeyEvent.VK_ALL_CANDIDATES = 256;
//			case KeyEvent.VK_ALPHANUMERIC = 240;
//			case KeyEvent.VK_ALT_GRAPH = 65406;
//			case KeyEvent.VK_BACK_QUOTE = 192;
//			case KeyEvent.VK_CLEAR = 12;
//			case KeyEvent.VK_CODE_INPUT = 258;
//			case KeyEvent.VK_COMPOSE = 65312;
//			case KeyEvent.VK_CONVERT = 28;
//			case KeyEvent.VK_COPY = 65485;
//			case KeyEvent.VK_CUT = 65489;
//			case KeyEvent.VK_DEAD_ABOVEDOT = 134;
//			case KeyEvent.VK_DEAD_ABOVERING = 136;
//			case KeyEvent.VK_DEAD_ACUTE = 129;
//			case KeyEvent.VK_DEAD_BREVE = 133;
//			case KeyEvent.VK_DEAD_CARON = 138;
//			case KeyEvent.VK_DEAD_CEDILLA = 139;
//			case KeyEvent.VK_DEAD_CIRCUMFLEX = 130;
//			case KeyEvent.VK_DEAD_DIAERESIS = 135;
//			case KeyEvent.VK_DEAD_DOUBLEACUTE = 137;
//			case KeyEvent.VK_DEAD_GRAVE = 128;
//			case KeyEvent.VK_DEAD_IOTA = 141;
//			case KeyEvent.VK_DEAD_MACRON = 132;
//			case KeyEvent.VK_DEAD_OGONEK = 140;
//			case KeyEvent.VK_DEAD_SEMIVOICED_SOUND = 143;
//			case KeyEvent.VK_DEAD_TILDE = 131;
//			case KeyEvent.VK_DEAD_VOICED_SOUND = 142;
//			case KeyEvent.VK_DECIMAL = 110;
//			case KeyEvent.VK_EURO_SIGN = 516;
//			case KeyEvent.VK_FINAL = 24;
//			case KeyEvent.VK_FIND = 65488;
//			case KeyEvent.VK_FULL_WIDTH = 243;
//			case KeyEvent.VK_GREATER = 160;
//			case KeyEvent.VK_HALF_WIDTH = 244;
//			case KeyEvent.VK_HIRAGANA = 242;
//			case KeyEvent.VK_INPUT_METHOD_ON_OFF = 263;
//			case KeyEvent.VK_INVERTED_EXCLAMATION_MARK = 518;
//			case KeyEvent.VK_JAPANESE_HIRAGANA = 260;
//			case KeyEvent.VK_JAPANESE_KATAKANA = 259;
//			case KeyEvent.VK_JAPANESE_ROMAN = 261;
//			case KeyEvent.VK_KANA = 21;
//			case KeyEvent.VK_KANA_LOCK = 262;
//			case KeyEvent.VK_KANJI = 25;
//			case KeyEvent.VK_KATAKANA = 241;
//			case KeyEvent.VK_META = 157;
//			case KeyEvent.VK_MODECHANGE = 31;
//			case KeyEvent.VK_NONCONVERT = 29;
//			case KeyEvent.VK_NUMBER_SIGN = 520;
//			case KeyEvent.VK_PREVIOUS_CANDIDATE = 257;
//			case KeyEvent.VK_PROPS = 65482;
//			case KeyEvent.VK_ROMAN_CHARACTERS = 245;
//			case KeyEvent.VK_SEMICOLON = 59;
//			case KeyEvent.VK_SEPARATER = 108;
//			case KeyEvent.VK_UNDEFINED = 0;
//			case KeyEvent.VK_UNDO = 65483;
	    }
	}

	/**
	 * Return the native keycode and character
	 * corresponding to the AWT keycode
	 * if the keycode = -1 ==> the awt keycode was correct but it is not supported in the native layer
	 * if the keycode = -2 ==> teh awt keycode was wrong!
	 */
	public static TranslatedKey convertAWTToNativeKey(int keycode) {
		int nativeKeycode;
		char nativeChar;

	    switch (keycode) {
	    	// First group: basic keycode set for PPro compliance
	    	// See the PPro Javadoc for java.awt.event.KeyEvent
	    	// and the "Basic Key Codes".
			case KeyEvent.VK_0: 				nativeKeycode = nativeChar = '0'; break;
			case KeyEvent.VK_1: 				nativeKeycode = nativeChar = '1'; break;
			case KeyEvent.VK_2:  				nativeKeycode = nativeChar = '2'; break;
			case KeyEvent.VK_3:  				nativeKeycode = nativeChar = '3'; break;
			case KeyEvent.VK_4:  				nativeKeycode = nativeChar = '4'; break;
			case KeyEvent.VK_5:  				nativeKeycode = nativeChar = '5'; break;
			case KeyEvent.VK_6:  				nativeKeycode = nativeChar = '6'; break;
			case KeyEvent.VK_7:  				nativeKeycode = nativeChar = '7'; break;
			case KeyEvent.VK_8:  				nativeKeycode = nativeChar = '8'; break;
			case KeyEvent.VK_9:  				nativeKeycode = nativeChar = '9'; break;
			case KeyEvent.VK_A:  				nativeKeycode = nativeChar = 'A'; break;
			case KeyEvent.VK_B:  				nativeKeycode = nativeChar = 'B'; break;
			case KeyEvent.VK_C:  				nativeKeycode = nativeChar = 'C'; break;
			case KeyEvent.VK_D:  				nativeKeycode = nativeChar = 'D'; break;
			case KeyEvent.VK_E:  				nativeKeycode = nativeChar = 'E'; break;
			case KeyEvent.VK_F:  				nativeKeycode = nativeChar = 'F'; break;
			case KeyEvent.VK_G:  				nativeKeycode = nativeChar = 'G'; break;
			case KeyEvent.VK_H:  				nativeKeycode = nativeChar = 'H'; break;
			case KeyEvent.VK_I:  				nativeKeycode = nativeChar = 'I'; break;
			case KeyEvent.VK_J:  				nativeKeycode = nativeChar = 'J'; break;
			case KeyEvent.VK_K:  				nativeKeycode = nativeChar = 'K'; break;
			case KeyEvent.VK_L:  				nativeKeycode = nativeChar = 'L'; break;
			case KeyEvent.VK_M:  				nativeKeycode = nativeChar = 'M'; break;
			case KeyEvent.VK_N:  				nativeKeycode = nativeChar = 'N'; break;
			case KeyEvent.VK_O:  				nativeKeycode = nativeChar = 'O'; break;
			case KeyEvent.VK_P:  				nativeKeycode = nativeChar = 'P'; break;
			case KeyEvent.VK_Q:  				nativeKeycode = nativeChar = 'Q'; break;
			case KeyEvent.VK_R:  				nativeKeycode = nativeChar = 'R'; break;
			case KeyEvent.VK_S:  				nativeKeycode = nativeChar = 'S'; break;
			case KeyEvent.VK_T:  				nativeKeycode = nativeChar = 'T'; break;
			case KeyEvent.VK_U:					nativeKeycode = nativeChar = 'U'; break;
			case KeyEvent.VK_V:					nativeKeycode = nativeChar = 'V'; break;
			case KeyEvent.VK_W:					nativeKeycode = nativeChar = 'W'; break;
			case KeyEvent.VK_X:					nativeKeycode = nativeChar = 'X'; break;
			case KeyEvent.VK_Y:					nativeKeycode = nativeChar = 'Y'; break;
			case KeyEvent.VK_Z:					nativeKeycode = nativeChar = 'Z'; break;
			case KeyEvent.VK_ENTER:				nativeKeycode = OS.KEYCODE_ENTER; nativeChar = '\u0000'; break;
			case KeyEvent.VK_BACK_SPACE:		nativeKeycode = OS.KEYCODE_BACKSPACE; nativeChar = '\b'; break;
			case KeyEvent.VK_TAB:				nativeKeycode = OS.KEYCODE_TAB; nativeChar = '\t'; break;
			case KeyEvent.VK_SPACE:				nativeKeycode = nativeChar = ' '; break;
			case KeyEvent.VK_LEFT:				nativeKeycode = OS.KEYCODE_ARROW_LEFT; nativeChar = '\u0000'; break;
			case KeyEvent.VK_RIGHT:				nativeKeycode = OS.KEYCODE_ARROW_RIGHT; nativeChar = '\u0000'; break;
			case KeyEvent.VK_UP:				nativeKeycode = OS.KEYCODE_ARROW_UP; nativeChar = '\u0000'; break;
			case KeyEvent.VK_DOWN:				nativeKeycode = OS.KEYCODE_ARROW_DOWN; nativeChar = '\u0000'; break;

			// Second group: other keycodes which UGL supports
			case KeyEvent.VK_F1:				nativeKeycode = OS.KEYCODE_F1; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F2:				nativeKeycode = OS.KEYCODE_F2; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F3:				nativeKeycode = OS.KEYCODE_F3; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F4:				nativeKeycode = OS.KEYCODE_F4; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F5:				nativeKeycode = OS.KEYCODE_F5; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F6:				nativeKeycode = OS.KEYCODE_F6; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F7:				nativeKeycode = OS.KEYCODE_F7; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F8:				nativeKeycode = OS.KEYCODE_F8; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F9:				nativeKeycode = OS.KEYCODE_F9; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F10:				nativeKeycode = OS.KEYCODE_F10; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F11:				nativeKeycode = OS.KEYCODE_F11; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F12:				nativeKeycode = OS.KEYCODE_F12; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F13:				nativeKeycode = OS.KEYCODE_F13; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F14:				nativeKeycode = OS.KEYCODE_F14; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F15:				nativeKeycode = OS.KEYCODE_F15; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F16:				nativeKeycode = OS.KEYCODE_F16; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F17:				nativeKeycode = OS.KEYCODE_F17; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F18:				nativeKeycode = OS.KEYCODE_F18; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F19:				nativeKeycode = OS.KEYCODE_F19; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F20:				nativeKeycode = OS.KEYCODE_F20; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F21:				nativeKeycode = OS.KEYCODE_F21; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F22:				nativeKeycode = OS.KEYCODE_F22; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F23:				nativeKeycode = OS.KEYCODE_F23; nativeChar = '\u0000'; break;
			case KeyEvent.VK_F24:				nativeKeycode = OS.KEYCODE_F24; nativeChar = '\u0000'; break;
			case KeyEvent.VK_PLUS:				nativeKeycode = nativeChar = '+'; break;
			case KeyEvent.VK_MINUS:				nativeKeycode = nativeChar = '-'; break;
			case KeyEvent.VK_ASTERISK:			nativeKeycode = nativeChar = '*'; break;
			case KeyEvent.VK_AMPERSAND:			nativeKeycode = nativeChar = '&'; break;
			case KeyEvent.VK_AT:				nativeKeycode = nativeChar = '@'; break;
			case KeyEvent.VK_COLON:				nativeKeycode = nativeChar = ':'; break;
			case KeyEvent.VK_SEMICOLON:			nativeKeycode = nativeChar = ';'; break;
			case KeyEvent.VK_COMMA:				nativeKeycode = nativeChar = ','; break;
			case KeyEvent.VK_BRACELEFT:			nativeKeycode = nativeChar = '{'; break;
			case KeyEvent.VK_BRACERIGHT:		nativeKeycode = nativeChar = '}'; break;
			case KeyEvent.VK_DOLLAR:			nativeKeycode = nativeChar = '$'; break;
			case KeyEvent.VK_EXCLAMATION_MARK:	nativeKeycode = nativeChar = '!'; break;
			case KeyEvent.VK_EQUALS:			nativeKeycode = nativeChar = '='; break;
			case KeyEvent.VK_LEFT_PARENTHESIS:	nativeKeycode = nativeChar = '('; break;
			case KeyEvent.VK_RIGHT_PARENTHESIS:	nativeKeycode = nativeChar = ')'; break;
			case KeyEvent.VK_NUMBER_SIGN:		nativeKeycode = nativeChar = '#'; break;
			case KeyEvent.VK_ADD:				nativeKeycode = OS.KEYCODE_KEYPAD_ADD; nativeChar = '0'; break;
			case KeyEvent.VK_MULTIPLY:			nativeKeycode = OS.KEYCODE_KEYPAD_MULTIPLY; nativeChar = '0'; break;
			case KeyEvent.VK_SUBTRACT:			nativeKeycode = OS.KEYCODE_KEYPAD_SUBTRACT; nativeChar = '0'; break;
			case KeyEvent.VK_DIVIDE:			nativeKeycode = OS.KEYCODE_KEYPAD_DIVIDE; nativeChar = '0'; break;
			case KeyEvent.VK_DECIMAL:			nativeKeycode = OS.KEYCODE_KEYPAD_DECIMAL; nativeChar = '0'; break;
			case KeyEvent.VK_NUMPAD0:			nativeKeycode = OS.KEYCODE_KEYPAD_0; nativeChar = '0'; break;
			case KeyEvent.VK_NUMPAD1:			nativeKeycode = OS.KEYCODE_KEYPAD_1; nativeChar = '1'; break;
			case KeyEvent.VK_NUMPAD2:			nativeKeycode = OS.KEYCODE_KEYPAD_2; nativeChar = '2'; break;
			case KeyEvent.VK_NUMPAD3:			nativeKeycode = OS.KEYCODE_KEYPAD_3; nativeChar = '3'; break;
			case KeyEvent.VK_NUMPAD4:			nativeKeycode = OS.KEYCODE_KEYPAD_4; nativeChar = '4'; break;
			case KeyEvent.VK_NUMPAD5:			nativeKeycode = OS.KEYCODE_KEYPAD_5; nativeChar = '5'; break;
			case KeyEvent.VK_NUMPAD6:			nativeKeycode = OS.KEYCODE_KEYPAD_6; nativeChar = '6'; break;
			case KeyEvent.VK_NUMPAD7:			nativeKeycode = OS.KEYCODE_KEYPAD_7; nativeChar = '7'; break;
			case KeyEvent.VK_NUMPAD8:			nativeKeycode = OS.KEYCODE_KEYPAD_8; nativeChar = '8'; break;
			case KeyEvent.VK_NUMPAD9:			nativeKeycode = OS.KEYCODE_KEYPAD_9; nativeChar = '9'; break;
			case KeyEvent.VK_OPEN_BRACKET:		nativeKeycode = nativeChar = '['; break;
			case KeyEvent.VK_CLOSE_BRACKET:		nativeKeycode = nativeChar = ']'; break;
			case KeyEvent.VK_UNDERSCORE:		nativeKeycode = nativeChar = '_'; break;
			case KeyEvent.VK_SLASH:				nativeKeycode = nativeChar = '/'; break;
			case KeyEvent.VK_BACK_SLASH:		nativeKeycode = nativeChar = '\\'; break;
			case KeyEvent.VK_CAPS_LOCK:			nativeKeycode = OS.KEYCODE_LOCK_CAPS; nativeChar = '\u0000'; break;
			case KeyEvent.VK_NUM_LOCK:			nativeKeycode = OS.KEYCODE_LOCK_NUM; nativeChar = '\u0000'; break;
			case KeyEvent.VK_SCROLL_LOCK:		nativeKeycode = OS.KEYCODE_LOCK_SCROLL; nativeChar = '\u0000'; break;
			case KeyEvent.VK_DELETE:			nativeKeycode = OS.KEYCODE_DELETE; nativeChar = '\u0000'; break;
			case KeyEvent.VK_END:				nativeKeycode = OS.KEYCODE_END; nativeChar = '\u0000'; break;
			case KeyEvent.VK_ESCAPE:			nativeKeycode = OS.KEYCODE_ESCAPE; nativeChar = '\u0000'; break;
			case KeyEvent.VK_HOME:				nativeKeycode = OS.KEYCODE_HOME; nativeChar = '\u0000'; break;
			case KeyEvent.VK_KP_DOWN:			nativeKeycode = OS.KEYCODE_ARROW_DOWN; nativeChar = '\u0000'; break;
			case KeyEvent.VK_KP_LEFT:			nativeKeycode = OS.KEYCODE_ARROW_LEFT; nativeChar = '\u0000'; break;
			case KeyEvent.VK_KP_RIGHT:			nativeKeycode = OS.KEYCODE_ARROW_RIGHT; nativeChar = '\u0000'; break;
			case KeyEvent.VK_KP_UP:				nativeKeycode = OS.KEYCODE_ARROW_UP; nativeChar = '\u0000'; break;
			case KeyEvent.VK_HELP:				nativeKeycode = OS.KEYCODE_HELP; nativeChar = '\u0000'; break;
			case KeyEvent.VK_INSERT:			nativeKeycode = OS.KEYCODE_INSERT; nativeChar = '\u0000'; break;
			case KeyEvent.VK_PAGE_DOWN:			nativeKeycode = OS.KEYCODE_PAGE_DOWN; nativeChar = '\u0000'; break;
			case KeyEvent.VK_PAGE_UP:			nativeKeycode = OS.KEYCODE_PAGE_UP; nativeChar = '\u0000'; break;
			case KeyEvent.VK_PAUSE:				nativeKeycode = OS.KEYCODE_PAUSE; nativeChar = '\u0000'; break;
			case KeyEvent.VK_PERIOD:			nativeKeycode = nativeChar = '.'; break;
			case KeyEvent.VK_QUOTE:				nativeKeycode = nativeChar = '\''; break;
			case KeyEvent.VK_QUOTEDBL:			nativeKeycode = nativeChar = '\"'; break;
			case KeyEvent.VK_BACK_QUOTE:		nativeKeycode = nativeChar = '`'; break;
			case KeyEvent.VK_PRINTSCREEN:		nativeKeycode = OS.KEYCODE_PRINTSCREEN; nativeChar = '\u0000'; break;

			case KeyEvent.VK_ALT:				nativeKeycode = OS.KEYCODE_ALT; nativeChar = '\u0000'; break;
			case KeyEvent.VK_CIRCUMFLEX:		nativeKeycode = nativeChar = '^'; break;
			case KeyEvent.VK_CONTROL:			nativeKeycode = OS.KEYCODE_CONTROL; nativeChar = '\u0000'; break;
			case KeyEvent.VK_SHIFT:				nativeKeycode = OS.KEYCODE_SHIFT; nativeChar = '\u0000'; break;

			// Third group: unsupported keycodes
			case KeyEvent.VK_STOP:
			case KeyEvent.VK_PASTE:
			case KeyEvent.VK_LESS:
			case KeyEvent.VK_CANCEL:
			case KeyEvent.VK_ACCEPT:
			case KeyEvent.VK_AGAIN:
			case KeyEvent.VK_ALL_CANDIDATES:
			case KeyEvent.VK_ALPHANUMERIC:
			case KeyEvent.VK_ALT_GRAPH:
			case KeyEvent.VK_CLEAR:
			case KeyEvent.VK_CODE_INPUT:
			case KeyEvent.VK_COMPOSE:
			case KeyEvent.VK_CONVERT:
			case KeyEvent.VK_COPY:
			case KeyEvent.VK_CUT:
			case KeyEvent.VK_DEAD_ABOVEDOT:
			case KeyEvent.VK_DEAD_ABOVERING:
			case KeyEvent.VK_DEAD_ACUTE:
			case KeyEvent.VK_DEAD_BREVE:
			case KeyEvent.VK_DEAD_CARON:
			case KeyEvent.VK_DEAD_CEDILLA:
			case KeyEvent.VK_DEAD_CIRCUMFLEX:
			case KeyEvent.VK_DEAD_DIAERESIS:
			case KeyEvent.VK_DEAD_DOUBLEACUTE:
			case KeyEvent.VK_DEAD_GRAVE:
			case KeyEvent.VK_DEAD_IOTA:
			case KeyEvent.VK_DEAD_MACRON:
			case KeyEvent.VK_DEAD_OGONEK:
			case KeyEvent.VK_DEAD_SEMIVOICED_SOUND:
			case KeyEvent.VK_DEAD_TILDE:
			case KeyEvent.VK_DEAD_VOICED_SOUND:
			case KeyEvent.VK_EURO_SIGN:
			case KeyEvent.VK_FINAL:
			case KeyEvent.VK_FIND:
			case KeyEvent.VK_FULL_WIDTH:
			case KeyEvent.VK_GREATER:
			case KeyEvent.VK_HALF_WIDTH:
			case KeyEvent.VK_HIRAGANA:
			case KeyEvent.VK_INPUT_METHOD_ON_OFF:
			case KeyEvent.VK_INVERTED_EXCLAMATION_MARK:
			case KeyEvent.VK_JAPANESE_HIRAGANA:
			case KeyEvent.VK_JAPANESE_KATAKANA:
			case KeyEvent.VK_JAPANESE_ROMAN:
			case KeyEvent.VK_KANA:
			case KeyEvent.VK_KANA_LOCK:
			case KeyEvent.VK_KANJI:
			case KeyEvent.VK_KATAKANA:
			case KeyEvent.VK_META:
			case KeyEvent.VK_MODECHANGE:
			case KeyEvent.VK_NONCONVERT:
			case KeyEvent.VK_PREVIOUS_CANDIDATE:
			case KeyEvent.VK_PROPS:
			case KeyEvent.VK_ROMAN_CHARACTERS:
			case KeyEvent.VK_SEPARATER:
			case KeyEvent.VK_UNDEFINED:
			case KeyEvent.VK_UNDO:
				nativeKeycode = -1; nativeChar = '\u0000'; break;
			default:
				nativeKeycode = -2; nativeChar = '\u0000'; break;
		}
	    return new TranslatedKey(nativeKeycode, nativeChar, 0);
	}

	public static int convertToMouseEventButton(int awtButton) {
		if ((awtButton & InputEvent.BUTTON1_DOWN_MASK) != 0) {
			return MouseEvent.BUTTON1;
		}
		if ((awtButton & InputEvent.BUTTON2_DOWN_MASK) != 0) {
			return MouseEvent.BUTTON2;
		}
		if ((awtButton & InputEvent.BUTTON3_DOWN_MASK) != 0) {
			return MouseEvent.BUTTON3;
		}
		return MouseEvent.NOBUTTON;
	}
}
