package com.ibm.oti.awt.image;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.awt.Rectangle;
import java.awt.image.ColorModel;
import java.awt.image.ImageConsumer;
import java.awt.image.ImageObserver;
import java.util.Hashtable;

import com.ibm.oti.awt.metal.image.ImagePeer;

abstract public class AbstractImageConsumer implements ImageConsumer {

	IImageImpl fImageImpl;
	boolean fPixelsReceived;

	public AbstractImageConsumer(IImageImpl imageImpl) {
		fImageImpl= imageImpl;
		flush();
	}

	public void imageComplete(int status) {
		ImagePeer imagePeer= null;
		if (pixelsReceived()) {
			fImageImpl.setStatus(IImageImpl.LOADING);
			imagePeer= createImagePeer();
		}

		if (imagePeer == null) {
			fImageImpl.setStatus(IImageImpl.ERRORED);
			fImageImpl.notifyObservers(ImageObserver.ERROR | ImageObserver.ABORT, 0, 0, 0, 0);
			flush();
			return;
		}

		Rectangle bounds= imagePeer.getBounds();

		fImageImpl.setImagePeer(imagePeer); // should handle the disposal of its image peer

		// Only STATICIMAGEDONE means that the entire image has been
		// loaded
		if (status == ImageConsumer.STATICIMAGEDONE) {
			fImageImpl.setStatus(IImageImpl.FINISHED);
			fImageImpl.notifyObservers(ImageObserver.ALLBITS, 0, 0, bounds.width, bounds.height);
		} else {
			fImageImpl.setStatus(IImageImpl.FRAME_DONE);
			fImageImpl.notifyObservers(ImageObserver.FRAMEBITS, 0, 0, bounds.width, bounds.height);
		}

		flush();
	}

	public void setProperties(Hashtable properties) {
		fImageImpl.setProperties(properties);
		fImageImpl.notifyObservers(ImageObserver.PROPERTIES, 0, 0, 0, 0);
	}

	public void setColorModel(ColorModel colorModel) {
	}

	public void setDimensions(int width, int height) {
		fImageImpl.notifyObservers(ImageObserver.WIDTH | ImageObserver.HEIGHT, 0, 0, width, height);
	}

	public void setHints(int hints) {
	}

	public void setPixels(int x, int y, int w, int h, ColorModel cm, int[] p, int off, int scan) {
		setPixelsReceived(true);
		fImageImpl.notifyObservers(ImageObserver.SOMEBITS, x, y, w, h);
	}

	public void setPixels(int x, int y, int w, int h, ColorModel cm, byte[] p, int off, int scan) {
		setPixelsReceived(true);
		fImageImpl.notifyObservers(ImageObserver.SOMEBITS, x, y, w, h);
	}

	boolean pixelsReceived() {
		return fPixelsReceived;
	}

	void setPixelsReceived(boolean pixelsReceived) {
		fPixelsReceived = pixelsReceived;
	}

	void flush() {
		setPixelsReceived(false);
	}

	abstract ImagePeer createImagePeer();
}
