package com.ibm.oti.awt.image;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.awt.Rectangle;
import java.awt.image.ColorModel;
import java.awt.image.ImageConsumer;
import java.awt.image.ImageObserver;
import java.awt.image.ImageProducer;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;

import com.ibm.oti.awt.metal.image.ImagePeer;

abstract public class AbstractImageProducer implements ImageProducer {

	Vector fConsumers= new Vector();

	public static void buildImageFromProducer(ImageProducer producer, IImageImpl target) {
		if (producer instanceof AbstractImageProducer) {
			((AbstractImageProducer)producer).startProduction(null, target);
		} else {
			producer.startProduction(new NativeImageConsumer(target));
		}
	}

	public synchronized void addConsumer(ImageConsumer imageConsumer) {
		if (isConsumer(imageConsumer))
			return;

		fConsumers.addElement(imageConsumer);
	}

	public synchronized boolean isConsumer(ImageConsumer imageConsumer) {
		return fConsumers.contains(imageConsumer);
	}

	public synchronized void removeConsumer(ImageConsumer imageConsumer) {
		fConsumers.removeElement(imageConsumer);
	}

	public void requestTopDownLeftRightResend(ImageConsumer imageConsumer) {
		//TODO Need to implement requestTopDownLeftRightResend()
	}

	abstract ImagePeer getImagePeer();

	private void startProduction(ImageConsumer consumer, IImageImpl target) {
		// Create the peer, do this before copying the consumer list so that
		// if a consumer is added while the image is loading then it will not have to wait
		// for the new startProduction call.
		ImagePeer tImagePeer= getImagePeer();

		// Copy the consumers locally so that this whole method does
		// not have to be synchronized just to prevent new consumers being added
		// during production.  Any consumers that get added after this point will
		// only be called during the next startProduction cycle
		ArrayList consumers;
		synchronized (this) {
			consumers = new ArrayList(fConsumers);
			fConsumers.removeAllElements();
		}
		if (consumer != null) {
			consumers.add(consumer);
		}

		if (tImagePeer == null) {
			if (target != null) {
				target.setStatus(IImageImpl.ERRORED);
				target.notifyObservers(ImageObserver.ERROR | ImageObserver.ABORT, 0, 0, 0, 0);
			}

			for (int i=0; i<consumers.size(); i++) {
				ImageConsumer ic= (ImageConsumer) consumers.get(i);
				ic.imageComplete(ImageConsumer.IMAGEERROR); // return only image error
			}
			return;
		}

		if (target != null) {
			target.setStatus(IImageImpl.LOADING);
		}

		ColorModel colorModel= getColorModel(tImagePeer);
		Hashtable properties= new Hashtable();

		Rectangle peerBounds = tImagePeer.getBounds();
		if (consumers.size() > 0) {
			Object pixels;
			if (tImagePeer.getDepth() <= 8) {
				pixels= tImagePeer.getPixelsLowColor();
			} else {
				pixels= tImagePeer.getPixelsHighColor();
			}

			for (int i=0; i<consumers.size(); i++) {
				ImageConsumer ic= (ImageConsumer) consumers.get(i);

				ic.setDimensions(peerBounds.width, peerBounds.height);
				ic.setProperties(properties);
				ic.setColorModel(colorModel);
				ic.setHints(ImageConsumer.SINGLEPASS | ImageConsumer.SINGLEFRAME);

				if (tImagePeer.getDepth() <= 8) {
					ic.setPixels(peerBounds.x, peerBounds.y, peerBounds.width, peerBounds.height,
						colorModel, (byte[])pixels, 0, peerBounds.width);
				}
				else {
					ic.setPixels(peerBounds.x, peerBounds.y, peerBounds.width, peerBounds.height,
						colorModel, (int[])pixels, 0, peerBounds.width);
				}

				ic.imageComplete(ImageConsumer.STATICIMAGEDONE);
			}
		}

		if (target == null) {
			tImagePeer.dispose();
		} else {
			target.setImagePeer(tImagePeer);
			target.setProperties(properties);
			target.notifyObservers(ImageObserver.PROPERTIES, 0, 0, 0, 0);
			target.notifyObservers(ImageObserver.WIDTH | ImageObserver.HEIGHT, 0, 0, peerBounds.width, peerBounds.height);
			target.setStatus(IImageImpl.FINISHED);
			target.notifyObservers(ImageObserver.ALLBITS, 0, 0, peerBounds.width, peerBounds.height);
		}
	}

	public void startProduction(ImageConsumer imageConsumer) {
		startProduction(imageConsumer, null);
	}

	ColorModel getColorModel(ImagePeer peer) {
		return peer.getColorModel();
	}

}
