package com.ibm.oti.awt.image;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2005  All Rights Reserved
 */

import java.awt.GraphicsConfiguration;
import java.awt.image.BufferedImage;

public class BufferedImageHandler {

	private static IImageFactory gImageFactory;

	private static void initializeImageFactory() {
		// Touch BufferedImage so its static initializer will set the factory.
		try {
			Class.forName("java.awt.image.BufferedImage"); //$NON-NLS-1$
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}

	private static IImageFactory getImageFactory() {
		if (gImageFactory == null) {
			// lazy initialize instead of using static initializer because otherwise BufferImageHandler
			// and BufferedImage static initializers would be dependent on each other and cause a
			// deadlock when two different threads load each of the classes at the same time.
			initializeImageFactory();
		}
		return gImageFactory;
	}

	static public void setImageFactory(IImageFactory imageFactory) {
		BufferedImageHandler.gImageFactory= imageFactory;
	}

	static public BufferedImage createImage(GraphicsConfiguration graphicsConfiguration, int width, int height) {
		return getImageFactory().createImage(graphicsConfiguration, width, height);
	}

	static public IImageImpl getIImageImpl(Object image) {
		return getImageFactory().getIImageImpl(image);
	}

	static public IImageImpl createIImageImpl(Object image) {
		return getImageFactory().createIImageImpl(image);
	}
}
