package com.ibm.oti.awt.metal.graphics;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2004, 2005  All Rights Reserved
 */

import java.awt.Point;
import java.awt.image.*;

import com.ibm.oti.awt.metal.NativeObject;
import com.ibm.ugl.eswt.OS;
public class Device extends NativeObject {

	public static boolean convertToRGB(ColorModel colorModel) {
		boolean convert = false;

		if (colorModel instanceof DirectColorModel) {
			final int RedMask = 0xff0000;
			final int GreenMask = 0xff00;
			final int BlueMask = 0xff;
			final int AlphaMask = 0xff000000;
			DirectColorModel directColorModel = (DirectColorModel) colorModel;

			// Color conversion required if masks do not match UGL API requirements
			if (directColorModel.getRedMask() != RedMask ||
				directColorModel.getGreenMask() != GreenMask ||
				directColorModel.getBlueMask() != BlueMask ||
				directColorModel.getAlphaMask() != AlphaMask)

			{
				convert = true;
			}
		} else if (colorModel.getPixelSize() > 8) {
			// Always use color model to convert to RGB when more than 8bpp
			// are used. Saves palette space.
			convert = true;
		}
		return convert;
	}

	public String[] getFontList(String faceName, boolean scalable) {
		String[] typefaces = OS.Device_GetTypefaces(this.fPeerHandle,scalable);
		if (faceName != null) {
			for (int i=0; i<typefaces.length; i++) {
				if (faceName.equals(typefaces[i])) {
					return new String[] { faceName };
				}
			}
			// did not find it
			return new String[0];
		}
		return typefaces;
	}

	public ColorModel getDefaultColorModel() {
		ColorModel colorModel = null;
		int depth = OS.Device_GetDepth(fPeerHandle);

		if (OS.Device_IsDirect(fPeerHandle)) {
			int[] masks = OS.Device_GetDirectPaletteMasks(fPeerHandle);
			if (masks != null) {
				colorModel = new DirectColorModel(depth, masks[OS.PALETTE_MASK_RED], masks[OS.PALETTE_MASK_GREEN], masks[OS.PALETTE_MASK_BLUE], masks[OS.PALETTE_MASK_ALPHA]);
			} // TODO: should throw exception if 'masks' is null?
		} else {
			int numColors = (int) Math.pow(2, depth);
			byte[] red = new byte[numColors];
			byte[] green = new byte[numColors];
			byte[] blue = new byte[numColors];

			OS.Device_GetIndexedPalette(fPeerHandle, red, green, blue);
			colorModel = new IndexColorModel(depth, numColors, red, green, blue);
		}
		return colorModel;
	}

	public Point getDPI() {
		int[] point = OS.Device_GetDPI(this.fPeerHandle);
		java.awt.Point returnValue = new java.awt.Point(point[OS.INDEX_X],point[OS.INDEX_Y]);
		return returnValue;
	}

	public Device(int handle) {
		super(handle);
	}

	public void dispose() {
		if (fPeerHandle == 0) return;
		OS.Device_Dispose(fPeerHandle);
		fPeerHandle = 0;
	}

	public int getAvailableAcceleratedMemory() {
		if (fPeerHandle == 0) return 0;
		return OS.Device_GetAvailableAcceleratedMemory(fPeerHandle);
	}
}
