package com.ibm.oti.awt.metal.graphics;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2004, 2005  All Rights Reserved
 */

import java.awt.Font;

import com.ibm.oti.awt.metal.NativeObject;
import com.ibm.ugl.eswt.OS;
public class FontPeer extends NativeObject {

	int referenceCount;

	public FontPeer(int handle) {
		super(handle);
	}

	public FontPeer(Device device, String fontFamily, int height, int style) {
		super(OS.Font_New((device == null ? 0 : device.fPeerHandle), fontFamily, height, convertToNativeStyle(style)));
	}

	public FontPeer(Device device, int logicalFont, int height, int style) {
		super(OS.Font_NewLogical((device == null ? 0 : device.fPeerHandle), logicalFont, height, convertToNativeStyle(style)));
	}

	public void addReference() {
		referenceCount++;
	}

	public int removeReference() {
		referenceCount--;
		return referenceCount;
	}

	public int stringWidth(String string) {
		return OS.Font_StringAdvance(fPeerHandle, string);
	}

	public int charWidth(char ch) {
		return OS.Font_GetAdvanceWidth(fPeerHandle, ch);
	}

	public void dispose() {
		if (fPeerHandle == 0) return;
		OS.Font_Dispose(this.fPeerHandle);
		fPeerHandle = 0;
	}

	public int getAscent() {
		int returnValue = OS.Font_GetAscent(this.fPeerHandle);
		return returnValue;
	}

	public int getLeading() {
		int returnValue = OS.Font_GetLeading(this.fPeerHandle);
		return returnValue;
	}

	public int getDescent() {
		int returnValue = OS.Font_GetDescent(this.fPeerHandle);
		return returnValue;
	}

	public String getName() {
		return OS.Font_GetName(fPeerHandle);
	}

	public int getSizeInPoints() {
		return OS.Font_GetSizeInPoints(fPeerHandle);
	}

	public int getStyle() {
		int pproStyle = 0;
		int uglStyle = OS.Font_GetStyle(fPeerHandle);
		if ((uglStyle & OS.FONT_STYLE_BOLD) == OS.FONT_STYLE_BOLD) pproStyle |= Font.BOLD;
		if ((uglStyle & OS.FONT_STYLE_ITALIC) == OS.FONT_STYLE_ITALIC) pproStyle |= Font.ITALIC;

		return pproStyle;
	}

	public int getExtraStyles() {
		int ppro11Style = 0;
		int uglStyle = OS.Font_GetStyle(fPeerHandle);
		if ((uglStyle & OS.FONT_STYLE_UNDERLINED) == OS.FONT_STYLE_UNDERLINED) ppro11Style |= OS.FONT_STYLE_UNDERLINED;
		if ((uglStyle & OS.FONT_STYLE_STRIKETHROUGH) == OS.FONT_STYLE_STRIKETHROUGH) ppro11Style |= OS.FONT_STYLE_STRIKETHROUGH;

		return ppro11Style;
	}

	static int convertToNativeStyle(int pproStyle) {
	    int nativeStyle = 0;
	    if ((pproStyle & Font.BOLD) == Font.BOLD) {
	        nativeStyle |= OS.FONT_STYLE_BOLD;
	    }
	    if ((pproStyle & Font.ITALIC) == Font.ITALIC) {
	        nativeStyle |= OS.FONT_STYLE_ITALIC;
	    }
	    if ((pproStyle & OS.FONT_STYLE_UNDERLINED) == OS.FONT_STYLE_UNDERLINED) {
	        nativeStyle |= OS.FONT_STYLE_UNDERLINED;
	    }
	    if ((pproStyle & OS.FONT_STYLE_STRIKETHROUGH) == OS.FONT_STYLE_STRIKETHROUGH) {
	        nativeStyle |= OS.FONT_STYLE_STRIKETHROUGH;
	    }
	    return nativeStyle;
	}
}
