package com.ibm.oti.connection.socket;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2000, 2005  All Rights Reserved
 */

import com.ibm.oti.util.Inet6Util;
import java.security.AccessController;
import com.ibm.oti.util.PriviAction;

public abstract class Socket {
	// signals that when SO_LINGER is enabled and shutdown() is called, a subsequent call to closesocket() will unnecessarily hang
	public static final int FLAG_BROKEN_SO_LINGER_SHUTDOWN = 8;
public static native int getSocketFlags();
public static String parseURL(String spec, int[] returnPort, boolean portRequired, boolean allowNoPort) {
	if (spec.startsWith("//")) spec = spec.substring(2);
	int index = spec.lastIndexOf(':');
	int endOfIPv6Addr = spec.lastIndexOf(']');
	/* if there is no colon, or it is part of the IPv6 address contained in [] */
	if ((index == -1) || (index < endOfIPv6Addr)) {
		if (portRequired)
			// K00a6 = Must specify port
			throw new IllegalArgumentException(com.ibm.oti.util.Msg.getString("K00a6"));
		return spec;
	}
	if ( (-1 != spec.indexOf('/')) || (-1 != spec.indexOf('@')) || (index != spec.indexOf(':')) ||
			(-1 != spec.indexOf('?')) || (-1 != spec.indexOf(';'))) {
		// K039d = host cannot contain '/', '@', '?', ';', ':'
		throw new IllegalArgumentException(com.ibm.oti.util.Msg.getString("K039d"));
	}
	int port;
	String portString = spec.substring(index + 1, spec.length());
	if (allowNoPort && portString.equals("")) port = 0;
	else {
		try {
			port = Integer.parseInt(portString);
		} catch (NumberFormatException e) {
			// K00a7 = Invalid port: {0}
			throw new IllegalArgumentException(com.ibm.oti.util.Msg.getString("K00a7", portString));
		}
	}
	if (port < 0 || port > 65535)
		// K0325 = Port out of range: {0}
		throw new IllegalArgumentException(com.ibm.oti.util.Msg.getString("K0325", port));
	returnPort[0] = port;
	return spec.substring(0, index);
}
}
