package com.ibm.oti.crypto;

import java.io.IOException;

import com.ibm.j9.bluez.crypto.AESCipher;
import com.ibm.j9.bluez.crypto.CL3;

/**
 * Implements an AES Provider in the context of the <code>Provider</code> framework
 * <p>
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2000, 2006  All Rights Reserved
 * </p>
  */
public class AESProvider extends CL3BasedProvider {

	//TODO: make sure this value is correct
	private static final int AES_EFFECT_KEY_LENGTH_BITS = 128;	// required effective key length in bits

	/**
	 * Constructs a AESProvider supporting the effective key length provided
	 *
	 * @param effectiveKeyBitLength
	 *
	 * @throws IOException If effectiveKeyBitLength not supported by <code>AESProvider</code>
	 */
	public AESProvider( int effectiveKeyBitLength) throws IOException {
		super(Provider.ALG_AES, effectiveKeyBitLength);
		if (effectiveKeyBitLength != AES_EFFECT_KEY_LENGTH_BITS ) {
			// K01f9 = incorrect key length
			throw new IOException(com.ibm.oti.util.Msg.getString( "K01f9" ) ); //$NON-NLS-1$
		}
	}

	public Key createKey(byte[] keybytes) throws IOException {
		return new AESKey(this, keybytes);
	}

	void cl3Call(CL3 workingKey, int mode, byte[] iv, int ivOffset, byte[] data, int dataOffset, byte[] output, int outputOffset, int outputLength) {
		AESCipher.aes(workingKey, mode, iv, ivOffset, data, dataOffset, output, outputOffset, outputLength);
	}
}
