
package com.ibm.oti.net.www;

import java.util.Properties;
import java.io.*;
import java.security.AccessController;
import java.security.PrivilegedAction;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * Instances of this class map file extensions to MIME content
 * types based on a default MIME table.
 * <p>
 * The default values can be changed by modifying the
 * contents of the file lib/content-types.properties.
 *
 * @author		OTI
 * @version		initial
 */

public class MimeTable implements java.net.FileNameMap {

	public static final String UNKNOWN = "content/unknown";

	/**
	 * A hash table containing the mapping between extensions
	 * and mime types.
	 */
	public static final Properties types = new Properties();

/**
 * Contructs a MIME table using the default values defined in
 * this class.
 * <p>
 * It then augments this table by reading pairs of extensions and
 * matching content types from the file "types.properties", which
 * is represented in standard java.util.Properties.load(...) format.
 *
 * @author		OTI
 * @version		initial
 */
public MimeTable() {
	InputStream str = (InputStream)AccessController.doPrivileged(new PrivilegedAction() {
		public Object run() {
			return getContentTypes();
		}});

	if (str != null) {
		try {
			try {
				types.load(str);
			} finally {
				str.close();
			}
		} catch (IOException ex) {
		}
	}
}

private InputStream getContentTypes() {
	String userTable = System.getProperty("content.types.user.table");
	if (userTable != null) {
		try {
			return new FileInputStream(userTable);
		} catch (IOException e) {
		}
	}
	String javahome = System.getProperty("java.home");
	File contentFile = new File(javahome, "lib/content-types.properties");
	try {
		return new FileInputStream(contentFile);
	} catch (IOException e) {
		return null;
	}
}

/**
 * Determines the MIME type for the given filename.
 *
 * @author		OTI
 * @version		initial
 *
 * @return 		java.lang.String
 *					The mime type associated with the file's extension
 *					or null if no mapping is known.
 * @param 		filename java.lang.String
 *					The file whose extension will be mapped.
 */
public String getContentTypeFor(String filename) {
	if (filename.endsWith("/"))
		// a directory, return html
		return (String)types.get("html");
	int lastCharInExtension = filename.lastIndexOf('#');
	if (lastCharInExtension < 0)
		lastCharInExtension = filename.length();
	int firstCharInExtension = filename.lastIndexOf('.') + 1;
	String ext="";
	if (firstCharInExtension > filename.lastIndexOf('/'))
		ext = filename.substring(firstCharInExtension, lastCharInExtension);
	return (String)types.get(ext.toLowerCase());
}

}
