package com.ibm.oti.net.www.protocol.file;

import java.net.URL;
import java.net.URLConnection;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * This is the handler that is responsible for reading files from the file system.
 *
 * @author		OTI
 * @version		initial
 */

public class Handler extends java.net.URLStreamHandler {

/**
 * Answers a connection to the a file pointed by this <code>URL</code> in the file system
 *
 * @author		OTI
 * @version		initial
 *
 * @return java.net.URLConnection		A connection to the resource pointed by this url.
 * @param url java.net.URL				The URL to which the connection is pointing to
 *
 */
public URLConnection openConnection(URL url) {
	return new FileURLConnection(url);
}

/**
 * Parse the <code>string</code>str into <code>URL</code> u which already
 * have the context properties. The string generally have the following format:
 * <code><center>/c:/windows/win.ini</center></code>.
 *
 * @author		OTI
 * @version		initial
 *
 * @param u java.net.URL		The URL object that's parsed into
 * @param str java.lang.String	The string equivalent of the specification URL
 * @param start int				The index in the spec string from which to begin parsing
 * @param end int				The index to stop parsing
 *
 * @see 		java.net.URLStreamHandler#toExternalForm(URL)
 * @see 		java.net.URL
 */
protected void parseURL(URL u, String str, int start, int end) {
	if (end < start) return;
	String parseString = "";
	if (start < end)
		parseString = str.substring(start, end).replace('\\', '/');
	super.parseURL(u, parseString, 0, parseString.length());
}

/**
 * Processes any relative path modifiers in the path string
 *
 * @author		OTI
 * @version		initial
 *
 * @return java.lang.String				canonized path string
 * @param path java.lang.String			normal path string
 */
private static String canonize(String path) {
	int dirIndex = 0;
	String file = path;
	if (file.startsWith("./") && file.length() > 2) file = file.substring(2);
	if (file.endsWith("/.")) file = file.substring(0, file.length() - 2);
	while ((dirIndex = file.indexOf("/./", dirIndex)) >= 0) {
		file = file.substring(0, dirIndex) + file.substring(dirIndex + 2);
	}

	dirIndex = 0;
	while ((dirIndex = file.indexOf("/..", dirIndex)) >= 0 &&
		(dirIndex + 3 == file.length() || file.charAt(dirIndex + 3) == '/'))
	{
		if (dirIndex == 0) {
			file = file.substring(dirIndex + 3);
		} else {
			int last = file.lastIndexOf('/', dirIndex - 1);
			if (dirIndex + 4 > file.length())
				file = file.substring(0, last + 1);
			else
				file = file.substring(0, last + 1) + file.substring(dirIndex + 4);
		}
	}
	if (file.length() == 0 && path.length() > 0) {
		if (path.charAt(0) == '/') file = "/";
		else file = ".";
	}
	return file;
}
}
