package com.ibm.oti.net.www.protocol.jxe;

import java.io.*;
import java.net.*;
import com.ibm.oti.net.www.*;
import com.ibm.oti.vm.*;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * This class is responsible for connecting, getting content and input stream of the file.
 */

public class JxeURLConnection extends URLConnection {
	private InputStream      is;
	private int              length  = -1;
	private boolean          isDir = false;
	private Jxe jxe;

/**
 * Creates an instance of <code>FileURLConnection</code> for establishing a connection to
 * the file pointed by this <code>URL<code>
 */
public JxeURLConnection(URL u, Jxe jxe) {
	super(u);
	this.jxe = jxe;
}

/**
 * This methods will attempt to obtain the input stream of the file pointed by
 * this <code>URL</code>. If the file is a directory, it will return that directory
 * listing as an input stream.
 */
public void connect() throws IOException{
	String           resName;

	// K01a8 = jxe uuid not found
	if (null == jxe) throw new IOException(com.ibm.oti.util.Msg.getString("K01a8"));

	resName  = getURL().getFile();
	if (resName.startsWith("/")) {
		resName = resName.substring(1);
	}

	is = jxe.getResourceAsStream(resName);
	if (null == is) throw new FileNotFoundException(resName);

	length = is.available();
	connected = true;
}

/**
 * Answers the length of the file in bytes.
 */
public int getContentLength() {
	try {
		if (!connected) connect();
	} catch (IOException e) {
		// default is -1
	}
	return length;
}

/*
 * Answers the content type of the resource.
 */
public String getContentType() {
	try {
		if (!connected) connect();
	} catch (IOException e) {
		return MimeTable.UNKNOWN;
	}

	if (isDir) return "text/html";
	String result = guessContentTypeFromName(url.getFile());

	if (result == null) return MimeTable.UNKNOWN;

	return result;
}

/**
 * This protocol does not support output.
 */
public boolean getDoOutput() {
	return false;
}

/**
 * Answers the input stream of the object refered to by this <code>URLConnection</code>
 */
public InputStream getInputStream() throws IOException {
	if (!connected) connect();
	return is;
}

/**
 * Get the jxe.
 */
public Jxe getJxe() {
	return jxe;
}

/**
 * Answers the permission, in this case the subclass, FilePermission object
 * which represents the permission necessary for this URLConnection to
 * establish the connection.
 */
public java.security.Permission getPermission() throws IOException {
	return new JxePermission(getURL().getHost());
}

}
