
package com.ibm.oti.security.provider;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.security.*;

/**
 * This class implements a provider for the Java Cryptography Architecture.
 * For now we only provide a digest algorithm, SHA-1.
 *
 * @author		OTI
 * @version		initial
 */
public final class OTI extends java.security.Provider {
	private static final String NAME = "OTI";		// Provider name
	private static final double VERSION = 1.4;		// Version of the services provided

	private static final String INFO = NAME + " DSA key, parameter generation and signing; SHA-1 digest; SHA1PRNG SecureRandom; JKS keystore";	// generic description
/**
 * Constructs a new instance of the OTI provider.
 */
public OTI() {
	super (NAME, VERSION, INFO);
	registerServices();
}

/**
 * Register the services the receiver provides.
 */
private void registerServices() {
	AccessController.doPrivileged(new PrivilegedAction() {
		public Object run() {
			// SHA-1 message digest algorithm
			put("MessageDigest.SHA-1", "com.ibm.oti.security.provider.MessageDigestSHA");
			put("Alg.Alias.MessageDigest.SHA", "SHA-1");
			put("Alg.Alias.MessageDigest.SHA1", "SHA-1");
			put("Alg.Alias.MessageDigest.OID.1.3.14.3.2.26", "SHA-1"); // The "SUN" Provider does not have this alias
			put("Alg.Alias.MessageDigest.1.3.14.3.2.26", "SHA-1"); // The "SUN" Provider does not have this alias

			// MD5 message digest algorithm
			put("MessageDigest.MD5", "com.ibm.oti.security.provider.MessageDigestMD5");
			put("Alg.Alias.MessageDigest.1.2.840.113549.2.5", "MD5");
			put("Alg.Alias.MessageDigest.OID.1.2.840.113549.2.5", "MD5");

			// Secure Random
			put("SecureRandom.SHA1PRNG", "com.ibm.oti.security.provider.SecureRandomSHA1PRNG");

			// Algorithm parameter generator
			put("AlgorithmParameterGenerator.DSA", "com.ibm.oti.security.provider.AlgorithmParameterGeneratorDSA");

			// Algorithm parameters
			put("AlgorithmParameters.DSA", "com.ibm.oti.security.provider.AlgorithmParametersDSA");
			// aliases
			put("Alg.Alias.AlgorithmParameters.1.2.840.10040.4.1", "DSA");
			put("Alg.Alias.AlgorithmParameters.1.3.14.3.2.12", "DSA");

			// Key pair generator
			put("KeyPairGenerator.DSA", "com.ibm.oti.security.provider.KeyPairGeneratorDSA");
			// aliases
			put("Alg.Alias.KeyPairGenerator.OID.1.2.840.10040.4.1", "DSA");
			put("Alg.Alias.KeyPairGenerator.1.2.840.10040.4.1", "DSA");
			put("Alg.Alias.KeyPairGenerator.1.3.14.3.2.12", "DSA");

			// Key factory
			put("KeyFactory.DSA", "com.ibm.oti.security.provider.KeyFactoryDSA");
			// aliases
			put("Alg.Alias.KeyFactory.1.2.840.10040.4.1", "DSA");
			put("Alg.Alias.KeyFactory.1.3.14.3.2.12", "DSA");

			// Signature algorithm
			put("Signature.SHA1withDSA", "com.ibm.oti.security.provider.SignatureDSA");
			// aliases
			put("Alg.Alias.Signature.DSA", "SHA1withDSA");
			put("Alg.Alias.Signature.DSS", "SHA1withDSA");
			put("Alg.Alias.Signature.SHA/DSA", "SHA1withDSA");
			put("Alg.Alias.Signature.SHA1/DSA", "SHA1withDSA");
			put("Alg.Alias.Signature.SHA-1/DSA", "SHA1withDSA");
			put("Alg.Alias.Signature.SHAwithDSA", "SHA1withDSA");
			put("Alg.Alias.Signature.DSAwithSHA1", "SHA1withDSA"); // The "SUN" Provider does not have this alias
			put("Alg.Alias.Signature.DSAWithSHA1", "SHA1withDSA");
			put("Alg.Alias.Signature.SHA-1withDSA", "SHA1withDSA");
			put("Alg.Alias.Signature.OID.1.2.840.10040.4.3", "SHA1withDSA");
			put("Alg.Alias.Signature.1.2.840.10040.4.3", "SHA1withDSA");
			put("Alg.Alias.Signature.1.3.14.3.2.13", "SHA1withDSA");
			put("Alg.Alias.Signature.1.3.14.3.2.27", "SHA1withDSA");
			put("Alg.Alias.Signature.OID.1.3.14.3.2.13", "SHA1withDSA"); // The "SUN" Provider does not have this alias
			put("Alg.Alias.Signature.OID.1.3.14.3.2.27", "SHA1withDSA"); // The "SUN" Provider does not have this alias

			// KeyStore
			put("KeyStore.JKS", "com.ibm.oti.security.provider.KeyStore");

			// Certificate
			put("CertificateFactory.X509", "com.ibm.oti.security.provider.CertificateFactoryX509");
			// aliases
			put("Alg.Alias.CertificateFactory.X.509", "X509");

			// RSA key pair generator
            put("KeyPairGenerator.RSA", "com.ibm.oti.security.provider.KeyPairGeneratorRSA");

			// RSA key factory
			put("KeyFactory.RSA", "com.ibm.oti.security.provider.KeyFactoryRSA");
			put("Alg.Alias.KeyFactory.1.3.14.3.2.11", "RSA");

			// RSA digital signatures with MD2 digest
			put("Signature.MD2withRSA", "com.ibm.oti.security.provider.SignatureMD2withRSA");
			put("Alg.Alias.Signature.1.3.14.3.2.24", "MD2withRSA");
			put("Alg.Alias.Signature.OID.1.3.14.3.2.24", "MD2withRSA");

			// RSA digital signatures with MD5 digest
			put("Signature.MD5withRSA", "com.ibm.oti.security.provider.SignatureMD5withRSA");
			put("Alg.Alias.Signature.1.3.14.3.2.3", "MD5withRSA");
			put("Alg.Alias.Signature.OID.1.3.14.3.2.3", "MD5withRSA");
			put("Alg.Alias.Signature.1.3.14.3.2.25", "MD5withRSA");
			put("Alg.Alias.Signature.OID.1.3.14.3.2.25", "MD5withRSA");

			// RSA digital signatures with SHA-1 digest
			put("Signature.SHA1withRSA", "com.ibm.oti.security.provider.SignatureSHA1withRSA");
			put("Alg.Alias.Signature.SHAwithRSA", "SHA1withRSA");
			put("Alg.Alias.Signature.SHA-1withRSA", "SHA1withRSA");
			put("Alg.Alias.Signature.1.3.14.3.2.15", "SHA1withRSA");
			put("Alg.Alias.Signature.OID.1.3.14.3.2.15", "SHA1withRSA");
			put("Alg.Alias.Signature.1.3.14.3.2.29", "SHA1withRSA");
			put("Alg.Alias.Signature.OID.1.3.14.3.2.29", "SHA1withRSA");

			return null;
		}
	});
}
}
