package com.ibm.oti.util;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2005, 2006  All Rights Reserved
 */

/**
 * ListResourceBundle is the abstract superclass of classes which provide
 * resources by implementing the <code>getContents()</code> method to return the list
 * of resources.
 *
 * @author		OTI
 * @version		initial
 *
 * @see			ResourceBundle
 */

public abstract class ExtendedResourceBundle extends ResourceBundle {
	private Hashtable table;
/**
 * Constructs a new instance of this class.
 */
public ExtendedResourceBundle() {
}
/**
 * Answers an Object array which contains the resources of this
 * ListResourceBundle. Each element in the array is an array of
 * two elements, the first is the resource key and the second is
 * the resource.
 *
 * @return		a Object array containing the resources
 */
abstract protected Object[][] getContents();
/**
 * Answers the names of the resources contained in this ListResourceBundle.
 *
 * @return		an Enumeration of the resource names
 */
public Enumeration getKeys() {
	if (table == null) initializeTable();
	if (parent == null) return table.keys();
	return new Enumeration() {
		Enumeration local = table.keys();
		Enumeration pEnum = parent.getKeys();
		Object nextElement = null;
		private boolean findNext() {
			if (nextElement != null) return true;
			while (pEnum.hasMoreElements()) {
				String next = (String)pEnum.nextElement();
				if (!table.containsKey(next)) {
					nextElement = next;
					return true;
				}
			}
			return false;
		}
		public boolean hasMoreElements() {
			if (local.hasMoreElements()) return true;
			return findNext();
		}
		public Object nextElement() {
			if (local.hasMoreElements())
				return local.nextElement();
			if (findNext()) {
				Object result = nextElement;
				nextElement = null;
				return result;
			}
			// Cause an exception
			return pEnum.nextElement();
		}
	};
}
/**
 * Answers the named resource from this ResourceBundle.
 *
 * @param		key	the name of the resource
 * @return		the resource object
 *
 * @exception	MissingResourceException	when the resource is not found
 */
public final Object getObject(Integer key) {
	ExtendedResourceBundle last, theParent = this;
	do {
		Object result = theParent.handleGetObject(key);
		if (result != null) return result;
		last = theParent;
		theParent = (ExtendedResourceBundle) theParent.parent;
	} while (theParent != null);
	throw new MissingResourceException(null, last.getClass().getName(), key.toString());
}

/**
 * Answers the named resource from this ResourceBundle, or null if
 * the resource is not found.
 *
 * @param		key	the name of the resource
 * @return		the resource object
 */
public Object handleGetObject(String key) {
	if (table == null) initializeTable();
	return table.get(key);
}

public Object handleGetObject(Integer key) {

	if (table == null) initializeTable();
	return table.get(key);
}

private synchronized void initializeTable() {
	if (table == null) {
		Object[][] contents = getContents();
		table = new Hashtable(contents.length / 3 * 4 + 3);
		for (int i=0; i < contents.length; i++) {
			table.put(contents[i][0], contents[i][1]);
		}
	}
}
}
