
package com.ibm.oti.vm;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

import java.util.Hashtable;
import java.util.Enumeration;

/**
 * BootstrapClassLoaders load classes out of the file system,
 * from the directories and jars on the system class path.
 * The system class path can be set using the command line
 * option "-scp <paths>", and can be read from the system
 * property "com.ibm.oti.system.class.path".
 *
 * @author		OTI
 * @version		initial
 */

public final class BootstrapClassLoader extends AbstractClassLoader{

	static {
		com.ibm.oti.vm.VM.initializeVM();
		Class cl = Integer.class;
	}
	private static BootstrapClassLoader singleton;
	private static Object findClassLock = new Object();
	private Hashtable packages = new Hashtable();
	private volatile boolean defining = false;

/**
 * Prevents this class from being instantiated.
 * BootstrapClassLoaders may only be created by the VM.
 *
 * @author		OTI
 * @version		initial
 */
private BootstrapClassLoader() {
	int count = VM.getClassPathCount();
	types = new int[count];
	cache = new Object[count];
	parsedPath = new String[count];
	VM.initializeClassLoader(this, true);
}

/**
 * Invoked by the Virtual Machine when resolving class references.
 * Equivalent to loadClass(className, false);
 *
 * @return 		java.lang.Class
 *					the Class object.
 * @param 		className String
 *					the name of the class to search for.
 *
 * @exception	ClassNotFoundException
 *					If the class could not be found.
 */
public Class loadClass(String className) throws ClassNotFoundException {
	Class loadedClass;
	synchronized(findClassLock) {
		loadedClass = com.ibm.oti.vm.VM.findClassOrNull(className, this);
	}

	if (loadedClass != null) {
		String packageName = getPackageName(loadedClass);
		if (packageName != null) {
			int index = VM.getCPIndexImpl(loadedClass);
			if (super.getPackage(packageName) == null) {
				synchronized(packages) {
					if (!packages.containsKey(packageName)) {
						packages.put(packageName, new Integer(index));
					}
				}
			}
		}
	}

	return loadedClass;
}

public static ClassLoader singleton() {
	if (singleton == null)
	 	singleton = new BootstrapClassLoader();
	 else
	 	// K0084 = can only instantiate one BootstrapClassLoader
	 	throw new SecurityException(com.ibm.oti.util.Msg.getString("K0084"));
 	return singleton;
}

protected Package getPackage(String name) {
	if (packages.size() > 0) definePackages();
	return super.getPackage(name);
}

protected Package[] getPackages() {
	if (packages.size() > 0) definePackages();
	return super.getPackages();
}

private void definePackages() {
	if (defining) return;
	while (true) {
		Hashtable toDefine = null;
		synchronized(packages) {
			if (packages.size() == 0) break;
			// Capture packages loaded while calling definePackage()
			toDefine = (Hashtable)packages.clone();
			packages.clear();
		}
		if (toDefine != null) {
			defining = true;
			try {
				definePackages(toDefine);
			} finally {
				defining = false;
			}
		}
	}
}

private void definePackages(Hashtable toDefine) {
	Enumeration keys = toDefine.keys();
	while (keys.hasMoreElements()) {
		String packageName = (String)keys.nextElement();
		definePackage(packageName, ((Integer)toDefine.get(packageName)).intValue());
	}
}

}
