
/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2006  All Rights Reserved
 */

package com.ibm.oti.vm;

/**
 * The following class enables direct, unprotected access to memory from Java.
 */
public class DMA {

	public DMA() {
		if (VM.callerClassLoader() != null)
			throw new SecurityException();
	}

	/**
	 * The following method reads a signed 8 bit quantity from the specified
	 * memory address.
	 *
	 * @param address
	 *            the memory location
	 * @return the byte value from memory
	 */
	public native byte readByte(long address);

	/**
	 * The following method writes a signed 8 bit quantity to the specified
	 * memory address.
	 *
	 * @param address
	 *            the memory location
	 * @param value
	 *            the byte value to write
	 */
	public native void writeByte(long address, byte value);

	/**
	 * The following method reads a signed 16 bit quantity from the specified
	 * memory address. The address must be 16 bit aligned.
	 *
	 * @param address
	 *            the memory location
	 * @return the short value from memory
	 */
	public native short readShort(long address);

	/**
	 * The following method writes a signed 16 bit quantity to the specified
	 * memory address. The address must be 16 bit aligned.
	 *
	 * @param address
	 *            the memory location
	 * @param value
	 *            the short value to write
	 */
	public native void writeShort(long address, short value);

	/**
	 * The following method reads a signed 32 bit quantity from the specified
	 * memory address. The address must be 32 bit aligned.
	 *
	 * @param address
	 *            the memory location
	 * @return the int value from memory
	 */
	public native int readInt(long address);

	/**
	 * The following method writes a signed 32 bit quantity to the specified
	 * memory address. The address must be 32 bit aligned.
	 *
	 * @param address
	 *            the memory location
	 * @param value
	 *            the int value to write
	 */
	public native void writeInt(long address, int value);

	/**
	 * The following method reads a signed 64 bit quantity from the specified
	 * memory address. The address must be 64 bit aligned.
	 *
	 * @param address
	 *            the memory location
	 * @return the long value from memory
	 */
	public native long readLong(long address);

	/**
	 * The following method writes a signed 64 bit quantity to the specified
	 * memory address. The address must be 64 bit aligned.
	 *
	 * @param address
	 *            the memory location
	 * @param value
	 *            the long value to write
	 */
	public native void writeLong(long address, long value);
}
