package com.ibm.oti.vm;

import java.io.*;
import java.util.Hashtable;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * ExtensionClassLoaders load classes out of the file system,
 * from the jars on the extension class path.
 *
 * @author		OTI
 * @version		initial
 */

public final class ExtensionClassLoader extends AppClassLoader {

	private static ExtensionClassLoader singleton;

	private static Hashtable protectionDomainCache = new Hashtable(10);

/**
 * Prevents this class from being instantiated.
 * ExtensionClassLoaders should only be created in ClassLoader.<clinit>
 *
 * @author		OTI
 * @version		initial
 */
private ExtensionClassLoader() {
	super(false);
	int total = 0;
	String[][] list;
	String[] extDirPath;
	String extDirs = System.getProperty("java.ext.dirs");
	if (extDirs == null) {
		list = new String[1][];
		extDirPath = new String[1];
		String bootPath = System.getProperty("java.home", "");
		File extDir = new File(bootPath, "lib/ext");
		list[0] = extDir.list();
		if (list[0] != null) {
			total = list[0].length;
			extDirPath[0] = extDir.getPath() + File.separatorChar;
		}
	} else {
		int index = 0, count = 0, end = extDirs.length();
		while (index < end) {
			int next = extDirs.indexOf(File.pathSeparatorChar, index);
			if (next == -1) next = end;
			if (next - index > 0) count++;
			index = next + 1;
		}
		list = new String[count][];
		extDirPath = new String[count];
		index = count = 0;
		while (index < end) {
			int next = extDirs.indexOf(File.pathSeparatorChar, index);
			if (next == -1) next = end;
			if (next - index > 0) {
				String path = extDirs.substring(index, next);
				File extDir = new File(path);
				list[count] = extDir.list();
				if (list[count] != null) {
					total += list[count].length;
					extDirPath[count++] = extDir.getPath() + File.separatorChar;
				}
			}
			index = next + 1;
		}
	}
	if (list != null) {
		types = new int[total];
		cache = parsedPath = new String[total];
		StringBuffer extPath = new StringBuffer();
		for (int j=0; j<list.length; j++) {
			if (list[j] != null)
				for (int i=0; i<list[j].length; i++) {
					extPath.append(extDirPath[j]);
					extPath.append(list[j][i]);
					parsedPath[i] = extDirPath[j] + list[j][i];
					if (i != list.length - 1) extPath.append(';');
				}
		}
		com.ibm.oti.vm.VM.setClassPathImpl(this, extPath.toString());
	}
}

public static synchronized ClassLoader singleton() {
	if(singleton == null)
	 	singleton = new ExtensionClassLoader();
	 else
	 	// K0085 = can only instantiate one ExtensionClassLoader
	 	throw new InstantiationError(com.ibm.oti.util.Msg.getString("K0085"));
 	return singleton;
}

Hashtable getProtectionDomainCache() {
	return protectionDomainCache;
}
boolean addExitPermission() {
	return false;
}
}
