package com.ibm.oti.vm;

import java.io.*;

/**
 * <p>This interface defines the requirements of any output stream used
 * for the streaming reocation of jxes.</p>
 *
 * <pre>
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corp. 2000, 2004  All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * </pre>
 *
 * @author		OTI
 * @version		initial
 *
 * @see			JxeUtil
 */
public interface JxeOutputStream {

/**
 * Reads back an 8-bit <code>byte</code> value from this stream.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		offset	the offset from the start of the jxe to read from.
 * @return		byte		the next unsigned byte value from the source stream.
 * @exception 	java.io.IOException If the stream is already closed or another IOException occurs.
 */
public byte read8(int offset) throws IOException;

/**
 * Reads back an 16-bit <code>byte</code> value from this stream.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		offset	the offset from the start of the jxe to read from.
 * @return		short		the next short from the source stream at jxeOffset.
 * @exception 	java.io.IOException If the stream is already closed or another IOException occurs.
 */
public short read16(int offset) throws IOException;

/**
 * Reads back an 32-bit <code>byte</code> value from this stream.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		offset	the offset from the start of the jxe to read from.
 * @return		int		the next int from the source stream at jxeOffset.
 * @exception 	java.io.IOException If the stream is already closed or another IOException occurs.
 */
public int read32(int offset) throws IOException;

/**
 * Reads back an 64-bit value from this stream.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		offset	the offset from the start of the jxe to read from.
 * @return		long		the next long from the source stream at jxeOffset.
 * @exception 	java.io.IOException If the stream is already closed or another IOException occurs.
 */
public long read64(int offset) throws IOException;

/**
 * Writes <code>count</code> <code>bytes</code> from the byte array
 * <code>buffer</code> starting at <code>offset</code> to this
 * OutputStream.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		buffer		the buffer to be written
 * @param		offset		offset in buffer to get bytes
 * @param		count		number of bytes in buffer to write
 *
 * @exception 	java.io.IOException	If an error occurs attempting to write to this OutputStream.
 * @exception	java.lang.IndexOutOfBoundsException If offset or count are outside of bounds.
 */
public void write(byte buffer[], int offset, int count) throws IOException;

}
