
package com.ibm.oti.vm;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * ReflectObjectInvalidExceptions are thrown when an attempt is
 * made to reference a reflect object that has become invalid.
 * <p>
 * A reflect object becomes invalid when the constructor, method
 * or field that it models is dynamically changes and either
 * (a) ceases to exist or (b) could no longer be reached via
 * the original lookup method. For example, if a method is
 * obtained by using Class.getDeclaredMethod() and it moves
 * to another class, the object becomes invalid. If it was
 * obtained via Class.getMethods() and the method is no longer
 * public, it will also become invalid.
 *
 * @see java.lang.reflect.Constructor
 * @see java.lang.reflect.Field
 * @see java.lang.reflect.Method
 *
 * @author		OTI
 * @version		draft
 */

public class ReflectObjectInvalidException extends java.lang.RuntimeException {

/**
 * Constructs a new instance of this class with its
 * walkback and message filled in. The message is a set to
 * a default string.
 *
 * @author		OTI
 * @version		initial
 */
public ReflectObjectInvalidException () {
	// K0086 = Referenced reflect object is no longer valid
	super(com.ibm.oti.util.Msg.getString("K0086"));
}

/**
 * Constructs a new instance of this class with its
 * walkback and message filled in.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		message String
 *				The detail message for the exception.
 */
public ReflectObjectInvalidException (String message) {
	// K0087 = Referenced reflect object is no longer valid: {0}
	super(com.ibm.oti.util.Msg.getString("K0087", message));
}

}
