package com.ibm.oti.vm;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

public class URLAppClassLoader extends URLSystemClassLoader {
	private static URL[] classPath = getClassPath();

public URLAppClassLoader() {
	this(null);
}

public URLAppClassLoader(ClassLoader parent) {
	super(classPath, parent);
}

URLAppClassLoader(URL[] path, ClassLoader parent) {
	super(path, parent);
}

private static URL[] getClassPath() {
	String classPath = System.getProperty("java.class.path", ".");
	int index = 0, count = 0, end = classPath.length();
	while (index < end) {
		int next = classPath.indexOf(File.pathSeparatorChar, index);
		if (next == -1) next = end;
		if (next - index > 0) count++;
		index = next + 1;
	}
	URL[] urlPath = new URL[count];
	index = count = 0;
	while (index < end) {
		int next = classPath.indexOf(File.pathSeparatorChar, index);
		if (next == -1) next = end;
		if (next - index > 0) {
			String path = classPath.substring(index, next);
			try {
				boolean isInMemoryJxe = path.startsWith("jxe=");
				if (path.length() > 4 &&
					(path.substring(path.length() - 4).equalsIgnoreCase(".jxe") ||  isInMemoryJxe))
				{
					try {
						Jxe jxe = null;
						if ( isInMemoryJxe ){
							try {
								jxe = Jxe.fromPointer(Long.parseLong(path.substring(4),16));
							} catch (NumberFormatException e) {}
						} else {
							jxe = Jxe.fromFile(new File(path));
						}
						if (jxe != null) {
							urlPath[count] = new URL("jxe", jxe.getUuid(), -1, "/", new com.ibm.oti.net.www.protocol.jxe.Handler(jxe));
							count++;
							index = next + 1;
							continue;
						}
					} catch (JxeException e) {
					} catch (UnsatisfiedLinkError e) {}
				}
				File f = new File(path);
				path = f.getCanonicalPath();
				if (File.separatorChar != '/')
					path = path.replace(File.separatorChar, '/');
				if (f.isDirectory()) {
					if (!path.endsWith("/"))
						path = new StringBuffer(path.length() + 1).
							append(path).append('/').toString();
				}
				if (!path.startsWith("/"))
					path = new StringBuffer(path.length() + 1).
						append('/').append(path).toString();
				if (!path.startsWith("//")) {
					try {
						urlPath[count++] = new URL("file", null, -1, path, null);
					} catch (MalformedURLException e) {}
					index = next + 1;
					continue;
				}
				path = new StringBuffer(path.length() + 5).
					append("file:").append(path).toString();

				urlPath[count] = new URL(path);
				count++;
			} catch (IOException e) {}
		}
		index = next + 1;
	}
	if (count < urlPath.length) {
		URL[] paths = new URL[count];
		System.arraycopy(urlPath, 0, paths, 0, count);
		urlPath = paths;
	}
	return urlPath;
}

boolean addExitPermission() {
	return true;
}
}
