package com.ibm.oti.vm;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2001, 2005  All Rights Reserved
 */

import java.io.*;
import java.net.*;

/**
 * ExtensionClassLoaders load classes out of the file system,
 * from the jars on the extension class path.
 *
 * @author		OTI
 * @version		initial
 */
public final class URLExtensionClassLoader extends URLSystemClassLoader {
	private static final URL[] classPath = getClassPath();

public URLExtensionClassLoader() {
	this(null);
}

URLExtensionClassLoader(ClassLoader parent) {
	super(classPath, parent);
}

static URL[] getClassPath() {
	int total = 0;
	String[][] list;
	String[] extDirPath;

 	String extDirs = System.getProperty("java.ext.dirs");
	if (extDirs == null) {
		String bootPath = System.getProperty("java.home", "");
	    File defaultExt = new File( bootPath, "lib/ext" );
	    // Check <ive_dir>/lib/ext;<ive_dir>/lib/<config_dir>/opt-ext unless there
	    // is an override of the system property
		String configDirName = System.getProperty("com.ibm.oti.configuration.dir");
	    File opPkgExt = new File( bootPath, "lib/" + configDirName + "/opt-ext" );
	    extDirs = defaultExt.getPath() + File.pathSeparatorChar + opPkgExt.getPath();

		System.setProperty("java.ext.dirs", extDirs);
	}

	int index = 0, extCount = 0, end = extDirs.length();
	while (index < end) {
		int next = extDirs.indexOf(File.pathSeparatorChar, index);
		if (next == -1) next = end;
		if (next - index > 0) extCount++;
		index = next + 1;
	}
	list = new String[extCount][];
	extDirPath = new String[extCount];
	index = extCount = 0;
	while (index < end) {
		int next = extDirs.indexOf(File.pathSeparatorChar, index);
		if (next == -1) next = end;
		if (next - index > 0) {
			String path = extDirs.substring(index, next);
			File extDir = new File(path);
			list[extCount] = extDir.list();
			if (list[extCount] != null) {
				total += list[extCount].length;
				extDirPath[extCount++] = extDir.getPath() + File.separatorChar;
			}
		}
		index = next + 1;
	}

	URL[] parsedPath = new URL[total];
	if (list != null) {
		int count = 0;
		for (int j=0; j<list.length; j++) {
			if (list[j] != null)
				for (int i=0; i<list[j].length; i++) {
					String file = list[j][i], path = null;
					boolean foundJxe = false;
					if (file.length() > 4 &&
						file.substring(file.length() - 4).equalsIgnoreCase(".jxe"))
					{
						try {
							Jxe jxe = Jxe.fromFile(new File(extDirPath[j] + file));
							if (jxe != null) {
								parsedPath[count] = new URL("jxe", jxe.getUuid(), -1, "/", new com.ibm.oti.net.www.protocol.jxe.Handler(jxe));
								count++;
								continue;
							}
						} catch (IOException e) {
						} catch (JxeException e) {
						} catch (UnsatisfiedLinkError e) {}
					}
					if (!foundJxe)
						path = "file:" + extDirPath[j] + file;
					try {
						parsedPath[count] = new URL(path);
						count++;
					} catch (MalformedURLException e) {}
				}
		}
		if (count < parsedPath.length) {
			URL[] paths = new URL[count];
			System.arraycopy(parsedPath, 0, paths, 0, count);
			parsedPath = paths;
		}
	}
	return parsedPath;
}

boolean addExitPermission() {
	return false;
}
}
