package com.ibm.oti.vm;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.net.URL;
import java.net.URLClassLoader;
import java.security.CodeSource;
import java.security.PermissionCollection;

public abstract class URLSystemClassLoader extends URLClassLoader {
	private boolean checkingPackageAccess = false;
	private static final RuntimePermission permissionToExitVM =
		new RuntimePermission("exitVM");

URLSystemClassLoader() {
	super(null);
}

URLSystemClassLoader(URL[] path, ClassLoader parent) {
	super(path, parent);
}

public Class findClass(String className) throws ClassNotFoundException {
	Class newClass = super.findClass(className);
	return newClass;
}

/**
 * Overrides the loadClass() of <code>ClassLoader</code>. It calls the security
 * manager's <code>checkPackageAccess()</code> before attempting toload the class.
 *
 * @author		OTI
 * @version		initial
 *
 * @return 		java.lang.Class
 *					the Class object.
 * @param 		className String
 *					the name of the class to search for.
 * @param 		resolveClass boolean
 *					indicates if class should be resolved after loading.
 * @exception	ClassNotFoundException
 *					If the class could not be found.
 */
protected synchronized Class loadClass(String className, boolean resolveClass) throws ClassNotFoundException {
	SecurityManager sm = System.getSecurityManager();
	if (sm != null && !checkingPackageAccess) {
		int index = className.lastIndexOf('.');
		if (index > 0) { // skip if class is from a default package
			try {
				checkingPackageAccess = true;
				sm.checkPackageAccess(className.substring(0, index));
			} finally {
				checkingPackageAccess = false;
			}
		}
	}
	return super.loadClass(className, resolveClass);
}

abstract boolean addExitPermission();

/**
 * Answers the permissions for the given code source.
 *
 * @author		OTI
 * @version		initial
 *
 * @return PermissionCollection
 * @param codesource CodeSource
 */
protected PermissionCollection getPermissions(CodeSource codesource) {
	PermissionCollection pc = super.getPermissions(codesource);
	if (addExitPermission())
		pc.add(permissionToExitVM);
	return pc;
}
}
