package com.ibm.oti.xlet;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2005  All Rights Reserved
 */

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import com.ibm.oti.awt.MessageStrings;
/**
 * A Window for the Xlet Application Manager to display and control Xlets.
 */
class NewXletDialog extends Dialog {

	private XletApplicationManager manager;

	private final int CLASSPATH = 0;
	private final int JARFILE = 1;
	private TextField xletName;
	private TextField xletPath;
	private TextField xletArgs;
	private String selectedItem;

	NewXletDialog(Frame parent, XletApplicationManager mgr){
		super(parent, MessageStrings.getString("BBAWTXLET_FRAME_NEWXLET"), true); //$NON-NLS-1$

		manager = mgr;

	    setSize(parent.getWidth(), parent.getHeight() / 2);
	    setLocation(0, parent.getHeight() / 4);
		setup();
	}

	private Frame getFrame() {
		return (Frame)getParent();
	}

	private void setup() {
		// The Xlet setup panel should be wrapped in a ScrollPane
		// so that devices with small screens can still access all the
		// controls that are provided (bug 102872)
		Container scrollable = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
		scrollable.add(setupXletPanel());
		add(scrollable, "Center"); //$NON-NLS-1$

		Panel panel2 = setupButtonPanel();
		add(panel2, "South"); //$NON-NLS-1$

		//add an event listener to deal with the closing of the dialog
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				Window window = e.getWindow();
				if (window != null) {
					window.dispose();
				}
			}
		});
	}

	private Panel setupXletPanel() {

		//setup panel
		Panel panel = new Panel();
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		panel.setLayout(layout);

		// setup path button
		Button pathButton = new Button(MessageStrings.getString("BBAWTXLET_BUTTON_ADDTOPATH")); //$NON-NLS-1$
		pathButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				FileDialog fileDialog = new FileDialog(getFrame(), MessageStrings.getString("BBAWTXLET_FD_SELECTJAR"), FileDialog.LOAD); //$NON-NLS-1$
				fileDialog.setFile("*.jar"); //$NON-NLS-1$
				fileDialog.setVisible(true);
				toFront();   //return newXletFrame to front
				try{
					JarFile jarfile = new JarFile(fileDialog.getDirectory() + fileDialog.getFile());
					xletPath.setText(xletPath.getText() + File.pathSeparator + jarfile.getName());
					if (xletPath.getText().startsWith(File.pathSeparator)){
						xletPath.setText(xletPath.getText().substring(File.pathSeparator.length()));
					}
				}
				catch(IOException ioe){
					if (fileDialog.getFile() != null){
						displayMessage(getFrame(), MessageStrings.getString("BBAWTXLET_ERROR_INVALID_JAR1"), MessageStrings.getString("BBAWTXLET_ERROR_INVALID_JAR2"), ioe.getMessage()); //$NON-NLS-1$ //$NON-NLS-2$
					}
				}
			}
		});

		// setup xlet button
		//initially disabled, becomes enabled only when classpath is entered
		final Button xletButton = new Button(MessageStrings.getString("BBAWTXLET_BUTTON_FINDXLET")); //$NON-NLS-1$
		xletButton.setEnabled(false);
		xletButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				try {
					StringTokenizer strtok = new StringTokenizer(xletPath.getText(), File.pathSeparator);
					if (strtok.countTokens() == 0){
						return;
					}
					addClassFromPath(getFrame(), strtok);
					toFront();
				}
				catch(IOException ioe){}
				}
			});

		//Xlet path label
		Label pathLabel = new Label(MessageStrings.getString("BBAWTXLET_LABEL_PATH")); //$NON-NLS-1$
		c.weightx = 0.0;
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = 1;
		layout.setConstraints(pathLabel, c);
		panel.add(pathLabel);

		//Textfield for xlet path
		//Added TextListener to enable/disable Add Class from Classpath button
		c.anchor = GridBagConstraints.SOUTH;
		c.weightx = 1.0;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.fill = GridBagConstraints.HORIZONTAL;
		xletPath = new TextField(20);
		layout.setConstraints(xletPath, c);
		xletPath.addTextListener(new TextListener(){
			public void textValueChanged(TextEvent textEvent) {
				if (xletPath.getText().equals("")){ //$NON-NLS-1$
					xletButton.setEnabled(false);
				}
				else {
					xletButton.setEnabled(true);
				}
			}
		});
		panel.add(xletPath);

		//Add jar to classpath button
		c.anchor = GridBagConstraints.SOUTHEAST;
		c.weightx = 0.0;
		c.fill = GridBagConstraints.NONE;
		layout.setConstraints(pathButton, c);
		panel.add(pathButton);

		//Xlet Name label
		Label nameLabel = new Label(MessageStrings.getString("BBAWTXLET_LABEL_NAME")); //$NON-NLS-1$
		c.weightx = 0.0;
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = 1;
		layout.setConstraints(nameLabel, c);
		panel.add(nameLabel);

		//TextField for xlet name
		c.anchor = GridBagConstraints.SOUTH;
		c.weightx = 1.0;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.fill = GridBagConstraints.HORIZONTAL;
		xletName = new TextField(20);
		layout.setConstraints(xletName, c);
		panel.add(xletName);

		//Add class from classpath button
		c.anchor = GridBagConstraints.SOUTHEAST;
		c.weightx = 0.0;
		c.fill = GridBagConstraints.NONE;
		layout.setConstraints(xletButton, c);
		panel.add(xletButton);

		//Xlet args label
		Label argsLabel = new Label(MessageStrings.getString("BBAWTXLET_LABEL_ARGS")); //$NON-NLS-1$
		c.weightx = 0.0;
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.WEST;
		layout.setConstraints(argsLabel, c);
		panel.add(argsLabel);

		//TextField for Xlet args
		c.anchor = GridBagConstraints.SOUTH;
		c.weightx = 1.0;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.fill = GridBagConstraints.HORIZONTAL;
		xletArgs = new TextField(20);
		layout.setConstraints(xletArgs, c);
		panel.add(xletArgs);
		return panel;
	}

	private Panel setupButtonPanel() {
		//Bottom panel
		Panel panel = new Panel();

		//Ok Button loads the xlet
		Button okButton = new Button(MessageStrings.getString("BBAWTXLET_BUTTON_OK")); //$NON-NLS-1$
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				StringTokenizer tok = new StringTokenizer(xletArgs.getText());
				int argCount = tok.countTokens();
				String[] args = new String[argCount];
				for (int i=0; i < argCount; i++) {
					args[i] = tok.nextToken();
				}
				// need to check field values

				XletApplication xlet = new XletApplication(xletName.getText(), xletPath.getText(), args);
				try {
					manager.createXlet(xlet);
					xlet.initXlet();
					xlet.startXlet();
					dispose();
				} catch (Exception ex) {
					//show message
					displayMessage(getFrame(), MessageStrings.getString("BBAWTXLET_ERROR_INVALID_XLET"), MessageStrings.getString("BBAWTXLET_ERROR_COULDNT_LOAD") , ex.getMessage()); //$NON-NLS-1$ //$NON-NLS-2$
//					xlet.destroyXlet(true);
				}
			}
		});

		panel.add(okButton);

		//Cancel button disposes the dialog
		Button cancelButton = new Button(MessageStrings.getString("BBAWTXLET_BUTTON_CANCEL")); //$NON-NLS-1$
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});

		panel.add(cancelButton);
		return panel;
	}

	/**
	 * Helper method used to open jar files and return its file's paths
	 * as a String array
	 * @param jarfile jarfile to be opened
	 * @return paths to the jarfile entries as an array of Strings
	 */
	private String[] getJarEntriesAsArray(JarFile jarfile) {
		//Copy path entries for class files from jar file to vector
		Vector jarEntries = new Vector();
		Enumeration  enumeration = jarfile.entries();
		while( enumeration.hasMoreElements()){
			String element = ((JarEntry) enumeration.nextElement()).getName();
			if (element.endsWith(".class")){	 //$NON-NLS-1$
				jarEntries.add(element);
			}
		}
		return (String[]) (jarEntries.toArray(new String[jarEntries.size()]));
	}

	/**
	 * Set the size and location of the dialog relative to the parent frame
	 * @param parent - parent frame
	 * @param dialog - whose size and location is to be set
	 */
	private void setSizeAndLocation(final Frame parent, final Dialog dialog) {
		dialog.setSize(parent.getWidth(), parent.getHeight());
		dialog.setLocation(parent.getLocation().x + (parent.getWidth() / 4), parent.getLocation().y + (parent.getHeight() / 4));
	}

	/**
	 * This method pops up a filedialog/dialog with a list of jar entries based on the
	 * mode and lets the user select an xlet to be loaded
	 * @param parent the Frame on which to create a dialog
	 * @param names array of paths to the jarentries within a jar file
	 * @param title title of the dialog
	 * @param mode JARFILE or CLASSPATH
	 */
	private void selectXlet(final Frame parent, String[] names, String title, final int mode) {
		final Dialog dialog = new Dialog(parent, title, true);
		final java.awt.List list = new java.awt.List();

		list.setMultipleMode(false);

		//Ok button on the dialog
		Button okButton = new Button(MessageStrings.getString("BBAWTXLET_BUTTON_OK")); //$NON-NLS-1$
		okButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				selectedItem = list.getSelectedItem();
				if (mode  == CLASSPATH){
					if (selectedItem.endsWith(".jar")){ //$NON-NLS-1$
						try{
							JarFile jarfile = new JarFile(selectedItem);
							String[] jarEntries =  getJarEntriesAsArray(jarfile);
							dialog.dispose();
							selectXlet(parent, jarEntries, MessageStrings.getString("BBAWTXLET_MESSAGE_SELECT_XLET"), JARFILE); //$NON-NLS-1$
						}
						catch(IOException ex){}
					}
					else {
						FileDialog classFileDialog = new FileDialog(parent, MessageStrings.getString("BBAWTXLET_FD_SELECT_CLASS"), FileDialog.LOAD); //$NON-NLS-1$
						classFileDialog.setDirectory(selectedItem);
						classFileDialog.show();
						String name = classFileDialog.getDirectory() + classFileDialog.getFile();
						name = name.substring(0, name.length());
						name = name.substring(selectedItem.length());
						name = name.replace(File.separatorChar, '.');
						xletName.setText(name);
					}
				}
				else if (mode == JARFILE){
					if ((selectedItem != null) && (selectedItem.endsWith(".class"))) { //$NON-NLS-1$
						selectedItem = selectedItem.substring(0, selectedItem.length()-6);
						selectedItem = selectedItem.replace('/', '.');
						xletName.setText(selectedItem);
					}
				}
				dialog.dispose();
			}});

		//Cancel button on the dialog
		Button cancelButton = new Button(MessageStrings.getString("BBAWTXLET_BUTTON_CANCEL")); //$NON-NLS-1$
		cancelButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				dialog.dispose();
			}});

		for (int i = 0; i < names.length; i++){
			list.add(names[i]);
		}

		setSizeAndLocation(parent, dialog);
		dialog.setLayout(new BorderLayout());
		dialog.add(list, BorderLayout.CENTER);
		Panel panel = new Panel();
		panel.add(okButton);
		panel.add(cancelButton);

		dialog.add(panel, BorderLayout.SOUTH);
		dialog.show();
	}

	/**
	 * This method brings up a dialog on the parent frame with the specified
	 * title and message
	 * @param parent frame on which dialog is created
	 * @param title - title of dialog
	 * @param message - message to be shown
	 */
	private void displayMessage(final Frame parent, String title, String message, String exceptionMessage) {
		final Dialog dialog = new Dialog(parent, title , true);
		dialog.setLayout(new BorderLayout());
		Label label = new Label(message);
		label.setAlignment(Label.CENTER);
		Label exceptionMessageLabel = new Label(exceptionMessage);
		label.setAlignment(Label.CENTER);
		Panel p5 = new Panel();
		p5.setLayout(new BorderLayout());
		p5.add(label, BorderLayout.NORTH);
		p5.add(exceptionMessageLabel, BorderLayout.CENTER);
		dialog.add(p5, BorderLayout.CENTER);
		Panel p4 = new Panel();
		Button button = new Button(MessageStrings.getString("BBAWTXLET_BUTTON_OK")); //$NON-NLS-1$
		p4.add(button);
		dialog.add(p4, BorderLayout.SOUTH);
		dialog.setLocation(parent.getLocation().x, parent.getLocation().y + (parent.getHeight() / 4));
		dialog.pack();
		button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			dialog.dispose();
			parent.setVisible(true);
		}});

		//add an event listener to deal with the closing of the dialog
		dialog.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				Window window = e.getWindow();
				if (window != null) {
					window.dispose();
				}
		}});

		dialog.show();
	}

	/**
	 * This method prompts the user to select a class depending
	 * on the classpath entry/entries.
	 * @param parent - the parent frame
	 * @param strtok - classpath entires tokenized by pathseparators
	 */
	private void addClassFromPath(final Frame parent, StringTokenizer strtok) throws IOException {
		if (strtok.countTokens() == 1){
			String path = strtok.nextToken();
			if (path.endsWith(".jar")){ //$NON-NLS-1$
				JarFile jarfile = new JarFile(path);
				String[] jarEntries = getJarEntriesAsArray(jarfile);
				selectXlet(parent, jarEntries, MessageStrings.getString("BBAWTXLET_MESSAGE_SELECT_XLET"), JARFILE); //$NON-NLS-1$
				if (selectedItem != null){
					selectedItem = selectedItem.substring(0, selectedItem.length());
				}
				xletName.setText(selectedItem);
			} else {
				FileDialog classFileDialog = new FileDialog(parent, MessageStrings.getString("BBAWTXLET_FD_SELECT_CLASS"), FileDialog.LOAD); //$NON-NLS-1$
				classFileDialog.setDirectory(path);
				classFileDialog.show();
				if (classFileDialog.getFile() != null){
					String name = classFileDialog.getDirectory() + classFileDialog.getFile();
					name = name.substring(0, name.length()-6);
					name = name.substring(path.length());
					name = name.replace(File.separatorChar, '.');
					xletName.setText(name);
				}
			}
		} else {
			String[] classpaths = new String[strtok.countTokens()];
			int i = 0;
			while (strtok.hasMoreTokens()){
				classpaths[i] = strtok.nextToken();
				i++;
			}

			selectXlet(parent, classpaths, MessageStrings.getString("BBAWTXLET_MESSAGE_SELECT_CLASSPATH"), CLASSPATH); //$NON-NLS-1$

			if ((selectedItem != null) && (selectedItem.endsWith(".jar"))) { //$NON-NLS-1$
				JarFile jarfile = new JarFile(selectedItem);
				String[] jarEntries =  getJarEntriesAsArray(jarfile);
				selectXlet(parent, jarEntries, MessageStrings.getString("BBAWTXLET_MESSAGE_SELECT_XLET"), JARFILE); //$NON-NLS-1$
				xletName.setText(selectedItem);
			}
		}
	}
}
