package com.ibm.oti.xlet;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2006  All Rights Reserved
 */

import java.io.*;
import java.net.*;
import java.util.*;

import javax.microedition.xlet.Xlet;

import com.ibm.oti.awt.MessageStrings;

/**
 * The Xlet Application Manager manages Xlet applications.
 */
public class XletApplicationManager {

	private static XletApplicationManager Default;

	private XletApplicationManagerFrame frame;
	private Vector xletApps; 		// Collection of XletApplications, under control of this manager
	private Vector listeners;		// Collection of Xletlisteners, for Xlet changes

	public static XletApplicationManager getXletApplicationManager() {
		if (Default == null)
			Default = new XletApplicationManager();

		return Default;
	}

	private XletApplicationManager() {
		xletApps = new Vector();
		listeners = new Vector();
	}

	/**
	 * Initialize the Xlet Manager.
	 *
	 * Note: This method is called by the TCKXletManagerProxy.
	 */
	public void init(String[] args) {

		Vector initialXletApps;

		initialXletApps = getInitialXlets(args);
		initializeUI();
		XletApplication xlet = null;
		try {
			// For each specified Xlet, load and start it
			for (int i=0; i < initialXletApps.size(); i++) {
				xlet = (XletApplication) initialXletApps.elementAt(i);
				createXlet(xlet);
				xlet.initXlet();
				xlet.startXlet();
			}

		} catch (Exception ex) {
			ex.printStackTrace();
			xlet.destroyXlet(true);
		}

		if (initialXletApps.size() > 0)
			frame.setXletFocus((XletApplication)initialXletApps.firstElement());
	}

	/*
	 * Return the initial collection of Xlets, specified in the startup args.
	 */
	protected Vector getInitialXlets(String[] args) {

		String xletClassName = ""; //$NON-NLS-1$
		String xletPath = ""; //$NON-NLS-1$
		String[] xletArgs = new String[0];
		String xletArgsFilename;
		XletApplicationManager appManager = null;
		Vector initialXletApps = new Vector();

		// Parse the args and collect information on the initial Xlets to load and run.
		if ((args == null) || (args.length == 0)) {
			//--------------------------------------------------------------------------------
		 	// No initial Xlets specified, allow empty manager frame
		 	// -------------------------------------------------------------------------------

		} else if (args[0].startsWith("-name:")) { //$NON-NLS-1$
			//--------------------------------------------------------------------------------
		 	// Initial Xlets specified as command line args.  Parse the xlet startup parameters.
		 	// The keywords can repeat to specify multiple xlets.
		 	//--------------------------------------------------------------------------------
			int i = 0;
			while (i < args.length) {
				if (args[i].startsWith("-name:")) { //$NON-NLS-1$
					xletClassName = args[i].substring(6);
					i++;
				} else {
					usageError(MessageStrings.getFormattedString("BBAWTXLET_MISSING_KEYWORD", "name")); //$NON-NLS-1$ //$NON-NLS-2$
				}
				if (args[i].startsWith("-path:")) { //$NON-NLS-1$
					xletPath = args[i].substring(6);
					i++;
				} else {
					usageError(MessageStrings.getFormattedString("BBAWTXLET_MISSING_KEYWORD", "path")); //$NON-NLS-1$ //$NON-NLS-2$
				}

				if (i < (args.length)) { //-args is optional

					if (args[i].startsWith("-args:")) { //$NON-NLS-1$
						Vector v = new Vector();
						if (args[i].substring(6).length() > 0) {
							v.add(args[i].substring(6));
						}
						i++;
						while ((i < (args.length))
							&& (!args[i].startsWith("-name:"))) { //$NON-NLS-1$
							v.add(args[i]);
							i++;
						}

						xletArgs = (String[]) (v.toArray(new String[v.size()]));
					}
				}

				initialXletApps.add(new XletApplication(xletClassName, xletPath, xletArgs));
			}

		} else if (args[0].startsWith("-filename:")) { //$NON-NLS-1$
			//--------------------------------------------------------------------------------
		 	// Initial Xlets specified in a command file.  Parse the xlet startup parameters.
		 	// The keywords can repeat to specify multiple xlets, one xlet per line.
		 	//--------------------------------------------------------------------------------
			BufferedReader in = null;
			String argline = null;
			String arg;

			String optionFilename = args[0].substring(10);
			if (optionFilename.length() == 0) {
				usageError(MessageStrings.getFormattedString("BBAWTXLET_EMPTY_KEYWORD", "filename")); //$NON-NLS-1$ //$NON-NLS-2$
			}
			try {
				in = new BufferedReader(new FileReader(optionFilename));
			} catch (FileNotFoundException e) {
				usageError(MessageStrings.getFormattedString("BBAWTXLET_ERROR_KEYWORD", new Object[] {"filename", optionFilename})); //$NON-NLS-1$ //$NON-NLS-2$
			}

			try {
				argline = in.readLine();
			} catch (IOException e) {
			}

			while (argline != null) {

				StringTokenizer tokenizer = new StringTokenizer(argline);
				int count = tokenizer.countTokens();

				if (tokenizer.hasMoreTokens()) {
					arg = tokenizer.nextToken();
					if (arg.startsWith("-name:")) { //$NON-NLS-1$
						xletClassName = arg.substring(6);
					} else {
						usageError(MessageStrings.getFormattedString("BBAWTXLET_MISSING_KEYWORD", "name")); //$NON-NLS-1$ //$NON-NLS-2$
					}
				}

				if (tokenizer.hasMoreTokens()) {
					arg = tokenizer.nextToken();
					if (arg.startsWith("-path:")) { //$NON-NLS-1$
						xletPath = arg.substring(6);
					} else {
						usageError(MessageStrings.getFormattedString("BBAWTXLET_MISSING_KEYWORD", "path")); //$NON-NLS-1$ //$NON-NLS-2$
					}
				}

				if (tokenizer.hasMoreTokens()) {
					arg = tokenizer.nextToken();
					if (arg.startsWith("-args:")) { //$NON-NLS-1$
						Vector v = new Vector();
						if (arg.substring(6).length() > 0) {
							v.add(arg.substring(6));
						}

						while (tokenizer.hasMoreTokens()) {
							arg = tokenizer.nextToken();
							v.add(arg);
						}

						xletArgs = (String[]) (v.toArray(new String[v.size()]));
					}
				}

				if (!argline.equals("")){ //$NON-NLS-1$
					initialXletApps.add(new XletApplication(xletClassName, xletPath, xletArgs));
				}

				try {
					argline = in.readLine();
				} catch (IOException e) {
				}
			}

		} else {
			usageError(MessageStrings.getString("BBAWTXLET_INVALID_ARGUMENTS")); //$NON-NLS-1$
		}

		return initialXletApps;
	}

	/**
	 * Create and initialize a Frame for the Application Manager UI
	 */
	protected void initializeUI() {
		if (frame == null) {
			frame = new XletApplicationManagerFrame(this);
		}
	}

	/**
	 * Create the specified Xlet
	 */
	public void createXlet(XletApplication xletApp) throws Exception {
		ClassLoader xletClassLoader = xletApp.getClassLoader();

		Class xletClass = null;
		Xlet xlet = null;
		try {
			// Load xlet class
			try {
				xletClass = xletClassLoader.loadClass(xletApp.getName());
			} catch (ClassNotFoundException e) {
				throw new RuntimeException("Error creating Xlet: can not find Class '" + xletApp.getName() + "'. " + e.toString()); //$NON-NLS-1$ //$NON-NLS-2$
			}

			// Create an instance of the Xlet class
			xlet = (Xlet) xletClass.newInstance();
		} catch (Throwable t) {
			throw new RuntimeException("Error creating Xlet '" + xletApp.getName() + "':" + t.toString()); //$NON-NLS-1$ //$NON-NLS-2$
		}

		// Bind the Xlet Application to the real xlet
		xletApp.bind(xlet, this);
		xletApps.add(xletApp);

		// notify listeners that an xlet was added
		notifyXletAdded(xletApp);
	}

	/**
	 * Remove the Xlet
	 *
	 * @param 	xletApp XletApplication
	 */
	public void removeXlet(XletApplication xletApp) {
		XletApplication nextXlet = getNextXlet(xletApp);
		xletApps.remove(xletApp);

		notifyXletRemoved(xletApp, nextXlet);
	}

	/*
	 * Return an array of URLs, from the specified xletPath.
	 * Note that the URLs are separated by a semi-colon (NOT File.pathSeparator).
	 * This is done to avoid a conflict on Linux where the path separator is the
	 * same string that is used following the protocol identifier.
	 */
	protected static URL[] getURLs(String xletPath) {
		StringTokenizer tokenizer = new StringTokenizer(xletPath, ";" /*File.pathSeparator*/);
		URL[] xletPathURLs;

		Vector urls = new Vector();
		while (tokenizer.hasMoreTokens()) {

			String pathToken = tokenizer.nextToken();
			URL url = null;
			try {
				url = new URL(pathToken);
				urls.add(url);
				continue;
			} catch (MalformedURLException e) {
			}

			// Not a URL.  Construct a file URL
			File file = new File(pathToken);
			if (! file.exists()) {
				continue;
			}

			String fileName = ""; //$NON-NLS-1$
			try {
				fileName = file.getCanonicalPath();
			} catch (IOException e) {
			}
			fileName = fileName.replace('\\', '/');
			// Directories must end with '/'
			if ((file.isDirectory()) && (! fileName.endsWith("/"))) { //$NON-NLS-1$
					fileName = fileName.concat("/"); //$NON-NLS-1$
			}

			try {
				url = new URL("file", "", fileName); //$NON-NLS-1$ //$NON-NLS-2$
				urls.add(url);
				continue;
			} catch (MalformedURLException e) {
			}
		}

		xletPathURLs = (URL[]) (urls.toArray(new URL[urls.size()]));
		return xletPathURLs;
	}

	/**
	 * Returns the frame for the manager.
	 * @return XletApplicationManagerFrame
	 */
	public XletApplicationManagerFrame getFrame() {
		return frame;
	}

	/**
	 * Returns the current Xlets.
	 *
	 * @return 	Enumeration of XletApplications
	 */
	public Enumeration getXlets() {
		return xletApps.elements();
	}

	/**
	 * Answer whether the receiver is managing any Xlets
	 *
	 * @return		<code>true</code> if the receiver has Xlets,
	 *				<code>false</code> otherwise
	 */
	public boolean hasXlets() {
		return (!xletApps.isEmpty());
	}

	/**
	 * Given the specified Xlet, return the next Xlet being managed.
	 *
	 * @param 	xlet 	XletApplication current Xlet
	 * @return 	XletApplication next Xlet
	 */
	protected XletApplication getNextXlet(XletApplication xlet) {
		if (xletApps.isEmpty()) {
			return null;
		}
		int nextXletIndex = xletApps.indexOf(xlet) + 1;
		if (nextXletIndex >= xletApps.size()) {
			return (XletApplication) xletApps.get(0);
		}
		return (XletApplication) xletApps.get(nextXletIndex);
	}

	/**
	 * Add a listener for Xlet changes.
	 *
	 * @param 	listener 	XletListener
	 */
	public void addXletListener(XletListener listener) {
		listeners.add(listener);
	}

	/**
	 * Remove a listener for Xlet changes.
	 *
	 * @param 	listener 	XletListener
	 */
	public void removeXletListener(XletListener listener) {
		listeners.remove(listener);
	}

	protected void notifyXletAdded(XletApplication xlet) {
		for (int i=0; i < listeners.size(); i++) {
			((XletListener) listeners.elementAt(i)).xletAdded(xlet);
		}
	}

	protected void notifyXletRemoved(XletApplication removedXlet, XletApplication nextXlet) {
		for (int i=0; i < listeners.size(); i++) {
			((XletListener) listeners.elementAt(i)).xletRemoved(removedXlet, nextXlet);
		}
	}

	public void notifyXletStateChange(XletApplication xlet, XletState newState) {
		for (int i=0; i < listeners.size(); i++) {
			((XletListener) listeners.elementAt(i)).xletStateChanged(xlet, newState);
		}
	}

	/**
	 * Load an Xlet into the manager.
	 *
	 * Note: This method is called by the TCKXletManagerProxy.
	 */
	public XletApplication loadXlet(String className, String location, String[] args) throws ClassNotFoundException {

		XletApplication xletApp = new XletApplication(className, location, args);
		try {
			createXlet(xletApp);
		} catch (ClassNotFoundException ex) {
			throw ex;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		return xletApp;
	}

	/**
	 * Main method for command line invocation.
	 *
	 * @param 	args String[] command line args
	 */
	public static void main(String[] args) {
		getXletApplicationManager().init(args);
	}

	/**
	 * Startup error.  Prints how the method main should be used.
	 */
	protected static void usageError(String error) {
		StringBuffer usageString = new StringBuffer();
		usageString.append(MessageStrings.getFormattedString("BBAWTXLET_USAGE_ERROR", error)); //$NON-NLS-1$
		usageString.append("\n").append(MessageStrings.getString("BBAWTXLET_USAGE")); //$NON-NLS-1$ //$NON-NLS-2$
		usageString.append("\n\tXletApplicationManager {"); //$NON-NLS-1$
		usageString.append("\n\t\t-name: -path: [-args:] [-name: -path: [-args:]]..."); //$NON-NLS-1$
		usageString.append("\n\t\t| -filename: }"); //$NON-NLS-1$
		usageString.append("\n\n\t").append(MessageStrings.getString("BBAWTXLET_WHERE")); //$NON-NLS-1$ //$NON-NLS-2$
		usageString.append("\n\t\t").append(MessageStrings.getString("BBAWTXLET_REPEATED")); //$NON-NLS-1$ //$NON-NLS-2$
		usageString.append("\n\t\t\t-name:  = ").append(MessageStrings.getString("BBAWTXLET_NAME_DESC")); //$NON-NLS-1$ //$NON-NLS-2$
		usageString.append("\n\t\t\t-path:  = ").append(MessageStrings.getString("BBAWTXLET_PATH_DESC")); //$NON-NLS-1$ //$NON-NLS-2$
		usageString.append("\n\t\t\t-args:  = ").append(MessageStrings.getString("BBAWTXLET_ARGS_DESC")); //$NON-NLS-1$ //$NON-NLS-2$
		usageString.append("\n\n\t\t-filename:  ").append(MessageStrings.getString("BBAWTXLET_FILENAME_DESC")); //$NON-NLS-1$ //$NON-NLS-2$
		usageString.append("\n\t\t\t").append(MessageStrings.getString("BBAWTXLET_FILE_CONTENT")); //$NON-NLS-1$ //$NON-NLS-2$
		usageString.append("\n\t\t\t\t[-name: -path: [-args:]"); //$NON-NLS-1$

		System.err.println(usageString.toString());
		System.exit(-1);
	}
}
