package com.ibm.oti.xlet.ixc;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2006  All Rights Reserved
 */

import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.security.AccessControlContext;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;

import javax.microedition.xlet.XletContext;
import javax.microedition.xlet.ixc.StubException;

import com.ibm.oti.xlet.XletApplication;

public class IxcDispatcher {

	public static final byte ReturnValuePrefix = 0x01;
	public static final byte ReturnExceptionPrefix = 0x02;

	public static byte[] dispatch(long id,Method method, byte[] arg_bytes) throws StubException, RemoteException {
		return new IxcDispatcher().invoke(id,method,arg_bytes);
	}

	private byte[]invoke(long id,Method method, byte[] arg_bytes) throws StubException, RemoteException {

		Remote ro = IxcServerTable.getRemoteObjectForId(id);
		if (ro == null)
			throw new RemoteException("Did not find receiver in the object table: " + id); //$NON-NLS-1$

		// the context that is receiving the remote method call.
		XletContext calledContext = IxcServerTable.toContext(ro);

		Object[] args = readArgs(method, arg_bytes, ro, calledContext);

		XletObjectOutputStream out;
		ByteArrayOutputStream baosss;
		try {
			baosss = new ByteArrayOutputStream();
			out = new XletObjectOutputStream(baosss, calledContext);
		} catch (IOException e) {
			throw new RemoteException("Exception invoking method " + method.getName(), e); //$NON-NLS-1$
		}

		Object result;
		try {
			result = invokeMethod(method, ro, calledContext, args);

			try {
				out.writeByte(IxcDispatcher.ReturnValuePrefix);
			} catch (IOException e) {
				throw new RemoteException("Exception invoking method " + method.getName(), e); //$NON-NLS-1$
			}
		} catch (IllegalAccessException e) {
			throw new RemoteException("Exception invoking method " + method.getName(), e); //$NON-NLS-1$
		} catch (InvocationTargetException e) {
			try {
				out.writeByte(IxcDispatcher.ReturnExceptionPrefix);
			} catch (IOException ex) {
				throw new RemoteException("Exception invoking method " + method.getName(), ex); //$NON-NLS-1$
			}
			result = e.getTargetException();
		} catch (NoSuchMethodException e) {
			throw new RemoteException("Exception invoking method " + method.getName(), e); //$NON-NLS-1$
		} catch (IllegalArgumentException e) {
			throw new RemoteException("Exception invoking method "+ method.getName(), e); //$NON-NLS-1$
		}

		try {
			out.writeObject(result);
		} catch (StubException e) {
			throw e;
		} catch (IOException e) {
			throw new RemoteException("Exception while writing result  " + method.getName(), e); //$NON-NLS-1$
		}

		return baosss.toByteArray();
	}

	private Object invokeMethod(Method method, final Remote ro, XletContext calledContext, final Object[] args)
		throws IllegalAccessException, InvocationTargetException, NoSuchMethodException {
		Class[] paramtypes = method.getParameterTypes();
		Class[] newParamtypes = new Class [paramtypes.length];

		try {
			for (int i = 0; i < paramtypes.length; i++) {
				if (paramtypes[i].isPrimitive())
					newParamtypes[i] = paramtypes[i];
				else {
					String paramname = paramtypes[i].getName();
					newParamtypes[i] = calledContext.getClassLoader().loadClass(paramname);
				}
			}
		} catch (ClassNotFoundException e) {
			throw new IllegalArgumentException(e.getMessage());
		}

		final Method m = ro.getClass().getMethod(method.getName(),newParamtypes);
			AccessController.doPrivileged(new PrivilegedAction() {
				public Object run() {
					m.setAccessible(true);
					return null;
			}});

		final IllegalAccessException[] accessException = new IllegalAccessException[1];
		final InvocationTargetException[] invocationException = new InvocationTargetException[1];
		AccessControlContext securityContext = ((XletApplication)calledContext).getSecurityContext();
		Object res =  AccessController.doPrivileged(new PrivilegedAction() {
			public Object run() {
				try {
					return m.invoke(ro,args);
				} catch (IllegalArgumentException e) {
					throw e;
				} catch (IllegalAccessException e) {
					accessException[0] = e;
				} catch (InvocationTargetException e) {
					invocationException[0] = e;
				}
				return null;
			}
		}, securityContext);

		if (accessException[0] != null) throw accessException[0];
		if (invocationException[0] != null) throw invocationException[0];
		return res;
	}

	private Object[] readArgs(Method method, byte[] argBytes,Remote ro,XletContext calledContext)
		throws StubException, RemoteException {

		Object[] args;
		XletObjectInputStream in;
		try {
			in =
				new XletObjectInputStream(
					new ByteArrayInputStream(argBytes),
					calledContext);
			args = (Object[]) in.readObject();
		} catch (StubException e) {
			throw e;
		} catch (ClassNotFoundException e) {
			throw new RemoteException("Exception while reading arguments " + method.getName(), e); //$NON-NLS-1$
		} catch (IOException e) {
			throw new RemoteException("Exception while reading arguments " + method.getName(), e); //$NON-NLS-1$
		}
		return args;
	}
}
