package com.ibm.oti.xlet.ixc;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2005, 2006  All Rights Reserved
 */

import java.security.Permission;
import java.security.PermissionCollection;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Vector;

import javax.microedition.xlet.ixc.IxcPermission;

/**
 * PermuissionCollections for Xlets
 *
 */
public class IxcPermissionCollection extends PermissionCollection {
	Vector binds;
	Vector lookups;

	private static final long serialVersionUID = -2154300303877389853L;

	class IxcPermissionEnumeration implements Enumeration {
		Enumeration bindsEnum, lookupsEnum;
		Object nextElement;
		IxcPermissionEnumeration(Enumeration bindsEnum, Enumeration lookupsEnum) {
			super();
			this.bindsEnum = bindsEnum;
			this.lookupsEnum = lookupsEnum;
			this.nextElement = findNextElement();
		}
		public boolean hasMoreElements() {
			return nextElement != null;
		}
		public Object nextElement() {
			if (nextElement == null) {
				throw new NoSuchElementException();
			} else {
				Object result = nextElement;
				nextElement = findNextElement();
				return result;
			}
		}
		private Object findNextElement() {
			if (bindsEnum != null) {
				if (bindsEnum.hasMoreElements()) return bindsEnum.nextElement();
				this.bindsEnum = null;
			}
			if (lookupsEnum != null) {
				if (lookupsEnum.hasMoreElements()) return lookupsEnum.nextElement();
				this.lookupsEnum = null;
			}
			return null;
		}
	}
	/**
	 *
	 */
	public IxcPermissionCollection() {
		super();
		this.binds = new Vector();
		this.lookups = new Vector();
	}

	/* (non-Javadoc)
	 * @see java.security.PermissionCollection#add(java.security.Permission)
	 */
	public void add(Permission permission) {
		if (!(permission instanceof IxcPermission)) throw new IllegalArgumentException(permission.toString());

		if (isReadOnly()) throw new IllegalStateException();

		String actions = permission.getActions();
		if (actions.indexOf("bind") > -1) binds.addElement(permission);
		if (actions.indexOf("lookup") > -1) lookups.addElement(permission);
}

	/* (non-Javadoc)
	 * @see java.security.PermissionCollection#elements()
	 */
	public Enumeration elements() {
		if (binds.size() != 0) {
			if (lookups.size() != 0) {
				return new IxcPermissionEnumeration(binds.elements(), lookups.elements());
			} else {
				return binds.elements();
			}
		} else {
			return lookups.elements();
		}
	}

	/* (non-Javadoc)
	 * @see java.security.PermissionCollection#implies(java.security.Permission)
	 */
	public boolean implies(Permission permission) {
		if (permission instanceof IxcPermission) {
			String actions = permission.getActions();
			if (actions.indexOf("bind") > -1) {
				if (!pathNameImpliedByOneOf(binds, (IxcPermission)permission)) return false;
			}
			if (actions.indexOf("lookup") > -1) {
				if (!pathNameImpliedByOneOf(lookups, (IxcPermission)permission)) return false;
			}
			return true;
		}
		return false;
	}
	private boolean pathNameImpliedByOneOf(Vector permissions, IxcPermission other) {
		int len = permissions.size();
		IxcPermission p;
		for (int i = 0; i < len; i++) {
			p = (IxcPermission)(permissions.elementAt(i));
			if (IxcUtil.pathNameImplies(p, other)) return true;
		}
		return false;
	}

}
