package com.ibm.oti.xlet.ixc;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2005  All Rights Reserved
 */

import java.io.Serializable;
import java.lang.reflect.*;
import java.rmi.Remote;
import java.rmi.RemoteException;

import javax.microedition.xlet.ixc.StubException;

public class IxcProxyFactory {

	protected static Remote createStub(Remote ro, long id, long contextId) throws StubException {
		try {
			validate(ro);

			ClassLoader roCloader = ro.getClass().getClassLoader();
			Class[] interfaces = IxcUtil.getAllRemoteInterfaces(ro.getClass());

			InvocationHandler h = new XInvocationHandler(id, contextId);
			Remote stub = (Remote) Proxy.newProxyInstance(roCloader, interfaces, h);
			return stub;

		} catch (StubException e) {
			throw e;
		} catch (Exception e) {
			throw new StubException("Exception while creating stub ", e); //$NON-NLS-1$
		}
	}

	protected static void validate(Remote o) throws StubException {
		Class clazz = o.getClass();
		Class[] Interfaces = IxcUtil.getAllRemoteInterfaces(clazz);
		Method[] methods = IxcUtil.findMethods(clazz);
		for(int i=0;i<methods.length;i++) {
			validateMethod(methods[i]);
		}
	}

	private static void validateMethod(Method method) throws StubException {
		Class params[] = method.getParameterTypes();
		validateParams(params);
		Class returnType = method.getReturnType();
		validateParams(new Class[] {returnType});
		Class[] exceptions = method.getExceptionTypes();
		validateExceptions(exceptions, method);
	}

	private static void validateParams(Class[] params) throws StubException {
		if (params.length == 0)
			return;
		for (int i = 0; i < params.length; i++) {
			Class param = params[i];
			if (validateParam(param) == false)
				throw new StubException("Invalid parameter or return type " + param); //$NON-NLS-1$
		}
	}

	private static boolean validateParam(Class param) {
		if (param.isPrimitive())
			return true;

		if (param.isInterface()) {
			if (Remote.class.isAssignableFrom(param) || Serializable.class.isAssignableFrom(param))
				return true;
			else {
				return false;
			}
		}

		// if this is a serializable object
		if (Serializable.class.isAssignableFrom(param))
			return true;

		Class [] interfaces = param.getInterfaces();

		return false;
	}

	private static void validateExceptions(Class[] classes, Method method) throws StubException {
		boolean remoteExceptionFound = false;
		for (int i = 0; i < classes.length; i++) {
			if (RemoteException.class.isAssignableFrom(classes[i])) {
				remoteExceptionFound = true;
				break;
			}
		}

		if (remoteExceptionFound == false)
			throw new StubException("Remote method does not have RemoteException in its signature " + method.toString()); //$NON-NLS-1$
	}
}
