package com.ibm.oti.xlet.ixc;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2005  All Rights Reserved
 */

import java.lang.reflect.Proxy;
import java.rmi.Remote;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.microedition.xlet.XletContext;
import javax.microedition.xlet.ixc.StubException;

public class IxcServerTable {

	private static Hashtable idToRo = new Hashtable();

	private static Hashtable ContextIdToContext = new Hashtable();

	private static Hashtable roToStub = new Hashtable();

	private static Hashtable roToContext = new Hashtable();

	private static long idcount=0 ;
	private static long contextIdcount=0 ;

	protected static Remote toStub(Remote o, XletContext exporter_context) throws StubException {
		// check if the object to export is already a stub
		// his would be the case if this is a remote object created within another xletcontext,
		// and imported by this xletcontext earlier.
		 if (o  instanceof Proxy) {
			return o;
		}

		// since it is not a stub, now we know this is a remote object created within this xletcontext.
		// check if this object has already been exported
		// it might have been exported previously since it was parameter or return value of a remote
		// method call to or from this xletcontext, or it was bound tot he registry by this context.
		if (isExported(o)) {
			Remote stub = (Remote)roToStub.get(o);
			return stub;
		} else {
		// since the remote object has not been exported before, do it now.
			Remote stub = exportObject(o, exporter_context);
			return stub;
		}
	}

	protected static Object resolveStub(Remote stub, XletContext importerContext) {
		XletContext context_ro = toContext(getRemoteObjectForStub(stub));

		if (context_ro == importerContext) {
			return getRemoteObjectForStub(stub);
		} else {
			changeImporterContextForStub(stub, importerContext);
			return stub;
		}
	}

	protected static Remote getRemoteObjectForStub(Remote stub) {
		if (stub instanceof Proxy) {
			long id =  ((XInvocationHandler)Proxy.getInvocationHandler((Proxy)stub)).id;
			return (Remote) idToRo.get(new Long(id));
		}
		return null;
	}

	protected static Remote getRemoteObjectForId(long id) {
		return (Remote) idToRo.get(new Long(id));
	}

	protected static XletContext toContext(Remote o) {
		//if object is a stub, return the context associated with it
		if (o instanceof Proxy) {
			long contextId =  ((XInvocationHandler)Proxy.getInvocationHandler((Proxy)o)).contextId;
			return (XletContext) ContextIdToContext.get(new Long(contextId));
		} else {
			//if object is a remote object, return the context that exported it
			return (XletContext)roToContext.get(o);
		}
	}

	protected static XletContext getContextForId(long contextId) {
		return (XletContext) ContextIdToContext.get(new Long(contextId));
	}

	protected static boolean isExported(Remote o) {
		if (roToContext.get(o)!=null)
			return true;
		else
			return false;
	}

	protected static Remote exportObject(Remote ro, XletContext context) throws StubException {
		if (ro==null)
			throw new NullPointerException();

		long id = idcount++;
		long contextId = getContextId(context);

		Remote stub = IxcProxyFactory.createStub(ro, id, contextId);
		idToRo.put(new Long(id), ro);
		roToStub.put(ro, stub);
		roToContext.put(ro,context);

		return stub;
	}

	protected static long getContextId(XletContext context) {
		// enumaration ContextIdToContext
		// if it is not there yet, create an entry otherwise return back the id;
		long id = 0;
		int i = 0;
		Long[] ids = new Long[ContextIdToContext.size()];
		synchronized(ContextIdToContext) {
			Enumeration keys = ContextIdToContext.keys();
			while (keys.hasMoreElements())
				ids[i++] = (Long)keys.nextElement();
		}

		for (int j =0; j<ids.length; j++) {
			if (ContextIdToContext.get(ids[j]) == context) {
				Long a = ids[j];
				id =a.longValue();
				break;
			}
		}

		if (id ==0) {
			contextIdcount ++;
			id =contextIdcount;
			ContextIdToContext.put(new Long(id), context);
		}

		return id;
	}

	protected static void changeImporterContextForStub(Object object, XletContext receiverContext)  {
		((XInvocationHandler)Proxy.getInvocationHandler((Proxy)object)).contextId = IxcServerTable.getContextId(receiverContext);
	}
}
