package com.ibm.oti.xlet.ixc;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2006  All Rights Reserved
 */

import java.lang.reflect.Method;
import java.rmi.Remote;
import java.util.*;
import javax.microedition.xlet.ixc.IxcPermission;

/**
 * This class has a set of methods used by ixcProxyFactory..
 *
 * @author      IBM
 * @version		initial
 */
public class IxcUtil {
	static final Class objClass = Object.class;
	static final Class remoteClass = Remote.class;

	/**
	 * Answers an array with all Remote interfaces implemented
	 * by <code>c</code>.
	 *
	 * @author      IBM
	 * @version		initial
	 */
	public static Class[] getAllRemoteInterfaces(Class c) {
		Vector remoteInterfaces = new Vector();
		getAllRemoteInterfaces(c, new Hashtable(16), remoteInterfaces);
		Class r[] = new Class[remoteInterfaces.size()];
		for(int i=0;i<r.length;i++)
			r[i] = (Class)remoteInterfaces.elementAt(i);
		return r;
	}

	/*
	 * Answers a Method array with the methods
	 * that must be implemented by the stub.
	 */
	protected static Method[] findMethods(Class remoteClass) {
		TreeMap meth = new TreeMap();
		Class ints[] = IxcUtil.getAllRemoteInterfaces(remoteClass);
		for(int i =0;i<ints.length;i++) {
			Method[] m = ints[i].getMethods();
			for(int j=0;j<m.length;j++) {
				String sig = operationString1(m[j]);
				if (!meth.containsKey(sig))
					meth.put(sig, new Object[]{m[j], ints[i]});
			}
		}
		Method[] methods = new Method[meth.size()];
		Class[]  interfaces = new Class[meth.size()];
		int i=0;
		Iterator it = meth.values().iterator();
		while(it.hasNext()) {
			Object[] value = (Object[])it.next();
			methods[i] = (Method)value[0];
			interfaces[i] = (Class)value[1];
			i++;
		}
		return methods;
	}

	/*
	 * Answers the second part of the operation string.
	 * Ex: Answers (int, java.lang.String, double)
	 * for String foo(int i,String s);
	 * Ex: Answers (int, java.lang.String, double)[]
	 * for String[] foo(int i,String s);
	*/
	protected static String operationString1 (Method m) {
		String r = m.getName() + "("; //$NON-NLS-1$
		Class[] pTypes = m.getParameterTypes();
		for(int i=0;i<pTypes.length;i++) {
			r = r + getTypeName(pTypes[i]);
			if(i + 1 < pTypes.length)
				r = r + ", "; //$NON-NLS-1$
		}
		r = r + ")"; //$NON-NLS-1$
		if(m.getReturnType().isArray()) {
			int dim = m.getReturnType().getName().lastIndexOf('[') + 1;
			for(int i=0;i<dim;i++)
				r = r + "[]"; //$NON-NLS-1$
		}
		return r;
	}

	/*
	 * Answers <code>c</code>'s qualified name.
	 */
	protected static String getTypeName(Class c) {
		if(c.isPrimitive()) {
			if(c == boolean.class) return "boolean"; //$NON-NLS-1$
			if(c == byte.class) return "byte"; //$NON-NLS-1$
			if(c == char.class) return "char"; //$NON-NLS-1$
			if(c == short.class) return "short"; //$NON-NLS-1$
			if(c == int.class) return "int"; //$NON-NLS-1$
			if(c == long.class) return "long"; //$NON-NLS-1$
			if(c == float.class) return "float"; //$NON-NLS-1$
			if(c == double.class) return "double"; //$NON-NLS-1$
			if(c == void.class) return "void"; //$NON-NLS-1$
			return null;
		}
		if(!c.isArray())
			return c.getName();

		String tName = c.getName();
		int bIndex = tName.lastIndexOf('[');
		if(tName.charAt(bIndex + 1) == 'L') {
			tName = tName.substring(bIndex + 2,tName.length() - 1);
		} else {
			char type = tName.charAt(bIndex + 1);
			if(type == 'Z') tName = "boolean"; //$NON-NLS-1$
			if(type == 'B') tName = "byte"; //$NON-NLS-1$
			if(type == 'C') tName = "char"; //$NON-NLS-1$
			if(type == 'S') tName = "short"; //$NON-NLS-1$
			if(type == 'I') tName = "int"; //$NON-NLS-1$
			if(type == 'J') tName = "long"; //$NON-NLS-1$
			if(type == 'F') tName = "float"; //$NON-NLS-1$
			if(type == 'D') tName = "double"; //$NON-NLS-1$
			if(type == 'V') tName = "void"; //$NON-NLS-1$
		}
		for(int i=0;i<=bIndex;i++) {
			tName = tName + "[]"; //$NON-NLS-1$
		}
		return tName;
	}

	/**
	 * Adds all Remote interfaces implemented by <code>c</code> in the
	 * specified Hashtable.
	 *
	 * @author      IBM
	 * @version		initial
	 */
	private static boolean getAllRemoteInterfaces(Class c, Hashtable foundInterfaces, Vector result) {
		if (c == remoteClass) return true;
		boolean remote = false;
		Class supercl = c.getSuperclass();
		if (supercl != null && supercl != objClass)
			remote = getAllRemoteInterfaces(supercl, foundInterfaces, result);
		Class cInterfaces[] = c.getInterfaces();
		for(int i=0;i<cInterfaces.length;i++) {
			Class cInt = cInterfaces[i];
			if (cInt.isAssignableFrom(remoteClass) || foundInterfaces.containsKey(cInt)) {
				remote = true;
				continue;
			}
			if (getAllRemoteInterfaces(cInt, foundInterfaces, result)) {
				remote = true;
				foundInterfaces.put(cInt, cInt);
				result.addElement(cInt);
			}
		}
		return remote;
	}
	/**
	 * Returns true if the first pathName implies the second regarding the specs of an IxcPermission
	 *
	 * @param pathName
	 * @param otherPathName
	 * @return
	 */
	public static boolean pathNameImplies(IxcPermission p1, IxcPermission p2) {
		String pathName = p1.getName();
		String otherPathName = p2.getName();
		int len = pathName.length();
		if (pathName.equals(otherPathName)) return true;
		// An asterisk may appear by itself, or if immediately preceded by a "/" may appear at the end of a name, to signify a wildcard match.
		int index = pathName.indexOf('*');
		if (index == 0) {
			// ==> the name must be equal to "*" to be a valid wildcard
			return (len == 1); // A "*" accepts any other pathName
		}
		if ((index == (len - 1)) && (pathName.charAt(len -2) == '/')) {
			String nameWithoutWildcard = pathName.substring(0, len-1);
			if (otherPathName.startsWith(nameWithoutWildcard)) return true;
		}
		return false;
	}

}
