package com.ibm.oti.xlet.ixc;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2005  All Rights Reserved
 */

import java.io.*;
import java.lang.reflect.*;
import java.rmi.RemoteException;
import java.rmi.UnexpectedException;
import java.security.AccessController;
import java.security.PrivilegedAction;

import javax.microedition.xlet.XletContext;
import javax.microedition.xlet.ixc.StubException;

public class XInvocationHandler implements InvocationHandler, Serializable {

public static final byte ReturnValuePrefix = 0x01;
public static final byte ReturnExceptionPrefix = 0x02;

	// the id for the remote Object
	long id ;

	// the id for the context that has imported this stub.
	// it is also the context that will make remote method calls
	// to the remote object using this stub.
	long contextId;

	public XInvocationHandler(long id, long contextId) {
		this.id =id;
		this.contextId = contextId;
	}

	public boolean equals(Object o) {

		if (o instanceof XInvocationHandler) {
			XInvocationHandler xo = (XInvocationHandler) o;
			return id == xo.id;
		}

		return false;
	}

	public int hashCode() {
		return (int)id;
	}

	public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
		Object result;
		if (method.getDeclaringClass() == Object.class) {
			result = delegate(method, args);
			if (result != null)
				return result;
		}

		XletContext callerContext = IxcServerTable.getContextForId(contextId);
		ClassLoader cls =null;
		if (args!=null && args.length==1)
			cls = args[0].getClass().getClassLoader();

		byte[] argBytes = writeArgs(args, callerContext, method);

		byte[] resultArray = IxcDispatcher.dispatch(id, method, argBytes);

		result = readResult(method, callerContext, resultArray);

		return result;
	}

	private Object readResult(Method method, XletContext callerContext,byte[] resultArray) throws RemoteException, Exception {

		XletObjectInputStream in;
		Object result;
		try {
			in =
				new XletObjectInputStream(
					new ByteArrayInputStream(resultArray),
					callerContext);
		} catch (IOException e) {
			throw new RemoteException("Exception while reading result " + method.getName(), e); //$NON-NLS-1$
		}

		readReturnValueHeader(in, method);

		try {

			result = in.readObject();
		} catch (ClassNotFoundException e) {
			throw new RemoteException("Exception while reading result " + method.getName(), e); //$NON-NLS-1$
		} catch (IOException e) {
			throw new RemoteException("Exception while reading result " + method.getName(), e); //$NON-NLS-1$
		}
		return result;
	}

	protected byte[] writeArgs(Object[] args, XletContext callerContext, Method method) throws RemoteException {
		//write the args in a stream..
		ByteArrayOutputStream baosss = new ByteArrayOutputStream();

		try {
			XletObjectOutputStream out =
			new XletObjectOutputStream(baosss, callerContext);
			out.writeObject(args);
		} catch (StubException e) {
			throw e;
		} catch (IOException e) {
			throw new RemoteException("Exception while writing arguments " + method.getName(), e); //$NON-NLS-1$
		}

		byte[] argBytes = baosss.toByteArray();
		return argBytes;
	}

	private Object delegate(Method method, Object[] args) throws NoSuchMethodException {
		if (method.equals(Object.class.getMethod("equals", new Class[] {Object.class}))) { //$NON-NLS-1$
			if (args[0] instanceof Proxy)
				return new Boolean(this.equals(Proxy.getInvocationHandler((Proxy)args[0])));
			else
				return new Boolean(false);
		}

		if (method.equals(Object.class.getMethod("hashCode", null))) { //$NON-NLS-1$
			return new Integer(this.hashCode());
		}

		return null;
	}

	private void readReturnValueHeader (ObjectInput ois, Method method) throws Exception {
		//Read ReturnValuePrefix || ReturnExceptionPrefix
		byte prefix = ois.readByte();
		if(prefix == ReturnExceptionPrefix) {
			Exception ex;
			try {
				ex = (Exception)ois.readObject();
			} catch (Exception e) {
				throw new java.rmi.RemoteException("Exception occurred when reading the exception that occurred on the server"); //$NON-NLS-1$
			}

			if (ex instanceof java.rmi.RemoteException) {
				throw ex;
			}

			boolean exceptionIsDeclared = false;
			Class[] exceptions = method.getExceptionTypes();
			for (int i = 0; i < exceptions.length; i++) {
				if (exceptions[i].isAssignableFrom(ex.getClass())) {
					exceptionIsDeclared = true;
					break;
				}
			}
			if (exceptionIsDeclared) {
				// not sure, should we really fill in the stack trace?
				ex.fillInStackTrace();
				throw ex;
			}

			if (ex instanceof RuntimeException)
				throw ex;
			 else
				throw new UnexpectedException("Unexpected exception occurred", ex);		 //$NON-NLS-1$
		}
	}
}
