package com.ibm.oti.xlet.ixc;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2005  All Rights Reserved
 */

import java.io.*;
import java.lang.reflect.Proxy;
import java.rmi.Remote;
import java.security.AccessController;
import java.security.PrivilegedAction;

import javax.microedition.xlet.XletContext;
import javax.microedition.xlet.ixc.StubException;

import com.ibm.oti.xlet.XletApplication;

public class XletObjectInputStream extends ObjectInputStream {

private XletContext receiverContext;
private ClassLoader receiverClassLoader;

/**
 * Constructs a new instance of this class.
 *
 * @author      IBM
 * @version		initial
 */
public XletObjectInputStream(InputStream in, XletContext context)
	throws StreamCorruptedException, IOException {
	super(in);
	receiverContext = context;
	receiverClassLoader = ((XletApplication)context).getClassLoader();

	AccessController.doPrivileged(new PrivilegedAction() {
		public Object run() {
			return enable();
		}});
}

	private Object enable() {
		enableResolveObject(true);
		return null;
}

	protected Object resolveObject(Object object) throws IOException {
		if ((object instanceof Remote) && (object instanceof Proxy)) {
			return IxcServerTable.resolveStub((Remote)object,receiverContext);
		}
		return object;
	}

	protected Class resolveClass(ObjectStreamClass osClass) throws IOException, ClassNotFoundException {
		// resolve the class using the receiverClassLoader
		// (ObjectInputStream uses VM.getNonBootstrapClassLoader() by default)
		return Class.forName(osClass.getName(), true, receiverClassLoader);
	}

	protected Class resolveProxyClass(String[] interfaceNames) throws IOException, ClassNotFoundException {
		// resolve the ProxyClass using the receiverClassLoader
		// (ObjectInputStream uses VM.getNonBootstrapClassLoader() by default)
		try {
			Class[] interfaces = new Class[interfaceNames.length];
			for (int i = 0; i < interfaceNames.length; i++) {
				interfaces[i] =
					Class.forName(interfaceNames[i], false, receiverClassLoader);
			}

			try {
				return Proxy.getProxyClass(receiverClassLoader, interfaces);
			} catch (IllegalArgumentException e) {
				throw new ClassNotFoundException(e.toString(), e);
			}

		} catch (ClassNotFoundException e) {
			throw new StubException("Stub could not be read: ", e); //$NON-NLS-1$
		}
	}
}
