package com.ibm.oti.xlet.ixc;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2005  All Rights Reserved
 */

import java.io.*;
import java.rmi.Remote;
import java.security.AccessController;
import java.security.PrivilegedAction;

import javax.microedition.xlet.XletContext;

public class XletObjectOutputStream extends ObjectOutputStream {

	XletContext senderContext;

	/**
	 * Constructs a new XletObjectOutputStream on the OutputStream <code>output</code>.  All
	 * writes are now filtered through this stream.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		output		The non-null OutputStream to filter writes on.
	 *
	 * @exception		IOException		If an IO exception happened when writing the object stream header
	 */
	public XletObjectOutputStream(OutputStream output, XletContext senderContext) throws IOException {
		super(output);
		this.senderContext = senderContext;
		AccessController.doPrivileged(new PrivilegedAction() {
			public Object run() {
				return enable();
			}});
	}

	private Object enable() {
		enableReplaceObject(true);
		return null;
	}

	/**
	 * Computes the replacement object for the original object
	 * <code>object</code> and answers the replacement.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		object		Original object for which a replacement may be defined.
	 *
	 * @return		Object		A possibly new replacement object for <code>object</code>
	 *
	 * @exception		java.io.IOException		If any IO problem occurred when trying to resolve the object.
	 *
	 * @see			enableReplaceObject()
	 * @see			ObjectInputStream.enableResolveObject()
	 * @see			ObjectInputStream.resolveObject()
	 */
	protected Object replaceObject(Object object) throws IOException {
		if(object instanceof Remote)
			return IxcServerTable.toStub((Remote)object, senderContext);
		else
			return object;
	}
}
