package com.ibm.ugl;

import java.security.AccessController;
import java.security.PrivilegedExceptionAction;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2004, 2005  All Rights Reserved
 */

/**
 * The UGL OS classes need to be able to live in a
 * project with a CLDC classpath.  System.loadLibrary()
 * is missing from that configuration, so there is a
 * compatibility class to prevent a compile error.
 */
public class UGLCompatibility {

	public static UnsatisfiedLinkError loadLibrary(final String library) {
		try {
			// We need to protect the loadLibary call in case
			// loadLibary() is not allow by the current SecurityManager
            AccessController.doPrivileged(new PrivilegedExceptionAction() {
                public Object run() throws Exception {
                	System.loadLibrary(library);
                    return null;
                }
            });
            return null;
		} catch(Exception exception) {
			throw new UnsatisfiedLinkError(library);
		} catch (UnsatisfiedLinkError error) {
			return error;
		}
	}
}
