package java.applet;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.net.URL;

/**
 * This is an interface implemented by an applet's containing environment, and
 * it is not generally used by applet programmers.  An <code>AppletStub</code>
 * is associated with each applet in a given environment or applet context.
 * A given applet uses its applet stub as a means of interacting with the
 * environment regarding data and operations relating specifically to itself
 * as one of the potentially many applets existing in the context.  An
 * <code>AppletContext</code> object, on the other hand may be shared by
 * many applets and provides methods which allow these applets to discover
 * information about their environment or context.
 *
 * @author		IBM
 * @version		initial
 *
 * @see	Applet
 * @see	AppletContext
 */

public interface AppletStub {

	/**
	 * Requests that receiver's applet be resized.
	 * The containing environment may ignore this request.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param		width	int
	 *				The intended new width for the applet.
	 * @param		height int
	 *				The intended new height for the applet.
	 */
	void appletResize(int width, int height);

	/**
	 * Answers the receiver's applet's applet context.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @return		AppletContext
	 *				this applet's applet context
	 *
	 * @see			AppletContext
	 */
	AppletContext getAppletContext();

	/**
	 * Answers the receiver's applet's code base URL.
	 * <p>
	 * If the 'code URL' is the URL indicating the location of the
	 * receiver's applet's class file, then the 'code base URL' is
	 * the part of the code URL excluding the applet's class filename.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @return		URL
	 *				the code base URL of this applet
	 *
	 */
	URL getCodeBase();

	/**
	 * Answers the receiver's applet's document base URL.
	 * <p>
	 * If the 'document URL' is the URL of the document in which the
	 * receiver's applet was embedded, then the 'document base URL' is
	 * the part of the document URL excluding the documents name.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @return		URL
	 *				the document base URL of this applet
	 *
	 */
	URL getDocumentBase();

	/**
	 * Answers the value of the applet parameter named <code>name</code>.
	 * Applet parameters are typically specified in the HTML document containing
	 * the applet.  If this is the case, this method will return the <code>value</code>
	 * attribute of a <code>param</code> tag (nested within this applet's
	 * <code>applet</code> tag) whose <code>name</code> attribute is <code>name</code>.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param		name String
	 *				the name of the parameter whose value is to be returned
	 *
	 * @return		String
	 *				the value of the parameter with the specified name, or
	 *				<code>null</code> if the named parameter does not exist
	 */
	String getParameter(String name);

	/**
	 * Answers true if the receiver's applet is active and false otherwise.
	 * An applet becomes active just before its <code>start()</code> method
	 * is called and ceases to be active just before its <code>stop()</code>
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @return		whether or not the receiver's applet is active.
	 *
	 */
	boolean isActive();

}
