package java.awt;
/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2005  All Rights Reserved
 */

import java.awt.event.AWTEventListener;
import java.util.Vector;
/**
 * Internal class used to broadcast AWTEvent to the AWTEventListener
 */
class AWTEventBroadcaster {
	AWTEventListener[] listeners;

	class AWTEventListenerMulticaster implements AWTEventListener {
		AWTEventListener l1;
		AWTEventListener l2;

		AWTEventListenerMulticaster(AWTEventListener l1, AWTEventListener l2) {
			super();
			this.l1 = l1;
			this.l2 = l2;
		}
		public void eventDispatched(AWTEvent event) {
			l1.eventDispatched(event);
			l2.eventDispatched(event);
		}
		AWTEventListener removeAWTEventListener(AWTEventListener listener) {
			if (listener == l1) return l2;
			if (listener == l2) return l1;
			if (l1 instanceof AWTEventListenerMulticaster) {
				this.l1 = ((AWTEventListenerMulticaster)l1).removeAWTEventListener(listener);
			} else
			if (l2 instanceof AWTEventListenerMulticaster) {
				this.l2 = ((AWTEventListenerMulticaster)l2).removeAWTEventListener(listener);
			}
			return this;
		}

	}
	/**
	 *
	 */
	public AWTEventBroadcaster() {
		super();
		this.listeners = new AWTEventListener[AWTEvent.MAX_LOG2 + 1];
	}
	public void addAWTEventListener(AWTEventListener listener, long eventMask) {
		if ((eventMask & AWTEvent.ACTION_EVENT_MASK) != 0) 			addAWTEventListener(AWTEvent.ACTION_LOG2, listener);
		if ((eventMask & AWTEvent.ADJUSTMENT_EVENT_MASK) != 0) 		addAWTEventListener(AWTEvent.ADJUSTMENT_LOG2, listener);
		if ((eventMask & AWTEvent.COMPONENT_EVENT_MASK) != 0) 		addAWTEventListener(AWTEvent.COMPONENT_LOG2, listener);
		if ((eventMask & AWTEvent.CONTAINER_EVENT_MASK) != 0) 		addAWTEventListener(AWTEvent.CONTAINER_LOG2, listener);
		if ((eventMask & AWTEvent.FOCUS_EVENT_MASK) != 0) 			addAWTEventListener(AWTEvent.FOCUS_LOG2, listener);
		if ((eventMask & AWTEvent.ITEM_EVENT_MASK) != 0) 			addAWTEventListener(AWTEvent.ITEM_LOG2, listener);
		if ((eventMask & AWTEvent.KEY_EVENT_MASK) != 0) 			addAWTEventListener(AWTEvent.KEY_LOG2, listener);
		if ((eventMask & AWTEvent.MOUSE_EVENT_MASK) != 0) 			addAWTEventListener(AWTEvent.MOUSE_LOG2, listener);
		if ((eventMask & AWTEvent.MOUSE_MOTION_EVENT_MASK) != 0)	addAWTEventListener(AWTEvent.MOUSE_MOTION_LOG2, listener);
		if ((eventMask & AWTEvent.TEXT_EVENT_MASK) != 0) 			addAWTEventListener(AWTEvent.TEXT_LOG2, listener);
		if ((eventMask & AWTEvent.WINDOW_EVENT_MASK) != 0) 			addAWTEventListener(AWTEvent.WINDOW_LOG2, listener);
		if ((eventMask & AWTEvent.INPUT_METHOD_EVENT_MASK) != 0)	addAWTEventListener(AWTEvent.INPUT_METHOD_LOG2, listener);
		if ((eventMask & AWTEvent.PAINT_EVENT_MASK) != 0) 			addAWTEventListener(AWTEvent.PAINT_LOG2, listener);
		if ((eventMask & AWTEvent.INVOCATION_EVENT_MASK) != 0) 		addAWTEventListener(AWTEvent.INVOCATION_LOG2, listener);
		if ((eventMask & AWTEvent.MOUSE_WHEEL_EVENT_MASK) != 0) 	addAWTEventListener(AWTEvent.MOUSE_WHEEL_LOG2, listener);
		if ((eventMask & AWTEvent.WINDOW_FOCUS_EVENT_MASK) != 0) 	addAWTEventListener(AWTEvent.WINDOW_FOCUS_LOG2, listener);
	}
	public void removeAWTEventListener(AWTEventListener listener) {
		if (listener == null) return;
		int len = listeners.length;
		for (int i = 0; i < len; i++) {
			listeners[i] = removeAWTEventListener(listeners[i], listener);
		}
	}
	void addAWTEventListener(int index, AWTEventListener listener) {
		listeners[index] = addAWTEventListener(listeners[index], listener);
	}
	AWTEventListener addAWTEventListener(AWTEventListener l1, AWTEventListener l2) {
		if (l1 == null) return l2;
		if (l2 == null) return l1;
		return new AWTEventListenerMulticaster(l1, l2);
	}
	AWTEventListener removeAWTEventListener(AWTEventListener multicaster, AWTEventListener listener) {
		if (multicaster == null || multicaster == listener) return null;
		if (!(multicaster instanceof AWTEventListenerMulticaster)) return multicaster;
		return ((AWTEventListenerMulticaster) multicaster).removeAWTEventListener(listener);
	}
	void dispatchEvent(AWTEvent awtEvent) {
		int index = awtEvent.getEventMaskLog2();
		if (index == -1) return;
		AWTEventListener listener = listeners[index];
		if (listener == null) return;
		listener.eventDispatched(awtEvent);
	}

	AWTEventListener[] getAWTEventListeners() {
		int len = listeners.length;
		Vector result = new Vector();
		for (int i = 0; i < len; i++) {
			getAWTEventListeners(listeners[i], result);
		}
		return (AWTEventListener[])(result.toArray(new AWTEventListener[result.size()]));
	}

	AWTEventListener[] getAWTEventListeners(long eventMask) {
		Vector result = new Vector();

		if ((eventMask & AWTEvent.ACTION_EVENT_MASK) != 0) 			getAWTEventListeners(listeners[AWTEvent.ACTION_LOG2], result);
		if ((eventMask & AWTEvent.ADJUSTMENT_EVENT_MASK) != 0) 		getAWTEventListeners(listeners[AWTEvent.ADJUSTMENT_LOG2], result);
		if ((eventMask & AWTEvent.COMPONENT_EVENT_MASK) != 0) 		getAWTEventListeners(listeners[AWTEvent.COMPONENT_LOG2], result);
		if ((eventMask & AWTEvent.CONTAINER_EVENT_MASK) != 0) 		getAWTEventListeners(listeners[AWTEvent.CONTAINER_LOG2], result);
		if ((eventMask & AWTEvent.FOCUS_EVENT_MASK) != 0) 			getAWTEventListeners(listeners[AWTEvent.FOCUS_LOG2], result);
		if ((eventMask & AWTEvent.ITEM_EVENT_MASK) != 0) 			getAWTEventListeners(listeners[AWTEvent.ITEM_LOG2], result);
		if ((eventMask & AWTEvent.KEY_EVENT_MASK) != 0) 			getAWTEventListeners(listeners[AWTEvent.KEY_LOG2], result);
		if ((eventMask & AWTEvent.MOUSE_EVENT_MASK) != 0) 			getAWTEventListeners(listeners[AWTEvent.MOUSE_LOG2], result);
		if ((eventMask & AWTEvent.MOUSE_MOTION_EVENT_MASK) != 0)	getAWTEventListeners(listeners[AWTEvent.MOUSE_MOTION_LOG2], result);
		if ((eventMask & AWTEvent.TEXT_EVENT_MASK) != 0) 			getAWTEventListeners(listeners[AWTEvent.TEXT_LOG2], result);
		if ((eventMask & AWTEvent.WINDOW_EVENT_MASK) != 0) 			getAWTEventListeners(listeners[AWTEvent.WINDOW_LOG2], result);
		if ((eventMask & AWTEvent.INPUT_METHOD_EVENT_MASK) != 0)	getAWTEventListeners(listeners[AWTEvent.INPUT_METHOD_LOG2], result);
		if ((eventMask & AWTEvent.PAINT_EVENT_MASK) != 0) 			getAWTEventListeners(listeners[AWTEvent.PAINT_LOG2], result);
		if ((eventMask & AWTEvent.INVOCATION_EVENT_MASK) != 0) 		getAWTEventListeners(listeners[AWTEvent.INVOCATION_LOG2], result);
		if ((eventMask & AWTEvent.MOUSE_WHEEL_EVENT_MASK) != 0) 	getAWTEventListeners(listeners[AWTEvent.MOUSE_WHEEL_LOG2], result);
		if ((eventMask & AWTEvent.WINDOW_FOCUS_EVENT_MASK) != 0) 	getAWTEventListeners(listeners[AWTEvent.WINDOW_FOCUS_LOG2], result);

		return (AWTEventListener[])(result.toArray(new AWTEventListener[result.size()]));
	}

	void getAWTEventListeners(AWTEventListener listener, Vector result) {
		if (listener == null) return;
		if (listener instanceof AWTEventListenerMulticaster) {
			AWTEventListenerMulticaster listenerMulticater = (AWTEventListenerMulticaster)listener;
			getAWTEventListeners(listenerMulticater.l1, result);
			getAWTEventListeners(listenerMulticater.l2, result);
		} else {
			// ==> listener is *only* aninstance of AWTEventListener
			if (!result.contains(listener)) result.addElement(listener);
		}
	}
}
