package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.awt.event.*;
import java.io.*;
import java.util.EventListener;

/**
 * Combines multiple listeners into one listener.
 *
 * @author		IBM
 * @version		initial
 */

public class AWTEventMulticaster
	implements
		ActionListener,
		AdjustmentListener,
		ComponentListener,
		ContainerListener,
		FocusListener,
		ItemListener,
		InputMethodListener,
		KeyListener,
		MouseListener,
		MouseMotionListener,
		MouseWheelListener,
		TextListener,
		WindowFocusListener,
		WindowListener {

	// The first event listener
	protected final EventListener a;

	// The second event listener
	protected final EventListener b;

	/**
	 * Combines the specified listeners in one listener.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	listener1	the first listener
	 * @param	listener2	the second listener
	 *
	 * @return	a java.awt.event.ActionListener
	 *
	 * @see		<code>ActionListener</code>
	 */
	public static ActionListener add(
		ActionListener listener1,
		ActionListener listener2) {
		return (ActionListener) addInternal(listener1, listener2);
	}

	/**
	 * Combines the specified listeners in one listener.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	listener1	the first listener
	 * @param	listener2	the second listener
	 *
	 * @return	a java.awt.event.AdjustmentListener
	 *
	 * @see		<code>AdjustmentListener</code>
	 */
	public static AdjustmentListener add(
		AdjustmentListener listener1,
		AdjustmentListener listener2) {
		return (AdjustmentListener) addInternal(listener1, listener2);
	}

	/**
	 * Combines the specified listeners in one listener.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	listener1	the first listener
	 * @param	listener2	the second listener
	 *
	 * @return	a java.awt.event.ComponentListener
	 *
	 * @see		<code>ComponentListener</code>
	 */
	public static ComponentListener add(
		ComponentListener listener1,
		ComponentListener listener2) {
		return (ComponentListener) addInternal(listener1, listener2);
	}

	/**
	 * Combines the specified listeners in one listener.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	listener1	the first listener
	 * @param	listener2	the second listener
	 *
	 * @return	a java.awt.event.ContainerListener
	 *
	 * @see		<code>ContainerListener</code>
	 */
	public static ContainerListener add(
		ContainerListener listener1,
		ContainerListener listener2) {
		return (ContainerListener) addInternal(listener1, listener2);
	}

	/**
	 * Combines the specified listeners in one listener.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	listener1	the first listener
	 * @param	listener2	the second listener
	 *
	 * @return	a java.awt.event.FocusListener
	 *
	 * @see		<code>FocusListener</code>
	 */
	public static FocusListener add(
		FocusListener listener1,
		FocusListener listener2) {
		return (FocusListener) addInternal(listener1, listener2);
	}

	/**
	 * Combines the specified listeners in one listener.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	listener1	the first listener
	 * @param	listener2	the second listener
	 *
	 * @return	a java.awt.event.ItemListener
	 *
	 * @see		<code>ItemListener</code>
	 */
	public static ItemListener add(
		ItemListener listener1,
		ItemListener listener2) {
		return (ItemListener) addInternal(listener1, listener2);
	}

	/**
	 * Combines the specified listeners in one listener.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	listener1	the first listener
	 * @param	listener2	the second listener
	 *
	 * @return	a java.awt.event.KeyListener
	 *
	 * @see		<code>KeyListener</code>
	 */
	public static KeyListener add(
		KeyListener listener1,
		KeyListener listener2) {
		return (KeyListener) addInternal(listener1, listener2);
	}

	/**
	 * Combines the specified listeners in one listener.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	listener1	the first listener
	 * @param	listener2	the second listener
	 *
	 * @return	a java.awt.event.MouseListener
	 *
	 * @see		<code>MouseListener</code>
	 */
	public static MouseListener add(
		MouseListener listener1,
		MouseListener listener2) {
		return (MouseListener) addInternal(listener1, listener2);
	}

	/**
	 * Combines the specified listeners in one listener.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	listener1	the first listener
	 * @param	listener2	the second listener
	 *
	 * @return	a java.awt.event.MouseMotionListener
	 *
	 * @see		<code>MouseMotionListener</code>
	 */
	public static MouseMotionListener add(
		MouseMotionListener listener1,
		MouseMotionListener listener2) {
		return (MouseMotionListener) addInternal(listener1, listener2);
	}

	/**
	 * Combines the specified listeners in one listener.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	listener1	the first listener
	 * @param	listener2	the second listener
	 *
	 * @return	a java.awt.event.TextListener
	 *
	 * @see		<code>TextListener</code>
	 */
	public static TextListener add(
		TextListener listener1,
		TextListener listener2) {
		return (TextListener) addInternal(listener1, listener2);
	}

	/**
	 * Combines the specified listeners in one listener.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	listener1	the first listener
	 * @param	listener2	the second listener
	 *
	 * @return	a java.awt.event.WindowListener
	 *
	 * @see		<code>WindowListener</code>
	 */
	public static WindowListener add(
		WindowListener listener1,
		WindowListener listener2) {
		return (WindowListener) addInternal(listener1, listener2);
	}

	/**
	 * Combines the specified listeners in one listener.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	listener1	the first listener
	 * @param	listener2	the second listener
	 *
	 * @return	a java.util.EventListener
	 *
	 * @see		<code>EventListener</code>
	 */
	protected static EventListener addInternal(
		EventListener listener1,
		EventListener listener2) {
		if (listener1 == null)
			return listener2;
		if (listener2 == null)
			return listener1;

		return new AWTEventMulticaster(listener1, listener2);
	}

	/**
	 * Removes the specified listener from the list.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	multicaster the AwtEventMulicaster holding the listener
	 * @param	listener	the listener to be removed from the multicaster
	 *
	 * @return	a java.awt.event.ActionListener
	 *
	 * @see		<code>ActionListener</code>
	 */
	public static ActionListener remove(
		ActionListener multicaster,
		ActionListener listener) {
		return (ActionListener) removeInternal(multicaster, listener);
	}

	/**
	 * Removes the specified listener from the list.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	multicaster the AwtEventMulicaster holding the listener
	 * @param	listener	the listener to be removed from the multicaster
	 *
	 * @return	a java.awt.event.AdjustmentListener
	 *
	 * @see		<code>AdjustmentListener</code>
	 */
	public static AdjustmentListener remove(
		AdjustmentListener multicaster,
		AdjustmentListener listener) {
		return (AdjustmentListener) removeInternal(multicaster, listener);
	}

	/**
	 * Removes the specified listener from the list.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	multicaster the AwtEventMulicaster holding the listener
	 * @param	listener	the listener to be removed from the multicaster
	 *
	 * @return	a java.awt.event.ComponentListener
	 *
	 * @see		<code>ComponentListener</code>
	 */
	public static ComponentListener remove(
		ComponentListener multicaster,
		ComponentListener listener) {
		return (ComponentListener) removeInternal(multicaster, listener);
	}

	/**
	 * Removes the specified listener from the list.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	multicaster the AwtEventMulicaster holding the listener
	 * @param	listener	the listener to be removed from the multicaster
	 *
	 * @return	a java.awt.event.ContainerListener
	 *
	 * @see		<code>ContainerListener</code>
	 */
	public static ContainerListener remove(
		ContainerListener multicaster,
		ContainerListener listener) {
		return (ContainerListener) removeInternal(multicaster, listener);
	}

	/**
	 * Removes the specified listener from the list.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	multicaster the AwtEventMulicaster holding the listener
	 * @param	listener	the listener to be removed from the multicaster
	 *
	 * @return	a java.awt.event.FocusListener
	 *
	 * @see		<code>FocusListener</code>
	 */
	public static FocusListener remove(
		FocusListener multicaster,
		FocusListener listener) {
		return (FocusListener) removeInternal(multicaster, listener);
	}

	/**
	 * Removes the specified listener from the list.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	multicaster the AwtEventMulicaster holding the listener
	 * @param	listener	the listener to be removed from the multicaster
	 *
	 * @return	a java.awt.event.ItemListener
	 *
	 * @see		<code>ItemListener</code>
	 */
	public static ItemListener remove(
		ItemListener multicaster,
		ItemListener listener) {
		return (ItemListener) removeInternal(multicaster, listener);
	}

	/**
	 * Removes the specified listener from the list.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	multicaster the AwtEventMulicaster holding the listener
	 * @param	listener	the listener to be removed from the multicaster
	 *
	 * @return	a java.awt.event.KeyListener
	 *
	 * @see		<code>KeyListener</code>
	 */
	public static KeyListener remove(
		KeyListener multicaster,
		KeyListener listener) {
		return (KeyListener) removeInternal(multicaster, listener);
	}

	/**
	 * Removes the specified listener from the list.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	multicaster the AwtEventMulicaster holding the listener
	 * @param	listener	the listener to be removed from the multicaster
	 *
	 * @return	a java.awt.event.MouseListener
	 *
	 * @see		<code>MouseListener</code>
	 */
	public static MouseListener remove(
		MouseListener multicaster,
		MouseListener listener) {
		return (MouseListener) removeInternal(multicaster, listener);
	}

	/**
	 * Removes the specified listener from the list.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	multicaster the AwtEventMulicaster holding the listener
	 * @param	listener	the listener to be removed from the multicaster
	 *
	 * @return	a java.awt.event.MouseMotionListener
	 *
	 * @see		<code>MouseMotionListener</code>
	 */
	public static MouseMotionListener remove(
		MouseMotionListener multicaster,
		MouseMotionListener listener) {
		return (MouseMotionListener) removeInternal(multicaster, listener);
	}

	/**
	 * Removes the specified listener from the list.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	multicaster the AwtEventMulicaster holding the listener
	 * @param	listener	the listener to be removed from the multicaster
	 *
	 * @return	a java.awt.event.TextListener
	 *
	 * @see		<code>TextListener</code>
	 */
	public static TextListener remove(
		TextListener multicaster,
		TextListener listener) {
		return (TextListener) removeInternal(multicaster, listener);
	}

	/**
	 * Removes the specified listener from the list.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	multicaster the AwtEventMulicaster holding the listener
	 * @param	listener	the listener to be removed from the multicaster
	 *
	 * @return	a java.awt.event.WindowListener
	 *
	 * @see		<code>WindowListener</code>
	 */
	public static WindowListener remove(
		WindowListener multicaster,
		WindowListener listener) {
		return (WindowListener) removeInternal(multicaster, listener);
	}

	/**
	 * Removes the specified listener from the list.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	multicaster the AwtEventMulicaster holding the listener
	 * @param	listener	the listener to be removed from the multicaster
	 *
	 * @return	a java.util.EventListener
	 *
	 * @see		<code>EventListener</code>
	 */
	protected static EventListener removeInternal(
		EventListener multicaster,
		EventListener listener) {
		if (multicaster == null || multicaster == listener)
			return null;
		if (!(multicaster instanceof AWTEventMulticaster))
			return multicaster;
		return ((AWTEventMulticaster) multicaster).remove(listener);
	}

	/**
	 * Serialize the specified listener in the specified stream using the
	 * specified key.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	stream
	 * @param	key
	 * @param	lsitener
	 *
	 * @see		<code>ObjectOutputStream</code>
	 */
	protected static void save(
		ObjectOutputStream stream,
		String key,
		EventListener listener)
		throws IOException {
		if (listener instanceof AWTEventMulticaster)
			 ((AWTEventMulticaster) listener).saveInternal(stream, key);
		else if (listener instanceof Serializable) {
			stream.writeObject(key);
			stream.writeObject(listener);
		}
	}

	/**
	 * Constructs a new <code>AWTEventMulticaster</code> with the
	 * specified listener.
	 *
	 * @author        IBM
	 * @version 	initial
	 *
	 * @param	eventmulticaster	the first event listener
	 * @param	eventListener2	the second event listener
	 */
	protected AWTEventMulticaster(
		EventListener eventListener1,
		EventListener eventListener2) {
		this.a = eventListener1;
		this.b = eventListener2;
	}

	/**
	 * Called when an action event occurs on the actionEvent's
	 * source object.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	actionEvent	the event that occured
	 *
	 * @see		<code>ActionEvent</code>
	 */
	public void actionPerformed(ActionEvent actionEvent) {
		((ActionListener) a).actionPerformed(actionEvent);
		((ActionListener) b).actionPerformed(actionEvent);
	}

	/**
	 * Called when an adjustment event occurs on the adjustmentEvent's
	 * source object.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	adjustmentEvent	the event that occured
	 *
	 * @see		<code>AdjustmentEvent</code>
	 */
	public void adjustmentValueChanged(AdjustmentEvent adjustmentEvent) {
		((AdjustmentListener) a).adjustmentValueChanged(adjustmentEvent);
		((AdjustmentListener) b).adjustmentValueChanged(adjustmentEvent);
	}

	/**
	 * Called when the componentEvent's source object is hidden.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	componentEvent	the event that occured
	 *
	 * @see		<code>ComponentEvent</code>
	 */
	public void componentHidden(ComponentEvent componentEvent) {
		((ComponentListener) a).componentHidden(componentEvent);
		((ComponentListener) b).componentHidden(componentEvent);
	}

	/**
	 * Called when the componentEvent's source object is moved.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	componentEvent	the event that occured
	 *
	 * @see		<code>ComponentEvent</code>
	 */
	public void componentMoved(ComponentEvent componentEvent) {
		((ComponentListener) a).componentMoved(componentEvent);
		((ComponentListener) b).componentMoved(componentEvent);
	}

	/**
	 * Called when the componentEvent's source object is resized.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	componentEvent	the event that occured
	 *
	 * @see		<code>ComponentEvent</code>
	 */
	public void componentResized(ComponentEvent componentEvent) {
		((ComponentListener) a).componentResized(componentEvent);
		((ComponentListener) b).componentResized(componentEvent);
	}

	/**
	 * Called when the componentEvent's source object is shown.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	componentEvent	the event that occured
	 *
	 * @see		<code>ComponentEvent</code>
	 */
	public void componentShown(ComponentEvent componentEvent) {
		((ComponentListener) a).componentShown(componentEvent);
		((ComponentListener) b).componentShown(componentEvent);
	}

	/**
	 * Called when the containerEvent's source object is added
	 * to its container.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	containerEvent	the event that occurred
	 *
	 * @see		<code>ContainerEvent</code>
	 */
	public void componentAdded(ContainerEvent containerEvent) {
		((ContainerListener) a).componentAdded(containerEvent);
		((ContainerListener) b).componentAdded(containerEvent);
	}

	/**
	 * Called when the containerEvent's source object is removed
	 * from its container.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	containerEvent	the event that occurred
	 *
	 * @see		<code>ContainerEvent</code>
	 */
	public void componentRemoved(ContainerEvent containerEvent) {
		((ContainerListener) a).componentRemoved(containerEvent);
		((ContainerListener) b).componentRemoved(containerEvent);
	}

	/**
	 * Called when the focusEvent's source object gains keyboard focus.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	focusEvent	the event that occured
	 *
	 * @see		<code>FocusEvent</code>
	 */
	public void focusGained(FocusEvent focusEvent) {
		((FocusListener) a).focusGained(focusEvent);
		((FocusListener) b).focusGained(focusEvent);
	}

	/**
	 * Called when the focusEvent's source object loses keyboard focus.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	focusEvent	the event that occured
	 *
	 * @see		<code>FocusEvent</code>
	 */
	public void focusLost(FocusEvent focusEvent) {
		((FocusListener) a).focusLost(focusEvent);
		((FocusListener) b).focusLost(focusEvent);
	}

	/**
	 * Called when an item event occurs on the itemEvent's
	 * source object.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	itemEvent	the event that occured
	 *
	 * @see		<code>ItemEvent</code>
	 */
	public void itemStateChanged(ItemEvent itemEvent) {
		((ItemListener) a).itemStateChanged(itemEvent);
		((ItemListener) b).itemStateChanged(itemEvent);
	}

	/**
	 * Called when a key has been pressed for the receiver.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	keyEvent	the event that occured
	 *
	 * @see		<code>KeyEvent</code>
	 */
	public void keyPressed(KeyEvent keyEvent) {
		((KeyListener) a).keyPressed(keyEvent);
		((KeyListener) b).keyPressed(keyEvent);
	}

	/**
	 * Called when a key has been released for the receiver.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	keyEvent	the event that occured
	 *
	 * @see		<code>KeyEvent</code>
	 */
	public void keyReleased(KeyEvent keyEvent) {
		((KeyListener) a).keyReleased(keyEvent);
		((KeyListener) b).keyReleased(keyEvent);
	}

	/**
	 * Called when a key has been typed for the receiver.
	 * Key typed events represent the action of both pressing
	 * and releasing the key.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	keyEvent	the event that occured
	 *
	 * @see		<code>KeyEvent</code>
	 */
	public void keyTyped(KeyEvent keyEvent) {
		((KeyListener) a).keyTyped(keyEvent);
		((KeyListener) b).keyTyped(keyEvent);
	}

	/**
	 * Called when a mouse click occurs on the receiver.
	 * This happens when a mouse button is pressed and released.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseClicked(MouseEvent mouseEvent) {
		((MouseListener) a).mouseClicked(mouseEvent);
		((MouseListener) b).mouseClicked(mouseEvent);
	}

	/**
	 * Called when a mouse drag motion occurs for the receiver.
	 * This happens when a mouse button is pressed and then the
	 * mouse is moved while the button is still pressed.
	 * These events are continually posted to a listener until the
	 * mouse button is released, indicating that dragging is done.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseDragged(MouseEvent mouseEvent) {
		((MouseMotionListener) a).mouseDragged(mouseEvent);
		((MouseMotionListener) b).mouseDragged(mouseEvent);
	}

	/**
	 * Called when mouse pointer enters the receiver.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseEntered(MouseEvent mouseEvent) {
		((MouseListener) a).mouseEntered(mouseEvent);
		((MouseListener) b).mouseEntered(mouseEvent);
	}

	/**
	 * Called when mouse pointer exits the receiver.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseExited(MouseEvent mouseEvent) {
		((MouseListener) a).mouseExited(mouseEvent);
		((MouseListener) b).mouseExited(mouseEvent);
	}

	/**
	 * Called when mouse motion occurs on the receiver.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseMoved(MouseEvent mouseEvent) {
		((MouseMotionListener) a).mouseMoved(mouseEvent);
		((MouseMotionListener) b).mouseMoved(mouseEvent);
	}

	/**
	 * Called when a mouse button is pressed on the receiver.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mousePressed(MouseEvent mouseEvent) {
		((MouseListener) a).mousePressed(mouseEvent);
		((MouseListener) b).mousePressed(mouseEvent);
	}

	/**
	 * Called when a mouse button is released on the receiver.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseReleased(MouseEvent mouseEvent) {
		((MouseListener) a).mouseReleased(mouseEvent);
		((MouseListener) b).mouseReleased(mouseEvent);
	}

	/**
	 * Remove the specified event listener, and answer the remaining list.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	listener	the listener
	 *
	 * @return	a java.util.EventListener
	 */
	protected EventListener remove(EventListener eventListener) {
		if (eventListener == a)
			return b;
		if (eventListener == b)
			return a;

		EventListener result;
		result = removeInternal(b, eventListener);
		if (result != b) {
			return new AWTEventMulticaster(a, result);
		}
		result = removeInternal(a, eventListener);
		if (result != a) {
			return new AWTEventMulticaster(result, b);
		}

		return this;
	}

	/**
	 * Serialize the receiver in the specified stream using the
	 * specified key.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	stream
	 * @param	key
	 *
	 * @see		<code>ObjectOutputStream</code>
	 */
	protected void saveInternal(ObjectOutputStream stream, String key)
		throws IOException {
		save(stream, key, a);
		save(stream, key, b);
	}

	/**
	 * Called when a text event occurs on the textEvent's
	 * source object.  This indicates that a component's
	 * text has changed.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	textEvent	the event that occured
	 *
	 * @see		<code>TextEvent</code>
	 */
	public void textValueChanged(TextEvent textEvent) {
		((TextListener) a).textValueChanged(textEvent);
		((TextListener) b).textValueChanged(textEvent);
	}

	/**
	 * Called when a window is activated.  This means the window
	 * comes to the forground and it or one of its children is
	 * receiving keyboard input.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	windowEvent	the event that occured
	 *
	 * @see		<code>WindowEvent</code>
	 */
	public void windowActivated(WindowEvent windowEvent) {
		((WindowListener) a).windowActivated(windowEvent);
		((WindowListener) b).windowActivated(windowEvent);
	}

	/**
	 * Called when a window is closed.  This means the window
	 * will no longer be visible on the screen.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	windowEvent	the event that occured
	 *
	 * @see		<code>WindowEvent</code>
	 */
	public void windowClosed(WindowEvent windowEvent) {
		((WindowListener) a).windowClosed(windowEvent);
		((WindowListener) b).windowClosed(windowEvent);
	}

	/**
	 * Called when a window is closing.  This means the window
	 * may still be visible on the screen but a user is attempting
	 * to close it.  This could be using the mouse or keyboard
	 * or other input device.
	 * <p>
	 * If a listener does not dispose the window in response to receiving
	 * this event the window will not be closed.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	windowEvent	the event that occured
	 *
	 * @see		<code>WindowEvent</code>
	 */
	public void windowClosing(WindowEvent windowEvent) {
		((WindowListener) a).windowClosing(windowEvent);
		((WindowListener) b).windowClosing(windowEvent);
	}

	/**
	 * Called when a window is de-activated.  This means the window
	 * is no longer the forground window and another window or
	 * or one of its children is receiving keyboard input.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	windowEvent	the event that occured
	 *
	 * @see		<code>WindowEvent</code>
	 */
	public void windowDeactivated(WindowEvent windowEvent) {
		((WindowListener) a).windowDeactivated(windowEvent);
		((WindowListener) b).windowDeactivated(windowEvent);
	}

	/**
	 * Called when a window is de-iconified.  This means the window
	 * is no longer iconified but will appear drawn in its regular
	 * state on the screen.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	windowEvent	the event that occured
	 *
	 * @see		<code>WindowEvent</code>
	 */
	public void windowDeiconified(WindowEvent windowEvent) {
		((WindowListener) a).windowDeiconified(windowEvent);
		((WindowListener) b).windowDeiconified(windowEvent);
	}

	/**
	 * Called when a window is iconified.  This means the window
	 * is no longer drawn in its regular state on the screen but
	 * will appear as an icon.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	windowEvent	the event that occured
	 *
	 * @see		<code>WindowEvent</code>
	 */
	public void windowIconified(WindowEvent windowEvent) {
		((WindowListener) a).windowIconified(windowEvent);
		((WindowListener) b).windowIconified(windowEvent);
	}

	/**
	 * Called when a window is initially opened.  This means the window
	 * will now be visible on the screen.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	windowEvent	the event that occured
	 *
	 * @see		<code>WindowEvent</code>
	 */
	public void windowOpened(WindowEvent windowEvent) {
		((WindowListener) a).windowOpened(windowEvent);
		((WindowListener) b).windowOpened(windowEvent);
	}

	public void inputMethodTextChanged(InputMethodEvent inputMethodEvent) {
		((InputMethodListener) a).inputMethodTextChanged(inputMethodEvent);
		((InputMethodListener) b).inputMethodTextChanged(inputMethodEvent);
	}

	public void caretPositionChanged(InputMethodEvent inputMethodEvent) {
		((InputMethodListener) a).caretPositionChanged(inputMethodEvent);
		((InputMethodListener) b).caretPositionChanged(inputMethodEvent);
	}

	public void mouseWheelMoved(MouseWheelEvent mouseWheelEvent) {
		((MouseWheelListener) a).mouseWheelMoved(mouseWheelEvent);
		((MouseWheelListener) b).mouseWheelMoved(mouseWheelEvent);
	}

	public void windowGainedFocus(WindowEvent windowFocusEvent) {
		((WindowFocusListener) a).windowGainedFocus(windowFocusEvent);
		((WindowFocusListener) b).windowGainedFocus(windowFocusEvent);
	}

	public void windowLostFocus(WindowEvent windowFocusEvent) {
		((WindowFocusListener) a).windowLostFocus(windowFocusEvent);
		((WindowFocusListener) b).windowLostFocus(windowFocusEvent);
	}

	public static WindowFocusListener add(WindowFocusListener listener1, WindowFocusListener listener2) {
		return (WindowFocusListener) addInternal(listener1, listener2);
	}

	public static InputMethodListener add(InputMethodListener listener1, InputMethodListener listener2) {
		return (InputMethodListener) addInternal(listener1, listener2);
	}

	public static MouseWheelListener add(MouseWheelListener listener1, MouseWheelListener listener2) {
		return (MouseWheelListener) addInternal(listener1, listener2);
	}

	public static WindowFocusListener remove(WindowFocusListener multicaster, WindowFocusListener listener) {
		return (WindowFocusListener) removeInternal(multicaster, listener);
	}

	public static InputMethodListener remove(InputMethodListener multicaster, InputMethodListener listener) {
		return (InputMethodListener) removeInternal(multicaster, listener);
	}

	public static MouseWheelListener remove(MouseWheelListener multicaster, MouseWheelListener listener) {
		return (MouseWheelListener) removeInternal(multicaster, listener);
	}

	public static EventListener[] getListeners(EventListener eventListener, Class listenerType) {
		// Answer all chained listeners that are instances of listenerType

		if (eventListener == null) {
			return new EventListener[] {};
		}
		if (!(EventListener.class.isAssignableFrom(listenerType))) {
			throw new ClassCastException("Instances of listenerType are not java.util.EventListener instances");
		}
		if (!(eventListener instanceof AWTEventMulticaster)) {
			if (listenerType.isAssignableFrom(eventListener.getClass())) {
				return new EventListener[] {eventListener};
			} else {
				return new EventListener[] {};
			}
		}
		EventListener[] a = getListeners(((AWTEventMulticaster) eventListener).a, listenerType);
		EventListener[] b = getListeners(((AWTEventMulticaster) eventListener).b, listenerType);
		EventListener[] result = new EventListener[a.length + b.length];
		System.arraycopy(a, 0, result, 0, a.length);
		System.arraycopy(b, 0, result, a.length, b.length);
		return result;
	}

}
