package java.awt;

import java.awt.event.AdjustmentListener;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This interface should be implemented by all classes
 * which model the choosing of a numeric value within
 * a specified range.  Adjustables have a proportional
 * indicator which represtents the proportional amount
 * of content displayed.
 *
 * @author		IBM
 * @version		initial
 */
public abstract interface Adjustable {

	// Adjustable orientation constants
	public static final int HORIZONTAL = 0;
	public static final int VERTICAL = 1;
	public static final int NO_ORIENTATION = 2;
	/**
	 * Adds an <code>AdjustmentListener</code> to the receiver.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	adjustmentListener	the <code>AdjustmentListener</code> to be added.
	 *
	 * @see		java.awt.event.AdjustmentListener
	 * @see		java.awt.event.AdjustmentEvent
	 */
	public void addAdjustmentListener(AdjustmentListener adjustmentListener);

	/**
	 * Answer the block increment value for the receiver.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @return	an int	the block increment
	 */
	public int getBlockIncrement();

	/**
	 * Answer the maximum value for the receiver.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @return	an int	the maximum
	 */
	public int getMaximum();

	/**
	 * Answer the minimum value for the receiver.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @return	an int	the minimum
	 */
	public int getMinimum();

	/**
	 * Answer the orientation value for the receiver.
	 * The possible values are:
	 *	- <code>HORIZONTAL</code>
	 *	- <code>VERTICAL</code>
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @return	an int	the orientation
	 */
	public int getOrientation();

	/**
	 * Answer the unit increment value for the receiver.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @return	an int	the unit increment
	 */
	public int getUnitIncrement();

	/**
	 * Answer the value for the receiver.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @return	an int	the value
	 */
	public int getValue();

	/**
	 * Return the length of the indicator (eg. the scroll thumb)
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @return	an int	the visible amount
	 */
	public int getVisibleAmount();

	/**
	 * Removes an <code>AdjustmentListener</code> from the receiver.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	adjustmentListener	the <code>AdjustmentListener</code> to be removed.
	 *
	 * @see		java.awt.event.AdjustmentListener
	 * @see		java.awt.event.AdjustmentEvent
	 */
	public void removeAdjustmentListener(AdjustmentListener adjustmentListener);

	/**
	 * Set the block increment value for the receiver.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	blockIncrement	the block increment
	 */
	public void setBlockIncrement(int blockIncrement);

	/**
	 * Set the maximum value for the receiver.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	maximum	the maximum
	 */
	public void setMaximum(int maximum);

	/**
	 * Set the minimum value for the receiver.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	minimum	the minimum
	 */
	public void setMinimum(int minimum);

	/**
	 * Set the unit increment value for the receiver.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	unitIncrement	the unit increment
	 */
	public void setUnitIncrement(int unitIncrement);

	/**
	 * Set the value for the receiver.
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	value	the value
	 */
	public void setValue(int value);

	/**
	 * Set the length of the indicator (eg. the scroll thumb).
	 *
	 * @author		IBM
	 * @version		initial
	 *
	 * @param	visibleAmount	the visible amount
	 */
	public void setVisibleAmount(int visibleAmount);

}
