package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

//------------------------------------------------------------
/**
 *
 */
public final class AlphaComposite implements Composite {

	//------------------------------------------------------------
	/**
	 *
	 */
	public static final int CLEAR = 1;

	//------------------------------------------------------------
	/**
	 *
	 */
	public static final int SRC = 2;

	//------------------------------------------------------------
	/**
	 *
	 */
	public static final int SRC_OVER = 3;

	//------------------------------------------------------------
	/**
	 *
	 */
	public static final AlphaComposite Clear = getInstance(CLEAR);

	//------------------------------------------------------------
	/**
	 *
	 */
	public static final AlphaComposite Src = getInstance(SRC);

	//------------------------------------------------------------
	/**
	 *
	 */
	public static final AlphaComposite SrcOver = getInstance(SRC_OVER);

	private int rule = CLEAR;
	private float alpha = 1.0f;

	private AlphaComposite(int rule, float alpha) {
		if ((alpha < 0.0f) || (alpha > 1.0f)) {
			// The spec only mentions that the value of
			// alpha MUST be between 0 and 1.  However,
			// the Sun JRE throws an exception for out
			// of bounds alphas and the TCK does not
			// check otherwise.  In this undefined case, we
			// should follow precedent rather than silently
			// ignore the invalid value
			throw new IllegalArgumentException("Alpha value (" + alpha + ") is out of range");
		}

		this.rule = rule;
		this.alpha = alpha;
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public static AlphaComposite getInstance(int rule) {
		return getInstance(rule, 1.0f);
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public static AlphaComposite getInstance(int rule, float alpha) {
		switch (rule) {
			case CLEAR :
			case SRC :
			case SRC_OVER :
				break;
			default :
				throw new IllegalArgumentException("Compositing rule not found"); //$NON-NLS-1$
		}
		return new AlphaComposite(rule, alpha);
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public float getAlpha() {
		return alpha;
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public int getRule() {
		return rule;
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public int hashCode() {
		return ((int) getAlpha() * 100) + getRule();
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public boolean equals(Object object) {
		if (object == this)
			return true;
		if (!(object instanceof AlphaComposite))
			return false;
		return (getAlpha() == ((AlphaComposite) object).getAlpha())
			&& (getRule() == ((AlphaComposite) object).getRule());
	}

	public String toString() {
		String str = "AlphaComposite [rule=";
		switch (rule) {
			case SRC: str += "SRC"; break;
			case SRC_OVER: str += "SRC_OVER"; break;
			case CLEAR: str += "CLEAR"; break;
			default: str += "Unknown";
		}

		str += " alpha=" + alpha + "]";
		return str;
	}

}
