package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import com.ibm.ugl.eswt.OS;

/**
 * A <code>Canvas</code> is a graphical interface used to implement custom
 * widgets by rendering its displayable area and listening to keyboard and mouse
 * input.
 * <p>
 * By default, this component fills its displayable area with its background
 * color. Subclasses should override <code>paint(Graphics)</code> to perform
 * custom drawings.
 *
 * @author		IBM
 * @version		initial
 */
public class Canvas extends Component {

	static final long serialVersionUID = -2284879212465893870L;

	transient GraphicsConfiguration graphicsConfiguration = null;

	/**
	 * Constructs a new <code>Canvas</code>.
	 *
	 * @author	IBM
	 * @version	initial
	 */
	public Canvas() {
	}

	/**
	 * Constructs a new <code>Canvas</code> with the specified <code>GraphicsConfiguration</code>.
	 *
	 * @author	IBM
	 * @version	initial
	 */
	public Canvas(GraphicsConfiguration config) {
		this();
		this.graphicsConfiguration = config;
	}

	/**
	 * Create the underlying widget for the receiver.
	 *
	 * @author	IBM
	 * @version	initial
	 */
	int _createPeer(Container hwParent) {
		return OS.Composite_New(hwParent.peer, 0);
	}

	/**
	 * Answer whether the receiver can be used as a canvas
	 */
	boolean isPaintable() {
		return true;
	}

	Dimension getMinimumSizeImpl() {
		return getSize();
	}

	String classNonlocalizedName() {
		return "canvas"; //$NON-NLS-1$
	}

	public boolean isLightweight() {
		return false;
	}

	public boolean isOpaque() {
		return true;
	}

	public boolean isValid(){
		return true;
	}

	/**
	 * Called when regions of this component need to be
	 * redrawn.  The graphics clip region represents the damaged area.
	 * <p>
	 * Subclasses should override this method to perform custom
	 * drawings.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param	graphics	the graphics context used for drawing
	 */
	public void paint(Graphics graphics) {
		super.paint(graphics);
	}

	public GraphicsConfiguration getGraphicsConfiguration() {
		if (graphicsConfiguration != null) {
			return graphicsConfiguration;
		} else {
			return super.getGraphicsConfiguration();
		}
	}

}
