package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

import java.io.Serializable;

/**
 * A <code>CheckboxGroup</code> represents a group of <code>Checkbox</code>.
 * Only one checkbox owned by the same group can be selected at a
 * given time.
 *
 * @author		IBM
 * @version		initial
 *
 * @see			java.io.Serializable
 * @see			java.awt.Checkbox
 */
public class CheckboxGroup implements Serializable {

	static final long serialVersionUID = 3729780091441768983L;

	Checkbox selectedCheckbox;

	/**
	 * Constructs a new <code>CheckboxGroup</code>
	 *
	 * @author	IBM
	 * @version	initial
	 */
	public CheckboxGroup() {
	}

	/**
	 * Answers the selected checkbox for the receiver.
	 *
	 * @author	IBM
	 * @version	initial
	 */
	public Checkbox getSelectedCheckbox() {
		return selectedCheckbox;
	}

	/**
	 * Answers the selected checkbox for the receiver.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @deprecated use <code>getSelectedCheckbox()</code> instead
	 */
	public Checkbox getCurrent() {
		return getSelectedCheckbox();
	}

	/**
	 * Sets the selected checkbox for the receiver.
	 * If checkbox belongs to a different group, the method does nothing.
	 *
	 * @author	IBM
	 * @version	initial
	 */
	public void setSelectedCheckbox(Checkbox checkbox) {
		if (selectedCheckbox == checkbox) {
			return;
		}
		if ((checkbox != null) && (checkbox.getCheckboxGroup() != this)) {
			return;
		}

		Checkbox oldSelection = selectedCheckbox;

		selectedCheckbox = checkbox;

		if (oldSelection != null) oldSelection.setState(false);

		if ((selectedCheckbox != null) && (!selectedCheckbox.getState())) {
			selectedCheckbox.setState(true);
		}
	}

	/**
	 * Sets the selected checkbox for the receiver.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @deprecated use <code>setSelectedCheckbox(Checkbox)</code> instead
	 */
	public void setCurrent(Checkbox checkbox) {
		setSelectedCheckbox(checkbox);
	}

	/**
	 * Answers a string containing a concise, human-readable
	 * description of the receiver with relevant information.
	 *
	 * @author		OTI
	 * @version		initial
	 *
	 * @return	a java.lang.String which is a printable representation for the receiver
	 */
	public String toString() {
		return getClass().getName() + "[selectedCheckbox=" + getSelectedCheckbox() + "]"; //$NON-NLS-1$ //$NON-NLS-2$
	}

	void updateSelectedCheckbox(Checkbox checkbox, boolean state) {
		if (state) {
			setSelectedCheckbox(checkbox);
		} else {
			if (checkbox == selectedCheckbox) {
				selectedCheckbox = null;
			}
		}
	}

}
