package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2006  All Rights Reserved
 */

import java.awt.datatransfer.*;
import java.io.IOException;

import com.ibm.ugl.eswt.OS;

/**
 * Comment for <code>ClipboardImpl</code>.
 */
class ClipboardImpl extends Clipboard {

	/**
	 * Constructs a new <code>Clipboard</code> with the
	 * specified name.
	 *
	 * @author        IBM
	 * @version 	initial
	 *
	 */
	public ClipboardImpl() {
		super("System Clipboard"); //$NON-NLS-1$
	}

	/**
	 * Answers the current contents of the receiver.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @return	a <code>Transferable</code> which is the current contents
	 *				of the receiver
	 */
	public synchronized Transferable getContents(Object o) {
		String text = getText();
		if (text != null)
			return new java.awt.datatransfer.StringSelection(text);

		Transferable result = super.getContents(o);
		if (result instanceof StringSelection)
			return null;

		return result;
	}

	String getText() {
		return OS.Clipboard_GetText();
	}

	/**
	 * Sets the contents of the receiver to the given <code>Transferable</code>.
	 * The specified <code>ClipboardOwner</code> will be notified
	 * if the contents of the receiver is altered later on.
	 *
	 * @author        IBM
	 * @version		initial
	 *
	 * @param	contents	the contents for the receiver
	 * @param	owner		the owner for the receiver
	 */
	public synchronized void setContents(Transferable contents, ClipboardOwner owner) {
		super.setContents(contents, owner);

		try {
			String newContents = (String)contents.getTransferData(DataFlavor.stringFlavor);
			if (newContents != null) {
				setText(newContents);
			}
		} catch (UnsupportedFlavorException e) {
		} catch (IOException e) {
		}
	}

	void setText(String text) {
		OS.Clipboard_SetText(text);
	}

}
