package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2004, 2005  All Rights Reserved
 */

public class DefaultFocusTraversalPolicy extends ContainerOrderFocusTraversalPolicy {

	public DefaultFocusTraversalPolicy() {
	}
	/**
	 * Determines whether a Component can be a focus owner.  In order to be a
	 * focus owner, the Component must be visible, displayable, and enabled.
	 * If client code has explicitly set the focusability of the Component
	 * (by either overriding Component.isFocusTraversable() or Component.isFocusable(),
	 * or by calling Component.setFocusable()), then the Component will be accepted
	 * if and only if it is focusable.  If the Component is relying on default
	 * focusability, then all Canvases, Labels, Panels, Scrollbars, ScrollPanes,
	 * Windows, and lightweight Components will be rejected.
	 */
	protected boolean accept(Component component) {
	    if (!component.isVisible()) return false;
	    if (!component.isEnabled()) return false;
	    if (!component.isDisplayable()) return false;
        // TODO PPro 1.1 Focus - how to track when client code has
        // overridden isFocusTraversable() or isFocusable() methods?
	    if (component.isFocusTraversableOverridden == 1) {
	        // client code has overridden focus traversability
	        return component.isFocusable();
	    } else {
	        if (component instanceof Label) return false;
	        if (component instanceof Canvas) return false;
	        if (component instanceof Panel) return false;
	        if (component instanceof Scrollbar) return false;
	        if (component instanceof ScrollPane) return false;
	        if (component instanceof Window) return false;
	        if (component.isLightweight()) return false;
	        return true;
	    }
	}
}
