package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * This class represents an event in the 1.0 event
 * model.
 *
 * @see	AWTEvent
 *
 * @deprecated	use <code>AWTEvent</code> instead
 */

public class Event implements java.io.Serializable {

static final long serialVersionUID = 5488922509400504703L;

/**
 * The object that generated the receiver.
 */
public Object target;

/**
 * The receiver's specific argument.
 */
public Object arg;

/**
 * The event type of the receiver..
 */
public int id;

/**
 * The time when this event occured.
 */
public long when;

/**
 * The x coordinate of the receiver.
 */
public int x;

/**
 * The y coordinate of the receiver.
 */
public int y;

/**
 * The number of times this event has happend in a certain amount of time.
 */
public int clickCount;

/**
 * The key pressed.
 */
public int key;

/**
 * The state of the modifier keys.
 */
public int modifiers;

/**
 * The next event.
 */
public Event evt;

boolean consumed;

/**
 * Key constants
 */
public static final int BACK_SPACE = 8;
public static final int TAB = 9;
public static final int ENTER = 10;
public static final int ESCAPE = 27;
public static final int DELETE = 127;
public static final int HOME = 1000;
public static final int END = 1001;
public static final int PGUP = 1002;
public static final int PGDN = 1003;
public static final int UP = 1004;
public static final int DOWN = 1005;
public static final int LEFT = 1006;
public static final int RIGHT = 1007;
public static final int F1 = 1008;
public static final int F2 = 1009;
public static final int F3 = 1010;
public static final int F4 = 1011;
public static final int F5 = 1012;
public static final int F6 = 1013;
public static final int F7 = 1014;
public static final int F8 = 1015;
public static final int F9 = 1016;
public static final int F10 = 1017;
public static final int F11 = 1018;
public static final int F12 = 1019;
public static final int PRINT_SCREEN = 1020;
public static final int SCROLL_LOCK = 1021;
public static final int CAPS_LOCK = 1022;
public static final int NUM_LOCK = 1023;
public static final int PAUSE = 1024;
public static final int INSERT = 1025;

/**
 * Key modifiers constants
 */
public static final int SHIFT_MASK = 1;
public static final int CTRL_MASK = 2;
public static final int META_MASK = 4;
public static final int ALT_MASK = 8;

/**
 * Event type constants
 */
public static final int WINDOW_DESTROY = 201;
public static final int WINDOW_EXPOSE = 202;
public static final int WINDOW_ICONIFY = 203;
public static final int WINDOW_DEICONIFY = 204;
public static final int WINDOW_MOVED = 205;

public static final int KEY_PRESS = 401;
public static final int KEY_RELEASE = 402;
public static final int KEY_ACTION = 403;
public static final int KEY_ACTION_RELEASE = 404;

public static final int MOUSE_DOWN = 501;
public static final int MOUSE_UP = 502;
public static final int MOUSE_MOVE = 503;
public static final int MOUSE_ENTER = 504;
public static final int MOUSE_EXIT = 505;
public static final int MOUSE_DRAG = 506;

public static final int SCROLL_LINE_UP = 601;
public static final int SCROLL_LINE_DOWN = 602;
public static final int SCROLL_PAGE_UP = 603;
public static final int SCROLL_PAGE_DOWN = 604;
public static final int SCROLL_ABSOLUTE = 605;
public static final int SCROLL_BEGIN = 606;
public static final int SCROLL_END = 607;

public static final int LIST_DESELECT = 702;
public static final int LIST_SELECT = 701;

public static final int ACTION_EVENT = 1001;
public static final int LOAD_FILE = 1002;
public static final int SAVE_FILE = 1003;
public static final int GOT_FOCUS = 1004;
public static final int LOST_FOCUS = 1005;

/**
 * Constructs a new <code>InputEvent</code> using the
 * specified values.
 *
 * @author      IBM
 * @version 	initial
 *
 * @param	target	the target component of the receiver
 * @param	id		the event type of the receiver
 * @param	arg		the argument of the receiver
 *
 */
public Event(Object target, int id, Object arg) {
	this.target = target;
	this.id = id;
	this.arg = arg;
}

/**
 * Constructs a new <code>InputEvent</code> using the
 * specified values.
 *
 * @author      IBM
 * @version 	initial
 *
 * @param	target		the target component of the receiver
 * @param	when		the time the event occured
 * @param	id			the event type of the receiver
 * @param	x			the x coordinate of the receiver
 * @param	y			the y coordinate of the receiver
 * @param	key			the key pressed of the receiver
 * @param	modifiers	the modifiers for the receiver
 *
 */
public Event(Object target, long when, int id, int x, int y, int key, int modifiers) {
	this.target = target;
	this.when = when;
	this.id = id;
	this.x = x;
	this.y = y;
	this.key = key;
	this.modifiers = modifiers;
}

/**
 * Constructs a new <code>InputEvent</code> using the
 * specified values.
 *
 * @author      IBM
 * @version 	initial
 *
 * @param	target		the target component of the receiver
 * @param	when		the time the event occured
 * @param	id			the event type of the receiver
 * @param	x			the x coordinate of the receiver
 * @param	y			the y coordinate of the receiver
 * @param	key			the key pressed of the receiver
 * @param	modifiers	the modifiers for the receiver
 * @param	arg			the argument of the receiver
 *
 */
public Event(Object target, long when, int id, int x, int y, int key, int modifiers, Object arg) {
	this(target, when, id, x, y, key, modifiers);
	this.arg = arg;
}

/**
 * Answer the whether the Ctrl key was pressed when the event
 * that the receiver represents occured.
 *
 * @author      IBM
 * @version		initial
 *
 * @return	a boolean	<code>true</code> if Shift was pressed
 *						<code>false</code> otherwise
 *
 * @see <code>InputEvent</code>
 *
 * @deprecated	use <code>InputEvent.isControlDown()</code> instead
 */
public boolean controlDown() {
	return (modifiers & CTRL_MASK) != 0;
}

/**
 * Answer the whether the Meta key was pressed when the event
 * that the receiver represents occured.
 *
 * @author      IBM
 * @version		initial
 *
 * @return	a boolean	<code>true</code> if Shift was pressed
 *						<code>false</code> otherwise
 *
 * @see <code>InputEvent</code>
 *
 * @deprecated	use <code>InputEvent.isMetaDown()</code> instead
 */
public boolean metaDown() {
	return (modifiers & META_MASK) != 0;
}

/**
 * Answer a string with the relevant information describing the state
 * of the receiver.  This will not be <code>null</code>.
 *
 * @author      IBM
 * @version		initial
 *
 * @return	a java.lang.String	the param string.
 *
 * @see	<code>AWTEvent</code>
 *
 * @deprecated	use <code>AWTEvent.paramString()</code> instead
 */
protected String paramString() {
	StringBuffer buffer = new StringBuffer();
	buffer.append("id="); //$NON-NLS-1$
	buffer.append(id);
	buffer.append(",x="); //$NON-NLS-1$
	buffer.append(x);
	buffer.append(",y="); //$NON-NLS-1$
	buffer.append(y);
	buffer.append(",target="); //$NON-NLS-1$
	buffer.append(target);
	if (key != 0) {
		buffer.append(",key="); //$NON-NLS-1$
		buffer.append(key);
	}
	if (modifiers != 0) {
		buffer.append(",mods="); //$NON-NLS-1$
		buffer.append(modifiers);
	}
	if (clickCount != 0) {
		buffer.append(",clickCount="); //$NON-NLS-1$
		buffer.append(clickCount);
	}
	if (arg != null) {
		buffer.append(",arg="); //$NON-NLS-1$
		buffer.append(arg);
	}
	return buffer.toString();
}

/**
 * Answer the whether the Shift key was pressed when the event
 * that the receiver represents occured.
 *
 * @author      IBM
 * @version		initial
 *
 * @return	a boolean	<code>true</code> if Shift was pressed
 *						<code>false</code> otherwise
 *
 * @see <code>InputEvent</code>
 *
 * @deprecated	use <code>InputEvent.isShiftDown()</code> instead
 */
public boolean shiftDown() {
	return (modifiers & SHIFT_MASK) != 0;
}

/**
 * Answers a string containing a concise, human-readable
 * description of the receiver's relevant information.
 *
 * @author      IBM
 * @version		initial
 *
 * @return	a java.lang.String which is a printable representation for the receiver
 *
 * @see	<code>AWTEvent</code>
 *
 * @deprecated	use <code>AWTEvent.toString()</code> instead
 */
public String toString() {
	return getClass().getName() + "[" + paramString() + "]"; //$NON-NLS-1$ //$NON-NLS-2$
}

/**
 * Translate the location of the receiver by adding the
 * specified x and y values.
 *
 * @author      IBM
 * @version		initial
 *
 * @param	x	the x translation value
 * @param	y	the y translation value
 *
 * @see <code>MouseEvent</code>
 *
 * @deprecated	use <code>MouseEvent.translatePoint(int, int)</code> instead
 */
public void translate(int dx, int dy) {
	this.x += dx;
	this.y += dy;
}

}
