package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

import java.awt.event.FocusEvent;
import java.io.Serializable;

import com.ibm.ugl.eswt.OS;

class FocusManager implements Serializable {

	static final long serialVersionUID = 2491878825643557906L;

	Window window;
	Component curr;
	Component last;
	Component[] lastRequested = new Component[1];

	public FocusManager(Window window) {
		this.window = window;
	}

	public Component getLastFocus() {
		return last;
	}

	public Component getCurrentFocus() {
		return curr;
	}

	public void focusGained(Component component) {
		last = component;
		curr = component;
	}

	public void focusLost(Component component) {
		if (component == curr) {
			curr = null;
		}
	}

	public Object getRequestedLock() {
		return lastRequested;
	}

	public Component getLastRequested() {
		return lastRequested[0];
	}

	public void requestFocusEmulated(Component newComp, Component oldComp, boolean temporary) {
		focusLost(oldComp);
		oldComp.postEvent(new FocusEvent(oldComp, FocusEvent.FOCUS_LOST, temporary, newComp));

		focusGained(newComp);
		newComp.postEvent(new FocusEvent(newComp, FocusEvent.FOCUS_GAINED, temporary, oldComp));

		lastRequested[0] = newComp;
	}

	public boolean requestFocus(Component comp) {
		lastRequested[0] = comp;

		comp.acquirePeerReadLock();
		try {
			if (comp.peer != 0) {
				OS.Control_ForceFocus(comp.peer);
				return true;
			} else {
				return false;
			}
		} finally {
			comp.releasePeerReadLock();
		}

	}

}
