package java.awt;

import java.util.ArrayList;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2004, 2005  All Rights Reserved
 */

public abstract class FocusTraversalPolicy {

	public FocusTraversalPolicy() {
	}

	public abstract Component getComponentAfter(Container container0, Component component1);

	public abstract Component getComponentBefore(Container container0, Component component1);

	public abstract Component getFirstComponent(Container container0);

	public abstract Component getLastComponent(Container container0);

	public abstract Component getDefaultComponent(Container container0);

	/**
	 * Returns the Component that should receive the focus when a Window is made
	 * visible for the first time. This method is only used when the window initially
	 * becomes visible.  After the window loses and subsequently regains
	 * focus, the Window's most recent focused Component will become the window's
	 * focus owner.  The default implementation of this method will return the
	 * the default Component.
	 */
	public Component getInitialComponent(Window window) {
		if (window == null) throw new IllegalArgumentException("window cannot be null");
		Component cmpo = getDefaultComponent(window);
		// Although this is not documented in the spec, the tck indicates that
		// if the default component is null, the focus traversal policy will
		// override that behavior and attempt to provide an initial component.
		if (cmpo == null) {
			cmpo = getInitialDefaultComponent(window);
		}
		return cmpo;
	}
	ArrayList getPreorderFocusCycle(Container root, boolean topLevel) {
	    ArrayList list = new ArrayList();
	    list.add(root);
	    if (!topLevel && root.isFocusCycleRoot()) return list;
		Component[] components = root.getComponents();
		for (int i=0; i<components.length; i++) {
		    Component cmpo = components[i];
		    if (cmpo instanceof Container) {
		        list.addAll(getPreorderFocusCycle((Container)cmpo, false));
		    } else {
		        list.add(cmpo);
		    }
		}
		return list;
	}
	Component getInitialDefaultComponent(Container focusCycleRoot) {
		ArrayList list = getPreorderFocusCycle(focusCycleRoot, true);
		for (int i=0; i<list.size(); i++) {
		    Component cmpo = (Component)list.get(i);
		    if (cmpo.isDisplayable() && cmpo.isFocusable() && cmpo.isEnabled()) return cmpo;
		}
		return null;
	}

}
