package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.VolatileImage;

/**
 * <code>GraphicsConfiguration</code> is an abstract base class,
 * concrete subclasses of which represent the attributes of
 * a particular graphics destination (e.g. monitor, printer).
 *
 * @author	IBM
 * @version	initial
 */
public abstract class GraphicsConfiguration {

	//------------------------------------------------------------
	/**
	 *
	 */
	protected GraphicsConfiguration() {
		super();
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public abstract BufferedImage createCompatibleImage(int width, int height);

	//------------------------------------------------------------
	/**
	 *
	 */
	public abstract Rectangle getBounds();

	//------------------------------------------------------------
	/**
	 *
	 */
	public abstract ColorModel getColorModel();

	//------------------------------------------------------------
	/**
	 *
	 */
	public abstract GraphicsDevice getDevice();

	public abstract VolatileImage createCompatibleVolatileImage(int width, int height);

	public VolatileImage createCompatibleVolatileImage(int width, int height, ImageCapabilities capabilities) throws AWTException {
		if ((capabilities != null) && (capabilities.isAccelerated() != getImageCapabilities().isAccelerated())) {
			throw new AWTException("Could not meet the supplied image capabilities");
		}
		return new VolatileImageImpl(this, width, height, capabilities, null);
	}

	public ImageCapabilities getImageCapabilities() {
		return new ImageCapabilities(false);
	}
}
