package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2002, 2005  All Rights Reserved
 */

import com.ibm.oti.awt.Util;

class GraphicsDeviceImpl extends GraphicsDevice {

	private int type;
	private String idString;
	private GraphicsConfiguration defaultConfiguration;
	private GraphicsConfiguration[] configurations;
	private Window fullScreenWindow;
	private Rectangle oldFullScreenBounds;

	protected GraphicsDeviceImpl() {
		this(TYPE_RASTER_SCREEN, "defaultDisplay"); //$NON-NLS-1$

		defaultConfiguration = new GraphicsConfigurationImpl(
			Util.getDisplay().getBounds(),
			Util.getDisplay().getDefaultColorModel(),
			this);
		configurations = new GraphicsConfiguration[] { defaultConfiguration };
	}

	GraphicsDeviceImpl(int type, String idString) {
		super();
		this.type = type;
		this.idString = idString;
	}

	public GraphicsConfiguration[] getConfigurations() {
		return configurations;
	}

	public GraphicsConfiguration getDefaultConfiguration() {
		return defaultConfiguration;
	}

	public String getIDstring() {
		return idString;
	}

	public int getType() {
		return type;
	}

	/* (non-Javadoc)
	 * @see java.awt.GraphicsDevice#isFullScreenSupported()
	 */
	public boolean isFullScreenSupported() {
		return Util.getDisplay().isFullScreenSupported();
	}

	/* (non-Javadoc)
	 * @see java.awt.GraphicsDevice#setFullScreenWindow(java.awt.Window)
	 */
	public void setFullScreenWindow(Window window) {
		if (fullScreenWindow == window) return;
		if (isFullScreenSupported()) {
			if (window != null) window.acquirePeerReadLock();
			try {
				// It is expected that the native platform will set the bounds of
				// the current full screen window accordingly
				Util.getDisplay().setFullScreenWindow(window == null ? 0 : window.peer);
			} finally {
				if (window != null) window.releasePeerReadLock();
			}
		} else {
			Rectangle oldBounds = oldFullScreenBounds;
			if (window == null) {
				oldFullScreenBounds = null;
			} else {
				oldFullScreenBounds = window.getBounds();
				Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
				window.setBounds(0, 0, screenSize.width, screenSize.height);
				window.toFront();
			}
			if (fullScreenWindow != null) {
				// Do this after setting the new window's bounds, to avoid flash
				fullScreenWindow.setBounds(oldBounds);
			}
		}
		fullScreenWindow = window;
	}

	/* (non-Javadoc)
	 * @see java.awt.GraphicsDevice#getFullScreenWindow()
	 */
	public Window getFullScreenWindow() {
		return fullScreenWindow;
	}

	/* (non-Javadoc)
	 * @see java.awt.GraphicsDevice#getAvailableAcceleratedMemory()
	 */
	public int getAvailableAcceleratedMemory() {
		return Util.getDisplay().getAvailableAcceleratedMemory();
	}
}
