package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.awt.image.BufferedImage;
import java.util.Locale;

/**
 * <code>GraphicsEnvironment</code> is an abstract base class,
 * concrete subclasses of which manage the graphics devices
 * and fonts available on a particular platform.
 *
 * @author      IBM
 * @version		initial
 */
public abstract class GraphicsEnvironment {

	static GraphicsEnvironment localGraphicsEnv;

	/**
	 *
	 */
	public static GraphicsEnvironment getLocalGraphicsEnvironment() {
		synchronized (GraphicsEnvironment.class) {
			if (localGraphicsEnv == null)
				localGraphicsEnv = new GraphicsEnvironmentImpl();

			return localGraphicsEnv;
		}
	}

	/**
	 *
	 */
	protected GraphicsEnvironment() {
	}

	/**
	 * Create a graphics context for the input image
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param	image the buffered image
	 *
	 * @return a <code>Graphics2D</code> for drawing on the image
	 */
	public abstract Graphics2D createGraphics(BufferedImage image);

	/**
	 * Answer all available font family names for this environment
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @return a <code>String[]</code> containing font family names
	 *
	 * @see	Font#getFamily()
	 * @since	JDK 1.2
	 */
	public abstract String[] getAvailableFontFamilyNames();

	/**
	 * Answer all available, localized font family names for this environment
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param locale the <code>Locale</code> representing a geographic region
	 *
	 * @return a <code>String[]</code> containing font family names
	 *
	 * @see	Font#getFamily()
	 * @since	JDK 1.2
	 */
	public abstract String[] getAvailableFontFamilyNames(Locale locale);

	/**
	 * Answer the default screen device for this environment
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @return GraphicsDevice
	 */
	public abstract GraphicsDevice getDefaultScreenDevice()
	throws HeadlessException
	;

	/**
	 * Answer all screen devices for this environment
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @return  a <code>GraphicsDevice[]</code>
	 */
	public abstract GraphicsDevice[] getScreenDevices()
	throws HeadlessException
	;

	public static boolean isHeadless() {
		return getLocalGraphicsEnvironment().getHeadless();
	}

	abstract boolean getHeadless();

	abstract void setHeadless(boolean headless);
}
