package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.awt.image.BufferedImage;
import java.util.Locale;

import com.ibm.oti.awt.Util;

class GraphicsEnvironmentImpl extends GraphicsEnvironment {

	private GraphicsDevice[] screenDevices = {new GraphicsDeviceImpl()};
	private boolean headless;

	public GraphicsEnvironmentImpl() {
		headless = false;
	}

	public Graphics2D createGraphics(BufferedImage image) {
		return (Graphics2D) image.getGraphics();
	}

	public String[] getAvailableFontFamilyNames() {
		String[] allFonts = getFontList(true); // use scalable like JDK

		// ensure unique list like JDK
		java.util.Hashtable nameSet = new java.util.Hashtable();

		for (int i = 0; i < allFonts.length; i++) {
			String next = allFonts[i];
			nameSet.put(next, next);
		}

		// add the logical fonts to the list
		String[] fontFamilies = Font.getFontList();
		for (int i = 0; i < fontFamilies.length; i++) {
			String next = fontFamilies[i];
			nameSet.put(next, next);
		}

		String[] names = new String[nameSet.size()];
		java.util.Enumeration unique = nameSet.keys();
		for (int i = 0; i < names.length; i++) {
			names[i] = (String) unique.nextElement();
		}
		sort(names);

		return names;
	}

	public String[] getAvailableFontFamilyNames(Locale locale) {
		return getAvailableFontFamilyNames(); // For now
	}

	public GraphicsDevice getDefaultScreenDevice() {
		if (getHeadless()) {
			throw new HeadlessException();
		}
		return screenDevices[0];
	}

	public GraphicsDevice[] getScreenDevices() {
		if (getHeadless()) {
			throw new HeadlessException();
		}
		return screenDevices;
	}

	String[] getFontList(final boolean scalable) {
		return Util.getDisplay().getFontList(null, scalable);
	}

	void sort(String[] items) {
		/* Shell Sort from K&R, pg 108 */
		int length = items.length;
		for (int gap = length / 2; gap > 0; gap /= 2) {
			for (int i = gap; i < length; i++) {
				for (int j = i - gap; j >= 0; j -= gap) {
					if (items[j] .toLowerCase() .compareTo(items[j + gap].toLowerCase())> 0) {
						String swap = items[j];
						items[j] = items[j + gap];
						items[j + gap] = swap;
					}
				}
			}
		}
	}

	boolean getHeadless() {
		return headless;
	}

	void setHeadless(boolean headless) {
		this.headless = headless;
	}
}
