package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.io.Serializable;

public class GridBagConstraints implements Cloneable, Serializable {

	static final long serialVersionUID = -1000070633030801713L;

	public int anchor;
	public int fill;
	public int gridx;
	public int gridy;
	public int gridwidth;
	public int gridheight;
	public Insets insets;
	public int ipadx;
	public int ipady;
	public double weightx;
	public double weighty;

	int tempX;
	int tempY;
	int tempWidth;
	int tempHeight;
	int minWidth;
	int minHeight;

	public static final int NONE = 0;
	public static final int BOTH = 1;
	public static final int HORIZONTAL = 2;
	public static final int VERTICAL = 3;

	public static final int RELATIVE = -1;
	public static final int REMAINDER = 0;

	public static final int CENTER = 10;
	public static final int NORTH = 11;
	public static final int NORTHEAST = 12;
	public static final int EAST = 13;
	public static final int SOUTHEAST = 14;
	public static final int SOUTH = 15;
	public static final int SOUTHWEST = 16;
	public static final int WEST = 17;
	public static final int NORTHWEST = 18;

	public GridBagConstraints() {
		this(RELATIVE, RELATIVE, 1, 1, 0, 0, CENTER, NONE, new Insets(0, 0, 0, 0), 0, 0);
	}

	GridBagConstraints(
		int gridx, int gridy, int gridwidth, int gridheight,
		double weightx, double weighty,
		int anchor,
		int fill,
		Insets insets,
		int ipadx, int ipady)
	{
		this.gridx = gridx;
		this.gridy = gridy;
		this.gridwidth = gridwidth;
		this.gridheight = gridheight;
		this.weightx = weightx;
		this.weighty = weighty;
		this.anchor = anchor;
		this.fill = fill;
		this.insets = insets;
		this.ipadx = ipadx;
		this.ipady = ipady;
	}

	/**
	 * Answers a new instance of the same class as the receiver,
	 * whose slots have been filled in with the values in the
	 * slots of the receiver.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @return		a <code>GridBagLayout</code> which is a shallow copy of this object
	 *
	 * @throws		OutOfMemoryError	if there is not enough memory to allocate the clone
	 *
	 * @see			java.lang.Cloneable
	 */
	public Object clone() {
		try {
			GridBagConstraints clone = (GridBagConstraints)super.clone();
			clone.insets = (Insets)insets.clone();
			return clone;
		} catch (CloneNotSupportedException e) {
			return null;
		}
	}

}
