package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */
import java.io.Serializable;
import java.util.Hashtable;

public class GridBagLayout implements LayoutManager2, Serializable {

	static final long serialVersionUID = 8838754796412211005L;

	public double columnWeights[];

	public int columnWidths[];

	public double rowWeights[];

	public int rowHeights[];

	protected Hashtable comptable;

	protected GridBagConstraints defaultConstraints;

	GridBagLayoutInfo layoutInfo;

	protected static final int MINSIZE = 1;

	protected static final int PREFERREDSIZE = 2;

	protected static final int MAXGRIDSIZE = 512;

	/**
	 * Constructs a new <code>GridBagLayout</code>.
	 *
	 * @author IBM
	 * @version initial
	 */
	public GridBagLayout() {
		this.comptable = new Hashtable();
		this.defaultConstraints = new GridBagConstraints();
	}

	double add(double values[], int start, int end) {
		double result = 0;
		for (int i = start; i < end; i++)
			result += values[i];
		return result;
	}

	double add(double values[]) {
		return add(values, 0, values.length);
	}

	int add(int values[]) {
		return add(values, 0, values.length);
	}

	int add(int values[], int start, int end) {
		int result = 0;
		for (int i = start; i < end; i++)
			result += values[i];
		return result;
	}

	/**
	 * Adds the specified <code>Component</code> to the receiver using the
	 * given name.
	 *
	 * @author IBM
	 * @version initial
	 *
	 * @param name
	 *            the name which identifies the component
	 * @param component
	 *            the component to be added
	 *
	 * @see <code>Container.add()</code>
	 *
	 * @deprecated use <code>addLayoutComponent(Component, Object)</code>
	 *             instead
	 *
	 */
	public void addLayoutComponent(String name, Component component) {
		setConstraints(component, defaultConstraints);
	}

	/**
	 * Adds the specified <code>Component</code> to the receiver using the
	 * given constraints.
	 * <p>
	 * The constraints for a <code>GridBagLayout</code> must be a
	 * <code>GridBagConstraints</code> object.
	 *
	 * @author IBM
	 * @version initial
	 *
	 * @param component
	 *            the component to be added
	 * @param constraints
	 *            the constraints object for the given component
	 *
	 * @throws IllegalArgumentException
	 *             if the constraints is invalid
	 *
	 * @see <code>Container.add()</code>
	 */
	public void addLayoutComponent(Component component, Object constraints) {
		if (constraints instanceof String) {
			addLayoutComponent((String) constraints, component);
			return;
		}
		if (constraints instanceof GridBagConstraints) {
			setConstraints(component, (GridBagConstraints) constraints);
		} else if (constraints != null) {
			throw new IllegalArgumentException();
		}
	}

	protected void AdjustForGravity(GridBagConstraints c, Rectangle r) {
		switch (c.fill) {
		case GridBagConstraints.BOTH:
			r.x += c.insets.left;
			r.y += c.insets.top;
			r.width -= c.insets.left + c.insets.right;
			r.height -= c.insets.top + c.insets.bottom;
			break;
		case GridBagConstraints.HORIZONTAL:
			r.x += c.insets.left;
			r.width -= c.insets.left + c.insets.right;
			int compHeight = Math.min(r.height, c.minHeight);
			switch (c.anchor) {
			case GridBagConstraints.EAST:
			case GridBagConstraints.WEST:
			case GridBagConstraints.CENTER:
				r.y += c.insets.top + ((r.height - c.insets.top - c.insets.bottom) - compHeight) / 2;
				break;
			case GridBagConstraints.NORTHWEST:
			case GridBagConstraints.NORTHEAST:
			case GridBagConstraints.NORTH:
				r.y += c.insets.top;
				break;
			case GridBagConstraints.SOUTHEAST:
			case GridBagConstraints.SOUTHWEST:
			case GridBagConstraints.SOUTH:
				r.y += r.height - c.insets.bottom - compHeight;
			default:
			}
			r.height = compHeight;
			break;
		case GridBagConstraints.VERTICAL:
			r.y += c.insets.top;
			r.height -= c.insets.top + c.insets.bottom;
			int compWidth = Math.min(r.width, c.minWidth);
			switch (c.anchor) {
			case GridBagConstraints.NORTH:
			case GridBagConstraints.SOUTH:
			case GridBagConstraints.CENTER:
				r.x += c.insets.left + ((r.width - c.insets.left - c.insets.right) - compWidth) / 2;
				break;
			case GridBagConstraints.NORTHEAST:
			case GridBagConstraints.SOUTHEAST:
			case GridBagConstraints.EAST:
				r.x += r.width - c.insets.right - compWidth;
				break;
			case GridBagConstraints.NORTHWEST:
			case GridBagConstraints.SOUTHWEST:
			case GridBagConstraints.WEST:
				r.x += c.insets.left;
			default:
			}
			r.width = compWidth;
			break;
		case GridBagConstraints.NONE:
		default:
			compWidth = Math.min(r.width, c.minWidth);
			compHeight = Math.min(r.height, c.minHeight);
			switch (c.anchor) {
			case GridBagConstraints.NORTHWEST:
				r.x += c.insets.left;
				r.y += c.insets.top;
				break;
			case GridBagConstraints.NORTH:
				r.x += c.insets.left + ((r.width - c.insets.left - c.insets.right) - compWidth) / 2;
				r.y += c.insets.top;
				break;
			case GridBagConstraints.NORTHEAST:
				r.x += r.width - c.insets.right - compWidth;
				r.y += c.insets.top;
				break;
			case GridBagConstraints.WEST:
				r.x += c.insets.left;
				r.y += c.insets.top + ((r.height - c.insets.top - c.insets.bottom) - compHeight) / 2;
				break;
			case GridBagConstraints.CENTER:
				r.x += c.insets.left + ((r.width - c.insets.left - c.insets.right) - compWidth) / 2;
				r.y += c.insets.top + ((r.height - c.insets.top - c.insets.bottom) - compHeight) / 2;
				break;
			case GridBagConstraints.EAST:
				r.x += r.width - c.insets.right - compWidth;
				r.y += c.insets.top + ((r.height - c.insets.top - c.insets.bottom) - compHeight) / 2;
				break;
			case GridBagConstraints.SOUTHWEST:
				r.x += c.insets.left;
				r.y += r.height - c.insets.bottom - compHeight;
				break;
			case GridBagConstraints.SOUTH:
				r.x += c.insets.left + ((r.width - c.insets.left - c.insets.right) - compWidth) / 2;
				r.y += r.height - c.insets.bottom - compHeight;
				break;
			case GridBagConstraints.SOUTHEAST:
				r.x += r.width - c.insets.right - compWidth;
				r.y += r.height - c.insets.bottom - compHeight;
				break;
			default:
			}
			r.width = compWidth;
			r.height = compHeight;
			break;
		}
	}

	protected void ArrangeGrid(Container container) {
		// long start,end;
		// System.out.println("Measuring time GridBagLayout ArrangeGrid1");
		// start= System.currentTimeMillis();

		GridBagLayoutInfo info = GetLayoutInfo(container, PREFERREDSIZE);
		Dimension size = container.getSize();
		Dimension prefSize = GetMinSize(container, info);

		// here is the check to see whether the preferred size of all children
		// will fit in the parent container. If the combined preferred size is
		// larger than the container size, use minimum size to calculate the
		// size
		// of each grid.
		if (prefSize.height > size.height || prefSize.width > size.width) {
			info = GetLayoutInfo(container, MINSIZE);
		}

		if (info == null)
			return;

		Insets insets = container.getInsets();

		int widths[] = info.columnWidths;
		int heights[] = info.rowHeights;

		int totalWidth = add(info.columnWidths);
		int totalHeight = add(info.rowHeights);
		int availableWidth = size.width - insets.left - insets.right;
		int availableHeight = size.height - insets.top - insets.bottom;
		int extraWidth = availableWidth - totalWidth;
		int extraHeight = availableHeight - totalHeight;

		double totalColumnWeights = add(info.columnWeights);
		double totalRowWeights = add(info.rowWeights);

		info.origin = new Point(insets.left, insets.top);

		if (totalColumnWeights == 0) {
			info.origin.x += extraWidth / 2;
		} else {
			if (extraWidth != 0) {
				for (int i = 0; i < widths.length; i++) {
					widths[i] += extraWidth
							* (info.columnWeights[i] / totalColumnWeights);
				}
			}
		}
		if (totalRowWeights == 0) {
			info.origin.y += extraHeight / 2;
		} else {
			if (extraHeight != 0) {
				for (int i = 0; i < info.rowHeights.length; i++) {
					heights[i] += extraHeight
							* (info.rowWeights[i] / totalRowWeights);
				}
			}
		}

		// end= System.currentTimeMillis();
		// com.ibm.oti.awt.GraphicsTimerLog.getDefault().logTime(
		// com.ibm.oti.awt.GraphicsTimerLog.GridBagLayout_ArrangeGrid1,
		// (end-start));
		// System.out.println("ok:"+(end-start));

		// System.out.println("Measuring time GridBagLayout ArrangeGrid2");
		// start= System.currentTimeMillis();
		int ncomponents = container.getComponentCount();
		Rectangle area = new Rectangle();

		// container.getPeer().setVisible(false);

		for (int i = 0; i < ncomponents; i++) {
			// long lstart,lend;
			Component component = container.getComponent(i);
			// System.out.println("Measuring time GridBagLayout ArrangeGrid21");
			// lstart= System.currentTimeMillis();
			GridBagConstraints c = lookupConstraints(component);
			// lend= System.currentTimeMillis();
			// com.ibm.oti.awt.GraphicsTimerLog.getDefault().logTime(
			// / com.ibm.oti.awt.GraphicsTimerLog.GridBagLayout_ArrangeGrid21,
			// (lend-lstart));
			// System.out.println("ok:"+(end-start));

			area.x = info.origin.x;
			for (int j = 0; j < c.tempX; j++)
				area.x += widths[j];

			area.y = info.origin.y;
			for (int j = 0; j < c.tempY; j++)
				area.y += heights[j];

			area.width = 0;
			int lastCell = c.tempX + c.tempWidth;
			for (int j = c.tempX; j < lastCell; j++)
				area.width += widths[j];

			area.height = 0;
			lastCell = c.tempY + c.tempHeight;
			for (int j = c.tempY; j < lastCell; j++)
				area.height += heights[j];

			// System.out.println("Measuring time GridBagLayout ArrangeGrid22");
			// lstart= System.currentTimeMillis();
			AdjustForGravity(c, area);
			// lend= System.currentTimeMillis();
			// com.ibm.oti.awt.GraphicsTimerLog.getDefault().logTime(
			// com.ibm.oti.awt.GraphicsTimerLog.GridBagLayout_ArrangeGrid22,
			// (lend-lstart));
			// System.out.println("ok:"+(end-start));

			// System.out.println("Measuring time GridBagLayout ArrangeGrid23");
			// lstart= System.currentTimeMillis();
			// System.out.println("Calling peer setBounds from GridBagLayout");
			component.setBounds(area.x, area.y, area.width, area.height);
			// lend= System.currentTimeMillis();
			// com.ibm.oti.awt.GraphicsTimerLog.getDefault().logTime(
			// com.ibm.oti.awt.GraphicsTimerLog.GridBagLayout_ArrangeGrid23,
			// (lend-lstart) );
			// System.out.println("ok:"+(end-start));

		}

		// container.getPeer().setVisible(true);

		// end= System.currentTimeMillis();
		// com.ibm.oti.awt.GraphicsTimerLog.getDefault().logTime(
		// com.ibm.oti.awt.GraphicsTimerLog.GridBagLayout_ArrangeGrid2,
		// (end-start));
		// System.out.println("ok:"+(end-start));

		layoutInfo = info;
	}

	public GridBagConstraints getConstraints(Component component) {
		return (GridBagConstraints) lookupConstraints(component).clone();
	}

	/**
	 * Answer the preferred vertical alignment for the specified
	 * <code>Container</code>.
	 *
	 * @author IBM
	 * @version initial
	 *
	 * @return a <code>float</code> which is the alignment value
	 *
	 * @see <code>Container.getAlignmentX()</code>
	 */
	public float getLayoutAlignmentX(Container container) {
		return Component.CENTER_ALIGNMENT;
	}

	/**
	 * Answer the preferred horizontal alignment for the specified
	 * <code>Container</code>.
	 *
	 * @author IBM
	 * @version initial
	 *
	 * @return a <code>float</code> which is the alignment value
	 *
	 * @see <code>Container.getAlignmentY()</code>
	 */
	public float getLayoutAlignmentY(Container container) {
		return Component.CENTER_ALIGNMENT;
	}

	/**
	 * Answers the row heights and column widths for the receiver.
	 *
	 * @author IBM
	 * @version initial
	 *
	 * @return int[][] array of column widths and row heights
	 */
	public int[][] getLayoutDimensions() {
		if (layoutInfo == null)
			return null;
		return new int[][] { layoutInfo.columnWidths, layoutInfo.rowHeights };
	}

	/**
	 * Answer a <code>GridBagLayoutInfo</code> describing width, height and
	 * row/column weights of all cells in the grid bag. This is accomplished by
	 * looping over the children of the specified <code>Container</code> and
	 * determining their position and size in the grid bag.
	 *
	 * @author IBM
	 * @version initial
	 *
	 * @param container
	 *            the container to be laid out
	 * @param sizeflag
	 *            one of <code>MINSIZE</code> or <code>PREFERREDSIZE</code>
	 *            determining which size to query for the children
	 *
	 * @return a <code>GridBagLayoutInfo</code> which represents the layout
	 *         information
	 */
	GridBagLayoutInfo GetLayoutInfo(Container container, int sizeflag) {
		int ncomponents = container.getComponentCount();
		if (ncomponents == 0)
			return null;

		GridBagLayoutInfo info = new GridBagLayoutInfo();

		GridBagConstraints lastConstraintsInRow[] = new GridBagConstraints[MAXGRIDSIZE];
		GridBagConstraints lastConstraintsInColumn[] = new GridBagConstraints[MAXGRIDSIZE];

		int numberOfColumns = 0;
		int numberOfRows = 0;

		// Loop through the children and determine their position and size
		// in the grid bag
		for (int i = 0; i < ncomponents; i++) {
			GridBagConstraints c = lookupConstraints(container.getComponent(i));

			c.tempX = c.gridx;
			c.tempY = c.gridy;
			c.tempWidth = c.gridwidth;
			c.tempHeight = c.gridheight;

			// *** Determine the grid x and y for the child.

			// If gridx is RELATIVE then place the child in the first available
			// column in a row that is not full. If both gridx and gridy are
			// RELATIVE, then x is given priority and gridy is assigned the row
			// where an available column was found. A row becames full when a
			// child
			// whose gridwidth value is REMAINDER is added.
			GridBagConstraints previousConstr = null;
			if (c.gridx == GridBagConstraints.RELATIVE) {
				int row = (c.gridy == GridBagConstraints.RELATIVE) ? 0
						: c.tempY;
				// Find a row whose last free cell is occupied by a child
				// whose gridwidth is not REMAINDER.
				outerloop: for (; row < numberOfRows; row++) {
					previousConstr = lastConstraintsInRow[row];
					if (previousConstr == null
							|| previousConstr.gridwidth != GridBagConstraints.REMAINDER) {
						// If the child occupies more than one row, need to
						// check every row
						int end = row
								+ ((c.gridheight < 0) ? numberOfRows
										: c.tempHeight);
						for (int j = row + 1; j < end; j++) {
							GridBagConstraints constr = lastConstraintsInRow[j];
							if (constr != null) {
								if (constr.tempX + constr.tempWidth > previousConstr.tempX
										+ previousConstr.tempWidth)
									continue outerloop;
							}
						}
						break;
					}
				}
				// If the loop above reaches the number of rows, the grid has to
				// grow.
				// Find the last column that is not full. The gridx of this
				// child will
				// be the one to the right of that column.
				if (row == numberOfRows) {
					previousConstr = null;
					int column = numberOfColumns - 1;
					for (; column >= 0; column--) {
						GridBagConstraints constr = lastConstraintsInColumn[column];
						if (constr != null) {
							if (constr.gridheight == GridBagConstraints.REMAINDER) {
								previousConstr = constr;
								break;
							} else if (constr.gridheight == GridBagConstraints.RELATIVE
									&& c.gridheight == GridBagConstraints.REMAINDER) {
								previousConstr = constr;
								break;
							}
						}
					}
				}
				// If the previous cell is occupied by a child whose gridwidth
				// is RELATIVE,
				// then this child will occupied the last cell in this row, but
				// the last
				// cell can not be determined now, since the number of columns
				// is not known.
				if (previousConstr == null)
					c.tempX = 0;
				else if (previousConstr.gridwidth != GridBagConstraints.RELATIVE) {
					if (previousConstr.gridheight == GridBagConstraints.RELATIVE
							&& c.gridheight == GridBagConstraints.REMAINDER) {
						c.tempX = previousConstr.tempX;
					} else
						c.tempX = previousConstr.tempX
								+ previousConstr.tempWidth;
				}

				if (c.gridy == GridBagConstraints.RELATIVE)
					c.tempY = row;
			} else if (c.gridy == GridBagConstraints.RELATIVE) {
				// Find a column whose last free cell is occupied by a child
				// whose gridheight is not REMAINDER.
				int column = c.tempX;
				outerloop: for (; column < numberOfColumns; column++) {
					previousConstr = lastConstraintsInColumn[column];
					if (previousConstr == null
							|| previousConstr.gridheight != GridBagConstraints.REMAINDER) {
						// If the child occupies more than one column, need to
						// check every column
						int end = column
								+ ((c.gridwidth < 0) ? numberOfColumns
										: c.tempWidth);
						for (int j = column + 1; j < end; j++) {
							GridBagConstraints constr = lastConstraintsInColumn[j];
							if (constr != null) {
								if (constr.tempY + constr.tempHeight > previousConstr.tempY
										+ previousConstr.tempHeight)
									continue outerloop;
							}
						}
						break;
					}
				}
				// If the loop above reaches the number of columns, the grid has
				// to grow.
				// Find the last row that is not full. The gridy of this child
				// will
				// be the one below of that row.
				if (column == numberOfColumns) {
					previousConstr = null;
					int row = numberOfRows - 1;
					for (; row >= 0; row--) {
						GridBagConstraints constr = lastConstraintsInRow[row];
						if (previousConstr != null) {
							if (previousConstr.gridwidth == GridBagConstraints.REMAINDER) {
								previousConstr = constr;
								break;
							} else if (previousConstr.gridwidth == GridBagConstraints.RELATIVE
									&& c.gridwidth == GridBagConstraints.REMAINDER) {
								previousConstr = constr;
								break;
							}
						}
					}
				}
				// If the previous cell is occupied by a child whose gridheight
				// is RELATIVE,
				// then this child will occupied the last cell in this row, but
				// the last
				// cell can not be determined now, since the number of columns
				// is not known.
				if (previousConstr == null)
					// c.tempY = 0;
					c.tempY = i; // bug 98709
				else if (previousConstr.gridheight != GridBagConstraints.RELATIVE) {
					if (previousConstr.gridwidth == GridBagConstraints.RELATIVE
							&& c.gridwidth == GridBagConstraints.REMAINDER) {
						c.tempY = previousConstr.tempY;
					} else
						c.tempY = previousConstr.tempY
								+ previousConstr.tempHeight;
				}
			}

			int firstColumn = c.tempX;
			int firstRow = c.tempY;
			int endColumn = c.tempX + c.tempWidth;
			int endRow = c.tempY + c.tempHeight;

			// Update the number of columns. If the gridwidth value of this
			// child
			// is RELATIVE, there will be at least two more columns. One for
			// this
			// child (may more) and one for the REMAINDER that follows. If the
			// gridwidth value of this child is REMAINDER, there will be at
			// least
			// another column. Otherwise, the number of columns is the maximum
			// column this child occupies. This applies to rows.
			if (c.gridwidth == GridBagConstraints.RELATIVE) {
				endColumn = c.tempX + 2;
				numberOfColumns = Math.max(numberOfColumns, c.tempX + 2);
			} else if (c.gridwidth == GridBagConstraints.REMAINDER) {
				if (c.tempX == GridBagConstraints.RELATIVE) {
					c.tempWidth = 1;
					firstColumn = numberOfColumns - 1;
				} else
					numberOfColumns = Math.max(numberOfColumns, c.tempX + 1);
				endColumn = numberOfColumns;
			} else
				numberOfColumns = Math.max(numberOfColumns, endColumn);

			if (c.gridheight == GridBagConstraints.RELATIVE) {
				endRow = c.tempY + 2;
				numberOfRows = Math.max(numberOfRows, c.tempY + 2);
			} else if (c.gridheight == GridBagConstraints.REMAINDER) {
				if (c.tempY == GridBagConstraints.RELATIVE) {
					c.tempHeight = 1;
					firstRow = numberOfRows - 1;
				} else
					numberOfRows = Math.max(numberOfRows, c.tempY + 1);
				endRow = numberOfRows;
			} else
				numberOfRows = Math.max(numberOfRows, endRow);

			// Update the last free column in the rows spanned by this child
			for (int row = firstRow; row < endRow; row++) {
				GridBagConstraints constraints = lastConstraintsInRow[row];
				if (constraints == null
						|| constraints.tempX + constraints.tempWidth <= endColumn)
					lastConstraintsInRow[row] = c;
			}
			// Update the last free row in the columns spanned by this child
			for (int column = firstColumn; column < endColumn; column++) {
				GridBagConstraints constraints = lastConstraintsInColumn[column];
				if (constraints == null
						|| constraints.tempY + constraints.tempHeight <= endRow)
					lastConstraintsInColumn[column] = c;
			}

		}

		info.columnWidths = new int[numberOfColumns];
		info.rowHeights = new int[numberOfRows];
		info.columnWeights = new double[numberOfColumns];
		info.rowWeights = new double[numberOfRows];

		for (int i = 0; i < ncomponents; i++) {
			Component component = container.getComponent(i);

			if (!component.isVisible()) {
				continue;
			}

			GridBagConstraints c = lookupConstraints(component);

			// Retrieve the child size depending on the desired flag
			Dimension compSize;
			if (sizeflag == MINSIZE)
				compSize = component.getMinimumSize();
			else
				compSize = component.getPreferredSize();

			// Add the internal padding to the child size
			c.minWidth = compSize.width + c.ipadx;
			c.minHeight = compSize.height + c.ipady;

			// A child whose gridwidth/gridheight value is RELATIVE occupies the
			// remaining cells
			// in its row/columns, except the last one. A child whose
			// gridwidth/gridheight value is
			// REMAINDER occupies the remaining cells in its row/columns. If a
			// RELATIVE is
			// followed by a REMAINDER, the REMAINDER width/height is 1.
			if (c.gridwidth == GridBagConstraints.RELATIVE)
				c.tempWidth = numberOfColumns - c.tempX - 1;
			else if (c.gridwidth == GridBagConstraints.REMAINDER) {
				if (c.tempX == GridBagConstraints.RELATIVE) {
					c.tempX = numberOfColumns - 1;
					c.tempWidth = 1;
				} else
					c.tempWidth = numberOfColumns - c.tempX;
			}
			if (c.gridheight == GridBagConstraints.RELATIVE)
				c.tempHeight = numberOfRows - c.tempY - 1;
			else if (c.gridheight == GridBagConstraints.REMAINDER) {
				if (c.tempY == GridBagConstraints.RELATIVE) {
					c.tempY = numberOfRows - 1;
					c.tempHeight = 1;
				} else
					c.tempHeight = numberOfRows - c.tempY;
			}

			// In order to calculate the width of a column, first the components
			// that
			// occupy only that column are considered. The width of the column
			// is the
			// maximum width of those components. This applies to row heights
			// and
			// row/column weights.
			if (c.tempWidth == 1) {
				info.columnWidths[c.tempX] = Math.max(
						info.columnWidths[c.tempX], c.minWidth + c.insets.left
								+ c.insets.right);
				info.columnWeights[c.tempX] = Math.max(
						info.columnWeights[c.tempX], c.weightx);
			}
			if (c.tempHeight == 1) {
				info.rowHeights[c.tempY] = Math.max(info.rowHeights[c.tempY],
						c.minHeight + c.insets.top + c.insets.bottom);
				info.rowWeights[c.tempY] = Math.max(info.rowWeights[c.tempY],
						c.weighty);
			}
		}

		// For children that occupy more than one column, if the child width
		// is greater than the combined width of the columns spanned by the
		// child, then the extra width is distribuited proportionally to each
		// column. If the combined width is 0, than the last column gets the
		// whole extra width. This applies to row heights and row/column
		// weights.
		for (int i = 0; i < ncomponents; i++) {
			GridBagConstraints c = lookupConstraints(container.getComponent(i));

			if (c.tempWidth > 1) {
				int startCol = c.tempX;
				int endCol = c.tempX + c.tempWidth - 1;

				double combinedWeight = add(info.columnWeights, startCol,
						endCol + 1);
				if (c.weightx > combinedWeight) {
					if (combinedWeight == 0) {
						info.columnWeights[endCol] = c.weightx;
					} else {
						double factor = 1 + ((c.weightx - combinedWeight) / combinedWeight);
						for (int j = startCol; j <= endCol; j++)
							info.columnWeights[j] *= factor;
						// recalculate the combined weight since columnWeights have changed
						combinedWeight = add(info.columnWeights, startCol, endCol + 1);
					}
				}

				int combinedWidth = add(info.columnWidths, startCol, endCol + 1);
				int desiredWidth = c.minWidth + c.insets.left + c.insets.right;

				if (desiredWidth > combinedWidth) {
					int extraWidth = desiredWidth - combinedWidth;
					if (combinedWeight == 0) {
						info.columnWidths[endCol] = Math.max(
								info.columnWidths[endCol], desiredWidth
										- add(info.columnWidths, startCol,
												endCol));
					} else {
						for (int j = startCol; j <= endCol; j++)
							info.columnWidths[j] += extraWidth
									* (info.columnWeights[j] / combinedWeight);
					}
				}
			}
			if (c.tempHeight > 1) {
				int startRow = c.tempY;
				int endRow = c.tempY + c.tempHeight - 1;

				double combinedWeight = add(info.rowWeights, startRow,
						endRow + 1);
				if (c.weighty > combinedWeight) {
					if (combinedWeight == 0) {
						info.rowWeights[endRow] = c.weighty;
					} else {
						double factor = 1 + ((c.weighty - combinedWeight) / combinedWeight);
						for (int j = startRow; j <= endRow; j++)
							info.rowWeights[j] *= factor;
						// recalculate the combined weight since rowWeights have changed
						combinedWeight = add(info.rowWeights, startRow, endRow + 1);
					}
				}

				int combinedHeight = add(info.rowHeights, startRow, endRow + 1);
				int desiredHeight = c.minHeight + c.insets.top
						+ c.insets.bottom;

				if (desiredHeight > combinedHeight) {
					int extraWidth = desiredHeight - combinedHeight;
					if (combinedWeight == 0) {
						info.rowHeights[endRow] = Math
								.max(info.rowHeights[endRow],
										desiredHeight
												- add(info.rowHeights,
														startRow, endRow));
					} else {
						for (int j = startRow; j <= endRow; j++)
							info.rowHeights[j] += extraWidth
									* (info.rowWeights[j] / combinedWeight);
					}
				}
			}
		}

		// If override values are specified, use them.
		info.columnWidths = mergeOverrideValues(info.columnWidths,
				numberOfColumns, columnWidths);
		info.rowHeights = mergeOverrideValues(info.rowHeights, numberOfRows,
				rowHeights);
		info.columnWeights = mergeOverrideValues(info.columnWeights,
				numberOfColumns, columnWeights);
		info.rowWeights = mergeOverrideValues(info.rowWeights, numberOfRows,
				rowWeights);

		return info;
	}

	public Point getLayoutOrigin() {
		return (layoutInfo != null ? layoutInfo.origin : new Point(0, 0));
	}

	public double[][] getLayoutWeights() {
		if (layoutInfo == null)
			return null;
		return new double[][] { layoutInfo.columnWeights, layoutInfo.rowWeights };
	}

	Dimension GetMinSize(Container container, GridBagLayoutInfo info) {
		int width = 0;
		int height = 0;

		if (info != null) {
			width = add(info.columnWidths);
			height = add(info.rowHeights);
		}

		Insets insets = container.getInsets();
		return new Dimension(width + insets.left + insets.right, height
				+ insets.top + insets.bottom);
	}

	/**
	 * Invalidates the layout for the specified <code>Container</code>. Any
	 * cached information should be discarded.
	 *
	 * @author IBM
	 * @version initial
	 *
	 * @param container
	 *            the <code>Container</code> to invalidate the layout
	 *
	 * @see <code>Container.invalidate()</code>
	 */
	public void invalidateLayout(Container container) {
		// NOP
	}

	public void layoutContainer(Container container) {
		ArrangeGrid(container);
	}

	public Point location(int x, int y) {
		Point origin = layoutInfo.origin;

		// Find x grid
		int currentX = origin.x;
		int widths[] = layoutInfo.columnWidths;
		int gridx = 0;
		for (; gridx < widths.length; gridx++) {
			currentX += widths[gridx];
			if (x < currentX)
				break;
		}

		// Find y grid
		int currentY = origin.y;
		int heights[] = layoutInfo.rowHeights;
		int gridy = 0;
		for (; gridy < heights.length; gridy++) {
			currentY += heights[gridy];
			if (y < currentY)
				break;
		}

		return new Point(gridx, gridy);
	}

	protected GridBagConstraints lookupConstraints(Component component) {
		GridBagConstraints constraints = (GridBagConstraints) comptable
				.get(component);
		if (constraints == null) {
			setConstraints(component, defaultConstraints);
			constraints = (GridBagConstraints) comptable.get(component);
		}
		return constraints;
	}

	/**
	 * Calculates the maximum size for the specified <code>Container</code>.
	 *
	 * @author IBM
	 * @version initial
	 *
	 * @param container
	 *            the <code>Container</code> to calculate the maximum size
	 *
	 * @return a <code>Dimension</code> which is the maximum size for the
	 *         specified <code>Container</code>
	 *
	 * @see <code>Container.getMaximumSize()</code>
	 */
	public Dimension maximumLayoutSize(Container container) {
		return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
	}

	int[] mergeOverrideValues(int[] values, int valuesLength,
			int[] overrideValues) {
		int[] result;
		if (overrideValues == null) {
			if (values.length == valuesLength)
				result = values;
			else {
				result = new int[valuesLength];
				System.arraycopy(values, 0, result, 0, valuesLength);
			}
		} else {
			int overrideLength = overrideValues.length;
			int length = Math.max(valuesLength, overrideLength);
			result = new int[length];
			for (int i = 0; i < length; i++) {
				result[i] = Math.max((i < valuesLength) ? values[i] : 0,
						(i < overrideLength) ? overrideValues[i] : 0);
			}
		}
		return result;
	}

	double[] mergeOverrideValues(double[] values, int valuesLength,
			double[] overrideValues) {
		double[] result;
		if (overrideValues == null) {
			if (values.length == valuesLength)
				result = values;
			else {
				result = new double[valuesLength];
				System.arraycopy(values, 0, result, 0, valuesLength);
			}
		} else {
			int overrideLength = overrideValues.length;
			result = new double[valuesLength];
			for (int i = 0; i < valuesLength; i++) {
				result[i] = Math.max(values[i],
						(i < overrideLength) ? overrideValues[i] : 0);
			}
		}
		return result;
	}

	public Dimension minimumLayoutSize(Container container) {
		return GetMinSize(container, GetLayoutInfo(container, MINSIZE));
	}

	public Dimension preferredLayoutSize(Container container) {
		return GetMinSize(container, GetLayoutInfo(container, PREFERREDSIZE));
	}

	/**
	 * Removes the specified <code>Component</code> from the receiver.
	 *
	 * @author IBM
	 * @version initial
	 *
	 * @param component
	 *            the component to be removed
	 *
	 * @see <code>Container.remove()</code>
	 */
	public void removeLayoutComponent(Component component) {
		comptable.remove(component);
	}

	public void setConstraints(Component component,
			GridBagConstraints constraints) {
		comptable.put(component, constraints == null ? defaultConstraints
				.clone() : constraints.clone());
	}

	/**
	 * Answers a string containing a concise, human-readable description of the
	 * receiver's relevant information.
	 *
	 * @author IBM
	 * @version initial
	 *
	 * @return a java.lang.String which is a printable representation for the
	 *         receiver
	 */
	public String toString() {
		return getClass().getName();
	}

	protected void adjustForGravity(GridBagConstraints c, Rectangle r) {
		AdjustForGravity(c, r);
	}

	protected void arrangeGrid(Container container) {
		ArrangeGrid(container);
	}
}
