package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

import java.io.Serializable;

/**
 * <code>GridLayout</code> is a <code>LayoutManager</code> that
 * lays out the components in a grid.  Each cell in the grid
 * has the same size.
 *
 * @author		IBM
 * @version		initial
 *
 * @see		<code>LayoutManager</code>
 *
 */
public class GridLayout implements LayoutManager, Serializable {

	static final long serialVersionUID = -7411804673224730901L;

	// The number of rows in the grid
	int rows;

	// The number of columns in the grid
	int cols;

	// The horizontal gap between components
	int hgap;

	// The vertical gap between components
	int vgap;

	/**
	 * Constructs a new <code>GridLayout</code> with one
	 * row.
	 *
	 * @author	IBM
	 * @version	initial
	 */
	public GridLayout() {
		this(1, 0);
	}

	/**
	 * Constructs a new <code>GridLayout</code> with the specified
	 * number of rows and columns.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param	rows	the number of rows for the receiver
	 * @param	columns	the number of columns for the receiver
	 */
	public GridLayout(int rows, int columns) {
		this(rows, columns, 0, 0);
	}

	/**
	 * Constructs a new <code>GridLayout</code> with the specified
	 * number of rows/columns and specified horizontal/vertical gaps.
	 * <p>
	 * One of rows or columns must be greater than zero.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param	rows	the number of rows for the receiver
	 * @param	columns	the number of columns for the receiver
	 * @param	hgap	the horizontal gap
	 * @param	vgap	the vertical gap
	 *
	 * @throws	IllegalArgumentException	if both columns and rows are zero
	 */
	public GridLayout(int rows, int columns, int hgap, int vgap) {
		if (rows == 0 && columns == 0)
			throw new IllegalArgumentException();

		this.rows = rows;
		this.cols = columns;
		this.hgap = hgap;
		this.vgap = vgap;
	}

	/**
	 * Answers the number of rows.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @return		an <code>int</code> which is the number of rows in the receiver
	 *
	 * @see			<code>setRows(int)</code>
	 */
	public int getRows() {
		return rows;
	}

	/**
	 * Sets the number of rows.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param	rows		the desired number of rows in the recevier
	 *
	 * @throws	IllegalArgumentException	if both columns and rows are zero
	 *
	 * @see			<code>getRows()</code>
	 */
	public void setRows(int rows) {
		if (rows == 0 && cols == 0)
			throw new IllegalArgumentException();

		this.rows = rows;
	}

	/**
	 * Answers the number of columns.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @return		an <code>int</code> which is the number of columns in the receiver
	 *
	 * @see			<code>setColumns(int)</code>
	 */
	public int getColumns() {
		return cols;
	}

	/**
	 * Sets the number of columns.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param	columns		the desired number of columns in the recevier
	 *
	 * @throws	IllegalArgumentException	if both columns and rows are zero
	 *
	 * @see			<code>getColumns()</code>
	 */
	public void setColumns(int columns) {
		if (columns == 0 && rows == 0)
			throw new IllegalArgumentException();

		this.cols = columns;
	}

	/**
	 * Answers the horizontal gap between components.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @return		an <code>int</code> which is the horizontal gap
	 *
	 * @see			<code>setHgap(int)</code>
	 */
	public int getHgap() {
		return hgap;
	}

	/**
	 * Sets the horizontal gap between components.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param		hgap	the desired horizontal gap
	 *
	 * @see			<code>getHgap()</code>
	 */
	public void setHgap(int hgap) {
		this.hgap = hgap;
	}

	/**
	 * Answers the vertical gap between components.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @return		an <code>int</code> which is the vertaical gap
	 *
	 * @see			<code>setVgap(int)</code>
	 */
	public int getVgap() {
		return vgap;
	}

	/**
	 * Sets the vertical gap between components.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param		vgap	the desired vertical gap
	 *
	 * @see			<code>getHgap()</code>
	 */
	public void setVgap(int vgap) {
		this.vgap = vgap;
	}

	/**
	 * Adds the specified <code>Component</code> to the receiver
	 * using the given name.
	 * <p>
	 * The receiver does not need to keep the component or name in order
	 * to perform the layout operation.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param		name		the name which identifies the component
	 * @param		component	the component to be added
	 *
	 * @see			<code>Container.add()</code>
	 */
	public void addLayoutComponent(String name, Component component) {
	}

	/**
	 * Removes the specified <code>Component</code> from the receiver.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param		component	the component to be removed
	 *
	 * @see			<code>Container.remove()</code>
	 */
	public void removeLayoutComponent(Component component) {
	}

	/**
	 * Calculates the preferred size for the specified <code>Container</code>.
	 * The preferred size is the size with which the container can fully display
	 * itself and its children.
	 * <p>
	 * The preferred size of a container laid out using a <code>GridLayout</code>
	 * is the largest preferred size of each individual child multiplied by the
	 * number of rows or columns, plus the desired gaps between the children.
	 * <p>
	 * The container insets is included.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param		container	the <code>Container</code> to calculate the preferred size
	 *
	 * @return		a <code>Dimension</code> which is the preferred size
	 *				for the specified <code>Container</code>
	 *
	 * @see			<code>Container.preferredSize()</code>
	 */
	public Dimension preferredLayoutSize(Container container) {
		int width = 0;
		int height = 0;
		int ncomponents = container.getComponentCount();
		for (int i=0; i<ncomponents; i++) {
			Dimension size = container.getComponent(i).getPreferredSize();
			width = Math.max(width, size.width);
			height = Math.max(height, size.height);
		}

		int r, c;
		if (rows != 0) {
			r = rows;
			c = (int)Math.ceil((float)ncomponents / r);
		} else {
			c = cols;
			r = (int)Math.ceil((float)ncomponents / c);
		}

		Insets insets = container.getInsets();
		return new Dimension(
			(width * c) + insets.left + insets.right + ((c - 1) * hgap),
			(height * r) + insets.top + insets.bottom + ((r - 1) * vgap));
	}

	/**
	 * Calculates the minimum size for the specified <code>Container</code>.
	 * <p>
	 * The minimum size of a container laid out using a <code>GridLayout</code>
	 * is the largest minimum size of each individual child multiply by the
	 * number of rows or columns plus the desired gaps between the children.
	 * <p>
	 * The container insets is included.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param		container	the <code>Container</code> to calculate the minimum size
	 *
	 * @return		a <code>Dimension</code> which is the minimum size
	 *				for the specified <code>Container</code>
	 *
	 * @see			<code>Container.minimumSize()</code>
	 */
	public Dimension minimumLayoutSize(Container container) {
		int width = 0;
		int height = 0;
		int ncomponents = container.getComponentCount();
		for (int i=0; i<ncomponents; i++) {
			Dimension size = container.getComponent(i).getMinimumSize();
			width = Math.max(width, size.width);
			height = Math.max(height, size.height);
		}

		int r, c;
		if (rows != 0) {
			r = rows;
			c = (int)Math.ceil((float)ncomponents / r);
		} else {
			c = cols;
			r = (int)Math.ceil((float)ncomponents / c);
		}

		Insets insets = container.getInsets();
		return new Dimension(
			(width * c) + insets.left + insets.right + ((c - 1) * hgap),
			(height * r) + insets.top + insets.bottom + ((r - 1) * vgap));
	}

	/**
	 * Lays out the children of the specified <code>Container</code>.
	 * <p>
	 * The size of each cell is determined by dividing the container
	 * client area in equal portions considering the desired number
	 * of rows and columns.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param		container	the <code>Container</code> to be laid out
	 *
	 * @see			<code>Container.doLayout()</code>
	 */
	public void layoutContainer(Container container) {
		int ncomponents = container.getComponentCount();
		if (ncomponents <= 0)
			return;

		Dimension size = container.getSize();
		Insets insets = container.getInsets();

		int r, c;
		if (rows != 0) {
			r = rows;
			c = (int)Math.ceil((float)ncomponents / r);
		} else {
			c = cols;
			r = (int)Math.ceil((float)ncomponents / c);
		}

		int width = (size.width - insets.left - insets.right - ((c - 1) * hgap)) / c;
		int height = (size.height - insets.top - insets.bottom - ((r - 1) * vgap)) / r;

		int y = insets.top;
		int count = 0;

		for (int row=0; row<r; row++) {
			int x = insets.left;
			for (int column=0; column<c; column++) {
				Component component = container.getComponent(count);
				component.setBounds(x, y, width, height);
				x += width + hgap;
				if (++count == ncomponents) {
					return;
				}
			}
			y += height + vgap;
		}
	}

	/**
	 * Answers a string containing a concise, human-readable
	 * description of the receiver's relevant information.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @return	a java.lang.String which is a printable representation for the receiver
	 */
	public String toString() {
		return getClass().getName() +
			"[rows=" + getRows() + ",columns=" + getColumns() + //$NON-NLS-1$ //$NON-NLS-2$
			",hgap=" + getHgap() + ",vgap=" + getVgap() + "]"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

}
