package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.io.Serializable;

/**
 * A <code>Insets</code> represents the size of the borders in a
 * <code>Container</code>. Borders are the space left between the
 * the edges of the container and its client area which is the
 * area where its children are laid out.
 *
 * @author      IBM
 * @version		initial
 *
 * @see			java.io.Serializable
 * @see			java.awt.Container
 */

public class Insets implements Cloneable, Serializable {

	static final long serialVersionUID = -2272572637695466749L;

	/**
	 * The size of the top border.
	 */
	public int top;
	/**
	 * The size of the bottom border.
	 */
	public int bottom;
	/**
	 * The size of the left border.
	 */
	public int left;
	/**
	 * The size of the right border.
	 */
	public int right;

	/**
	 * Constructs a new <code>Insets</code> with specified
	 * top, left, bottom and right values.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param	top		the top border for the receiver
	 * @param	left	the left border for the receiver
	 * @param	bottom	the bottom border for the receiver
	 * @param	right	the right border for the receiver
	 */
	public Insets(int top, int left, int bottom, int right) {
		this.top = top;
		this.left = left;
		this.bottom = bottom;
		this.right = right;
	}

	/**
	 * Answers a new instance of the same class as the receiver,
	 * whose slots have been filled in with the values in the
	 * slots of the receiver.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @return		a Insets which is a shallow copy of this object
	 *
	 * @throws		OutOfMemoryError	if there is not enough memory to allocate the clone
	 *
	 * @see			java.lang.Cloneable
	 */
	public Object clone() {
		try {
			return super.clone();
		} catch (CloneNotSupportedException e) {
			return null;
		}
	}

	/**
	 * Compares the argument to the receiver, and answers true
	 * if the argument is a <code>Insets</code> and its top, left,
	 * bottom and right borders are equals to the receiver's top, left,
	 * bottom and right borders.

	 * @author      IBM
	 * @version		initial
	 *
	 * @param		object	Object	the object to compare with this dimension
	 *
	 * @return		<code>true</code> if the object is the same as this object
	 *				<code>false</code> otherwise
	 */
	public boolean equals(Object object) {
		if (object == this)
			return true;
		if (!(object instanceof Insets))
			return false;
		Insets insets = (Insets) object;
		return top == insets.top
			&& left == insets.left
			&& bottom == insets.bottom
			&& right == insets.right;
	}

	/**
	 * Generate the hashCode() of this instance.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @return		<code>int</code> hashCode value
	 *
	 */
	public int hashCode() {
		return top ^ bottom ^ left ^ right;
	}

	/**
	 * Answers a string containing a concise, human-readable
	 * description of the receiver's relevant information.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @return	a java.lang.String which is a printable representation for the receiver
	 */
	public String toString() {
		return getClass().getName() + "[top=" + top + ",left=" + left + //$NON-NLS-1$ //$NON-NLS-2$
		",bottom=" + bottom + ",right=" + right + "]"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

}
