package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * <code>ItemSelectable</code> is an interface for classes whose
 * objects fire <code>ItemEvent</code> when items are selected or
 * deselected.
 *
 * @author      IBM
 * @version		initial
 *
 * @see java.awt.event.ItemListener
 * @see java.awt.event.ItemEvent
 */

import java.awt.event.ItemListener;

public interface ItemSelectable {

	/**
	 * Adds a <code>ItemListener</code> to the receiver.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		itemListener	the <code>ItemListener</code> to be added.
	 *
	 * @see			java.awt.event.ItemListener
	 * @see			java.awt.event.ItemEvent
	 */
	public void addItemListener(ItemListener itemListener);

	/**
	 * Answers an array containg the selected items. If there are no selected
	 * items,  the return value is <code>null</code>.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @return	a <code>Object[]</code> which contains the selected items or <code>null</code>
	 */
	public Object[] getSelectedObjects();

	/**
	 * Removes an <code>ItemListener</code> from the receiver.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		itemListener	the <code>ItemListener</code> to be removed.
	 *
	 * @see			java.awt.event.ItemListener
	 * @see			java.awt.event.ItemEvent
	 */
	public void removeItemListener(ItemListener itemListener);

}
