package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This interface should be implemented by classes
 * which are able to lay out children of a <code>Container</code>.
 *
 * @author      IBM
 * @version		initial
 *
 */
public interface LayoutManager {

	/**
	 * Adds the specified <code>Component</code> to the receiver
	 * using the given name.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		name		the name which identifies the component
	 * @param		component	the component to be added
	 *
	 * @see			<code>Container.add()</code>
	 */
	public void addLayoutComponent(String name, Component component);

	/**
	 * Lays out the children of the specified <code>Container</code>.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		container	the <code>Container</code> to be laid out
	 *
	 * @see			<code>Container.doLayout()</code>
	 */
	public void layoutContainer(Container container);

	/**
	 * Calculates the minimum size for the specified <code>Container</code>.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		container	the <code>Container</code> to calculate the minimum size
	 *
	 * @return		a <code>Dimension</code> which is the minimum size
	 *				for the specified <code>Container</code>
	 *
	 * @see			<code>Container.getMinimumSize()</code>
	 */
	public Dimension minimumLayoutSize(Container container);

	/**
	 * Calculates the preferred size for the specified <code>Container</code>.
	 * The preferred size is the size with which the container can fully display
	 * itself and its children.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		container	the <code>Container</code> to calculate the preferred size
	 *
	 * @return		a <code>Dimension</code> which is the preferred size
	 *				for the specified <code>Container</code>
	 *
	 * @see			<code>Container.getPreferredSize()</code>
	 */
	public Dimension preferredLayoutSize(Container container);

	/**
	 * Removes the specified <code>Component</code> from the receiver.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		component	the component to be removed
	 *
	 * @see			<code>Container.remove()</code>
	 */
	public void removeLayoutComponent(Component component);

}
