package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This interface should be implemented by classes which
 * are able to lay out children of a <code>Container</code>.
 * <p>
 * <code>LayoutManager2</code> is an extention of <code>LayoutManager</code>
 * which introduces the concepts of constraints. Constraints are objects
 * that describe how a component should be laid out.
 *
 * @author      IBM
 * @version		initial
 *
 * @see		<code>LayoutManager</code>
 *
 */
public interface LayoutManager2 extends java.awt.LayoutManager {

	/**
	 * Adds the specified <code>Component</code> to the receiver
	 * using the given constraints.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		component	the component to be added
	 * @param		constraints	the constraints object for the given component
	 *
	 * @see			<code>Container.add()</code>
	 */
	public void addLayoutComponent(Component component, Object constraints);

	/**
	 * Answer the preferred vertical alignment for the specified
	 * <code>Container</code>.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @return		a <code>float</code> which is the alignment value
	 *
	 * @see			<code>Container.getAlignmentX()</code>
	 */
	public float getLayoutAlignmentX(Container container);

	/**
	 * Answer the preferred horizontal alignment for the specified
	 * <code>Container</code>.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @return		a <code>float</code> which is the alignment value
	 *
	 * @see			<code>Container.getAlignmentY()</code>
	 */
	public float getLayoutAlignmentY(Container container);

	/**
	 * Invalidates the layout for the specified <code>Container</code>.  Any
	 * cached information should be discarded.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		container	the <code>Container</code> to invalidate the layout
	 *
	 * @see			<code>Container.invalidate()</code>
	 */
	public void invalidateLayout(Container container);

	/**
	 * Calculates the maximum size for the specified <code>Container</code>.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		container	the <code>Container</code> to calculate the maximum size
	 *
	 * @return		a <code>Dimension</code> which is the maximum size
	 *				for the specified <code>Container</code>
	 *
	 * @see			<code>Container.getMaximumSize()</code>
	 */
	public Dimension maximumLayoutSize(Container container);

}
