package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import com.ibm.oti.awt.image.BufferedImageHandler;
import com.ibm.oti.awt.image.IImageImpl;

// private class representing a media object being
// tracked by a <code>MediaTracker</code>.
class MediaEntry {

	// the media object being tracked
	Image image;

	// ID for the tracked media object
	int id;

	// width and height to scale image to
	int width;
	int height;

	// next and previous media entry in the list
	public MediaEntry next;
	public MediaEntry previous;

	/**
	 * Constructs a new <code>MediaEntry</code> to
	 * represent a tracked media object.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 */
	public MediaEntry(Image image, int id) {
		this.image = image;
		this.id = id;
	}

	/**
	 * Constructs a new <code>MediaEntry</code> to
	 * represent a tracked media object.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 */
	public MediaEntry(Image image, int id, int width, int height) {
		this.image = image;
		this.id = id;
		this.width = width;
		this.height = height;
	}

	/**
	 * get the current status
	 */
	public int getStatus() {
		IImageImpl iImageImpl = BufferedImageHandler.createIImageImpl(image);
		int status = iImageImpl.getStatus();
		switch (status) {
			case IImageImpl.NOT_STARTED: 	return 0;
			case IImageImpl.LOADING: 		return MediaTracker.LOADING;
			case IImageImpl.FRAME_DONE: 	return MediaTracker.COMPLETE;
			case IImageImpl.FINISHED: 		return MediaTracker.COMPLETE;
			case IImageImpl.ABORTED:		return MediaTracker.ABORTED;
			case IImageImpl.ERRORED:		return MediaTracker.ERRORED;
			default: throw new RuntimeException("Unknown image state");
		}
	}

}
