package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

import com.ibm.ugl.eswt.OS;

/**
 * A <code>Panel</code> is a container that has no borders.
 * <p>
 * By default, this container fills its displayable area with its background
 * color. Subclasses should override <code>paint(Graphics)</code> to perform
 * custom drawings.
 *
 * @author		IBM
 * @version		initial
 */
public class Panel extends Container {

	static final long serialVersionUID = -2728009084054400034L;

	static FlowLayout defaultLayout = new FlowLayout();

	/**
	 * Constructs a new <code>Panel</code> with a <code>FlowLayout</code> as
	 * layout manager.
	 *
	 * @author	IBM
	 * @version	initial
	 */
	public Panel() {
		this(defaultLayout);
	}

	/**
	 * Constructs a new <code>Panel</code> with the specified
	 * layout manager.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param	layoutManager	 the layout manager for the receiver
	 */
	public Panel(LayoutManager layoutManager) {
		super(layoutManager);
	}

	/**
	 * Create the underlying widget for the receiver.
	 *
	 * @author      IBM
	 * @version		initial
	 */
	int _createPeer(Container hwParent) {
		return OS.Composite_New(hwParent.peer, 0);
	}

	/**
	 * Answer whether the receiver can be used as a canvas
	 */
	boolean isPaintable() {
		return true;
	}

	public boolean isLightweight() {
		return false;
	}

	public boolean isOpaque() {
		return true;
	}

	String classNonlocalizedName() {
		return "panel"; //$NON-NLS-1$
	}

	/**
	 * Answer a <code>Dimension</code> representing the size with which
	 * the receiver can fully display itself and its children if applicable.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @return	a java.awt.Dimension which is the receiver's preferred size
	 */
	Dimension getPreferredSizeImpl() {
		LayoutManager layout = getLayout();
		if (layout == null && (getWidth() != 0) && (getHeight() != 0)) {
			return new Dimension(getWidth(), getHeight());
		} else {
			return super.getPreferredSizeImpl();
		}
	}
}
