package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.io.Serializable;

import com.ibm.oti.awt.Util;
import com.ibm.ugl.eswt.OS;

/**
 * A <code>SystemColor</code> represents system RGB value.
 * NEED
 *
 * @author		IBM
 * @version		initial
 *
 * @see			java.io.Serializable
 */
public final class SystemColor extends Color implements Serializable {

	static final long serialVersionUID = 4503142729533789064L;

	public static final int NUM_COLORS = 26;

	/**
	 * Constants that specify the index of the system color.
	 */
	public static final int DESKTOP 				= 0;
	public static final int ACTIVE_CAPTION 			= 1;
	public static final int ACTIVE_CAPTION_TEXT 	= 2;
	public static final int ACTIVE_CAPTION_BORDER 	= 3;
	public static final int INACTIVE_CAPTION 		= 4;
	public static final int INACTIVE_CAPTION_TEXT 	= 5;
	public static final int INACTIVE_CAPTION_BORDER = 6;
	public static final int WINDOW 					= 7;
	public static final int WINDOW_BORDER 			= 8;
	public static final int WINDOW_TEXT 			= 9;
	public static final int MENU 					= 10;
	public static final int MENU_TEXT 				= 11;
	public static final int TEXT 					= 12;
	public static final int TEXT_TEXT 				= 13;
	public static final int TEXT_HIGHLIGHT 			= 14;
	public static final int TEXT_HIGHLIGHT_TEXT 	= 15;
	public static final int TEXT_INACTIVE_TEXT 		= 16;
	public static final int CONTROL 				= 17;
	public static final int CONTROL_TEXT 			= 18;
	public static final int CONTROL_HIGHLIGHT 		= 19;
	public static final int CONTROL_LT_HIGHLIGHT 	= 20;
	public static final int CONTROL_SHADOW 			= 21;
	public static final int CONTROL_DK_SHADOW 		= 22;
	public static final int SCROLLBAR 				= 23;
	public static final int INFO 					= 24;
	public static final int INFO_TEXT 				= 25;

	public static final SystemColor activeCaption = new SystemColor(ACTIVE_CAPTION);
	public static final SystemColor activeCaptionBorder = new SystemColor(ACTIVE_CAPTION_BORDER);
	public static final SystemColor activeCaptionText = new SystemColor(ACTIVE_CAPTION_TEXT);
	public static final SystemColor control = new SystemColor(CONTROL);
	public static final SystemColor controlDkShadow = new SystemColor(CONTROL_DK_SHADOW);
	public static final SystemColor controlHighlight = new SystemColor(CONTROL_HIGHLIGHT);
	public static final SystemColor controlLtHighlight = new SystemColor(CONTROL_LT_HIGHLIGHT);
	public static final SystemColor controlShadow = new SystemColor(CONTROL_SHADOW);
	public static final SystemColor controlText = new SystemColor(CONTROL_TEXT);
	public static final SystemColor desktop = new SystemColor(DESKTOP);
	public static final SystemColor inactiveCaption = new SystemColor(INACTIVE_CAPTION);
	public static final SystemColor inactiveCaptionBorder = new SystemColor(INACTIVE_CAPTION_BORDER);
	public static final SystemColor inactiveCaptionText = new SystemColor(INACTIVE_CAPTION_TEXT);
	public static final SystemColor info = new SystemColor(INFO);
	public static final SystemColor infoText = new SystemColor(INFO_TEXT);
	public static final SystemColor menu = new SystemColor(MENU);
	public static final SystemColor menuText = new SystemColor(MENU_TEXT);
	public static final SystemColor scrollbar = new SystemColor(SCROLLBAR);
	public static final SystemColor text = new SystemColor(TEXT);
	public static final SystemColor textHighlight = new SystemColor(TEXT_HIGHLIGHT);
	public static final SystemColor textHighlightText = new SystemColor(TEXT_HIGHLIGHT_TEXT);
	public static final SystemColor textInactiveText = new SystemColor(TEXT_INACTIVE_TEXT);
	public static final SystemColor textText = new SystemColor(TEXT_TEXT);
	public static final SystemColor window = new SystemColor(WINDOW);
	public static final SystemColor windowBorder = new SystemColor(WINDOW_BORDER);
	public static final SystemColor windowText = new SystemColor(WINDOW_TEXT);

	/**
	 * Constructs a new <code>SystemColor</code> with the specified
	 * index.
	 *
	 * @author      IBM
	 * @version 	initial
	 */
	SystemColor(int index) {
		super(0);
		this.value = index;
	}

	/**
	 * Answers the RGB value of the receiver in the default <code>ColorSpace</code>.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @return	an <code>int</code> which is the receiver's RGB value
	 */
	public int getRGB() {
		if (colorHandle == 0) {
			colorHandle = Util.getDisplay().getSystemColor(value);
		}

		int rgb = OS.Color_GetRed(colorHandle) << 16;
		rgb |= OS.Color_GetGreen(colorHandle) << 8;
		rgb |= OS.Color_GetBlue(colorHandle);
		rgb |= 0xff000000;

		return rgb;
	}

	/**
	 * Answers a string containing a concise, human-readable
	 * description of the receiver's relevant information.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @return	a java.lang.String which is a printable representation for the receiver
	 */
	public String toString() {
		return getClass().getName() + "[i=" + value + "]"; //$NON-NLS-1$ //$NON-NLS-2$
	}

	int getPeer() {
		// Override behavior from Color.
		// Avoid attempting to get the color through the colorCache.
		// SystemColors are not managed by the cache, but rather by Display.
		if (colorHandle == 0) {
			colorHandle = Util.getDisplay().getSystemColor(value);
		}

		return colorHandle;
	}

	void disposePeer() {
		// Override behavior from Color.
		// System colors should not be disposed even if finalized.
		// In general, finalization won't occur anyway. However, one exception
		// is if the SystemColor is serialized and then deserialized. If the
		// deserialized color is allowed to finalize, it will dispose the same
		// colorhandle being used by the real/original SystemColor.
		// Note that the SystemColors will explicitly be disposed by Display
		// when the display itself is disposed.
	}
}
