package java.awt;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.awt.datatransfer.Clipboard;
import com.ibm.ugl.eswt.OS;
import java.awt.event.AWTEventListener;
import java.awt.event.InputEvent;
import java.awt.image.*;
import java.net.URL;

import com.ibm.oti.awt.KeyStrings;

/**
 * The Toolkit class defines methods that you can invoke globally
 * and directly.  It is a collection of service methods for such things
 * as asking for characteristics of the windowing system, creating
 * images, reading system properties, and retrieving the AWT
 * event queue.  To retrieve the Toolkit instance for this implementation,
 * invoke the static method <CODE>getDefaultToolkit()</CODE>
 */
public abstract class Toolkit {

	AWTEventBroadcaster awtEventBroadcaster;
	private static Toolkit Default;

	static {
		ToolkitImpl.reference();
	}
	/**
	 * Used internally to provide the Current Event queue without
	 * passing all the permission checks!
	 */
	static EventQueue getCurrentSystemEventQueue() {
		return Toolkit.getDefaultToolkit().getSystemEventQueueImpl().getCurrentEventQueue();
	}
	public Toolkit() {
		super();
		this.awtEventBroadcaster = new AWTEventBroadcaster();
	}
	/**
	 * @param systemColors
	 */
	// TODO javadoc
	protected void loadSystemColors(int[] systemColors)
	throws HeadlessException
	{
		if (GraphicsEnvironment.isHeadless()) {
			throw new HeadlessException();
		}
		//Util.notImplementedYet("java/awt/Toolkit","loadSystemColors([I)V");
		// old implementation was empty, so this is too, for now
	}

	/**
	 * @return
	 */
	// TODO javadoc
	public abstract Dimension getScreenSize()
		throws HeadlessException
		;

	/**
	 * @return
	 */
	// TODO javadoc
	public abstract int getScreenResolution()
		throws HeadlessException
	;

	/**
	 * @return
	 */
	// TODO javadoc
	public abstract ColorModel getColorModel()
		throws HeadlessException
	;

	/**
	 * Answer the fonts available to applications on
	 * this system.
	 *
	 * @return String[] available fonts
	 * @deprecated
	 */
	public abstract String[] getFontList();

	/**
	 * Answer the font metrics for a given font
	 *
	 * @param font
	 * @return
	 * @deprecated
	 */
	public abstract FontMetrics getFontMetrics(Font font);

	/**
	 *
	 */
	// TODO javadoc
	public abstract void sync();

	/**
	 * @return
	 */
	// TODO javadoc
	public static synchronized Toolkit getDefaultToolkit() {
		if (Default == null)
			Default = new java.awt.ToolkitImpl();

		return Default;
	}

	/**
	 * @param fileName
	 * @return
	 */
	// TODO javadoc
	public abstract Image getImage(String fileName);

	/**
	 * Create an image from the specified URL.  Image is not loaded
	 * immediately with pixel data.  The image must be of JPEG, GIF, or
	 * PNG format.  Requests for images at the same URL answer the
	 * same Image instance.
	 *
	 * Returns <code>null</code> if the URL is not valid.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param url URL indicating image location
	 * @return Image image instance that will, at some point, contain pixel data
	 */
	public abstract Image getImage(URL url);

	/**
	 * @param image
	 * @param width
	 * @param height
	 * @param imageObserver
	 * @return
	 */
	// TODO javadoc
	public abstract boolean prepareImage(Image image, int width, int height, ImageObserver imageObserver);

	/**
	 * Answer the status of <code>image</code> as it is being loaded.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param image the image to be checked
	 * @param width the width of the area of the image being checked (-1 means the whole width)
	 * @param height the height of the area of the image being checked (-1 means the whole height)
	 * @param imageObserver the observer to notify as checking occurs
	 *
	 * @return the status of the image being built
	 */
	public abstract int checkImage(Image image, int width, int height, ImageObserver imageObserver);

	/**
	 * Creates an image based on the input byte array
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param data  the image data
	 *
	 * @return Image
	 */
	public Image createImage(byte data[]) {
		return createImage(data, 0, data.length);
	}

	/**
	 * @param data
	 * @param offset
	 * @param length
	 * @return
	 */
	// TODO javadoc
	public abstract Image createImage(byte data[], int offset, int length);

	/**
	 * @param imageProducer
	 * @return
	 */
	// TODO javadoc
	public abstract Image createImage(ImageProducer imageProducer);

	/**
	 * Creates a new image instance based on the image data found at the
	 * indicated file name.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param fileName the name of the image file from which image data is retrieved
	 * @return Image
	 */
	public abstract Image createImage(String fileName);

	/**
	 * Creates a new image instance based on the image data found at the
	 * indicated URL.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param url the URL from which the image data is retrieved
	 * @return Image
	 */
	public abstract Image createImage(URL url);

	/**
	 * Causes the system hardware to emit a short sound (if supported)
	 *
	 * @author      IBM
	 * @version		initial
	 */
	public abstract void beep();

	/**
	 * Answer the system clipboard.
	 *
	 * @author      IBM
	 * @version		initial
	 */
	public abstract Clipboard getSystemClipboard()
		throws HeadlessException
	;

	/**
	 * @return
	 */
	// TODO javadoc
	public int getMenuShortcutKeyMask()
		throws HeadlessException
	{
		if (GraphicsEnvironment.isHeadless()) {
			throw new HeadlessException();
		}
		return InputEvent.CTRL_MASK;
	}

	public static String getProperty(String name, String defaultValue) {
		String value = KeyStrings.getString(name);
		return (value == null) ? defaultValue : value;
	}

	/**
	 * @return
	 */
	// TODO javadoc
	public final EventQueue getSystemEventQueue() {
		final SecurityManager sm = System.getSecurityManager();
		if (sm != null) {
			sm.checkAwtEventQueueAccess();
		}
		return getSystemEventQueueImpl();
	}

	/**
	 * @return
	 */
	// TODO javadoc
	protected abstract EventQueue getSystemEventQueueImpl();

	/**
	 * @param awtEventListener
	 * @param eventMask
	 */
	// TODO javadoc
	public void addAWTEventListener(AWTEventListener awtEventListener, long eventMask) {
		final SecurityManager sm = System.getSecurityManager();
		if (sm != null) {
			sm.checkPermission(AWTPermission.PermissionToListenToAllAWTEvents);
		}
        synchronized (awtEventBroadcaster) {
        	awtEventBroadcaster.addAWTEventListener(awtEventListener, eventMask);
        }
	}

	/**
	 * @param awtEventListener
	 */
	// TODO javadoc
	public void removeAWTEventListener(AWTEventListener awtEventListener) {
		final SecurityManager sm = System.getSecurityManager();
		if (sm != null) {
			sm.checkPermission(AWTPermission.PermissionToListenToAllAWTEvents);
		}
        synchronized (awtEventBroadcaster) {
        	awtEventBroadcaster.removeAWTEventListener(awtEventListener);
        }
	}
	void dispatchEventToAWTEventListener(AWTEvent awtEvent) {
        synchronized (awtEventBroadcaster) {
        	awtEventBroadcaster.dispatchEvent(awtEvent);
        }
	}

	public Insets getScreenInsets(GraphicsConfiguration graphicsConfiguration) throws HeadlessException {
		if (GraphicsEnvironment.isHeadless()) {
			throw new HeadlessException();
		}
		if (graphicsConfiguration == null) {
			throw new NullPointerException();
		}
		return _getScreenInsets();
	}

	Insets _getScreenInsets() {
		return new Insets(0, 0, 0, 0);
	}

	public AWTEventListener[] getAWTEventListeners() {
		final SecurityManager sm = System.getSecurityManager();
		if (sm != null) {
			sm.checkPermission(AWTPermission.PermissionToListenToAllAWTEvents);
		}
        synchronized (awtEventBroadcaster) {
        	return awtEventBroadcaster.getAWTEventListeners();
        }
	}

	public AWTEventListener[] getAWTEventListeners(long eventMask) {
		final SecurityManager sm = System.getSecurityManager();
		if (sm != null) {
			sm.checkPermission(AWTPermission.PermissionToListenToAllAWTEvents);
		}
        synchronized (awtEventBroadcaster) {
        	return awtEventBroadcaster.getAWTEventListeners(eventMask);
        }
	}

	// Return true if the state indicated is supported
	// by this toolkit
	public boolean isFrameStateSupported(int state)
			throws HeadlessException {
		if (GraphicsEnvironment.isHeadless()) {
			throw new HeadlessException();
		}
		// TODO PPRO 1.1 need to handle the various states
		// use preprocessor to reduce allowed states
		// for PBPro.
		int capability = 0;
		switch (state) {
			case Frame.ICONIFIED:
				capability = OS.CAPABILITY_SHELL_MINMAX;
				break;
			default:
				return true;
		}
		return (OS.Platform_GetCapability(capability) != 0);
	}
}
