package java.awt.color;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.io.Serializable;

import com.ibm.oti.awt.ColorSpace_sRGB;

/**
 * Instances of this class define the color space of a given
 * Color, Image, BufferedImage, or GraphicsDevice.
 */
public abstract class ColorSpace implements Serializable {

	/**
	 * Any of the family of RGB color spaces.
	 */
	public static final int TYPE_RGB = 5;

	/**
	 * The sRGB color space
	 */
	public static final int CS_sRGB = 1000;

	private int fType;
	private int fNumComponents;

	/**
	 * Create a color space of the indicated type with the
	 * specified number of color components
	 */
	protected ColorSpace(int type, int numcomponents) {
		super();

		fType          = type;
		fNumComponents = numcomponents;
	}

	/**
	 * Answer the color space indicated by the argument.
	 * Currently only CS_sRGB is supported.
	 */
	public static ColorSpace getInstance(int colorspace) {
		if (colorspace != CS_sRGB) {
			throw new IllegalArgumentException();
		}

		return new ColorSpace_sRGB();
	}

	/**
	 * Answer whether this instance represents the sRGB color space
	 */
	public boolean isCS_sRGB() {
		return false;
	}

	/**
	 * Answer the type of this color space (e.g. TYPE_RGB, etc.)
	 */
	public int getType() {
		return fType;
	}

	/**
	 * Answer the number of components in the receiver
	 */
	public int getNumComponents() {
		return fNumComponents;
	}

	/**
	 * Answer the component name at the given index
	 */
	public String getName(int index) {
		throw new RuntimeException("subclass must override"); //$NON-NLS-1$
	}

}
