package java.awt.datatransfer;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This class implements a platform independent Clipboard that is used to transfer data.
 *
 * @see java.awt.datatransfer.Transferable
 * @see java.awt.Toolkit#getSystemClipboard
 */
public class Clipboard {

	// The name of the receiver
	String name;

	/**
	 * The contents of the receiver
	 */
	protected Transferable contents;

	/**
	 * The owner of the current contents of the receiver
	 */
	protected ClipboardOwner owner;

	/**
	 * Constructs a new <code>Clipboard</code> with the specified name.
	 *
	 * @param name the receiver's name
	 */
	public Clipboard(String name) {
		this.name = name;
	}

	/**
	 * Answers the current contents of the receiver.
	 *
	 * @return	a <code>Transferable</code> which is the current contents
	 *				of the receiver
	 */
	public synchronized Transferable getContents(Object o) {
		return contents;
	}

	/**
	 * Answers the name of the receiver.
	 *
	 * @return	a <code>String</code> which it the name of the receiver
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the contents of the receiver to the given <code>Transferable</code>.
	 * The specified <code>ClipboardOwner</code> will be notified
	 * if the contents of the receiver is altered later on.
	 *
	 * @param	contents	the contents for the receiver
	 * @param	owner		the owner for the receiver
	 */
	public synchronized void setContents(
		Transferable contents,
		ClipboardOwner owner) {

		if (this.owner != null && this.owner != owner) {
			this.owner.lostOwnership(this, this.contents);
		}

		this.contents = contents;
		this.owner = owner;
	}

}
