package java.awt.datatransfer;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.io.IOException;

/**
 * This interface should be implemented by all classes
 * which want to provide data to a clipboard or more
 * generic data transfer mechanism.
 *
 * @see java.awt.datatransfer.Clipboard#setContents(Transferable, ClipboardOwner)
 */
public interface Transferable {

	/**
	 * Answers the data transfered using the specified <code>DataFlavor</code>.
	 *
	 * @param	dataFlavor	the desired flavor to retrieve the data transfered
	 *
	 * @return	an <code>Object</code> which is the data transfered
	 *
	 * @see		DataFlavor
	 */
	public Object getTransferData(DataFlavor dataFlavor)
		throws UnsupportedFlavorException, IOException;

	/**
	 * Answers all the data flavors supported by the receiver.
	 *
	 * @return	a <code>DataFlavor</code> array containing all the data flavors
	 * 				support
	 *
	 * @see		DataFlavor
	 */
	public DataFlavor[] getTransferDataFlavors();

	/**
	 * Answers whether the specified <code>DataFlavor</code>
	 * is supported.
	 *
	 * @param	dataFlavor	the <code>DataFlavor</code> to be checked
	 *
	 * @return	<code>true</code> if the <code>DataFlavor</code> is supported,
	 *			<code>false</code> otherwise
	 *
	 * @see		DataFlavor
	 */
	public boolean isDataFlavorSupported(DataFlavor dataFlavor);

}
