package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.awt.AWTEvent;
import java.util.EventListener;

/**
 * This interface should be implemented by all classes
 * which wish to watch AWTEvent events.
 * <p>
 * When an <code>AWTEventListener</code> is added to a
 * <code>Component</code> or <code>MenuComponent</code>
 * using the <code>addAWTEventListener</code>
 * method, AWTEvent events occurring on that component will be
 * delivered to the AWTEvent listener via the sending of the
 * <code>eventDispatched</code> method.
 * <p>
 * <code>AWTEvents</code> are designed to be watched as opposed
 * to listening to them and taking action.  They are intended as
 * a means to allow events to be tracked and to support accessibility.
 */
public interface AWTEventListener extends EventListener {

	/**
	 * Called when an AWTEvent event occurs on the AWTEvent's
	 * source object.
	 *
	 * @param	awtEvent	the event that occured
	 *
	 * @see		<code>AWTEvent</code>
	 */
	public abstract void eventDispatched(AWTEvent event);

}
