package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.EventListener;

/**
 * This interface should be implemented by all classes
 * which wish to receive Adjustment events.
 * <p>
 * When an <code>AdjustmentListener</code> is added to a
 * <code>Component</code> using the <code>addAdjustmentListener</code>
 * method, Adjustment events occurring on that component will be
 * delivered to the adjustment listener via the sending of the
 * <code>adjustmentValueChanged</code> method.
 */
public interface AdjustmentListener extends EventListener {

	/**
	 * Called when an adjustment event occurs on the adjustmentEvent's
	 * source object.
	 *
	 * @author		OTI
	 * @version		initial
	 *
	 * @param	adjustmentEvent	the event that occured
	 *
	 * @see		<code>AdjustmentEvent</code>
	 */
	public abstract void adjustmentValueChanged(AdjustmentEvent adjustmentEvent);

}
