package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This class is a convenience class written to as a
 * <code>ComponentListener</code> with empty implementations
 * for all callback methods.
 * <p>
 * The purpose of this class is to allow subclasses to implement
 * only those methods that they are interested in listening to
 * without having to re-implement empty versions of the remaining
 * callback methods.  ComponentListeners must normally implement
 * the following methods:
 * 	- <code>componentHidden</code>
 * 	- <code>componentMoved</code>
 * 	- <code>componentResized</code>
 * 	- <code>componentShown</code>
 * <p>
 *
 * @see	<code>ComponentListener</code>
 */
public abstract class ComponentAdapter implements ComponentListener {

	/**
	 * Constructs a new <code>ComponentAdapter</code>.
	 *
	 */
	public ComponentAdapter() {
	}

	/**
	 * Called when the componentEvent's source object is hidden.
	 *
	 * @param	componentEvent	the event that occured
	 *
	 * @see		<code>ComponentEvent</code>
	 */
	public void componentHidden(ComponentEvent componentEvent) {
	}

	/**
	 * Called when the componentEvent's source object is moved.
	 *
	 * @param	componentEvent	the event that occured
	 *
	 * @see		<code>ComponentEvent</code>
	 */
	public void componentMoved(ComponentEvent componentEvent) {
	}

	/**
	 * Called when the componentEvent's source object is resized.
	 *
	 * @param	componentEvent	the event that occured
	 *
	 * @see		<code>ComponentEvent</code>
	 */
	public void componentResized(ComponentEvent componentEvent) {
	}

	/**
	 * Called when the componentEvent's source object is shown.
	 *
	 * @param	componentEvent	the event that occured
	 *
	 * @see		<code>ComponentEvent</code>
	 */
	public void componentShown(ComponentEvent componentEvent) {
	}

}
