package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.EventListener;

/**
 * This interface should be implemented by all classes
 * which wish to receive component events.  Another way to
 * listen to <code>ComponentEvents</code> is to extend the
 * <code>ComponentAdapter</code> class which implements this
 * interface, and override the methods of interest.
 * <p>
 * When a <code>ComponentListener</code> is added to a
 * <code>Component</code> using the <code>addComponentListener</code>
 * method, component events occurring on that component will be
 * delivered to the component listener via the sending of one of
 * the following methods:
 * 	- <code>componentHidden</code>
 * 	- <code>componentMoved</code>
 * 	- <code>componentResized</code>
 * 	- <code>componentShown</code>
 * <p>
 * Component events will be handled by AWT regardless of the existence of
 * listeners, they are used to notify listeners of what is happening.
 */
public interface ComponentListener extends EventListener {

	/**
	 * Called when the componentEvent's source object is hidden.
	 *
	 * @param	componentEvent	the event that occured
	 *
	 * @see		<code>ComponentEvent</code>
	 */
	public void componentHidden(ComponentEvent componentEvent);

	/**
	 * Called when the componentEvent's source object is moved.
	 *
	 * @param	componentEvent	the event that occured
	 *
	 * @see		<code>ComponentEvent</code>
	 */
	public void componentMoved(ComponentEvent componentEvent);

	/**
	 * Called when the componentEvent's source object is resized.
	 *
	 * @param	componentEvent	the event that occured
	 *
	 * @see		<code>ComponentEvent</code>
	 */
	public void componentResized(ComponentEvent componentEvent);

	/**
	 * Called when the componentEvent's source object is shown.
	 *
	 * @param	componentEvent	the event that occured
	 *
	 * @see		<code>ComponentEvent</code>
	 */
	public void componentShown(ComponentEvent componentEvent);

}
