package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This class is a convenience class written to as a
 * <code>ContainerListener</code> with empty implementations
 * for all callback methods.
 * <p>
 * The purpose of this class is to allow subclasses to implement
 * only those methods that they are interested in listening to
 * without having to re-implement empty versions of the remaining
 * callback methods.  ContainerListeners must normally implement
 * the following methods:
 * 	- <code>componentAdded</code>
 * 	- <code>componentRemoved</code>
 * <p>
 *
 * @see	<code>ContainerListener</code>
 */
public abstract class ContainerAdapter implements ContainerListener {

	/**
	 * Constructs a new <code>ContainerAdapter</code>.
	 *
	 */
	public ContainerAdapter() {
	}

	/**
	 * Called when the containerEvent's source object is added
	 * to its container.
	 *
	 * @param	containerEvent	the event that occurred
	 *
	 * @see		<code>ContainerEvent</code>
	 */
	public void componentAdded(ContainerEvent containerEvent) {
	}

	/**
	 * Called when the containerEvent's source object is removed
	 * from its container.
	 *
	 * @param	containerEvent	the event that occurred
	 *
	 * @see		<code>ContainerEvent</code>
	 */
	public void componentRemoved(ContainerEvent containerEvent) {
	}

}
