package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.EventListener;

/**
 * This interface should be implemented by all classes
 * which wish to receive container events.  Another way to
 * listen to <code>ContainerEvents</code> is to extend the
 * <code>ContainerAdapter</code> class which implements this
 * interface, and override the methods of interest.
 * <p>
 * When a <code>ContainerListener</code> is added to a
 * <code>Container</code> using the <code>addContainerListener</code>
 * method, container events occurring on that container will be
 * delivered to the container listener via the sending of one of
 * the following methods:
 * 	- <code>componentAdded</code>
 * 	- <code>componentRemoved</code>
 * <p>
 * Container events will be handled by AWT regardless of the existence of
 * listeners, they are used to notify listeners of what is happening.
 */
public interface ContainerListener extends EventListener {

	/**
	 * Called when the containerEvent's source object is added
	 * to its container.
	 *
	 * @param	containerEvent	the event that occurred
	 *
	 * @see		<code>ContainerEvent</code>
	 */
	public void componentAdded(ContainerEvent containerEvent);

	/**
	 * Called when the containerEvent's source object is removed
	 * from its container.
	 *
	 * @param	containerEvent	the event that occurred
	 *
	 * @see		<code>ContainerEvent</code>
	 */
	public void componentRemoved(ContainerEvent containerEvent);

}
