package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.EventListener;

/**
 * This interface should be implemented by all classes
 * which wish to receive focus events.  Another way to
 * listen to <code>FocusEvents</code> is to extend the
 * <code>FocusAdapter</code> class which implements this
 * interface, and override the methods of interest.
 * <p>
 * When a <code>FocusListener</code> is added to a
 * <code>Component</code> using the <code>addFocusListener</code>
 * method, focus events occurring on that component will be
 * delivered to the focus listener via the sending of one of
 * the following methods:
 * 	- <code>focusGained</code>
 * 	- <code>focusLost</code>
 */
public interface FocusListener extends EventListener {

	/**
	 * Called when the focusEvent's source object gains keyboard focus.
	 *
	 * @param	focusEvent	the event that occured
	 *
	 * @see		<code>FocusEvent</code>
	 */
	public void focusGained(FocusEvent focusEvent);

	/**
	 * Called when the focusEvent's source object loses keyboard focus.
	 *
	 * @param	focusEvent	the event that occured
	 *
	 * @see		<code>FocusEvent</code>
	 */
	public void focusLost(FocusEvent focusEvent);

}
