package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This class is a convenience class written to as a
 * <code>KeyListener</code> with empty implementations
 * for all callback methods.
 * <p>
 * The purpose of this class is to allow subclasses to implement
 * only those methods that they are interested in listening to
 * without having to re-implement empty versions of the remaining
 * callback methods.  KeyListeners must normally implement
 * the following methods:
 * 	- <code>keyPressed</code>
 * 	- <code>keyReleased</code>
 * 	- <code>keyTyped</code>
 * <p>
 *
 * @see	<code>KeyListener</code>
 */

public abstract class KeyAdapter implements KeyListener {

	/**
	 * Constructs a new <code>KeyAdapter</code>.
	 */
	public KeyAdapter() {
	}

	/**
	 * Called when a key has been pressed for the receiver.
	 *
	 * @param	keyEvent	the event that occured
	 *
	 * @see		<code>KeyEvent</code>
	 */
	public void keyPressed(KeyEvent keyEvent) {
	}

	/**
	 * Called when a key has been released for the receiver.
	 *
	 * @param	keyEvent	the event that occured
	 *
	 * @see		<code>KeyEvent</code>
	 */
	public void keyReleased(KeyEvent keyEvent) {
	}

	/**
	 * Called when a key has been typed for the receiver.
	 * Key typed events represent the action of both pressing
	 * and releasing the key.
	 *
	 * @param	keyEvent	the event that occured
	 *
	 * @see		<code>KeyEvent</code>
	 */
	public void keyTyped(KeyEvent keyEvent) {
	}

}
