package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.EventListener;

/**
 * This interface should be implemented by all classes
 * which wish to receive key events.  Another way to
 * listen to <code>KeyEvents</code> is to extend the
 * <code>KeyAdapter</code> class which implements this
 * interface, and override the methods of interest.
 * <p>
 * When a <code>KeyListener</code> is added to a
 * <code>Component</code> using the <code>addKeyListener</code>
 * method, key events occurring on that component will be
 * delivered to the key listener via the sending of one of
 * the following methods:
 * 	- <code>keyPressed</code>
 * 	- <code>keyReleased</code>
 * 	- <code>keyTyped</code>
 */

public interface KeyListener extends EventListener {

	/**
	 * Called when a key has been pressed for the receiver.
	 *
	 * @param	keyEvent	the event that occured
	 *
	 * @see		<code>KeyEvent</code>
	 */
	public void keyPressed(KeyEvent keyEvent);

	/**
	 * Called when a key has been released for the receiver.
	 *
	 * @param	keyEvent	the event that occured
	 *
	 * @see		<code>KeyEvent</code>
	 */
	public void keyReleased(KeyEvent keyEvent);

	/**
	 * Called when a key has been typed for the receiver.
	 * Key typed events represent the action of both pressing
	 * and releasing the key.
	 *
	 * @param	keyEvent	the event that occured
	 *
	 * @see		<code>KeyEvent</code>
	 */
	public void keyTyped(KeyEvent keyEvent);

}
