package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This class is a convenience class written to as a
 * <code>MouseListener</code> with empty implementations
 * for all callback methods.
 * <p>
 * The purpose of this class is to allow subclasses to implement
 * only those methods that they are interested in listening to
 * without having to re-implement empty versions of the remaining
 * callback methods.  MouseListeners must normally implement
 * the following methods:
 * 	- <code>mouseClicked</code>
 * 	- <code>mouseEntered</code>
 * 	- <code>mouseExited</code>
 * 	- <code>mousePressed</code>
 * 	- <code>mouseReleased</code>
 * <p>
 *
 * @see	<code>MouseListener</code>
 */
public abstract class MouseAdapter implements MouseListener {

	/**
	 * Constructs a new <code>MouseAdapter</code>.
	 */
	public MouseAdapter() {
	}

	/**
	 * Called when a mouse click occurs on the receiver.
	 * This happens when a mouse button is pressed and released.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseClicked(MouseEvent mouseEvent) {
	}

	/**
	 * Called when mouse pointer enters the receiver.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseEntered(MouseEvent mouseEvent) {
	}

	/**
	 * Called when mouse pointer exits the receiver.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseExited(MouseEvent mouseEvent) {
	}

	/**
	 * Called when a mouse button is pressed on the receiver.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mousePressed(MouseEvent mouseEvent) {
	}

	/**
	 * Called when a mouse button is released on the receiver.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseReleased(MouseEvent mouseEvent) {
	}

}
