package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.EventListener;

/**
 * This interface should be implemented by all classes
 * which wish to receive mouse events.  Another way to
 * listen to <code>MouseEvents</code> is to extend the
 * <code>MouseAdapter</code> class which implements this
 * interface, and override the methods of interest.
 * <p>
 * When a <code>MouseListener</code> is added to a
 * <code>Component</code> using the <code>addMouseListener</code>
 * method, mouse events occurring on that component will be
 * delivered to the mouse listener via the sending of one of
 * the following methods:
 * 	- <code>mouseClicked</code>
 * 	- <code>mouseEntered</code>
 * 	- <code>mouseExited</code>
 * 	- <code>mousePressed</code>
 * 	- <code>mouseReleased</code>
 */
public interface MouseListener extends EventListener {

	/**
	 * Called when a mouse click occurs on the receiver.
	 * This happens when a mouse button is pressed and released.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseClicked(MouseEvent mouseEvent);

	/**
	 * Called when mouse pointer enters the receiver.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseEntered(MouseEvent mouseEvent);

	/**
	 * Called when mouse pointer exits the receiver.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseExited(MouseEvent mouseEvent);

	/**
	 * Called when a mouse button is pressed on the receiver.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mousePressed(MouseEvent mouseEvent);

	/**
	 * Called when a mouse button is released on the receiver.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseReleased(MouseEvent mouseEvent);
}
