package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This class is a convenience class written to as a
 * <code>MouseMotionListener</code> with empty implementations
 * for all callback methods.
 * <p>
 * The purpose of this class is to allow subclasses to implement
 * only those methods that they are interested in listening to
 * without having to re-implement empty versions of the remaining
 * callback methods.  MouseMotionListeners must normally implement
 * the following methods:
 * 	- <code>mouseClicked</code>
 * 	- <code>mouseEntered</code>
 * 	- <code>mouseExited</code>
 * 	- <code>mousePressed</code>
 * 	- <code>mouseReleased</code>
 * <p>
 *
 * @see	<code>MouseMotionListener</code>
 */
public abstract class MouseMotionAdapter implements MouseMotionListener {

	/**
	 * Constructs a new <code>MouseMotionAdapter</code>.
	 */
	public MouseMotionAdapter() {
	}

	/**
	 * Called when a mouse drag motion occurs for the receiver.
	 * This happens when a mouse button is pressed and then the
	 * mouse is moved while the button is still pressed.
	 * These events are continually posted to a listener until the
	 * mouse button is released, indicating that dragging is done.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseDragged(MouseEvent mouseEvent) {
	}

	/**
	 * Called when mouse motion occurs on the receiver.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public void mouseMoved(MouseEvent mouseEvent) {
	}

}
