package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.EventListener;

/**
 * This interface should be implemented by all classes
 * which wish to receive mouse motion events.  Another way to
 * listen to <code>MouseEvents</code> is to extend the
 * <code>MouseMotionAdapter</code> class which implements this
 * interface, and override the methods of interest.
 * <p>
 * When a <code>MouseMotionListener</code> is added to a
 * <code>Component</code> using the <code>addMouseMotionListener</code>
 * method, mouse motion events occurring on that component will be
 * delivered to the mouse motion listener via the sending of one of
 * the following methods:
 * 	- <code>mouseDragged</code>
 * 	- <code>mouseMoved</code>
 */
public interface MouseMotionListener extends EventListener {

	/**
	 * Called when a mouse drag motion occurs for the receiver.
	 * This happens when a mouse button is pressed and then the
	 * mouse is moved while the button is still pressed.
	 * These events are continually posted to a listener until the
	 * mouse button is released, indicating that dragging is done.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public abstract void mouseDragged(MouseEvent mouseEvent);

	/**
	 * Called when mouse motion occurs on the receiver.
	 *
	 * @param	mouseEvent	the event that occured
	 *
	 * @see		<code>MouseEvent</code>
	 */
	public abstract void mouseMoved(MouseEvent mouseEvent);

}
