package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.EventListener;

/**
 * This interface should be implemented by all classes
 * which wish to receive text events.
 * <p>
 * When an <code>TextListener</code> is added to a
 * <code>Component</code> using the <code>addTextListener</code>
 * method, text events occurring on that component will be
 * delivered to the text listener via the sending of the
 * <code>textValueChanged</code> method.
 */
public interface TextListener extends EventListener {

	/**
	 * Called when a text event occurs on the textEvent's
	 * source object.  This indicates that a component's
	 * text has changed.
	 *
	 * @param	textEvent	the event that occured
	 *
	 * @see		<code>TextEvent</code>
	 */
	public abstract void textValueChanged(TextEvent textEvent);

}
