package java.awt.font;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2004, 2005  All Rights Reserved
 */

import java.text.AttributedCharacterIterator.Attribute;
import java.io.InvalidObjectException;

public final class TextAttribute extends Attribute {

    static final long serialVersionUID = 7744112784117861702l;

	public static final TextAttribute FAMILY = new TextAttribute("family");
	public static final TextAttribute WEIGHT = new TextAttribute("weight");
	public static final Float WEIGHT_EXTRA_LIGHT = new Float(0.5f);
	public static final Float WEIGHT_LIGHT = new Float(0.75f);
	public static final Float WEIGHT_DEMILIGHT = new Float(0.875f);
	public static final Float WEIGHT_REGULAR = new Float(1.0f);
	public static final Float WEIGHT_SEMIBOLD = new Float(1.25f);
	public static final Float WEIGHT_MEDIUM = new Float(1.5f);
	public static final Float WEIGHT_DEMIBOLD = new Float(1.75f);
	public static final Float WEIGHT_BOLD = new Float(2.0f);
	public static final Float WEIGHT_HEAVY = new Float(2.25f);
	public static final Float WEIGHT_EXTRABOLD = new Float(2.5f);
	public static final Float WEIGHT_ULTRABOLD = new Float(2.75f);
	public static final TextAttribute POSTURE = new TextAttribute("posture");
	public static final Float POSTURE_REGULAR = new Float(0.0);
	public static final Float POSTURE_OBLIQUE = new Float(0.2);
	public static final TextAttribute SIZE = new TextAttribute("size");
	public static final TextAttribute FONT = new TextAttribute("font");
	public static final TextAttribute FOREGROUND = new TextAttribute("foreground");
	public static final TextAttribute UNDERLINE = new TextAttribute("underline");
	public static final Integer UNDERLINE_ON = new Integer(0);
	public static final TextAttribute STRIKETHROUGH = new TextAttribute("strikethrough");
	public static final Boolean STRIKETHROUGH_ON = new Boolean(true);
	public static final TextAttribute INPUT_METHOD_HIGHLIGHT = new TextAttribute("input method highlight");

	private TextAttribute(String name) {
	    super(name);
	}

	protected Object readResolve() throws InvalidObjectException {
		if (this.getClass() != TextAttribute.class)
			throw new InvalidObjectException("invalid class for text attribute");
		if (this.getName().equals(FAMILY.getName()))
			return FAMILY;
		if (this.getName().equals(WEIGHT.getName()))
			return WEIGHT;
		if (this.getName().equals(POSTURE.getName()))
			return POSTURE;
		if (this.getName().equals(SIZE.getName()))
			return SIZE;
		if (this.getName().equals(FONT.getName()))
			return FONT;
		if (this.getName().equals(FOREGROUND.getName()))
			return FOREGROUND;
		if (this.getName().equals(UNDERLINE.getName()))
			return UNDERLINE;
		if (this.getName().equals(STRIKETHROUGH.getName()))
			return STRIKETHROUGH;
		if (this.getName().equals(INPUT_METHOD_HIGHLIGHT.getName()))
			return INPUT_METHOD_HIGHLIGHT;
		throw new InvalidObjectException("unrecognized text attribute");
	}

}
