package java.awt.font;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2004, 2005  All Rights Reserved
 */

public final class TextHitInfo {

	int charIndex;
    boolean leadingEdge;

    TextHitInfo(int charIndex, boolean leadingEdge) {
        this.charIndex = charIndex;
        this.leadingEdge = leadingEdge;
    }

    public int getCharIndex() {
		return charIndex;
	}

	public boolean isLeadingEdge() {
		return leadingEdge;
	}

	public int getInsertionIndex() {
		return isLeadingEdge() ? getCharIndex() : (getCharIndex() + 1);
	}

	public int hashCode() {
		return charIndex;
	}

	public boolean equals(Object object) {
		if (object == this)
			return true;
		if (!(object instanceof TextHitInfo))
			return false;
		TextHitInfo info = (TextHitInfo) object;
		return (charIndex == info.charIndex &&
				leadingEdge == info.leadingEdge);
	}

	public String toString() {
		return "TextHitInfo[" + charIndex + (leadingEdge ? 'L' : 'T') + ']';  //$NON-NLS-1$ //$NON-NLS-2$
	}

	public static TextHitInfo leading(int charIndex) {
		return new TextHitInfo(charIndex, true);
	}

	public static TextHitInfo trailing(int charIndex) {
		return new TextHitInfo(charIndex, false);
	}

	public static TextHitInfo beforeOffset(int offset) {
		return trailing(offset-1);
	}

	public static TextHitInfo afterOffset(int offset) {
		return leading(offset);
	}

	public TextHitInfo getOtherHit() {
	    // getOtherHit for trailing(n) returns leading(n+1) and getOtherHit for leading(n) returns trailing(n-1)
		return isLeadingEdge() ? trailing(charIndex - 1) : leading(charIndex + 1);
	}

	public TextHitInfo getOffsetHit(int delta) {
		return new TextHitInfo(getCharIndex() + delta, isLeadingEdge());
	}

}
