package java.awt.image;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.awt.Rectangle;
import java.util.Hashtable;

//------------------------------------------------------------
/**
 *
 */
public class CropImageFilter extends ImageFilter {

	Rectangle cropRect;

	//------------------------------------------------------------
	/**
	 *
	 */
	public CropImageFilter(int x, int y, int width, int height) {
		this.cropRect = new Rectangle(x, y, width, height);
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public void setProperties(Hashtable properties) {
		setProperty(properties, "croprect", cropRect); //$NON-NLS-1$
		super.setProperties(properties);
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public void setDimensions(int width, int height) {
		super.setDimensions(cropRect.width, cropRect.height);
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public void setPixels(int x, int y, int width, int height, ColorModel colorModel, byte[] pixels, int offset, int scansize) {
		Rectangle inters = cropRect.intersection(new Rectangle(x, y, width, height));

		if (!inters.isEmpty()) {
			super.setPixels(
				inters.x - cropRect.x, inters.y - cropRect.y,
				inters.width, inters.height,
				colorModel, pixels,
				(inters.y - y) * scansize + (inters.x - x) + offset,
				scansize);
		}
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public void setPixels(int x, int y, int width, int height, ColorModel colorModel, int[] pixels, int offset, int scansize) {
		Rectangle inters = cropRect.intersection(new Rectangle(x, y, width, height));

		if (!inters.isEmpty()) {
			super.setPixels(
				inters.x - cropRect.x, inters.y - cropRect.y,
				inters.width, inters.height,
				colorModel, pixels,
				(inters.y - y) * scansize + (inters.x - x) + offset,
				scansize);
		}
	}

}
