package java.awt.image;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.Hashtable;

//------------------------------------------------------------
/**
 *
 */
public class FilteredImageSource implements ImageProducer {

	ImageProducer imageProducer;
	ImageFilter imageFilter;
	Hashtable consumers;

	//------------------------------------------------------------
	/**
	 *
	 */
	public FilteredImageSource(ImageProducer imageProducer, ImageFilter imageFilter) {
		this.imageProducer = imageProducer;
		this.imageFilter = imageFilter;
		this.consumers = new Hashtable();
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public synchronized void addConsumer(ImageConsumer imageConsumer) {
		if (isConsumer(imageConsumer))
			return;

		ImageFilter imageFilter = this.imageFilter.getFilterInstance(imageConsumer);
		imageProducer.addConsumer(imageFilter);
		consumers.put(imageConsumer, imageFilter);
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public synchronized boolean isConsumer(ImageConsumer imageConsumer) {
		return consumers.containsKey(imageConsumer);
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public synchronized void removeConsumer(ImageConsumer imageConsumer) {
		ImageFilter imageFilter = (ImageFilter)consumers.remove(imageConsumer);
		if (imageFilter == null)
			return;

		imageProducer.removeConsumer(imageFilter);
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public void startProduction(ImageConsumer imageConsumer) {
		addConsumer(imageConsumer);

		imageProducer.startProduction((ImageFilter)consumers.get(imageConsumer));
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public void requestTopDownLeftRightResend(ImageConsumer imageConsumer) {
		ImageFilter imageFilter = (ImageFilter)consumers.get(imageConsumer);
		if (imageFilter == null)
			return;

		imageFilter.resendTopDownLeftRight(imageProducer);
	}

}
