package java.awt.image;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.Hashtable;

/**
 * The interface for objects wanting to receive
 * image data from an ImageProducer.
 */
public interface ImageConsumer {

	/**
	 * Status states.
	 */
	public static final int IMAGEERROR = 1;
	public static final int SINGLEFRAMEDONE = 2;
	public static final int STATICIMAGEDONE = 3;
	public static final int IMAGEABORTED = 4;

	/**
	 * Hints describing how the pixels will be delivered.
	 */
	public static final int RANDOMPIXELORDER = 1;
	public static final int TOPDOWNLEFTRIGHT = 2;
	public static final int COMPLETESCANLINES = 4;
	public static final int SINGLEPASS = 8;
	public static final int SINGLEFRAME = 16;

	/**
	 * Specifies the dimensions of the image data that
	 * will be provided by the ImageProducer.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param 		width		the width of the image
	 * @param 		height		the height of the image
	 */
	public void setDimensions(int width, int height);

	/**
	 * Sets the list of properties associated with the image.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param		properties		list of properties for the image
	 */
	public void setProperties(Hashtable properties);

	/**
	 * Sets the color model used for most of the pixels
	 * provided by the ImageProducer.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param		colorModel		color model used by pixels being set
	 */
	public void setColorModel(ColorModel colorModel);

	/**
	 * Called by the ImageProducer, this method provides hints as to
	 * how the image data will be delivered.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param		hints		hint indicating how the pixels are to be delivered
	 */
	public void setHints(int hints);

	/**
	 * This method is called by the ImageProducer to deliver a range
	 * of pixels for the image being constructed.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param 		x			the x coordinate of the rectangle of pixels
	 * @param 		y			the y coordinate of the rectangle of pixels
	 * @param 		width		the width of the rectangle of pixels
	 * @param 		height		the height of the rectangle of pixels
	 * @param 		colorModel	color model to be used for converting pixels
	 * @param 		pixels		range of pixels being delivered
	 * @param 		offset		starting location within pixel array
	 * @param		scansize	the scanline size in the image
	 */
	public void setPixels(int x, int y, int width, int height, ColorModel colorModel, byte[] pixels, int offset, int scansize);

	/**
	 * This method is called by the ImageProducer to deliver a range
	 * of pixels for the image being constructed.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param 		x			the x coordinate of the rectangle of pixels
	 * @param 		y			the y coordinate of the rectangle of pixels
	 * @param 		width		the width of the rectangle of pixels
	 * @param 		height		the height of the rectangle of pixels
	 * @param 		colorModel	color model to be used for converting pixels
	 * @param 		pixels		range of pixels being delivered
	 * @param 		offset		starting location within pixel array
	 * @param		scansize	the scanline size in the image
	 */
	public void setPixels(int x, int y, int width, int height, ColorModel colorModel, int[] pixels, int offset, int scansize);

	/**
	 * This method is called by the ImageProducer when all data for an
	 * image has been delivered, or when an error has occurred in loading
	 * or producing the image.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param status status of pixel delivery
	 */
	public void imageComplete(int status);
}
