package java.awt.image;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.Hashtable;

/**
 *
 */
public class ImageFilter implements ImageConsumer, Cloneable {

	protected ImageConsumer consumer;

	/**
	 *
	 */
	public ImageFilter() {
	}

	/**
	 *
	 */
	public ImageFilter getFilterInstance(ImageConsumer imageConsumer) {
		consumer= imageConsumer;
		return (ImageFilter)clone();
	}

	/**
	 *
	 */
	public void setDimensions(int width, int height) {
		consumer.setDimensions(width, height);
	}

	/**
	 *
	 */
	public void setProperties(Hashtable properties) {
		setProperty(properties, "filters", this); //$NON-NLS-1$
		consumer.setProperties(properties);
	}

	void setProperty(Hashtable properties, Object key, Object value) {
		Object currentValue = properties.get(key);
		properties.put(key, currentValue == null ? value : currentValue + " " + value); //$NON-NLS-1$
	}

	/**
	 *
	 */
	public void setColorModel(ColorModel colorModel) {
		consumer.setColorModel(colorModel);
	}

	/**
	 *
	 */
	public void setHints(int hints) {
		consumer.setHints(hints);
	}

	/**
	 *
	 */
	public void setPixels(int x, int y, int width, int height, ColorModel colorModel, byte[] pixels, int offset, int scansize) {
		consumer.setPixels(x, y, width, height, colorModel, pixels, offset, scansize);
	}

	/**
	 *
	 */
	public void setPixels(int x, int y, int width, int height, ColorModel colorModel, int[] pixels, int offset, int scansize) {
		consumer.setPixels(x, y, width, height, colorModel, pixels, offset, scansize);
	}

	/**
	 *
	 */
	public void imageComplete(int status) {
		consumer.imageComplete(status);
	}

	/**
	 *
	 */
	public void resendTopDownLeftRight(ImageProducer imageProducer) {
		imageProducer.requestTopDownLeftRightResend(this);
	}

	/**
	 * Answers a new instance of the same class as the receiver,
	 * whose slots have been filled in with the values in the
	 * slots of the receiver.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @return	a ImageFilter which is a shallow copy of this object
	 *
	 * @throws	OutOfMemoryError	if there is not enough memory to allocate the clone
	 *
	 * @see		java.lang.Cloneable
	 */
	public Object clone() {
		try {
			return super.clone();
		} catch (CloneNotSupportedException e) {
			return null;
		}
	}
}
