package java.awt.image;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * ImageProducer defines the interface for producing the image data for Images.
 * Every image has a producer that can recreate the image when requested.
 */
public interface ImageProducer {

	/**
	 * Register an ImageConsumer which is to receive image data from the producer.
	 * Delivery may begin immediately or when startProduction is called.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param 		imageConsumer	consumer to which pixels are to be delivered
	 */
	public void addConsumer(ImageConsumer imageConsumer);

	/**
	 * Indicates whether the specified ImageConsumer is currently
	 * registered with the ImageProducer.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param 		imageConsumer	consumer to which pixels would be delivered
	 *
	 * @return		true if the imageConsumer is currently registered to receive pixels
	 */
	public boolean isConsumer(ImageConsumer imageConsumer);

	/**
	 * Unregister the specified ImageConsumer from this ImageProducer.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param 		imageConsumer	consumer to which pixels are no longer to be delivered
	 */
	public void removeConsumer(ImageConsumer imageConsumer);

	/**
	 * Register the specified ImageConsumer, begin immediate reconstruction
	 * of the image data and delivery that data to all registered consumers.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param 		imageConsumer	consumer to which pixels are to be delivered
	 */
	public void startProduction(ImageConsumer imageConsumer);

	/**
	 * Request this ImageProducer resend the image data in top-down, left-right
	 * order to all registered consumers.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param 		imageConsumer	consumer to which pixels are to be delivered
	 */
	public void requestTopDownLeftRightResend(ImageConsumer imageConsumer);
}
