package java.awt.image;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

//------------------------------------------------------------
/**
 *
 */
public abstract class RGBImageFilter extends ImageFilter {

	//------------------------------------------------------------
	/**
	 *
	 */
	protected ColorModel origmodel;

	//------------------------------------------------------------
	/**
	 *
	 */
	protected ColorModel newmodel;

	//------------------------------------------------------------
	/**
	 *
	 */
	protected boolean canFilterIndexColorModel;

	//------------------------------------------------------------
	/**
	 *
	 */
	public RGBImageFilter() {
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public void setColorModel(ColorModel colorModel) {
		ColorModel cm;

		if (canFilterIndexColorModel && colorModel instanceof IndexColorModel) {
			IndexColorModel newCM;

			newCM = filterIndexColorModel((IndexColorModel)colorModel);
			substituteColorModel(colorModel, newCM);
			cm    = newCM;
		} else {
			cm = ColorModel.getRGBdefault();
		}
		super.setColorModel(cm);
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public void substituteColorModel(ColorModel oldCM, ColorModel newCM) {
		this.origmodel = oldCM;
		this.newmodel  = newCM;
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public IndexColorModel filterIndexColorModel(IndexColorModel colorModel) {
		int    size = colorModel.getMapSize();
		byte[] rgbs = new byte[size*4];

		for (int i=0; i<size; i++) {
			int rgb    = filterRGB(-1, -1, colorModel.getRGB(i));
			int offset = i * 4;

			rgbs[offset + 0] = (byte)((rgb >> 16) & 0xff);
			rgbs[offset + 1] = (byte)((rgb >>  8) & 0xff);
			rgbs[offset + 2] = (byte)((rgb >>  0) & 0xff);
			rgbs[offset + 3] = (byte)((rgb >> 24) & 0xff);
		}

		return new IndexColorModel(colorModel.getPixelSize(), size, rgbs, 0, true, colorModel.getTransparentPixel());
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public void filterRGBPixels(int x, int y, int width, int height, int[] pixels, int offset, int scansize) {
		int i    = offset;
		int xend = x + width;
		int yend = y + height;

		for (int cy = y; cy < yend; cy++) {
			for (int cx = x; cx < xend; cx++) {
				pixels[i] = filterRGB(cx, cy, pixels[i]);
				i++;
			}
			i += scansize - width;
		}

		super.setPixels(x, y, width, height, ColorModel.getRGBdefault(), pixels, offset, scansize);
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public void setPixels(int x, int y, int width, int height, ColorModel colorModel, byte[] pixels, int offset, int scansize) {
		if (colorModel == origmodel && newmodel != null) {
			super.setPixels(x, y, width, height, newmodel, pixels, offset, scansize);
			return;
		}

		int[] tempPixels = new int[pixels.length];
		int   i          = offset;
		int   xend       = x + width;
		int   yend       = y + height;

		for (int cy = y; cy < yend; cy++) {
			for (int cx = x; cx < xend; cx++) {
				tempPixels[i] = colorModel.getRGB(pixels[i]);
				i++;
			}
			i += scansize - width;
		}

		filterRGBPixels(x, y, width, height, tempPixels, offset, scansize);
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public void setPixels(int x, int y, int width, int height, ColorModel colorModel, int[] pixels, int offset, int scansize) {
		if (colorModel == origmodel && newmodel != null) {
			super.setPixels(x, y, width, height, newmodel, pixels, offset, scansize);
			return;
		}

		int[] tempPixels = new int[pixels.length];
		int   i          = offset;
		int   xend       = x + width;
		int   yend       = y + height;

		for (int cy = y; cy < yend; cy++) {
			for (int cx = x; cx < xend; cx++) {
				tempPixels[i] = colorModel.getRGB(pixels[i]);
				i++;
			}
			i += scansize - width;
		}

		filterRGBPixels(x, y, width, height, tempPixels, offset, scansize);
	}

	//------------------------------------------------------------
	/**
	 *
	 */
	public abstract int filterRGB(int x, int y, int rgb);

}
