package java.awt.image;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.Hashtable;

/**
 *
 */
public class ReplicateScaleFilter extends ImageFilter {

	protected int destWidth;
	protected int destHeight;
	protected int srcWidth;
	protected int srcHeight;
	protected Object outpixbuf;
	protected int[] srccols;
	protected int[] srcrows;

	/**
	 *
	 */
	public ReplicateScaleFilter(int width, int height) {
		if (width == 0 || height == 0) throw new IllegalArgumentException();

		this.destWidth = width;
		this.destHeight = height;
	}

	/**
	 *
	 */
	public void setDimensions(int width, int height) {
		srcWidth = width;
		srcHeight = height;
		if (destWidth < 0)
			destWidth = width;
		if (destHeight < 0)
			destHeight = height;

		super.setDimensions(destWidth, destHeight);
	}

	/**
	 *
	 */
	public void setPixels(int x, int y, int width, int height, ColorModel colorModel, byte[] pixels, int offset, int scansize) {
		if (srccols == null || srcrows == null)
			createMapping();
		if (outpixbuf == null)
			outpixbuf = new byte[destWidth];

		byte[] row = (byte[])outpixbuf;

		int destX = destWidth * x / srcWidth;
		int destY = destHeight * y / srcHeight;
		int destXEnd = Math.max((destWidth * (x + width) / srcWidth), destX+1);
		int destYEnd = Math.max((destHeight * (y + height) / srcHeight), destY+1);

		int lastSrcRow = -1;
		for (int cy = destY; cy < destYEnd; cy++) {
			int curSrcRow = srcrows[cy];
			int off = (curSrcRow - y) * scansize - x + offset;
			if (curSrcRow != lastSrcRow) {
				for (int cx = destX; cx < destXEnd; cx++) {
					row[cx] = pixels[srccols[cx] + off];
				}
			}
			lastSrcRow = curSrcRow;
			super.setPixels(destX, cy, destXEnd - destX, 1, colorModel, row, destX, destXEnd - destX);
		}
	}

	/**
	 *
	 */
	public void setPixels(int x, int y, int width, int height, ColorModel colorModel, int[] pixels, int offset, int scansize) {
		if (srccols == null || srcrows == null)
			createMapping();
		if (outpixbuf == null)
			outpixbuf = new int[destWidth];

		int[] row = (int[])outpixbuf;

		int destX = destWidth * x / srcWidth;
		int destY = destHeight * y / srcHeight;
		int destXEnd = Math.max((destWidth * (x + width) / srcWidth), destX+1);
		int destYEnd = Math.max((destHeight * (y + height) / srcHeight), destY+1);

		int lastSrcRow = -1;
		for (int cy = destY; cy < destYEnd; cy++) {
			int curSrcRow = srcrows[cy];
			int off = (curSrcRow - y) * scansize - x + offset;
			if (curSrcRow != lastSrcRow) {
				for (int cx = destX; cx < destXEnd; cx++) {
					row[cx] = pixels[srccols[cx] + off];
				}
			}
			lastSrcRow = curSrcRow;
			super.setPixels(destX, cy, destXEnd - destX, 1, colorModel, row, destX, destXEnd - destX);
		}
	}

	/**
	 *
	 */
	public void setProperties(Hashtable properties) {
		setProperty(properties, "rescale", destWidth + "x" + destHeight); //$NON-NLS-1$ //$NON-NLS-2$
		super.setProperties(properties);
	}

	void createMapping() {
		srccols = new int[destWidth + 1];
		for (int x = 0; x <= destWidth; x++) {
			srccols[x] = x * srcWidth / destWidth;
		}
		srcrows = new int[destHeight + 1];
		for (int y = 0; y <= destHeight; y++) {
			srcrows[y] = y * srcHeight / destHeight;
		}
	}
}
