package java.awt.image;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2004, 2005  All Rights Reserved
 */

import java.awt.Image;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.ImageCapabilities;

public abstract class VolatileImage extends Image {

	public static final int IMAGE_INCOMPATIBLE = 2;
	public static final int IMAGE_OK = 0;
	public static final int IMAGE_RESTORED = 1;

	public VolatileImage() {
		super();
	}

	public abstract BufferedImage getSnapshot();

	public abstract int getWidth();

	public abstract int getHeight();

	/**
	 * Answers the <code>ImageProducer</code> that generates the
	 * pixels for the receiver.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @return	an <code>ImageProducer</code> which is the receiver's source
	 */
	public ImageProducer getSource() {
		return getSnapshot().getSource();
	}

	/**
	 * Releases all resources used by the receiver.
	 *
	 * @author	IBM
	 * @version	initial
	 */
	public void flush() {
		// NOP
	}

	/**
	 * Answers a <code>Graphics</code> used for drawing
	 * on the receiver or <code>null</code> if the receiver
	 * is not a off-screen image.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @return	a java.awt.Graphics for drawing
	 */
	public Graphics getGraphics() {
		return createGraphics();
	}

	/**
	 * Answers a <code>Graphics2D</code> used for drawing
	 * on the receiver or <code>null</code> if the receiver
	 * is not a off-screen image.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @return	a java.awt.Graphics2D for drawing
	 */
	public abstract Graphics2D createGraphics();

	public abstract int validate(GraphicsConfiguration gc);

	public abstract boolean contentsLost();

	public abstract ImageCapabilities getCapabilities();

}
