package java.beans;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.EventObject;

/**
 * An object which describes a change in one of an object's properties.
 */
public class PropertyChangeEvent extends EventObject {

	static final long serialVersionUID = 7042693688939648123L;

	/**
	 * The name of the property which is changing.
	 */
	private String propertyName;

	/**
	 * The old value of the property.
	 */
	private Object oldValue;

	/**
	 * The name value of the property.
	 */
	private Object newValue;

	/**
	 * The propagation ID for this change.
	 */
	private Object propagationId;

	/**
	 * Constructs a new instance of this class.
	 *
	 * @param		source        the object which is changing.
	 * @param		propertyName  the name of the property which is changing.
	 * @param		oldValue      the old value of the property.
	 * @param		newValue      the new value of the property.
	 */
	public PropertyChangeEvent(
		Object source,
		String propertyName,
		Object oldValue,
		Object newValue) {
		super(source);

		this.propertyName = propertyName;
		this.oldValue = oldValue;
		this.newValue = newValue;
	}

	/**
	 * Answers the name of the property which changed.
	 *
	 * @return	the name of the changed property.
	 */
	public String getPropertyName() {
		return propertyName;
	}

	/**
	 * Answesr the newly changed value of the property.
	 *
	 * @return the new value of the property
	 */
	public Object getNewValue() {
		return newValue;
	}

	/**
	 * Answers the previous value of the property.
	 *
	 * @return the old value of the property
	 */
	public Object getOldValue() {
		return oldValue;
	}

	/**
	 * Answers the receiver's propagation ID.
	 *
	 * @return the receiver's propagation ID
	 */
	public Object getPropagationId() {
		return propagationId;
	}

	/**
	 * Sets the receiver's propagation ID.
	 * @param propagationId the receiver's propagation ID
	 */
	public void setPropagationId(Object propID) {
		this.propagationId = propID;
	}

}
