package java.beans;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * Classes which implement this interface describe information about the need for and/or presence of a graphical user-interface.
 */

public interface Visibility {

	/**
	 * Answers true if the receiver is currently not using a graphical user-interface because of a call to dontUseGui.
	 * @return true if the receiver needs a GUI
	 *
	 * @see			Visibility#dontUseGui
	 */
	public boolean avoidingGui();

	/**
	 * Tells the receiver to avoid using a graphical user-interface to get its work done if at all possible.
	 *
	 * @see			Visibility#avoidingGui
	 */
	public void dontUseGui();

	/**
	 * Answers true if the receiver will only work if a graphical user-interface is available.
	 * @return true if the receiver needs a GUI
	 */
	public boolean needsGui();

	/**
	 * Tells the receiver it does not need to avoid using a graphical user-interface to get its work done.
	 *
	 * @see			Visibility#avoidingGui
	 */
	public void okToUseGui();

}
