
package java.io;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * 	An EmulatedFieldsForLoading is an object that represents a set of emulated fields for an object
 * being loaded. It is a concrete implementation for ObjectInputStream.GetField
 *
 * @author		OTI
 * @version		initial
 *
 * @see			ObjectInputStream.GetField
 * @see			EmulatedFieldsForDumping
 */
class EmulatedFieldsForLoading extends ObjectInputStream.GetField {
	private ObjectStreamClass streamClass;		// The class descriptor with the declared fields the receiver emulates
	private EmulatedFields emulatedFields;		// The actual representation, with a more powerful API (set&get)
/**
 * Constructs a new instance of EmulatedFieldsForDumping.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		streamClass	an ObjectStreamClass, defining the class for which to emulate fields.
 */
 EmulatedFieldsForLoading(ObjectStreamClass streamClass) {
	super();
	this.streamClass = streamClass;
	emulatedFields = new EmulatedFields (streamClass.getLoadFields(),
		streamClass.fields());
}
/**
 * Return a boolean indicating if the field named <code>name</code> has been
 * assigned a value explicitely (false) or if it still holds a default value for
 * the type (true) because it hasn't been assigned to yet.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		name		A String, the name of the field to test
 * @return		<code>true</code> if the field holds it default value, <code>false</code> otherwise.
 *
 * @throws		IOException 				If an IO error occurs
 * @throws		IllegalArgumentException 	If the corresponding field can not be found.
 */
 public  boolean defaulted (String name) throws IOException, IllegalArgumentException {
	return emulatedFields.defaulted(name);
}
/**
 * Return the actual EmulatedFields instance used by the receiver. We have the actual work in
 * a separate class so that the code can be shared. The receiver has to be of a subclass
 * of GetField.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		array of ObjectSlot the receiver represents.
 */
 EmulatedFields emulatedFields (){
	return emulatedFields;
}
/**
 * Find and return the byte value of a given field named <code>name</code> in the receiver.
 * If the field has not been assigned any value yet, the default value <code>defaultValue</code>
 * is returned instead.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		name			A String, the name of the field to find
 * @param		defaultValue	Return value in case the field has not been assigned to yet.
 * @return		the value of the given field if it has been assigned, or the default value otherwise
 *
 * @throws		IOException 				If an IO error occurs
 * @throws		IllegalArgumentException 	If the corresponding field can not be found.
 */
 public  byte get (String name, byte defaultValue) throws IOException, IllegalArgumentException {
	return emulatedFields.get (name, defaultValue);
}
/**
 * Find and return the char value of a given field named <code>name</code> in the receiver.
 * If the field has not been assigned any value yet, the default value <code>defaultValue</code>
 * is returned instead.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		name			A String, the name of the field to find
 * @param		defaultValue	Return value in case the field has not been assigned to yet.
 * @return		the value of the given field if it has been assigned, or the default value otherwise
 *
 * @throws		IOException 				If an IO error occurs
 * @throws		IllegalArgumentException 	If the corresponding field can not be found.
 */
 public  char get (String name, char defaultValue) throws IOException, IllegalArgumentException {
	return emulatedFields.get (name, defaultValue);
}
/**
 * Find and return the double value of a given field named <code>name</code> in the receiver.
 * If the field has not been assigned any value yet, the default value <code>defaultValue</code>
 * is returned instead.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		name			A String, the name of the field to find
 * @param		defaultValue	Return value in case the field has not been assigned to yet.
 * @return		the value of the given field if it has been assigned, or the default value otherwise
 *
 * @throws		IOException 				If an IO error occurs
 * @throws		IllegalArgumentException 	If the corresponding field can not be found.
 */
 public  double get (String name, double defaultValue) throws IOException, IllegalArgumentException {
	return emulatedFields.get (name, defaultValue);
}
/**
 * Find and return the float value of a given field named <code>name</code> in the receiver.
 * If the field has not been assigned any value yet, the default value <code>defaultValue</code>
 * is returned instead.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		name			A String, the name of the field to find
 * @param		defaultValue	Return value in case the field has not been assigned to yet.
 * @return		the value of the given field if it has been assigned, or the default value otherwise
 *
 * @throws		IOException 				If an IO error occurs
 * @throws		IllegalArgumentException 	If the corresponding field can not be found.
 */
 public  float get (String name, float defaultValue) throws IOException, IllegalArgumentException {
	return emulatedFields.get (name, defaultValue);
}
/**
 * Find and return the int value of a given field named <code>name</code> in the receiver.
 * If the field has not been assigned any value yet, the default value <code>defaultValue</code>
 * is returned instead.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		name			A String, the name of the field to find
 * @param		defaultValue	Return value in case the field has not been assigned to yet.
 * @return		the value of the given field if it has been assigned, or the default value otherwise
 *
 * @throws		IOException 				If an IO error occurs
 * @throws		IllegalArgumentException 	If the corresponding field can not be found.
 */
 public  int get (String name, int defaultValue) throws IOException, IllegalArgumentException {
	return emulatedFields.get (name, defaultValue);
}
/**
 * Find and return the long value of a given field named <code>name</code> in the receiver.
 * If the field has not been assigned any value yet, the default value <code>defaultValue</code>
 * is returned instead.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		name			A String, the name of the field to find
 * @param		defaultValue	Return value in case the field has not been assigned to yet.
 * @return		the value of the given field if it has been assigned, or the default value otherwise
 *
 * @throws		IOException 				If an IO error occurs
 * @throws		IllegalArgumentException 	If the corresponding field can not be found.
 */
 public  long get (String name, long defaultValue) throws IOException, IllegalArgumentException {
	return emulatedFields.get (name, defaultValue);
}
/**
 * Find and return the Object value of a given field named <code>name</code> in the receiver.
 * If the field has not been assigned any value yet, the default value <code>defaultValue</code>
 * is returned instead.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		name			A String, the name of the field to find
 * @param		defaultValue	Return value in case the field has not been assigned to yet.
 * @return		the value of the given field if it has been assigned, or the default value otherwise
 *
 * @throws		IOException 				If an IO error occurs
 * @throws		IllegalArgumentException 	If the corresponding field can not be found.
 */
 public  Object get (String name, Object defaultValue) throws IOException, IllegalArgumentException {
	return emulatedFields.get (name, defaultValue);
}
/**
 * Find and return the short value of a given field named <code>name</code> in the receiver.
 * If the field has not been assigned any value yet, the default value <code>defaultValue</code>
 * is returned instead.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		name			A String, the name of the field to find
 * @param		defaultValue	Return value in case the field has not been assigned to yet.
 * @return		the value of the given field if it has been assigned, or the default value otherwise
 *
 * @throws		IOException 				If an IO error occurs
 * @throws		IllegalArgumentException 	If the corresponding field can not be found.
 */
 public  short get (String name, short defaultValue) throws IOException, IllegalArgumentException {
	return emulatedFields.get (name, defaultValue);
}
/**
 * Find and return the boolean value of a given field named <code>name</code> in the receiver.
 * If the field has not been assigned any value yet, the default value <code>defaultValue</code>
 * is returned instead.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		name			A String, the name of the field to find
 * @param		defaultValue	Return value in case the field has not been assigned to yet.
 * @return		the value of the given field if it has been assigned, or the default value otherwise
 *
 * @throws		IOException 				If an IO error occurs
 * @throws		IllegalArgumentException 	If the corresponding field can not be found.
 */
public  boolean get (String name, boolean defaultValue) throws IOException, IllegalArgumentException {
	return emulatedFields.get (name, defaultValue);
}
/**
 * Return the class descriptor for which the emulated fields are defined.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		ObjectStreamClass	The class descriptor for which the emulated fields are defined.
 */
 public ObjectStreamClass getObjectStreamClass () {
	return streamClass;
}
}
