package java.io;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * FileDescriptor is the lowest level representation of a
 * File, Device, or Socket.  You can create any of the IO classes
 * which take a FileDescriptor as an argument by querying an open
 * Socket or File for the FileDescriptor.
 * <p>
 * The FileDescriptor class also contains static fields representing
 * Standard Input, Output and Error. You can use these directly if
 * desired but it is recommended you go through System.in, System.out,
 * and System.err streams respectively.
 * <p>
 * Applications should not create new FileDescriptors.
 *
 * @author		OTI
 * @version		initial
 *
 * @see			FileInputStream#getFD()
 * @see			FileOutputStream#getFD()
 * @see			RandomAccessFile#getFD()
 */
 public final class FileDescriptor {
	/** FileDescriptor representing Standard In */
	public static final FileDescriptor in = new FileDescriptor();
	/** FileDescriptor representing Standard Out */
	public static final FileDescriptor out = new FileDescriptor();
	/** FileDescriptor representing Standard Error */
	public static final FileDescriptor err = new FileDescriptor();

	/**
	 * Represents a link to any underlying OS resources for this FileDescriptor. A
	 * value of -1 indicates that this FileDescriptor is invalid.
	 */
	long descriptor = -1;

	private static native void oneTimeInitialization();

	static {
		in.descriptor = 0;
		out.descriptor = 1;
		err.descriptor = 2;

		oneTimeInitialization();
	}

/**
 * Constructs a new FileDescriptor containing an invalid handle. This constructor does nothing
 * interesting.  Provided for signature compatibility.
 *
 * @author		OTI
 * @version		initial
 */
public FileDescriptor() {
	super();
}

/**
 * Ensures that data which is buffered within the underlying implementation
 * is written out to the appropriate device before returning.
 *
 * @author		OTI
 * @version		initial
 *
 * @throws		SyncFailedException when the operation fails
 */
public native void sync() throws SyncFailedException;

/**
 * Answers a boolean indicating whether or not this FileDescriptor is valid.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		<code>true</code> if this FileDescriptor is valid, <code>false</code> otherwise
 */
public native boolean valid();

}
