
package java.io;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

/**
 * A problem was found with the class of one of the objects
 * being serialized or deserialized. These can be
 * <ul>
 * <li>The SUIDs of the class loaded by the VM and the
 *     serialized class info do not match</li>
 * <li>A serializable or externalizable object cannot be
 *     instantiated (when deserializing) because the empty
 *     constructor that needs to be run is not visible or
 *     fails.</li>
 * </ul>
 *
 * @author		OTI
 * @version		initial
 *
 * @see			ObjectInputStream #readObject()
 * @see			ObjectInputValidation#validateObject()
 */
public class InvalidClassException extends ObjectStreamException {

	private static final long serialVersionUID = -4333316296251054416L;

	/**
	 * The fully qualified name of the class that caused the problem
	 */
	public String classname;

/**
 * Constructs a new instance of this class with its
 * walkback and message filled in.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		detailMessage	The detail message for the exception.
 */
public InvalidClassException (String detailMessage) {
	super(detailMessage);
}

/**
 * Constructs a new instance of this class with its
 * walkback, message and the fully qualified name of the
 * class which caused the exception filled in.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		className		The detail message for the exception.
 * @param		detailMessage	The detail message for the exception.
 */
public InvalidClassException (String className, String detailMessage) {
	super(detailMessage);
	this.classname = className;
}

/**
 * Answers the extra information message which was provided
 * when the exception was created. If no message was provided
 * at creation time, then answer null. If a message was
 * provided and a class name which caused the exception, the
 * values are concatenated and returned.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		The receiver's message, possibly concatenated with the name of the class
 *				that caused the problem.
 */
public String getMessage () {
	String msg = super.getMessage();
	if (classname != null) {
		msg = classname + ';' + ' ' + msg;
	}
	return msg;
}

}
