
package java.io;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * Streams to be used with serialization to write objects must implement this interface.
 * ObjectOutputStream is one example.
 *
 * @author		OTI
 * @version		initial
 *
 * @see			ObjectOutputStream
 * @see			ObjectInput
 */
public interface ObjectOutput extends DataOutput {
/**
 * Close this ObjectOutput.  Concrete implementations of this class should
 * free any resources during close.
 *
 * @author		OTI
 * @version		initial
 *
 * @throws 		IOException	If an error occurs attempting to close this ObjectOutput.
 */
public void close() throws IOException;
/**
 * Flush this ObjectOutput.  Concrete implementations of this class should
 * ensure any pending writes are written out when this method is envoked.
 *
 * @author		OTI
 * @version		initial
 *
 * @throws 		IOException	If an error occurs attempting to flush this ObjectOutput.
 */
public void flush() throws IOException;
/**
 * Writes the entire contents of the byte array <code>buffer</code>
 * to this ObjectOutput.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		buffer		the buffer to be written
 *
 * @throws 	java.io.IOException	If an error occurs attempting to write to this ObjectOutput.
 */
public void write(byte[] buffer) throws IOException;
/**
 * Writes <code>count</code> <code>bytes</code> from this byte array
 * <code>buffer</code> starting at offset <code>index</code> to this
 * ObjectOutput.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		buffer		the buffer to be written
 * @param		offset		offset in buffer to get bytes
 * @param		count		number of bytes in buffer to write
 *
 * @throws 	java.io.IOException	If an error occurs attempting to write to this ObjectOutput.
 */
public void write(byte[] buffer, int offset, int count) throws IOException;
/**
 * Writes the specified int <code>value</code> to this ObjectOutput.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		value		the int to be written
 *
 * @throws 	java.io.IOException	If an error occurs attempting to write to this ObjectOutput.
 */
public void write(int value) throws IOException;
/**
 * Writes the specified object <code>obj</code> to this ObjectOutput.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		obj		the object to be written
 *
 * @throws 	java.io.IOException	If an error occurs attempting to write to this ObjectOutput.
 */
public void writeObject(Object obj) throws IOException;
}
