
package java.io;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

/**
 * This type of exception is thrown by readObject() when it
 * detects an exception marker in the input stream. This marker
 * indicates that when the object was being serialized, an
 * exception happened and this marker was inserted instead of
 * the original object. It is a way to "propagate" an exception
 * from the code that attempted to write the object to the code
 * that is attempting to read the object.
 *
 * @author		OTI
 * @version		initial
 *
 * @see			ObjectInputStream#readObject()
 */
public class WriteAbortedException extends ObjectStreamException {

	private static final long serialVersionUID = -3326426625597282442L;

	/**
	 * The exception that was caused when writeObject() was attempting to serialize the object
	 */
	public Exception detail;

/**
 * Constructs a new instance of this class with its
 * walkback, message and the exception which caused the
 * underlying problem when serializing the object filled in.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		detailMessage	the detail message for the exception.
 * @param 		rootCause 		exception that caused the problem when serializing the object.
 */
public WriteAbortedException (String detailMessage, Exception rootCause) {
	super(detailMessage);
	detail = rootCause;
	initCause(rootCause);
}

/**
 * Answers the extra information message which was provided
 * when the throwable was created. If no message was provided
 * at creation time, then answer null.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the receiver's message.
 */
public String getMessage () {
	String msg = super.getMessage();
	if (detail != null) {
		msg = msg + "; " + detail.toString();
	}
	return msg;
}

/**
 * Answers the cause of this Throwable, or null if there
 * is no cause.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the receiver's cause.
 */
public Throwable getCause() {
	return detail;
}
}
