
package java.lang;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

/**
 * This error is thrown when an exception occurs during
 * class initialization.
 *
 * @author		OTI
 * @version		initial
 */
public class ExceptionInInitializerError extends LinkageError {

	private static final long serialVersionUID = 1521711792217232256L;

	private Throwable exception;

/**
 * Constructs a new instance of this class with its
 * walkback filled in.
 *
 * @author		OTI
 * @version		initial
 */
public ExceptionInInitializerError () {
	super();
	initCause(null);
}

/**
 * Constructs a new instance of this class with its
 * walkback and message filled in.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		detailMessage String
 *				The detail message for the exception.
 */
public ExceptionInInitializerError (String detailMessage) {
	super (detailMessage);
	initCause(null);
}

/**
 * Constructs a new instance of this class with its
 * walkback and exception filled in. The exception should
 * be the one which originally occured in the class
 * initialization code.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		exception Throwable
 *				The exception which caused the problem.
 */
public ExceptionInInitializerError (Throwable exception) {
	super();
	this.exception = exception;
	initCause(exception);
}

/**
 * Answers the exception which was passed in when
 * the instance was created.
 *
 * @author		OTI
 * @version		initial
 */
public Throwable getException () {
	return exception;
}

/**
 * Answers the cause of this Throwable, or null if there
 * is no cause.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		Throwable
 *				The receiver's cause.
 */
public Throwable getCause() {
	return exception;
}
}
