
package java.lang;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * Instances of class Process provide control of and access
 * to platform processes.
 *
 * @author		OTI
 * @version		initial
 */
public abstract class Process {

/**
 * Terimates the receiver and closes any associated streams.
 *
 * @author		OTI
 * @version		initial
 */
abstract public void destroy();

/**
 * Answers the exit value of the receiving Process. It is available
 * only when the OS subprocess is finished.
 *
 * @author		OTI
 * @version		initial
 *
 * @return 		The exit value of the receiver.
 *
 * @throws		IllegalThreadStateException 	If the receiver has not terminated.
 *
 */
abstract public int exitValue();

/**
 * Answers the receiver's error output stream.
 * <p>
 * Note: This is an InputStream which allows reading of the
 * other threads "stderr".
 *
 * @author		OTI
 * @version		initial
 *
 * @return 		The error stream associated with the receiver
 */
abstract public java.io.InputStream getErrorStream();

/**
 * Answers the receiver's standard input stream
 * <p>
 * Note: This is an InputStream which allows reading from
 * the other process' "stdout".
 *
 * @author		OTI
 * @version		initial
 *
 * @return 		The receiver's process' stdin.
 */
abstract public java.io.InputStream getInputStream();

/**
 * Answers the receiver's standard output stream
 * <p>
 * Note: This is an OutputStream which allows writing to the
 * other process' "stdin".
 *
 * @author		OTI
 * @version		initial
 *
 * @return 		The receiver's process' stdout.
 */
abstract public java.io.OutputStream getOutputStream();

/**
 * Causes the calling thread to wait for the process associated
 * with the receiver to finish executing.
 *
 * @author		OTI
 * @version		initial
 *
 * @return 		The exit value of the Process being waited on
 *
 * @throws		InterruptedException 	If the calling thread is interrupted
 */
abstract public int waitFor() throws InterruptedException;

}
