
package java.lang.reflect;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

/**
 * This class provides a wrapper for an exception thrown by a Method or Constructor
 * invocation.
 *
 * @author		OTI
 * @version		initial
 *
 * @see		java.lang.reflect.Method#invoke
 * @see		java.lang.reflect.Constructor#newInstance
 */
public class InvocationTargetException extends Exception {

	private static final long serialVersionUID = 4085088731926701167L;

	private Throwable target;

/**
 * Constructs a new instance of this class with its
 * walkback filled in.
 *
 * @author		OTI
 * @version		initial
 */
protected InvocationTargetException () {
	super((Throwable)null);
}

/**
 * Constructs a new instance of this class with its
 * walkback and target exception filled in.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		exception Throwable
 *				The exception which occurred while running the Method or Constructor.
 */
public InvocationTargetException (Throwable exception) {
	super(null, exception);
	target = exception;
}

/**
 * Constructs a new instance of this class with its
 * walkback, target exception and message filled in.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		detailMessage String
 *				The detail message for the exception.
 * @param		exception Throwable
 *				The exception which occurred while running the Method or Constructor.
 */
public InvocationTargetException (Throwable exception, String detailMessage) {
	super(detailMessage, exception);
	target = exception;
}

/**
 * Answers the exception which caused the receiver to
 * be thrown.
 *
 * @author		OTI
 * @version		initial
 */
public Throwable getTargetException () {
	return target;
}

/**
 * Answers the cause of this Throwable, or null if there
 * is no cause.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		Throwable
 *				The receiver's cause.
 */
public Throwable getCause() {
	return target;
}
}
