
package java.lang.reflect;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2001, 2006  All Rights Reserved
 */

/**
 * This class provides a wrapper for an unexpected exception thrown by an InvocationHandler
 *
 * @author		OTI
 * @version		initial
 *
 * @see		java.lang.reflect.InvocationHandler#invoke
 */

public class UndeclaredThrowableException extends RuntimeException {

	private static final long serialVersionUID = 330127114055056639L;

	private Throwable undeclaredThrowable;

/**
 * Constructs a new instance of this class with its
 * walkback and target exception filled in.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		exception	The exception which occurred while loading the class.
 */
public UndeclaredThrowableException(Throwable exception) {
	super();
	this.undeclaredThrowable = exception;
	initCause(exception);
}

/**
 * Constructs a new instance of this class with its
 * walkback, target exception and message filled in.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		detailMessage String
 *				The detail message for the exception.
 * @param		exception Throwable
 *				The exception which occurred while loading the class.
 */
public UndeclaredThrowableException(Throwable exception, String detailMessage) {
	super(detailMessage);
	this.undeclaredThrowable = exception;
	initCause(exception);
}

/**
 * Answers the exception which caused the receiver to
 * be thrown.
 *
 * @author		OTI
 * @version		initial
 */
public Throwable getUndeclaredThrowable() {
	return undeclaredThrowable;
}

/**
 * Answers the cause of this Throwable, or null if there
 * is no cause.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		Throwable
 *				The receiver's cause.
 */
public Throwable getCause() {
	return undeclaredThrowable;
}
}
