package java.net;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

import java.io.IOException;

/**
 * This class converts the content of a certain format into a Java type Object.
 * It is implemented differently for each content type in each platform.
 * It is created by <code>ContentHandlerFactory</code>
 *
 * @author		OTI
 * @version		initial
 *
 * @see			ContentHandlerFactory
 * @see			URL
 * @see			URLConnection#getContent()
 */

abstract public class ContentHandler {
/**
 * Answers the object pointed by the specified URL Connection <code>uConn</code>.
 *
 * @author		OTI
 * @version		initial
 *
 * @return java.lang.Object			the object refered by <code>uConn</code>
 * @param uConn java.net.URLConnection	the url connection that points to the desired object
 * @exception java.io.IOException 	thrown if an IO error occurs during the retrieval of the object
 */
public abstract Object getContent(URLConnection uConn) throws IOException;

/**
 * Answers the object pointed by the specified URL Connection <code>uConn</code>.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		uConn
 *					java.net.URLConnection	the url connection that points to the desired object
 * @param		types
 *					The list of acceptable content types
 * @return 		Object
 *					The object of the resource pointed by this URL, or
 *					null if the content does not match a specified
 *					content type.
 *
 * @exception 	IOException
 *					If an error occured obtaining the content.
 */
public Object getContent(URLConnection uConn, Class[] types) throws IOException {
	Object content = getContent(uConn);
	Class cl = content.getClass();
	for (int i=0; i<types.length; i++) {
		if (cl == types[i]) return content;
	}
	return null;
}
}
