package java.net;
import java.util.Enumeration;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2005, 2006  All Rights Reserved
 */

/**
 * This class provides is used to pass the information required in an
 * ip_mreq or ip6_mreq structure to java natives.  We don't have accessor
 * methods as it is more straight forward in the natives to simply access
 * the fields directly
 *
 * @author		OTI
 * @version		initial
 */
final class GenericIPMreq {

	// private members
	private InetAddress 	multiaddr;
	private InetAddress 	interfaceAddr;
	private boolean 		isIPV6Address;
	private int				interfaceIdx;

	/**
	 * This constructor is used to create an instance of the object
	 *
	 * @param addr multicast address to join/leave
	 *
	 */
	GenericIPMreq(InetAddress addr)	{
		this.multiaddr = addr;
		this.interfaceAddr = null;
		this.interfaceIdx = 0;
		init();
	}

	/**
	 * This constructor is used to create an instance of the object
	 *
	 * @param addr multicast address to join/leave
	 * @param netInterface the NetworkInterface object identifying the interface on which to join/leave
	 *
	 */
	GenericIPMreq(InetAddress addr, NetworkInterface netInterface){
		this.multiaddr = addr;
		if (null != netInterface){
			this.interfaceIdx = netInterface.getIndex();

			// here we need to get the first IPV4 address as we only use it if we
			// are settting the interface for an IPV4 multicast socket.  For adds/drops on
			// IPV6 addresses we use the index within the networkInterface
			this.interfaceAddr = null;
			Enumeration theAddresses = netInterface.getInetAddresses();
			if ((addr instanceof Inet4Address) && (theAddresses != null)){
				boolean found = false;
				while((theAddresses.hasMoreElements())&&(found != true)){
					InetAddress theAddress = (InetAddress) theAddresses.nextElement();
					if (theAddress instanceof Inet4Address){
						this.interfaceAddr = theAddress;
						found = true;
					}
				}
			}
		}
		else
		{
			// network interface is null so we just want to defer the decision to
			// the system
			this.interfaceIdx = 0;
			this.interfaceAddr = null;
		}
		init();
	}

	/**
	 * This method does any required initialization for the constructors
	 *
	 */
	private void init()
	{
		// do any required initialization
		isIPV6Address = false;

		// set the flag indicating if the multicast address is an IPV6 address or not
		if ((this.multiaddr!= null) && (this.multiaddr instanceof Inet6Address)){
			isIPV6Address = true;
		}
	}
}

