package java.net;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2004, 2006  All Rights Reserved
 */

import java.io.IOException;
import java.io.ObjectInputStream;

public class InetSocketAddress extends SocketAddress {
	private static final long serialVersionUID = 5076001401234631237L;

	private String hostName;
	private InetAddress addr;
	private int port;

public InetSocketAddress(int port) {
	this((InetAddress)null, port);
}

public InetSocketAddress(InetAddress address, int port) {
	if (port < 0 || port > 65535)
		throw new IllegalArgumentException();
	if (address == null)
		addr = InetAddress.ANY;
	else
		addr = address;
	this.port = port;
}

public InetSocketAddress(String host, int port) {
	if (host == null || port < 0 || port > 65535)
		throw new IllegalArgumentException();
	this.port = port;
	try {
		addr = InetAddress.getByName(host);
	} catch (UnknownHostException e) {
		hostName = host;
	}
}

public final int getPort() {
	return port;
}

public final InetAddress getAddress() {
	return addr;
}

public final String getHostName() {
	if (addr != null) {
		return addr.getHostName();
	}
	return hostName;
}

public final boolean isUnresolved() {
	return addr == null;
}

public String toString() {
	String host;
	if (addr != null) {
		host = addr.toString();
	} else {
		host = hostName;
	}
	return host + ":" + port; //$NON-NLS-1$
}

public final boolean equals(Object socketAddr) {
	if (this == socketAddr) return true;
	if (!(socketAddr instanceof InetSocketAddress))
		return false;
	InetSocketAddress iSockAddr = (InetSocketAddress)socketAddr;

	// check the ports as we always need to do this
	if (port != iSockAddr.port){
		return false;
	}

	// we only use the hostnames in the comparison if the addrs were not resolved
	if ((addr == null)&&(iSockAddr.addr == null))
	{
		return hostName.equals(iSockAddr.hostName);
	} else {
		// addrs were resolved so use them for the comparison
		if (addr == null){
			// if we are here we know iSockAddr is not null so just return false
			return false;
		}
		return addr.equals(iSockAddr.addr);
	}
}

public final int hashCode() {
	if (addr == null)
		return hostName.hashCode() + port;
	return addr.hashCode() + port;
}

private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
	stream.defaultReadObject();
	if (addr == null) {
		try {
			addr = InetAddress.getByName(hostName);
		} catch (UnknownHostException e) {
		}
	}
}

}
