package java.net;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.io.FileDescriptor;

/**
 * This class was added so we can create sockets with options that are needed
 * for server sockets.  It just overrides create so that we call new natives which
 * only set the options required for server sockets.  In order to preserve behaviour
 * of older versions the create PlainSocketImpl was left as is and this new class
 * was added.  For newer versions an instance of this class is used, for earlier
 * versions the original PlainSocketImpl is used.
 */
class PlainServerSocketImpl extends PlainSocketImpl {

	/**
	 * Answer the result of attempting to create a server stream socket in
	 * the IP stack.  Any special options required for server sockets will
	 * be set by this method.
	 *
	 * @author		OTI
	 * @version		initial
	 *
	 * @param		aFD		the socket FileDescriptor
	 * @exception	SocketException	if an error occurs while creating the socket
	 */
	static native void createServerStreamSocketImpl(FileDescriptor aFD, boolean preferIPv4Stack)
		throws SocketException;

	/**
	 * Creates a new unconnected socket. If streaming is true,
	 * create a stream socket, else a datagram socket.
	 * The deprecated datagram usage is not supported and will throw an exception.
	 *
	 * @author		OTI
	 * @version		initial
	 *
	 * @param		streaming	true, if the socket is type streaming
	 * @exception	SocketException	if an error occurs while creating the socket
	 */
	protected void create(boolean streaming) throws SocketException {
		createServerStreamSocketImpl(fd, Socket.preferIPv4Stack());
	}
}
