package java.net;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

import java.util.Vector;

/**
 * SocketPermissionCollection is a class which holds a collection of SocketPermission objects
 * and can answer a boolean indicating whether or not a specific permissions is implied
 * by a SocketPermissionCollection.
 *
 * @author		OTI
 * @version		initial
 *
 */
final class SocketPermissionCollection extends java.security.PermissionCollection {
	private static final long serialVersionUID = 2787186408602843674L;
	public Vector permissions = new Vector();

// Constructs a new instance of this class.
public SocketPermissionCollection() {
	super();
}

// Adds the argument to the collection.
public void add(java.security.Permission permission) {
	if (isReadOnly()) throw new IllegalStateException();
	if (!(permission instanceof SocketPermission))
		throw new java.lang.IllegalArgumentException(permission.toString());
	permissions.addElement(permission);
}

// Answers an enumeration of the permissions
public java.util.Enumeration elements() {
	return permissions.elements();
}

// Answers if this permission collection implies <code>permission</code>. Basically it tests
// if <code>permission</code> is the subset of this collection.
public boolean implies(java.security.Permission permission) {
	if (!(permission instanceof SocketPermission))
		return false;
	SocketPermission sp, argPerm = (SocketPermission)permission;
	int pmask = argPerm.actionsMask;
	int allMask = 0;
	int i = 0, count = permissions.size();
	while((i < count) && ((allMask & pmask) != pmask)){
		sp = (SocketPermission)permissions.elementAt(i);
		if (sp.checkHost(argPerm)) {
			if ((sp.actionsMask & SocketPermission.SP_RESOLVE) == SocketPermission.SP_RESOLVE)
				allMask |= SocketPermission.SP_RESOLVE;
			//Only set flags if the port range and host can be implied
			if ((argPerm.portMin >= sp.portMin) && (argPerm.portMax <= sp.portMax)) {
				if ((sp.actionsMask & SocketPermission.SP_CONNECT) == SocketPermission.SP_CONNECT)
					allMask |= SocketPermission.SP_CONNECT;
				if ((sp.actionsMask & SocketPermission.SP_ACCEPT) == SocketPermission.SP_ACCEPT)
					allMask |= SocketPermission.SP_ACCEPT;
				if ((sp.actionsMask & SocketPermission.SP_LISTEN) == SocketPermission.SP_LISTEN)
					allMask |= SocketPermission.SP_LISTEN;
			}
		}
		++i;
	}

	return (allMask & pmask) == pmask;
}
}

