package java.net;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.io.*;

/**
 * This class is used to decode a string which is encoded in the
 * <code>application/x-www-form-urlencoded</code> MIME content type.
 *
 * @author		OTI
 * @version		initial
 */

public class URLDecoder {

/**
 * Decodes the string argument which is assumed to be encoded in the
 * <code>x-www-form-urlencoded</code> MIME content type.
 * <p>
 * '+' will be converted to space,
 * '%' and two following hex digit characters are converted
 *     to the equivalent byte value.
 * All other characters are passed through unmodified.
 * <p>
 * e.g. "A+B+C %24%25" -> "A B C $%"
 *
 * @author		OTI
 * @version		initial
 *
 * @param		s java.lang.String
 *					The encoded string.
 * @return		java.lang.String
 *					The decoded version.
 *
 * @deprecated use URLDecoder#decode(String, String) instead
 */
public static String decode(String s) {
	return com.ibm.oti.util.Util.decode(s, true);
}

/**
 * Decodes the string argument which is assumed to be encoded in the
 * <code>x-www-form-urlencoded</code> MIME content type using the specified encoding scheme.
 * <p>
 * '+' will be converted to space,
 * '%' and two following hex digit characters are converted
 *     to the equivalent byte value.
 * All other characters are passed through unmodified.
 *
 * <p>
 * e.g. "A+B+C %24%25" -> "A B C $%"
 *
 * @author		OTI
 * @version		initial
 *
 * @param		s java.lang.String
 *					The encoded string.
 * @param		enc java.lang.String
 *					The encoding scheme to use
 * @return		java.lang.String
 *					The decoded version.
 */
public static String decode(String s, String enc)
                     throws UnsupportedEncodingException {

	if (enc == null) throw new NullPointerException();
	StringBuffer result = new StringBuffer(s.length());
	ByteArrayOutputStream out = new ByteArrayOutputStream();
	for (int i = 0; i < s.length();) {
		char c = s.charAt(i);
		if (c == '+') result.append(' ');
		else if (c == '%') {
			out.reset();
			do {
				if (i + 2 >= s.length())
					// K01fe = Incomplete % sequence at: {0}
					throw new IllegalArgumentException(com.ibm.oti.util.Msg.getString("K01fe", i));
				int d1 = Character.digit(s.charAt(i+1), 16);
				int d2 = Character.digit(s.charAt(i+2), 16);
				if (d1 == -1 || d2 == -1)
					// K01ff = Invalid % sequence ({0}) at: {1}
					throw new IllegalArgumentException(com.ibm.oti.util.Msg.getString("K01ff", s.substring(i, i+3), String.valueOf(i)));
				out.write((byte)((d1 << 4) + d2));
				i += 3;
			} while (i < s.length() && s.charAt(i) == '%');
			result.append(out.toString(enc));
			continue;
		} else result.append(c);
		i++;
	}
	return result.toString();
}

}
