package java.rmi;
/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp 1998, 2005
 */

import java.io.*;

/**
 * This is the super class of all remote exceptions.
 * An instance of this class or its subclasses will be thrown
 * whenever an exception occurs during a remote call.
 *
 * @author		OTI
 * @version		initial
 */
public class RemoteException extends java.io.IOException {

private static final long serialVersionUID =-5148567311918794206L;

/**
 * The nested exception.
 */
public Throwable detail;

/**
 * Constructs a new instance of this class with its
 * walkback filled in.
 *
 * @author		OTI
 * @version		initial
 */
public RemoteException() {
	super();
}

/**
 * Constructs a new instance of this class with its
 * walkback and message filled in.
 *
 * @param		s String
 *					a message describing why the exception was thrown.
 *
 * @author		OTI
 * @version		initial
 */
public RemoteException(String s) {
	super(s);
}

/**
 * Constructs a new instance of this class with its
 * walkback, message and nested exception filled in.
 *
 * @param		s String
 *					a message describing why the exception was thrown.
 * @param		ex Throwable
 *					the exception being wrapped by this new instance.
 *
 * @author		OTI
 * @version		initial
 */
public RemoteException(String s, Throwable ex) {
	super(s);
	detail = ex;
}

/**
 * Answers the extra information message that was provided
 * when the receiver was created. If no message was provided
 * at creation time, then answer null.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the receiver's message.
 */
public String getMessage () {
	if((detail == null) || (detail.getMessage() == null))
		return super.getMessage();
	if(super.getMessage() == null)
		return detail.getMessage();
	return super.getMessage() + "::" + detail;
}

/**
 * Answers the nested exception.
 *
 * @author		IBM
 * @version		initial
 * @returns		the receiver's nested exception
 *
 */
public Throwable getCause() {
	return detail;
}

}
