
package java.security;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

/**
 * This runtime exception is thrown when an
 * access control check indicates that access
 * should not be granted.
 *
 * @author		OTI
 * @version		initial
 */
public class AccessControlException extends SecurityException {

	private static final long serialVersionUID = 5138225684096988535L;

	/**
	 * The permission object associated with the receiver.
	 */
	Permission perm;

/**
 * Constructs a new instance of this class with its
 * walkback and message filled in.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		detailMessage String
 *				The detail message for the exception.
 */
public AccessControlException (String detailMessage) {
	super(detailMessage);
}

/**
 * Constructs a new instance of this class with its
 * walkback, message and associated permission all
 * filled in.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		detailMessage String
 *				The detail message for the exception.
 * @param		perm Permission
 *				The failed permission.
 */
public AccessControlException (String detailMessage, Permission perm) {
	super(detailMessage);
	this.perm = perm;
}

/**
 * Answers the receiver's permission.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		Permission
 *				the receiver's permission
 */
public Permission getPermission () {
	return perm;
}

}
