
package java.security;

import java.security.spec.*;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 * This class is a Service Provider Interface for the
 * algorithm parameter generators to be supplied by providers.
 *
 * @author		OTI
 * @version		initial
 */

public abstract class AlgorithmParameterGeneratorSpi {

/**
 * Constructs a new instance of this class.
 *
 * @author		OTI
 * @version		initial
 */
public AlgorithmParameterGeneratorSpi () {
	super();
}

/**
 * Answers the newly generated parameters.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		AlgorithmParameters
 *					the newly generated parameters.
 */
protected abstract AlgorithmParameters engineGenerateParameters ();

/**
 * Initializes the receiver with the specified parameter size
 * and source of randomness.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		size int
 *					Size of the parameters.
 * @param		random SecureRandom
 *					New secure random to use.
 */
protected abstract void engineInit (int size, SecureRandom random);

/**
 * Initializes the receiver with the specified parameters and
 * source of randomness.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		genParamSpec AlgorithmParameterSpec
 *					parameters for this algorithm.
 * @param		random SecureRandom
 *					New secure random to use.
 */
protected abstract void engineInit(
	AlgorithmParameterSpec genParamSpec,
	SecureRandom random)
		throws InvalidAlgorithmParameterException;

}
