package java.security;

import java.io.*;
import java.security.spec.*;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

/**
 * This class is a Service Provider Interface for
 * algorithm parameters to be supplied by providers.
 *
 *
 * @author		OTI
 * @version		initial
 */

public abstract class AlgorithmParametersSpi {

/**
 * Constructs a new instance of this class.
 *
 * @author		OTI
 * @version		initial
 */
public AlgorithmParametersSpi () {
	super();
}

/**
 * Answers the parameters in their primary encoding format.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		byte[]
 *					the parameters in their primary encoding format.
 *
 * @exception	IOException
 *					if encoding errors are detected
 */

protected abstract byte[] engineGetEncoded() throws IOException;

/**
 * Answers the parameters in the specified format. If the
 * format is null, answers them in the primary format.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		format String
 *					the name of the desired encoding format.
 * @return		byte[]
 *					the parameters in their primary encoding format.
 *
 * @exception	IOException
 *					if encoding errors are detected
 */
protected abstract byte[] engineGetEncoded (String format) throws IOException;

/**
 * Answers the parameter specification for the receiver,
 * instantiated from the provided class.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		paramSpec Class
 *					class to use when instantiating the result.
 * @return		AlgorithmParameterSpec
 *					the parameter specification for the receiver.
 *
 * @exception	InvalidParameterSpecException
 *					if the requested specification is not
 *					appropriate for the receiver.
 */
protected abstract AlgorithmParameterSpec engineGetParameterSpec (Class paramSpec)
	throws InvalidParameterSpecException;

/**
 * Initializes the receiver with the given parameters which must
 * be encoded in the primary format.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		params byte[]
 *					parameters encoded in the primary format for the receiver.
 *
 * @exception	IOException
 *					if decoding errors are detected.
 */
protected abstract void engineInit(byte[] params) throws IOException;

/**
 * Initializes the receiver with the given parameters which must
 * be encoded in the named format.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		params byte[]
 *					parameters encoded in the named format.
 * @param		format String
 *					name of the format in which the parameters are encoded.
 *
 * @exception	IOException
 *					if decoding errors are detected.
 */
protected abstract void engineInit (byte[] params, String format) throws IOException;

/**
 * Initializes the receiver with the provided parameter specification.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		paramSpec AlgorithmParameterSpec
 *					the parameter specification to be used.
 * @exception	InvalidParameterSpecException
 *					if the provided specification is not appropriate
 *					for the receiver.
 */
protected abstract void engineInit (AlgorithmParameterSpec paramSpec)
	throws InvalidParameterSpecException;

/**
 * Answers a string containing a concise, human-readable
 * description of the receiver, describing the parameters.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		String
 *					a printable representation for the receiver,
 *					describing the parameters.
 */
protected abstract String engineToString();
}

