package java.security;

import java.io.*;
/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2001, 2002  All Rights Reserved
 */

/**
 * A certificate is a means fo a principal (the guarantor)
 * to link a public key to another principal.
 *
 * @author		OTI
 * @version		initial
 *
 * @deprecated Use java.security.cert.Certificate
 */
public interface Certificate {

/**
 * Decodes an input stream into a certificate.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	is	java.io.InputStream		the input stream to decode from.
 */
public void decode(java.io.InputStream is) throws KeyException, IOException;
/**
 * Encodes the certificate to the given output stream,
 * in a manner such that it can be decoded by the <code>decode</code> method.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	os	java.io.OutputStream		the Output stream to encode to.
 */
public void encode(java.io.OutputStream os) throws KeyException, IOException;
/**
 * Returns the name of the format used for encoding, e.g "X.509" or "PGP".
 *
 * @author		OTI
 * @version		initial
 *
 * @return	String	the name of the format.
 */
public String getFormat();
/**
 * Returns the guarantor of this certificate, i.e. the principal who guarantees
 * that the public key corresponds to another principal.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	Principal	the guarantor.
 */
public Principal getGuarantor();
/**
 * Returns the principal associated to the public key of this certificate.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	Principal	the principal corresponding to the public key.
 */
public Principal getPrincipal();
/**
 * Returns the public key contained in this certificate.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	PublicKey	the public key associated to the guarantor.
 */
public PublicKey getPublicKey();
/**
 * Returns the string representation of the contents of this certificate.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	detail	boolean	if true indicates that a detailed representation should be given.
 * @return	String	the string representation of this certificate.
 */
public String toString(boolean detail);

}
