package java.security;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

import java.io.Serializable;

/**
 * GuardedObject controls access to an object, by
 * checking all requests for the object with a Guard.
 *
 * @author		OTI
 * @version		initial
 */
public class GuardedObject implements Serializable {
	private static final long serialVersionUID = -5240450096227834308L;

	Object object;
	Guard guard;

/**
 * Constructs a GuardedObject to protect access to the
 * specified Object using the specified Guard.
 *
 * @param	guardedObject	the Object to guard
 * @param	theGuard		the Guard
 */
public GuardedObject(Object guardedObject, Guard theGuard) {
	object = guardedObject;
	guard = theGuard;
}

/**
 * Checks whether access should be granted to the
 * object. If access is granted, this method returns
 * the object. If it is not granted, then a
 * <code>SecurityException</code> is thrown.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the guarded object
 *
 * @exception	java.lang.SecurityException
 *					If access is not granted to the object
 */
public Object getObject() throws SecurityException {
	if (guard != null) guard.checkGuard(object);
	return object;
}
}
