package java.security;

import java.util.*;
import com.ibm.oti.util.PriviAction;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2001, 2006  All Rights Reserved
 */

/**
 * This represents a scope for identities.
 *
 * @author		OTI
 * @version		initial
 *
 * @deprecated Use java.security.KeyStore, java.security.cert package, and java.security.Principal
 */
public abstract class IdentityScope extends Identity {
	private static final long serialVersionUID = -2337346281189773310L;

	static IdentityScope systemScope;
/**
 * Constructs a new instance of this class.
 *
 * @author		OTI
 * @version		initial
 *
 */
protected IdentityScope() {
	super();
}
/**
 * Constructs a new instance of this class with the name provided.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	name	String	the name of this identity scope.
 */
public IdentityScope(String name) {
	super(name);
}
/**
 * Constructs a new instance of this class with the name and scope provided.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	name	String		the name of this identity scope.
 * @param	scope	IdentityScope	the scope of this identity scope.
 */
public IdentityScope(String name, IdentityScope scope) throws KeyManagementException {
	super(name, scope);
}
/**
 * Adds the given identity to this scope.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	identity	Identity	the identity to add to this scope.
 */
public abstract void addIdentity(Identity identity) throws KeyManagementException;
/**
 * Removes the given identity from this scope.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	identity	Identity	the identity to remove from this scope.
 */
public abstract void removeIdentity(Identity identity) throws KeyManagementException;
/**
 * Returns an Enumeration containing all the identities in this scope.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	Enumeration	the identities contained in this scope.
 */
public abstract Enumeration identities();
/**
 * Returns the identity of the given principal, or null if no such identity is found.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	principal	Principal	the principal which identity is to be returned.
 * @return	Identity	the identity of the given principal, or null if no such identity.
 */
public Identity getIdentity(Principal principal) {
	Enumeration en = identities();
	while (en.hasMoreElements()) {
		Identity current = (Identity)en.nextElement();
		if (current.getName().equals(principal.getName()))
			return current;
	}
	return null;
}
/**
 * Returns the identity corresponding to the given public key
 * or null if no such identity is found.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	publicKey	PublicKey	the public key which correponsping identity is to be returned.
 * @return	Identity	the identity corresponding to the given public key, or null if no such identity.
 */
public abstract Identity getIdentity(PublicKey publicKey);
/**
 * Returns the identity having this name, or null if no such identity is found.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	name	String	the name which identity is to be returned.
 * @return	Identity	the identity having the given name, or null if no such identity.
 */
public abstract Identity getIdentity(String name);
/**
 * Sets the system scope to the given identity scope.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	scope	IdentityScope	the scope to set as the system scope.
 */
protected static void setSystemScope(IdentityScope scope) {
	SecurityManager security = System.getSecurityManager();
	if (security != null)
		security.checkSecurityAccess("setSystemScope");
	systemScope = scope;
}
/**
 * Returns the number of identities in this scope.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	int	the number of identities in this scope.
 */
public abstract int size();
/**
 * Returns a string representation of this scope.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	String	a string representation of this scope.
 */
public String toString() {
	String result = "Name : " + this.getName();
	if (getScope() != null)
		result += "\nScope name : " + this.getScope().getName();
	result += "\nNumber of identities : " + this.size();
	return result;
}
/**
 * Returns the system scope.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	IdentityScope	the system scope.
 */
public static IdentityScope getSystemScope() {
	if (systemScope != null) return systemScope;
	String prop = (String)AccessController.doPrivileged(
	        PriviAction.getSecurityProperty("system.scope"));
	if (prop != null) {
		try {
			Class cl = Class.forName(prop);
			systemScope = (IdentityScope)cl.newInstance();
		} catch (ClassNotFoundException e) {
		} catch (IllegalAccessException e) {
		} catch (InstantiationException e) {
		}
	}
	return systemScope;
}

}

