
package java.security;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * Defines the basic properties of all key objects.
 *
 * @author		OTI
 * @version		initial
 *
 * @see			PublicKey
 * @see			PrivateKey
 */

public interface Key extends java.io.Serializable {

	// Set the version id so we are field-compatible with JDK.
	public static final long serialVersionUID = 6603384152749567654L;

/**
 * Answers the name of the algorithm that this key
 * will work with. If the algorithm is unknown, it
 * answers null.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		String
 *					the receiver's algorithm
 */
public abstract String getAlgorithm ();

/**
 * Answers the encoded form of the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		byte[]
 *					the encoded form of the receiver
 */
 public abstract byte[] getEncoded ();

/**
 * Answers the name of the format used to encode
 * the key, or null if it can not be encoded.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		String
 *					the receiver's encoding format
 */
public abstract String getFormat ();

}
