
package java.security;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This class is a Service Provider Interface (therefore the Spi suffix) for key algorithms
 * to be supplied by providers.
 *
 *
 * @author		OTI
 * @version		initial
 */
 import java.security.spec.*;

public abstract class KeyFactorySpi {
/**
 * Constructs a new instance of this class.
 *
 * @author		OTI
 * @version		initial
 */
public KeyFactorySpi() {
	super();
}
/**
 * Generates a private key for the given key specification.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		keySpec KeySpec	the key specification to be used for key generation.
 *
 * @exception	InvalidKeySpecException 	if the provided key specification is not appropriate
 */
protected abstract PrivateKey engineGeneratePrivate(KeySpec keySpec) throws InvalidKeySpecException;

/**
 * Generates a public key for the given key specification.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		keySpec KeySpec	the key specification to be used for key generation.
 *
 * @exception	InvalidKeySpecException 	if the provided key specification is not appropriate
 */
protected abstract PublicKey engineGeneratePublic(KeySpec keySpec) throws InvalidKeySpecException;

/**
 * Answers a key specification for a given key.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		key 		Key		the key for which the specification is to be computed.
 * @param		keySpec 	Class	the class of the result specification
 *
 * @return		a key specification for a given key. The class of the result object will be <code>keySpec</code>.
 * @exception	InvalidKeySpecException 	if the provided key specification is not appropriate
 */
 protected abstract KeySpec engineGetKeySpec(Key key, Class keySpec) throws InvalidKeySpecException;

/**
 * Translates a generic key (any provider, trusted or not) to an instance of
 * this key factory.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		key 		Key		the key for which the new key is to be computed.
 *
 * @return		a key for the given key.
 * @exception	InvalidKeyException if the specified key is not valid (improperly encoded, parameters missing, etc)
 */
protected abstract Key engineTranslateKey(Key key) throws InvalidKeyException;
}
