
package java.security;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This class is a Service Provider Interface (therefore the Spi suffix) for key generation algorithms
 * to be supplied by providers.
 *
 *
 * @author		OTI
 * @version		initial
 */

 import java.security.spec.*;

 public abstract class KeyPairGeneratorSpi {
/**
 * Constructs a new instance of this class.
 *
 * @author		OTI
 * @version		initial
 */
public KeyPairGeneratorSpi() {
	super();
}
/**
 * Answers a newly generated key pair.
 *
 * @author		OTI
 * @version		initial
 *
 *
 * @return		a newly generated key pair.
 *
 */
 public abstract KeyPair generateKeyPair();
/**
 * Initialize the receiver to use a given secure random generator,
 * and generate keys of a certain size.
 *
 * @author		OTI
 * @version		initial
 *
 *
 * @param	keySize	int				New size of keys, in bits
 * @param	secureRandom	SecureRandom	New secure random to use
 *
 */
public abstract void initialize(int keySize, SecureRandom secureRandom);
/**
 * Initialize the receiver to use a given secure random generator,
 * and generate keys from the provided set of parameters.
 *
 * @author		OTI
 * @version		initial
 *
 *
 * @param	params	AlgorithmParameterSpec	parameters for key generation
 * @param	secureRandom	SecureRandom			New secure random to use
 *
 */
 public void initialize(AlgorithmParameterSpec params, SecureRandom secureRandom) throws InvalidAlgorithmParameterException {
	throw new UnsupportedOperationException ();	// See JDK's Javadoc for class KeyPairGenerator
 }

}
