
package java.security;

import java.io.Serializable;
import java.util.Enumeration;
import com.ibm.oti.util.PriviAction;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * Abstract superclass of classes which are collections
 * of Permission objects.
 *
 * @author		OTI
 * @version		initial
 */
public abstract class PermissionCollection implements Serializable {

	/**
	 * Set to true if the collection is read only.
	 */
	private boolean readOnly = false;

/**
 * Constructs a new instance of this class.
 *
 * @author		OTI
 * @version		initial
 */
public PermissionCollection() {
}

/**
 * Adds the argument to the collection.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		permission java.security.Permission
 *					the permission to add to the collection.
 * @exception	IllegalStateException
 *					if the collection is read only.
 */
public abstract void add(Permission permission);

/**
 * Answers an enumeration of the permissions
 * in the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		Enumeration
 *					the permissions in the receiver.
 */
public abstract Enumeration elements();

/**
 * Indicates whether the argument permission is implied
 * by the permissions contained in the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		boolean
 *					<code>true</code> if the argument permission
 *					is implied by the permissions in the receiver,
 *					and <code>false</code> if it is not.
 * @param		permission java.security.Permission
 *					the permission to check
 */
public abstract boolean implies(Permission permission);

/**
 * Indicates whether new permissions can be added
 * to the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		boolean
 *					<code>true</code> if the receiver is read only
 *					<code>false</code> if new elements can
 *					still be added to the receiver.
 */
public boolean isReadOnly() {
	return readOnly;
}

/**
 * Marks the receiver as read only, so that no new
 * permissions can be added to it.
 *
 * @author		OTI
 * @version		initial
 */
public void setReadOnly() {
	readOnly = true;
}

/**
 * Answers a string containing a concise, human-readable
 * description of the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		a printable representation for the receiver.
 */
public String toString() {
	String newline = (String)AccessController.doPrivileged(new PriviAction("line.separator", "\n"));
	StringBuffer answer = new StringBuffer();
	answer.append( super.toString() );
	answer.append(" (");
	Enumeration perms = elements();
	if (perms.hasMoreElements())
		answer.append(newline);
	while (perms.hasMoreElements()) {
		Permission p = (Permission) perms.nextElement();
		answer.append("    ");
		answer.append(p);
		answer.append(newline);
	}
	answer.append(")");
	return answer.toString();
}

}
