
package java.security;

import java.util.Enumeration;
import java.util.Hashtable;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

/**
 * A simple Hashtable based collection of Permission objects.
 * <p>
 * The class' .implies method simply scans each permission
 * individually and asks if the permission should be granted.
 * No addition semantics is provided by the collection, so it is
 * not possible to grant permissions whose "grantedness" is
 * split across multiple stored Permissions.
 * <p>
 * Instances of this class can be used to store heterogeneous
 * collections of permissions, as long as it is not necessary
 * to remember when multiple occurances of .equal permissions
 * are added.
 *
 * @author		OTI
 * @version		initial
 */

class PermissionsHash extends PermissionCollection {
	private static final long serialVersionUID = -8491988220802933440L;

	/**
	 * A hashtable to store the elements of the collection.
	 */
	Hashtable perms = new Hashtable(8);

/**
 * Constructs a new instance of this class.
 *
 * @author		OTI
 * @version		initial
 */
public PermissionsHash(){
	super();
}

/**
 * Adds the argument to the collection.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		perm java.security.Permission
 *					the permission to add to the collection.
 * @exception	IllegalStateException
 *					if the collection is read only.
 */
public void add(Permission perm) {
	if (isReadOnly())
		throw new IllegalStateException();
	else
		perms.put(perm, perm);
}

/**
 * Answers an enumeration of the permissions
 * in the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		Enumeration
 *					the permissions in the receiver.
 */
public Enumeration elements() {
	return perms.keys();
}

/**
 * Indicates whether the argument permission is implied
 * by the permissions contained in the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		boolean
 *					<code>true</code> if the argument permission
 *					is implied by the permissions in the receiver,
 *					and <code>false</code> if it is not.
 * @param		perm java.security.Permission
 *					the permission to check
 */
public boolean implies(Permission perm) {
	Enumeration elemEnum = elements();
	while (elemEnum.hasMoreElements())
		if (((Permission) elemEnum.nextElement()).implies(perm))
			return true;
	return false;
}

}

