package java.security;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2001, 2006  All Rights Reserved
 */

/**
 * A Signer is an identity that can sign data.
 *
 * @author		OTI
 * @version		initial
 *
 * @deprecated Use java.security.KeyStore, java.security.cert package, and java.security.Principal
 */
public abstract class Signer extends Identity {
	private static final long serialVersionUID = -1763464102261361480L;

	PrivateKey privateKey;
/**
 * Constructs a new instance of this class.
 *
 * @author		OTI
 * @version		initial
 *
 */
protected Signer() {
}
/**
 * Constructs a new instance of this class with the name provided.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	name	String	the name of the Signer.
 */
public Signer(String name) {
	super(name);
}
/**
 * Constructs a new instance of this class with the name and scope provided.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	name	String	the name of the Signer.
 * @param	scope	IdentityScope	the scope of this Signer.
 */
public Signer(String name, IdentityScope scope) throws KeyManagementException {
	super(name, scope);
}
/**
 * Returns the private key of this signer.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	PrivateKey	the private key of this signer, or null if none.
 */
public PrivateKey getPrivateKey() {
	SecurityManager security = System.getSecurityManager();
	if (security != null)
		security.checkSecurityAccess("getSignerPrivateKey");
	return privateKey;
}
/**
 * Sets the private and public keys of this signer to the given key pair.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	keyPair	KeyPair	the pair containing the private and public keys.
 */
public final void setKeyPair(KeyPair keyPair) throws InvalidParameterException, KeyException {
	SecurityManager security = System.getSecurityManager();
	if (security != null)
		security.checkSecurityAccess("setSignerKeyPair");
	PublicKey pubKey = keyPair.getPublic();
	PrivateKey priKey = keyPair.getPrivate();
	if (priKey == null || pubKey == null)
		throw new InvalidParameterException();
	setPublicKey(pubKey);
	privateKey = priKey;
}
/**
 * Returns the String representation of this Signer.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	String	the string representation of this Signer.
 */
public String toString() {
	return super.toString();
}
}
