
package java.security.acl;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * This interface represents Access Control Lists, used to
 * guard access to resources. It is a collection of AclEntry.
 *
 * @author		OTI
 * @version		initial
 *
 * @see AclEntry
 */

 import java.util.*;
 import java.security.*;

public abstract interface Acl extends Owner {
/**
 * Adds an ACL entry to the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	caller	Principal	The caller principal, must also be the owner of this ACL
 * @param	entry	AclEntry	The ACL entry to be inserted.
 *
 * @exception NotOwnerException 	If the <code>caller</code> is not the owner of this ACL
 */
public boolean addEntry(Principal caller, AclEntry entry) throws NotOwnerException;
/**
 * Answers if the specified Principal has the specified Permission or not.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	principal	Principal	The principal for which the caller wants to perform the check
 * @param	permission	Permission	The permission to be checked
 *
 * @return	true if the Principal has teh Permission, false otherwise.
 */
public boolean checkPermission(Principal principal, Permission permission);
/**
 * Answers a collection of AclEntry for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	a collection of AclEntry for the receiver.
 */
public Enumeration entries();
/**
 * Answers the name of this ACL
 *
 * @author		OTI
 * @version		initial
 *
 * @return	String	the name of this ACL
 */
public String getName();
/**
 * Answers a collection of permissions for the specified principal.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	user	Principal	The principal for which the caller wants to obtain the Permissions
 *
 * @return	a collection of permissions for the specified principal.
 */
public Enumeration getPermissions(Principal user);
/**
 * Removes an ACL entry from the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	caller	Principal	The caller principal, must also be the owner of this ACL
 * @param	entry	AclEntry	The ACL entry to be removed.
 *
 * @return	true on sucess, false if the AclEntry was not part of teh receiver
 *
 * @exception NotOwnerException 	If the <code>caller</code> is not the owner of this ACL
 */
public boolean removeEntry(Principal caller, AclEntry entry) throws NotOwnerException;
/**
 * Sets the name of this ACL
 *
 * @author		OTI
 * @version		initial
 *
 * @param	caller	Principal	The caller principal, must also be the owner of this ACL
 * @param	name	String		The new name of this ACL
 *
 * @exception NotOwnerException 	If the <code>caller</code> is not the owner of this ACL
 */
public void setName(Principal caller, String name) throws NotOwnerException;
/**
 * Answers a string containing a concise, human-readable
 * description of the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		a printable representation for the receiver.
 */
public String toString ();
}
