
package java.security.acl;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * This interface represents one entry in an Access Control List
 *
 * @author		OTI
 * @version		initial
 *
 * @see Acl
 */

 import java.util.*;
 import java.security.*;

 public interface AclEntry extends Cloneable {
/**
 * Adds the specified permission to the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		permission	Permission	The permission to add.
 *
 * @return		true if the permission was added, false if the permission was already in the receiver
 */
public boolean addPermission(Permission permission);
/**
 * Checks if the specified permission is part of the permisisons held by the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		permission	Permission	The permission to verify.
 *
 * @return		true if the permission is part, false if the permission is not in the receiver
 */
public boolean checkPermission(Permission permission);
/**
 * Answers a new instance of the same class as the receiver,
 * whose slots have been filled in with the values in the
 * slots of the receiver.
 * <p>
 * Classes which wish to support cloning must specify that
 * they implement the Cloneable interface, since the native
 * implementation checks for this.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		a shallow copy of this object
 */
public Object clone();
/**
 * Answers the Principal for which permissions are granted or denied by the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the Principal for which permissions are granted or denied by the receiver.
 *				<code>null</code> indicates no Principal was set yet.
 */
public Principal getPrincipal();
/**
 * Answers whether the receiver represents a negative (denial) permission
 *
 * @author		OTI
 * @version		initial
 *
 * @return		true if the receiver represents a negative (denial) permission, false otherwise.
 */
public boolean isNegative();
/**
 * Answers an enumeration on the permissions for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		an enumeration on the permissions for the receiver.
 */
public Enumeration permissions();
/**
 * Removes the specified permission from the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		permission	Permission	The permission to remove.
 *
 * @return		true if the permission was removed, false otherwise
 */
public boolean removePermission(Permission permission);
/**
 * Sets the receiver as a negative (denial) permission
 *
 * @author		OTI
 * @version		initial
 *
 */
public void setNegativePermissions();
/**
 * Sets the Principal for which permissions are granted or denied by the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		user	Princiapl	the Principal for which permissions are granted or denied by the receiver.
 *
 * @return		true if the Principal was set, false if the Principal had already been set previously.
 */
public boolean setPrincipal(Principal user);
/**
 * Answers a string containing a concise, human-readable
 * description of the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		a printable representation for the receiver.
 */
public String toString();
}
