
package java.security.acl;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 * This interface is used for managing owners of ACLs.
 *
 * @author		OTI
 * @version		initial
 *
 * @see Principal
 * @see Acl
 */

 import java.security.*;

 public abstract interface Owner {
/**
 * Adds the specified owner to the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		caller	Principal	The caller principal, must also be the owner of the ACL
 * @param		owner	Principal	The principal to add.
 *
 * @exception	NotOwnerException	If the <code>caller</code> is not the owner of the ACL
 *
 * @return		true if the principal was added, false if the principal was already owner.
 */
public boolean addOwner(Principal caller, Principal owner) throws NotOwnerException;
/**
 * Adds the specified owner to the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		caller	Principal	The caller principal, must also be the owner of the ACL
 * @param		owner	Principal	The principal to remove.
 *
 * @exception	NotOwnerException	If the <code>caller</code> is not the owner of the ACL
 * @exception	LastOwnerException	If there is only one owner left.
 *
 * @return		true if the principal was removed, false if the principal was not part of the list.
 */
public boolean deleteOwner(Principal caller, Principal owner) throws NotOwnerException, LastOwnerException;
/**
 * Answers whether the given principal is the owner of the ACL or not.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		owner	Principal	The principal to check.
 *
 * @return		true if the principal is the owner, false otherwise
 */
public boolean isOwner(Principal owner);
}
