
package java.security.cert;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 *	This class represents Certificate Revocation Lists (CRLs). They are used
 * to indicate that a given Certificate has expired already.
 *
 * @author		OTI
 * @version		initial
 *
 * @see CertificateFactory
 */

 public abstract class CRL {
	private String type;		// Type of this CRL
/**
 * Constructs a new instance of this class for a given CRL type.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	type	String	type of this CRL
 */
protected CRL(String type) {
	super();
	this.type = type;
}
/**
 * Answers the type of this CRL.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	String	the type of this CRL.
 */
public final String getType() {
	return type;
}
/**
 * Answers if a given Certificate has been revoked or not.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	cert	Certificate	The Certificate to test
 *
 * @return	true
 *	 			if the certificate has been revoked
 *	 		false
 *	 			if the certificate has not been revoked yet
 */
public abstract boolean isRevoked(Certificate cert);
/**
 * Answers a string containing a concise, human-readable
 * description of the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		a printable representation for the receiver.
 */
public abstract String toString ();
}
