
package java.security.cert;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This class is a Service Provider Interface (therefore the Spi suffix) for certificate
 * factories to be supplied by providers.
 *
 *
 * @author		OTI
 * @version		initial
 */
 import java.io.*;
 import java.util.*;

public abstract class CertificateFactorySpi {
/**
 * Constructs a new instance of this class.
 *
 * @author		OTI
 * @version		initial
 */
public CertificateFactorySpi() {
	super();
}
/**
 * Generates and initializes a Certificate from data from the provided
 * input stream.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		inStream InputStream	Stream from where data is read to create the Certificate
 *
 * @return	Certificate 	an initialized Certificate
 * @exception	CertificateException 	if parsing problems are detected
 */
public abstract Certificate engineGenerateCertificate(InputStream inStream) throws CertificateException;
/**
 * Generates and initializes a collection of Certificates from data from the provided
 * input stream.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		inStream InputStream	Stream from where data is read to create the Certificates
 *
 * @return	Collection	 	an initialized collection of Certificates
 * @exception	CertificateException 	if parsing problems are detected
 */
public abstract Collection engineGenerateCertificates(InputStream inStream) throws CertificateException;
/**
 * Generates and initializes a Certificate Revocation List from data from the provided
 * input stream.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		inStream InputStream	Stream from where data is read to create the CRL
 *
 * @return	CRL 	an initialized Certificate Revocation List
 * @exception	CRLException 	if parsing problems are detected
 */
public abstract CRL engineGenerateCRL(InputStream inStream) throws CRLException;
/**
 * Generates and initializes a collection of Certificate Revocation List from data from the provided
 * input stream.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		inStream InputStream	Stream from where data is read to create the CRLs
 *
 * @return	Collection		an initialized collection of Certificate Revocation List
 * @exception	CRLException 	if parsing problems are detected
 */
public abstract Collection engineGenerateCRLs(InputStream inStream) throws CRLException;

/**
 * Returns an Iterator over the supported CertPath encodings (as Strings). The first
 * element is the default encoding.
 * @return Iterator  Iterator over supported CertPath encodings (as Strings)
 */
public Iterator engineGetCertPathEncodings()
{
	throw new UnsupportedOperationException();
}

/**
 * Generates a <code>CertPath</code> from data from the provided <code>InputStream</code>. The
 * default encoding is assumed.
 *
 * @param inStream  InputStream with PKCS7 or PkiPath encoded data
 *
 * @return CertPath	 a CertPath initialized from the provided data
 *
 * @throws CertificateException  if parsing problems are detected
 */
public CertPath engineGenerateCertPath(InputStream inStream) throws CertificateException
{
	Collection certificates = engineGenerateCertificates( inStream );
	List certList = new ArrayList( certificates );
	return engineGenerateCertPath( certList );
}

/**
 * Generates a <code>CertPath</code> from data from the provided <code>InputStream</code>. The
 * encoding is that specified by the encoding parameter.
 *
 * @param inStream  InputStream containing certificate path data in specified encoding
 * @param encoding  encoding of the data in the input stream
 *
 * @return CertPath  a CertPath initialized from the provided data
 *
 * @throws CertificateException  if parsing problems are detected
 * @throws UnsupportedOperationException  if the provider does not implement this method
 */
public CertPath engineGenerateCertPath(InputStream inStream, String encoding ) throws CertificateException
{
	throw new UnsupportedOperationException();
}

/**
 * Generates a <code>CertPath</code> from the provided List of Certificates.  The encoding is
 * the default encoding.
 *
 * @param certificates  List containing certificates in a format supported by the CertificateFactory
 *
 * @return CertPath  a CertPath initialized from the provided data
 *
 * @throws CertificateException  if parsing problems are detected
 * @throws UnsupportedOperationException  if the provider does not implement this method
 */
public CertPath engineGenerateCertPath(List certificates) throws CertificateException
{
	throw new UnsupportedOperationException();
}
}

