
package java.security.cert;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 *	This class defines extensions for X.509 v3 Certificates and v2 CRLs (Certificate Revocation Lists)
 *
 * @author		OTI
 * @version		initial
 *
 * @see X509Certificate
 */

import java.util.*;

public interface X509Extension {

/**
 * Return true if the receiver has an unsupported, critical extension. Return
 * false otherwise.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		boolean		true if the receiver has an unsupported, critical extension, false otherwise.
 */
public boolean hasUnsupportedCriticalExtension();

/**
 * Return all the OID extensions marked as critical for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		Set		all the OID extensions marked as critical for the receiver.
 */
public Set getCriticalExtensionOIDs();

/**
 * Return all the OID extensions marked as non-critical for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		Set		all the OID extensions marked as non-critical for the receiver.
 */
public Set getNonCriticalExtensionOIDs();

/**
 * Return the DER-encoded OCTECT String for the extension value defined by oid in the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		oid	String	The ASN.1 OID for which extension value will be fetched
 * @return		byte[]	the DER-encoded OCTECT String for the extension value defined by oid in the receiver.
 */
public byte[] getExtensionValue(String oid);

}
