
package java.security.interfaces;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 * This interface represents objects capable of generating DSA key pairs.
 *
 * @author		OTI
 * @version		initial
 *
 * @see KeyPairGenerator
 */

import java.security.*;

public abstract interface DSAKeyPairGenerator {
/**
 * Initialize the receiver with the given DSA parameters (p, q and g).
 *
 * @author		OTI
 * @version		initial
 *
 * @param		params	DSAParams		Parameters to use to generate the keys.
 * @param		random	SecureRandom	Source of randomness to use when generating the keys.
 *
 * @throws		InvalidParameterException	If the parameters passed are not valid.
 *
 */
 public void initialize(DSAParams params, SecureRandom random) throws InvalidParameterException;

/**
 * Initialize the receiver with the given modulus length.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		modlen		int			Modulus length in bits, multiple of 8, between 512 and 1024.
 * @param		genParams	boolean		<code>true</code> if the DSA parameters should be generated, <code>false</code> if
 *										pre-computed parameters should be used instead.
 * @param		random	SecureRandom	Source of randomness to use when generating the keys.
 *
 * @throws		InvalidParameterException	If the modulus length is not between 512 and 1024, or if <code>genparams</code>
 *											is <code>false</code> but there is no set of pre-computed parameters of the desired size.
 *
 */public void initialize(int modlen, boolean genParams, SecureRandom random) throws InvalidParameterException;

}
