
package java.security.interfaces;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 * This interface represents RSA private keys, as defined in the PKCS#1 standard.
 * Uses the Chinese Remainder Theorem (CRT) values.
 *
 *
 * @author		OTI
 * @version		initial
 *
 * @see RSAPrivateKey
 */

 import java.math.BigInteger;

public abstract interface RSAPrivateCrtKey extends RSAPrivateKey{

/**
 * Return the public exponent for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	the public exponent for the receiver.
 *
 */
 public BigInteger getPublicExponent();

/**
 * Return the value of "primeP" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	the value of "primeP" for the receiver.
 *
 */
 public BigInteger getPrimeP();

/**
 * Return the value of "primeQ" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	the value of "primeQ" for the receiver.
 *
 */
 public BigInteger getPrimeQ();

/**
 * Return the value of "primeExponentP" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	the value of "primeExponentP" for the receiver.
 *
 */
 public BigInteger getPrimeExponentP();

/**
 * Return the value of "getPrimeExponentQ" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	the value of "getPrimeExponentQ" for the receiver.
 *
 */
 public BigInteger getPrimeExponentQ();

/**
 * Return the value of "crtCoefficient" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	the value of "crtCoefficient" for the receiver.
 *
 */
 public BigInteger getCrtCoefficient();

}
