
package java.security.spec;

import java.security.interfaces.*;

import java.math.*;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This interface represents DSA-specific key parameters.
 *
 * @author		OTI
 * @version		initial
 *
 * @see 		DSAKey
 * @see 		Key
 * @see 		Signature
 */
public class DSAParameterSpec implements AlgorithmParameterSpec, DSAParams {

	private BigInteger p, q, g;	// The DSA parameters
/**
 * Return the DSA parameter "g" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	parameter "g" for the receiver.
 *
 */
public BigInteger getG(){
	return g;
}

/**
 * Return the DSA parameter "p" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	parameter "p" for the receiver.
 *
 */
public BigInteger getP(){
	return p;
}

/**
 * Return the DSA parameter "q" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	parameter "q" for the receiver.
 *
 */
public BigInteger getQ(){
	return q;
}

/**
 * Constructs a new instance of this class with the parameters as provided.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	p	BigInteger	The DSA prime
 * @param	q	BigInteger	The DSA sub-prime
 * @param	g	BigInteger	The DSA base
 */
public DSAParameterSpec(BigInteger p, BigInteger q, BigInteger g) {
	this.p = p;
	this.q = q;
	this.g = g;
}

}
