
package java.security.spec;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 * This class represents a DSA-specific private key and its parameters.
 *
 * @author		OTI
 * @version		initial
 *
 * @see 		Key
 * @see 		KeyFactory
 * @see 		KeySpec
 * @see 		DSAPublicKeySpec
 */

import java.math.*;

public class DSAPrivateKeySpec implements KeySpec {

	private BigInteger p, q, g, x;	// The DSA parameters and the private key
/**
 * Return the DSA parameter "g" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	parameter "g" for the receiver.
 *
 */
public BigInteger getG(){
	return g;
}

/**
 * Return the DSA parameter "p" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	parameter "p" for the receiver.
 *
 */
public BigInteger getP(){
	return p;
}

/**
 * Return the DSA parameter "q" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	parameter "q" for the receiver.
 *
 */
public BigInteger getQ(){
	return q;
}

/**
 * Return the value of the private key.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	the actual private key.
 *
 */
 public BigInteger getX() {
 	return x;
 }

/**
 * Constructs a new instance of this class with the parameters as provided.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	x	BigInteger	The actual DSA private key
 * @param	p	BigInteger	The DSA prime
 * @param	q	BigInteger	The DSA sub-prime
 * @param	g	BigInteger	The DSA base
 */
public DSAPrivateKeySpec(BigInteger x,BigInteger p, BigInteger q, BigInteger g) {
	this.x = x;
	this.p = p;
	this.q = q;
	this.g = g;
}

}
