
package java.security.spec;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 * This class represents a DSA-specific private key and its parameters.
 *
 * @author		OTI
 * @version		initial
 *
 * @see 		Key
 * @see 		KeyFactory
 * @see 		KeySpec
 * @see 		DSAPublicKeySpec
 */

import java.math.*;

public class DSAPublicKeySpec implements KeySpec {

	private BigInteger p, q, g, y;	// The DSA parameters and the public key
/**
 * Return the DSA parameter "g" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	parameter "g" for the receiver.
 *
 */
public BigInteger getG(){
	return g;
}

/**
 * Return the DSA parameter "p" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	parameter "p" for the receiver.
 *
 */
public BigInteger getP(){
	return p;
}

/**
 * Return the DSA parameter "q" for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	parameter "q" for the receiver.
 *
 */
public BigInteger getQ(){
	return q;
}

/**
 * Return the value of the public key.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	the actual public key.
 *
 */
 public BigInteger getY() {
 	return y;
 }

/**
 * Constructs a new instance of this class with the parameters as provided.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	y	BigInteger	The actual DSA public key
 * @param	p	BigInteger	The DSA prime
 * @param	q	BigInteger	The DSA sub-prime
 * @param	g	BigInteger	The DSA base
 */
public DSAPublicKeySpec(BigInteger y,BigInteger p, BigInteger q, BigInteger g) {
	this.y = y;
	this.p = p;
	this.q = q;
	this.g = g;
}

}
