
package java.security.spec;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * This class represents encoded keys, public of private.
 *
 * @author		OTI
 * @version		initial
 */
public abstract class EncodedKeySpec implements KeySpec {
	private byte[] encodedKey;	// Key in encoded format
/**
 * Constructs a new instance of this class with its encoded representation.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	encodedKey	byte[]	Encoded representation of the key
 */
public EncodedKeySpec(byte[] encodedKey) {
	super();
	if (encodedKey == null) {
		// K039e = encoded key spec is null
		throw new NullPointerException(com.ibm.oti.util.Msg.getString("K039e")); //$NON-NLS-1$
	}
	this.encodedKey = encodedKey;
}
/**
 * Return the encoded representation of the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	byte[]	Encoded representation of the key
 */
public byte[] getEncoded() {
	return (byte[])encodedKey.clone();
}
/**
 * Return the name of the format in which the receiver is encoded.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	String	name of the format in which the receiver is encoded.
 */
public abstract String getFormat();
}
