package java.security.spec;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2001, 2006  All Rights Reserved
 */

public class PSSParameterSpec implements AlgorithmParameterSpec {

	private int saltLength = 0;

	/**
	 * Creates a PSSParameterSpec with the specified salt length (as defined in PKCS1)
	 * @param saltLen length of salt (in bits) to be used in PSS encoding
	 * @throws IllegalArgumentException if saltLen < 0
	 */
	public PSSParameterSpec(int saltLen){
		if( saltLen < 0 ){
			throw new IllegalArgumentException( "Salt length < 0 " );
		}
		this.saltLength = saltLen;
	}

	/**
	 * Get the salt length of this PSSParameterSpec
	 * @return int salt length
	 */
	public int getSaltLength(){
		return this.saltLength;
	}

}
