package java.security.spec;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2001, 2004  All Rights Reserved
 */

import java.math.*;

/**
 * This class represents the parameters required to create RSA keys.
 *
 * @author		OTI
 * @version		initial
 *
 * @see 		java.security.Key
 * @see 		java.security.Signature
 */
public class RSAKeyGenParameterSpec implements AlgorithmParameterSpec {
	public static final BigInteger F0 = new BigInteger("3");
	public static final BigInteger F4 = new BigInteger("65537");
	private int keysize;
	private BigInteger publicExponent;

/**
 * Constructs a new instance of this class with the parameters as provided.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	size	int	The RSA key size
 * @param	exponent BigInteger	The RSA public exponent for this key
 */
public RSAKeyGenParameterSpec(int size, BigInteger exponent) {
	keysize = size;
	publicExponent = exponent;
}

/**
 * Return the RSA public exponent for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	public exponent for the receiver.
 *
 */
public BigInteger getPublicExponent() {
	return publicExponent;
}

/**
 * Return the RSA key size.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		int	the key size for the receiver.
 *
 */
public int getKeysize() {
	return keysize;
}

}
