package java.security.spec;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * This interface represents RSA-specific private keys.
 *
 * @author		OTI
 * @version		initial
 *
 * @see 		Key
 * @see 		Signature
 */

import java.math.*;

public class RSAPrivateKeySpec extends Object implements KeySpec {

	private BigInteger modulus;			// RSA modulus
	private BigInteger privateExponent;	// RSA private exponent

/**
 * Constructs a new instance of this class with the parameters as provided.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	modulus	BigInteger	The RSA modulus for this key
 * @param	privateExponent	BigInteger	The RSA private exponent for this key
 */
public RSAPrivateKeySpec(BigInteger modulus, BigInteger privateExponent) {
	this.modulus = modulus;
	this.privateExponent = privateExponent;
}

/**
 * Return the RSA modulus for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	modulus for the receiver.
 *
 */
public BigInteger getModulus() {
	return modulus;
}

/**
 * Return the RSA private exponent for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	private exponent for the receiver.
 *
 */
public BigInteger getPrivateExponent() {
	return privateExponent;
}

}
