package java.security.spec;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * This interface represents RSA-specific public keys.
 *
 * @author		OTI
 * @version		initial
 *
 * @see 		Key
 * @see 		Signature
 */

import java.math.*;

public class RSAPublicKeySpec extends Object implements KeySpec {

	private BigInteger modulus;			// RSA modulus
	private BigInteger publicExponent;	// RSA public exponent

/**
 * Constructs a new instance of this class with the parameters as provided.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	modulus	BigInteger	The RSA modulus for this key
 * @param	publicExponent BigInteger	The RSA public exponent for this key
 */
public RSAPublicKeySpec(BigInteger modulus, BigInteger publicExponent) {
	this.modulus = modulus;
	this.publicExponent = publicExponent;
}

/**
 * Return the RSA modulus for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	modulus for the receiver.
 *
 */
public BigInteger getModulus() {
	return modulus;
}

/**
 * Return the RSA public exponent for the receiver.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		BigInteger	public exponent for the receiver.
 *
 */
public BigInteger getPublicExponent() {
	return publicExponent;
}

}
