package java.text;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

import java.io.Serializable;
import java.io.InvalidObjectException;
import java.util.Set;
import java.util.Map;

/**
 * AttributedCharacterIterator
 *
 * @author		OTI
 * @version		initial
 */
public interface AttributedCharacterIterator extends CharacterIterator {

	public static class Attribute implements Serializable {
		private static final long serialVersionUID = -9142742483513960612L;

		public static final Attribute INPUT_METHOD_SEGMENT =
			new Attribute("input_method_segment");
		public static final Attribute LANGUAGE =
			new Attribute("language");
		public static final Attribute READING =
			new Attribute("reading");

		private String name;

		protected Attribute(String name) {
			this.name = name;
		}

		public final boolean equals(Object object) {
			return this == object;
		}

		protected String getName() {
			return name;
		}

		public final int hashCode() {
			return name.hashCode();
		}

		protected Object readResolve() throws InvalidObjectException {
			if (this.getClass() != Attribute.class)
				// K000c = cannot resolve subclasses
				throw new InvalidObjectException(com.ibm.oti.util.Msg.getString("K000c"));
			if (INPUT_METHOD_SEGMENT.name.equals(this.name))
				return INPUT_METHOD_SEGMENT;
			if (LANGUAGE.name.equals(this.name))
				return LANGUAGE;
			if (READING.name.equals(this.name))
				return READING;
			// K000d = Unknown attribute
			throw new InvalidObjectException(com.ibm.oti.util.Msg.getString("K000d"));
		}

		public String toString() {
			return getClass().getName() + '(' + getName() + ')';
		}
	}

public Set getAllAttributeKeys();
public Object getAttribute(Attribute attribute);
public Map getAttributes();
public int getRunLimit();
public int getRunLimit(Attribute attribute);
public int getRunLimit(Set attributes);
public int getRunStart();
public int getRunStart(Attribute attribute);
public int getRunStart(Set attributes);
}
