package java.text;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * CharacterIterator is used to sequence over a group of characters. The
 * iteration starts at the begin index in the group of character and continues
 * to one index before the end index.
 *
 * @author		OTI
 * @version		initial
 */

public interface CharacterIterator extends Cloneable {
	/**
	 * A constant which indicates there is no character.
	 */
	public static char DONE = '\uffff';

/**
 * Answers a new CharacterIterator with the same properties.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		a shallow copy of this CharacterIterator
 *
 * @see			java.lang.Cloneable
 */
public Object clone();
/**
 * Answers the character at the current index.
 *
 * @author		OTI
 * @version		initial
 *
 * @return the current character, or DONE if the current index is past the end
 */
public char current();
/**
 * Sets the current position to the begin index and answers
 * the character at the begin index.
 *
 * @author		OTI
 * @version		initial
 *
 * @return the character at the begin index
 */
public char first();
/**
 * Answers the begin index.
 *
 * @author		OTI
 * @version		initial
 *
 * @return the index of the first character to iterate
 */
public int getBeginIndex();
/**
 * Answers the end index.
 *
 * @author		OTI
 * @version		initial
 *
 * @return the index one past the last character to iterate
 */
public int getEndIndex();
/**
 * Answers the current index.
 *
 * @author		OTI
 * @version		initial
 *
 * @return the current index
 */
public int getIndex();
/**
 * Sets the current position to the end index - 1 and answers
 * the character at the current position.
 *
 * @author		OTI
 * @version		initial
 *
 * @return the character before the end index
 */
public char last();
/**
 * Increments the current index and returns the character at
 * the new index.
 *
 * @author		OTI
 * @version		initial
 *
 * @return the character at the next index, or DONE if the next index is past the end
 */
public char next();
/**
 * Decrements the current index and returns the character at
 * the new index.
 *
 * @author		OTI
 * @version		initial
 *
 * @return the character at the previous index, or DONE if the previous index is past the beginning
 */
public char previous();
/**
 * Sets the current index.
 *
 * @author		OTI
 * @version		initial
 *
 * @return the character at the new index, or DONE if the index is past the end
 *
 * @exception	IllegalArgumentException when the new index is less than the begin index
 *				or greater than the end index
 */
public char setIndex(int location);
}
