package java.text;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * CollationKey represents the collation order of a particular String for a specific
 * Collator. CollationKeys can be compared to determine the relative ordering of their
 * source Strings. This is useful when the Strings must be compared multiple times, as
 * in sorting.
 *
 * @author		OTI
 * @version		initial
 */

public final class CollationKey implements Comparable {
	private String string, key;

CollationKey(String string, String key) {
	this.string = string;
	this.key = key;
}
/**
 * Compare the receiver to the specified CollationKey to determine
 * the relative ordering.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		value	a CollationKey
 * @return		an int < 0 if this CollationKey is less than the specified CollationKey,
 *				0 if they are equal, and > 0 if this CollationKey is greater
 */
public int compareTo(CollationKey value) {
	return key.compareTo(value.key);
}
/**
 * Compare this CollationKey to the specified Object to determine
 * the relative ordering.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		object	an Object
 * @return		an int < 0 if this CollationKey is less than the specified CollationKey,
 *				0 if they are equal, and > 0 if this CollationKey is greater
 *
 * @exception	ClassCastException when object is not a CollationKey
 */
public int compareTo(Object object) {
	return key.compareTo(((CollationKey)object).key);
}
/**
 * Compares the specified object to this CollationKey and answer if they
 * are equal. The object must be an instance of CollationKey and have the
 * same source string and collation key. The instances of CollationKey must
 * have been created by the same Collator.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		object	the object to compare with this object
 * @return		true if the specified object is equal to this CollationKey, false otherwise
 *
 * @see			#hashCode
 */
public boolean equals(Object object) {
	if (!(object instanceof CollationKey)) return false;
	CollationKey collationKey = (CollationKey)object;
	return key.equals(collationKey.key);
}
/**
 * Answer the String from which this CollationKey was created.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		a String
 */
public String getSourceString() {
	return string;
}
/**
 * Answers an integer hash code for the receiver. Objects which are
 * equal answer the same value for this method.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the receiver's hash
 *
 * @see			#equals
 */
public int hashCode() {
	return key.hashCode();
}
/**
 * Answer the collation key as a byte array.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		an array of bytes
 */
public byte[] toByteArray() {
	int length = key.length();
	byte[] bytes = new byte[length * 2];
	for (int i=0; i<length; i++) {
		char ch = key.charAt(i);
		bytes[i*2] = (byte)(ch >> 8);
		bytes[i*2+1] = (byte)(ch & 0xff);
	}
	return bytes;
}
}
