package java.text;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

/**
 * FieldPosition is used to identify fields in formatted Strings.
 *
 * @author		OTI
 * @version		initial
 */

public class FieldPosition {
	private int myField, beginIndex, endIndex;
	private Format.Field myAttribute;

/**
 * Constructs a new FieldPosition on the specified field.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		field	the field to identify
 */
public FieldPosition(int field) {
	myField = field;
}

/**
 * Constructs a new FieldPosition on the specified Field attribute.
 *
 * @author		IBM
 * @version		initial
 *
 * @param		attribute	the field attribute to identify
 */
public FieldPosition(Format.Field attribute) {
	myAttribute = attribute;
	myField = -1;
}
/**
 * Constructs a new FieldPosition on the specified Field attribute and field id.
 *
 * @author		IBM
 * @version		initial
 * @param		attribute	the field attribute to identify
 * @param		field	the field to identify
 */
public FieldPosition(Format.Field attribute, int field) {
	myAttribute = attribute;
	myField = field;
}

void clear() {
	beginIndex = endIndex = 0;
}
/**
 * Compares the specified object to this FieldPosition and answer if they
 * are equal. The object must be an instance of FieldPosition with the
 * same field, begin index and end index.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		object	the object to compare with this object
 * @return		true if the specified object is equal to this fieldPosition, false otherwise
 *
 * @see			#hashCode
 */
public boolean equals(Object object) {
	if (!(object instanceof FieldPosition)) return false;
	FieldPosition pos = (FieldPosition)object;
	return myField == pos.myField &&
			myAttribute == pos.myAttribute &&
			beginIndex == pos.beginIndex &&	endIndex == pos.endIndex;
}
/**
 * Answers the index of the beginning of the field.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	the first index of the field
 */
public int getBeginIndex() {
	return beginIndex;
}
/**
 * Answers the index one past the end of the field.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	one past the index of the last character in the field
 */
public int getEndIndex() {
	return endIndex;
}
/**
 * Answers the field which is being identified.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	the field
 */
public int getField() {
	return myField;
}

/**
 * Answers the attribute which is being identified.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	the field
 */
public Format.Field getFieldAttribute() {
	return myAttribute;
}
/**
 * Answers an integer hash code for the receiver. Objects which are
 * equal answer the same value for this method.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the receiver's hash
 *
 * @see			#equals
 */
public int hashCode() {
	int attributeHash = (myAttribute == null)?0:myAttribute.hashCode();
	return attributeHash + myField*10 + beginIndex*100 + endIndex;
}
/**
 * Sets the index of the beginning of the field.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		index	the index of the first character in the field
 */
public void setBeginIndex(int index) {
	beginIndex = index;
}
/**
 * Sets the index of the end of the field.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		index	one past the index of the last character in the field
 */
public void setEndIndex(int index) {
	endIndex = index;
}
/**
 * Answers the string representation of this FieldPosition.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the string representation of this FieldPosition
 */
public String toString() {
	return getClass().getName()+
		"[attribute="+myAttribute+", field="+myField+ ", beginIndex="+beginIndex+", endIndex="+endIndex+"]";
}
}
