package java.text;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * ParsePosition is used to track the current position in a String being parsed.
 *
 * @author		OTI
 * @version		initial
 */

public class ParsePosition {
	private int currentPosition, errorIndex = -1;
/**
 * Constructs a new ParsePosition at the specified index.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		index the index to begin parsing
 */
public ParsePosition(int index) {
	currentPosition = index;
}
/**
 * Compares the specified object to this ParsePosition and answer if they
 * are equal. The object must be an instance of ParsePosition and have the
 * same index and error index.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		object	the object to compare with this object
 * @return		true if the specified object is equal to this ParsePosition, false otherwise
 *
 * @see			#hashCode
 */
public boolean equals(Object object) {
	if (!(object instanceof ParsePosition)) return false;
	ParsePosition pos = (ParsePosition)object;
	return currentPosition == pos.currentPosition && errorIndex == pos.errorIndex;
}
/**
 * Answers the index at which the parse could not continue.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	the index of the parse error, or -1 if there is no error
 */
public int getErrorIndex() {
	return errorIndex;
}
/**
 * Answers the current parse position.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	the current position
 */
public int getIndex() {
	return currentPosition;
}
/**
 * Answers an integer hash code for the receiver. Objects which are
 * equal answer the same value for this method.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the receiver's hash
 *
 * @see			#equals
 */
public int hashCode() {
	return currentPosition + errorIndex;
}
/**
 * Sets the index at which the parse could not continue.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		index the index of the parse error
 */
public void setErrorIndex(int index) {
	errorIndex = index;
}
/**
 * Sets the current parse position.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		index the current parse position
 */
public void setIndex(int index) {
	currentPosition = index;
}
/**
 * Answers the string representation of this FieldPosition.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the string representation of this FieldPosition
 */
public String toString() {
	return getClass().getName()+"[index="+currentPosition+", errorIndex="+errorIndex+"]";
}
}
