package java.util;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 * AbstractSet is an abstract implementation of the Set iterface. This implemenation
 * does not support adding. A subclass must implement the abstract methods iterator()
 * and size().
 *
 * @author		OTI
 * @version		initial
 */

public abstract class AbstractSet extends AbstractCollection implements Set {
/**
 * Constructs a new instance of this AbstractSet.
 *
 * @author		OTI
 * @version		initial
 */
protected AbstractSet() {
}
/**
 * Compares the specified object to this Set and answer if they
 * are equal. The object must be an instance of Set and contain
 * the same objects.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		object	the object to compare with this object
 * @return		true if the specified object is equal to this Set, false otherwise
 *
 * @see			#hashCode
 */
public boolean equals(Object object) {
	if (this == object) return true;
	if (object instanceof Set) {
		Set s = (Set)object;
		return size() == s.size() && containsAll(s);
	}
	return false;
}
/**
 * Answers an integer hash code for the receiver. Objects which are
 * equal answer the same value for this method.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the receiver's hash
 *
 * @see			#equals
 */
public int hashCode() {
	int result = 0;
	Iterator it = iterator();
	while (it.hasNext()) {
		Object next = it.next();
		result += next == null ? 0 : next.hashCode();
	}
	return result;
}

/**
 * Removes all occurrences in this Collection of each object in the
 * specified Collection.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		collection	the Collection of objects to remove
 * @return		true if this Collection is modified, false otherwise
 *
 * @exception	UnsupportedOperationException when removing from this Collection
 *				is not supported
 */
public boolean removeAll(Collection collection) {
	boolean result = false;
	if (size() <= collection.size()) {
		Iterator it = iterator();
		while (it.hasNext()) {
			if (collection.contains(it.next())) {
				it.remove();
				result = true;
			}
		}
	} else {
		Iterator it = collection.iterator();
		while (it.hasNext())
			result = remove(it.next()) || result;
	}
	return result;
}
}
