package java.util;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * Collection is the root of the collection hierarchy.
 *
 * @author		OTI
 * @version		initial
 */

public interface Collection {
/**
 * Adds the specified object to this Collection.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		object	the object to add
 * @return		true if this Collection is modified, false otherwise
 *
 * @exception	UnsupportedOperationException when adding to this Collection
 *				is not supported
 * @exception	ClassCastException when the class of the object is inappropriate
 *				for this Collection
 * @exception	IllegalArgumentException when the object cannot be added
 *				to this Collection
 */
public boolean add(Object object);
/**
 * Adds the objects in the specified Collection to this Collection.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		collection	the Collection of objects
 * @return		true if this Collection is modified, false otherwise
 *
 * @exception	UnsupportedOperationException when adding to this Collection
 *				is not supported
 * @exception	ClassCastException when the class of an object is inappropriate
 *				for this Collection
 * @exception	IllegalArgumentException when an object cannot be added
 *				to this Collection
 */
public boolean addAll(Collection collection);
/**
 * Removes all elements from this Collection, leaving it empty.
 *
 * @author		OTI
 * @version		initial
 *
 * @exception	UnsupportedOperationException when removing from this Collection
 *				is not supported
 *
 * @see			#isEmpty
 * @see			#size
 */
public void clear();
/**
 * Searches this Collection for the specified object.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		object	the object to search for
 * @return		true if object is an element of this Collection, false otherwise
 */
public boolean contains(Object object);
/**
 * Searches this Collection for all objects in the specified Collection.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		collection	the Collection of objects
 * @return		true if all objects in the specified Collection are elements of
 *				this Collection, false otherwise
 */
public boolean containsAll(Collection collection);
/**
 * Compares the argument to the receiver, and answers true
 * if they represent the <em>same</em> object using a class
 * specific comparison.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		object Object
 *					the object to compare with this object.
 * @return		boolean
 *					<code>true</code>
 *						if the object is the same as this object
 *					<code>false</code>
 *						if it is different from this object.
 * @see			#hashCode
 */
public boolean equals(Object object);
/**
 * Answers an integer hash code for the receiver. Objects which are
 * equal answer the same value for this method.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the receiver's hash
 *
 * @see			#equals
 */
public int hashCode();
/**
 * Answers if this Collection has no elements, a size of zero.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		true if this Collection has no elements, false otherwise
 *
 * @see			#size
 */
public boolean isEmpty();
/**
 * Answers an Iterator on the elements of this Collection.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		an Iterator on the elements of this Collection
 *
 * @see			Iterator
 */
public Iterator iterator();
/**
 * Removes the first occurrence of the specified object from this
 * Collection.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		object	the object to remove
 * @return		true if this Collection is modified, false otherwise
 *
 * @exception	UnsupportedOperationException when removing from this Collection
 *				is not supported
 */
public boolean remove(Object object);
/**
 * Removes all occurrences in this Collection of each object in the
 * specified Collection.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		collection	the Collection of objects to remove
 * @return		true if this Collection is modified, false otherwise
 *
 * @exception	UnsupportedOperationException when removing from this Collection
 *				is not supported
 */
public boolean removeAll(Collection collection);
/**
 * Removes all objects from this Collection that are not contained in the
 * specified Collection.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		collection	the Collection of objects to retain
 * @return		true if this Collection is modified, false otherwise
 *
 * @exception	UnsupportedOperationException when removing from this Collection
 *				is not supported
 */
public boolean retainAll(Collection collection);
/**
 * Answers the number of elements in this Collection.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the number of elements in this Collection
 */
public int size();
/**
 * Answers a new array containing all elements contained in this Collection.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		an array of the elements from this Collection
 */
public Object[] toArray();
/**
 * Answers an array containing all elements contained in this Collection.
 * If the specified array is large enough to hold the elements, the specified
 * array is used, otherwise an array of the same type is created. If the
 * specified array is used and is larger than this Collection, the array
 * element following the collection elements is set to null.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		array	the array
 * @return		an array of the elements from this Collection
 *
 * @exception	ArrayStoreException when the type of an element in this Collection
 *				cannot be stored in the type of the specified array
 */
public Object[] toArray(Object[] array);
}
