package java.util;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 * Dictionary is a abstract class which is the superclass of all classes that
 * associate keys with values, such as Hashtable.
 *
 * @author		OTI
 * @version		initial
 *
 * @see			Hashtable
 */
public abstract class Dictionary {
/**
 * Constructs a new instance of this class.
 *
 * @author		OTI
 * @version		initial
 */
public Dictionary() {
}
/**
 * Answers an Enumeration on the elements of this Dictionary.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		an Enumeration of the values of this Dictionary
 *
 * @see			#keys
 * @see			#size
 * @see			Enumeration
 */
abstract public Enumeration elements();
/**
 * Answers the value associated with the specified key in
 * this Dictionary.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		key	the key of the value returned
 * @return		the value associated with the specified key, null if the specified key
 *				does not exist
 *
 * @see			#put
 */
abstract public Object get(Object key);
/**
 * Answers if this Dictionary has no key/value pairs, a size of zero.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		true if this Dictionary has no key/value pairs, false otherwise
 *
 * @see			#size
 */
abstract public boolean isEmpty();
/**
 * Answers an Enumeration on the keys of this Dictionary.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		an Enumeration of the keys of this Dictionary
 *
 * @see			#elements
 * @see			#size
 * @see			Enumeration
 */
abstract public Enumeration keys();
/**
 * Associate the specified value with the specified key in this Dictionary.
 * If the key already exists, the old value is replaced.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		key	the key to add
 * @param		value	the value to add
 * @return		the old value associated with the specified key, null if the key did
 *				not exist
 *
 * @see			#elements
 * @see			#get
 * @see			#keys
 */
abstract public Object put(Object key, Object value);
/**
 * Remove the key/value pair with the specified key from this Dictionary.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		key	the key to remove
 * @return		the value associated with the specified key, null if the specified key
 *				did not exist
 *
 * @see			#get
 * @see			#put
 */
abstract public Object remove(Object key);
/**
 * Answers the number of key/value pairs in this Dictionary.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the number of key/value pairs in this Dictionary
 *
 * @see			#elements
 * @see			#keys
 */
abstract public int size();
}
