package java.util;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

import java.io.Serializable;

/**
 * EventObjects represent events. Typically applications
 * subclass this class to add event specific information.
 *
 * @author		OTI
 * @version		initial
 *
 * @see			EventListener
 */

public class EventObject implements Serializable {
	private static final long serialVersionUID = 5516075349620653480L;
	/**
	 * The event source.
	 */
	protected transient Object source;
/**
 * Constructs a new instance of this class.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		source		the object which fired the event
 */
public EventObject(Object source) {
	if (source != null) this.source = source;
	else throw new IllegalArgumentException();
}
/**
 * Answers the event source.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the object which fired the event
 */
public Object getSource() {
	return source;
}
/**
 * Answers the string representation of this EventObject.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the string representation of this EventObject
 */
public String toString() {
	return getClass().getName() + "[source=" + String.valueOf(source) + ']';
}
}
