/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2005, 2006  All Rights Reserved.
 */

package java.util;

import java.io.Serializable;

/**
 * LinkedHashSet is a variant on HashSet. Its entries are kept in a doubly-linked list.
 * The iteration order is the order in which entries were inserted.
 * <p>
 * Null elements are allowed, and all the optional Set operations are supported.
 * <p>
 * Like HashSet, LinkedHashSet is not thread safe, so access by multiple threads must be synchronized
 * by an external mechanism such as Collections.synchronizedSet.
 *
 * @author blainey
 */
public class LinkedHashSet extends HashSet implements Set, Cloneable, Serializable {
	private static final long serialVersionUID = -2851667679971038690L;

	/**
	 * Contructs a new empty instance of LinkedHashSet.
	 *
	 * @author		blainey
	 * @version		initial
	 */
	public LinkedHashSet() {
		super(new LinkedHashMap());
	}
	/**
	 * Constructs a new instance of LinkedHashSet with the specified capacity.
	 *
	 * @author		blainey
	 * @version		initial
	 *
	 * @param		capacity	the initial capacity of this HashSet
	 */
	public LinkedHashSet(int capacity) {
		super(new LinkedHashMap(capacity));
	}
	/**
	 * Constructs a new instance of LinkedHashSet with the specified capacity and load factor.
	 *
	 * @author		blainey
	 * @version		initial
	 *
	 * @param		capacity	the initial capacity
	 * @param		loadFactor	the initial load factor
	 */
	public LinkedHashSet(int capacity, float loadFactor) {
		super(new LinkedHashMap(capacity, loadFactor));
	}
	/**
	 * Constructs a new instance of LinkedHashSet containing the unique elements in the
	 * specified collection.
	 *
	 * @author		blainey
	 * @version		initial
	 *
	 * @param		collection	the collection of elements to add
	 */
	public LinkedHashSet(Collection collection) {
		super(new LinkedHashMap(collection.size() < 6 ? 11 : collection.size() * 2));
		Iterator it = collection.iterator();
		while (it.hasNext()) add(it.next());
	}

	/* overrides method in HashMap */
	HashMap createBackingMap(int capacity, float loadFactor) {
		return new LinkedHashMap(capacity, loadFactor);
	}

}
