package java.util;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 * An ListIterator is used to sequence over a List of objects. ListIterator
 * can move backwards or forwards through the List.
 *
 * @author		OTI
 * @version		initial
 */

public interface ListIterator extends Iterator {
/**
 * Inserts the specified object into the list between <code>next</code>
 * and <code>previous</code>. The object inserted will be the previous
 * object.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	object the object to insert
 *
 * @exception	UnsupportedOperationException when adding is not supported by
 *				the list being iterated
 * @exception	ClassCastException when the class of the object is inappropriate
 *				for the list
 * @exception	IllegalArgumentException when the object cannot be added
 *				to the list
 */
void add(Object object);
/**
 * Answers if there are more elements to iterate.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		true if there are more elements, false otherwise
 *
 * @see			#next
 */
public boolean hasNext();
/**
 * Answers if there are previous elements to iterate.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		true if there are previous elements, false otherwise
 *
 * @see			#previous
 */
public boolean hasPrevious();
/**
 * Answers the next object in the iteration.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the next object
 *
 * @exception	NoSuchElementException when there are no more elements
 *
 * @see			#hasNext
 */
public Object next();
/**
 * Answers the index of the next object in the iteration.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the index of the next object
 *
 * @exception	NoSuchElementException when there are no more elements
 *
 * @see			#next
 */
public int nextIndex();
/**
 * Answers the previous object in the iteration.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the previous object
 *
 * @exception	NoSuchElementException when there are no previous elements
 *
 * @see			#hasPrevious
 */
public Object previous();
/**
 * Answers the index of the previous object in the iteration.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the index of the previous object
 *
 * @exception	NoSuchElementException when there are no previous elements
 *
 * @see			#previous
 */
public int previousIndex();
/**
 * Removes the last object returned by <code>next</code> or <code>previous</code>
 * from the list.
 *
 * @author		OTI
 * @version		initial
 *
 * @exception	UnsupportedOperationException when removing is not supported by
 *				the list being iterated
 * @exception	IllegalStateException when <code>next</code> or <code>previous</code>
 *				have not been called, or <code>remove</code> or <code>add</code> have
 *				already been called after the last call to <code>next</code> or
 *				<code>previous</code>
 */
public void remove();
/**
 * Replaces the last object returned by <code>next</code> or <code>previous</code>
 * with the specified object.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	object the object to add
 *
 * @exception	UnsupportedOperationException when adding is not supported by
 *				the list being iterated
 * @exception	ClassCastException when the class of the object is inappropriate
 *				for the list
 * @exception	IllegalArgumentException when the object cannot be added
 *				to the list
 * @exception	IllegalStateException when <code>next</code> or <code>previous</code>
 *				have not been called, or <code>remove</code> or <code>add</code> have
 *				already been called after the last call to <code>next</code> or
 *				<code>previous</code>
 */
void set(Object object);
}
