package java.util;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

/**
 * Map has a set of keys, each key is mapped to a single value.
 *
 * @author		OTI
 * @version		initial
 */

public interface Map {
	/**
	 * Map.Entry is a key/value mapping which is contained in a Map.
	 *
	 * @author		OTI
	 * @version		initial
	 */
	public abstract static interface Entry {
		/**
		 * Compares the specified object to this Map.Entry and answer if they
		 * are equal. The object must be an instance of Map.Entry and have the
		 * same key and value.
		 *
		 * @author		OTI
		 * @version		initial
		 *
		 * @param		object	the object to compare with this object
		 * @return		true if the specified object is equal to this Map.Entry, false otherwise
		 *
		 * @see			#hashCode
		 */
		public boolean equals(Object object);
		/**
		 * Gets the key.
		 *
		 * @author		OTI
		 * @version		initial
		 *
		 * @return		the key
		 */
		public Object getKey();
		/**
		 * Gets the value.
		 *
		 * @author		OTI
		 * @version		initial
		 *
		 * @return		the value
		 */
		public Object getValue();
		/**
		 * Answers an integer hash code for the receiver. Objects which are
		 * equal answer the same value for this method.
		 *
		 * @author		OTI
		 * @version		initial
		 *
		 * @return		the receiver's hash
		 *
		 * @see			#equals
		 */
		public int hashCode();
		/**
		 * Sets the value.
		 *
		 * @author		OTI
		 * @version		initial
		 *
		 * @param		object	the old value
		 */
		public Object setValue(Object object);
	};
/**
 * Removes all elements from this Map, leaving it empty.
 *
 * @author		OTI
 * @version		initial
 *
 * @exception	UnsupportedOperationException when removing from this Map
 *				is not supported
 *
 * @see			#isEmpty
 * @see			#size
 */
public void clear();
/**
 * Searches this Map for the specified key.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		key	the object to search for
 * @return		true if <code>key</code> is a key of this Map, false otherwise
 */
public boolean containsKey(Object key);
/**
 * Searches this Map for the specified value.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		value the object to search for
 * @return		true if <code>value</code> is a value of this Map, false otherwise
 */
public boolean containsValue(Object value);
/**
 * Answers a Set of the mappings contained in this Map. Each
 * element in the set is a Map.Entry. The set is backed by this
 * Map so changes to one are relected by the other. The set does
 * not support adding.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		a Set of the mappings
 */
public Set entrySet();
/**
 * Compares the argument to the receiver, and answers true
 * if they represent the <em>same</em> object using a class
 * specific comparison.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		object Object
 *					the object to compare with this object.
 * @return		boolean
 *					<code>true</code>
 *						if the object is the same as this object
 *					<code>false</code>
 *						if it is different from this object.
 * @see			#hashCode
 */
public boolean equals(Object object);
/**
 * Answers the value of the mapping with the specified key.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		key	the key
 * @return		the value of the mapping with the specified key
 */
public Object get(Object key);
/**
 * Answers an integer hash code for the receiver. Objects which are
 * equal answer the same value for this method.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the receiver's hash
 *
 * @see			#equals
 */
public int hashCode();
/**
 * Answers if this Map has no elements, a size of zero.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		true if this Map has no elements, false otherwise
 *
 * @see			#size
 */
public boolean isEmpty();
/**
 * Answers a Set of the keys contained in this Map. The set is backed
 * by this Map so changes to one are relected by the other. The set does
 * not support adding.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		a Set of the keys
 */
public Set keySet();
/**
 * Maps the specified key to the specified value.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		key the key
 * @param		value the value
 * @return		the value of any previous mapping with the specified key or
 *				null if there was no mapping
 *
 * @exception	UnsupportedOperationException when adding to this Map
 *				is not supported
 * @exception	ClassCastException when the class of the key or value is inappropriate
 *				for this Map
 * @exception	IllegalArgumentException when the key or value cannot be added
 *				to this Map
 * @exception	NullPointerException when the key or value is null and this Map does
 * 				not support null keys or values
 */
public Object put(Object key, Object value);
/**
 * Copies every mapping in the specified Map to this Map.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		map the Map to copy mappings from
 *
 * @exception	UnsupportedOperationException when adding to this Map
 *				is not supported
 * @exception	ClassCastException when the class of a key or value is inappropriate
 *				for this Map
 * @exception	IllegalArgumentException when a key or value cannot be added
 *				to this Map
 * @exception	NullPointerException when a key or value is null and this Map does
 * 				not support null keys or values
 */
public void putAll(Map map);
/**
 * Removes a mapping with the specified key from this
 * Map.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		key	the key of the mapping to remove
 * @return		the value of the removed mapping or null if key is not a key
 *				in this Map
 *
 * @exception	UnsupportedOperationException when removing from this Map
 *				is not supported
 */
public Object remove(Object key);
/**
 * Answers the number of elements in this Map.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the number of elements in this Map
 */
public int size();
/**
 * Answers a Collection of the values contained in this Map. The
 * collection is backed by this Map so changes to one are relected
 * by the other. The collection does not support adding.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		a Collection of the values
 */
public Collection values();
}
