package java.util;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

import java.security.Permission;
import java.security.PermissionCollection;
import java.io.IOException;
import java.io.ObjectStreamField;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 * A PermissionCollection for holding PropertyPermissions.
 *
 * @author		OTI
 * @version		initial
 */

class PropertyPermissionCollection extends PermissionCollection {
	private static final long serialVersionUID = 7015263904581634791L;
	Hashtable permissions = new Hashtable(30);

public void add(Permission perm) {
	if (!isReadOnly()) {
		Permission previous = (Permission) permissions.put(perm.getName(), perm);
		// if the permission already existed but with only "read" or "write" set, then replace with both set
		if (previous != null && !previous.getActions().equals(perm.getActions()))
			permissions.put(perm.getName(), new PropertyPermission(perm.getName(), "read,write"));
	} else throw new IllegalStateException();
}

public Enumeration elements() {
	return permissions.elements();
}

public boolean implies(Permission perm) {
	Enumeration elemEnum = elements();
	while (elemEnum.hasMoreElements())
		if (((Permission) elemEnum.nextElement()).implies(perm))
			return true;
	// At this point, the only way it can succeed is if both read and write are set,
	// and these are separately granted by two different permissions with one
	// representing a parent directory.
	return perm.getActions().equals("read,write")
		&& implies(new PropertyPermission(perm.getName(), "read"))
		&& implies(new PropertyPermission(perm.getName(), "write"));
}
private static final ObjectStreamField[] serialPersistentFields = {
	new ObjectStreamField("permissions", Hashtable.class),
	new ObjectStreamField("all_allowed", Boolean.TYPE)
};
private void writeObject(ObjectOutputStream stream) throws IOException {
	ObjectOutputStream.PutField fields = stream.putFields();
	fields.put("permissions", permissions);
	fields.put("all_allowed", false);
	stream.writeFields();
}
private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
	ObjectInputStream.GetField fields = stream.readFields();
	permissions = (Hashtable)fields.get("permissions", null);
}
}
