package java.util;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2006  All Rights Reserved
 */

import java.io.InputStream;
import java.io.IOException;

/**
 * PropertyResourceBundle loads resources from an InputStream. All resources
 * are Strings. The resources must be of the form <code>key=value</code>, one resource per
 * line.
 *
 * @author		OTI
 * @version		initial
 *
 * @see			ResourceBundle
 * @see			Properties
 */

public class PropertyResourceBundle extends ResourceBundle {
	final Properties resources;
/**
 * Constructs a new instance of PropertyResourceBundle and loads the
 * properties file from the specified input stream.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		stream	the input stream
 */
public PropertyResourceBundle (InputStream stream) throws IOException {
	resources = new Properties();
	resources.load(stream);
}
/**
 * Answers the names of the resources contained in this PropertyResourceBundle.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		an Enumeration of the resource names
 */
public Enumeration getKeys() {
	if (parent == null) return resources.keys();
	return new Enumeration() {
		Enumeration local = resources.keys();
		Enumeration pEnum = parent.getKeys();
		Object nextElement = null;
		private boolean findNext() {
			if (nextElement != null) return true;
			while (pEnum.hasMoreElements()) {
				String next = (String)pEnum.nextElement();
				if (!resources.containsKey(next)) {
					nextElement = next;
					return true;
				}
			}
			return false;
		}
		public boolean hasMoreElements() {
			if (local.hasMoreElements()) return true;
			return findNext();
		}
		public Object nextElement() {
			if (local.hasMoreElements())
				return local.nextElement();
			if (findNext()) {
				Object result = nextElement;
				nextElement = null;
				return result;
			}
			// Cause an exception
			return pEnum.nextElement();
		}
	};
}
/**
 * Answers the named resource from this PropertyResourceBundle, or null if
 * the resource is not found.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		key	the name of the resource
 * @return		the resource object
 */
public Object handleGetObject(String key) {
	return resources.get(key);
}
}
