package java.util;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 * SortedMap is a Map where the iterators sequence in order of the sorted keys.
 *
 * @author		OTI
 * @version		initial
 */

public interface SortedMap extends Map {
/**
 * Answers the Comparator used to compare elements in this SortedMap.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	a Comparator or null if the natural order is used
 */
public Comparator comparator();
/**
 * Answer the first sorted key in this SortedMap.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the first sorted key
 *
 * @exception	NoSuchElementException when this SortedMap is empty
 */
public Object firstKey();
/**
 * Answers a SortedMap of the specified portion of this SortedMap
 * which contains keys less than the end key. The returned SortedMap is
 * backed by this SortedMap so changes to one are reflected by the other.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		endKey		the end key
 * @return		a submap where the keys are less than <code>endKey</code>
 *
 * @exception	ClassCastException when the class of the end key is inappropriate
 *				for this SubMap
 * @exception	NullPointerException when the end key is null and this SortedMap does
 * 				not support null keys
 */
public SortedMap headMap(Object endKey);
/**
 * Answers the last sorted key in this SortedMap.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the last sorted key
 *
 * @exception	NoSuchElementException when this SortedMap is empty
 */
public Object lastKey();
/**
 * Answers a SortedMap of the specified portion of this SortedMap which
 * contains keys greater or equal to the start key but less than the end
 * key. The returned SortedMap is backed by this SortedMap so changes to one
 * are reflected by the other.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		startKey	the start key
 * @param		endKey		the end key
 * @return		a submap where the keys are greater or equal to <code>startKey</code>
 *				and less than <code>endKey</code>
 *
 * @exception	ClassCastException when the class of the start or end key is inappropriate
 *				for this SubMap
 * @exception	NullPointerException when the start or end key is null and this SortedMap does
 * 				not support null keys
 * @exception	IllegalArgumentException when the start key is greater than the end key
 */
public SortedMap subMap(Object startKey, Object endKey);
/**
 * Answers a SortedMap of the specified portion of this SortedMap which
 * contains keys greater or equal to the start key. The returned SortedMap
 * is backed by this SortedMap so changes to one are reflected by the other.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		startKey	the start key
 * @return		a submap where the keys are greater or equal to <code>startKey</code>
 *
 * @exception	ClassCastException when the class of the start key is inappropriate
 *				for this SubMap
 * @exception	NullPointerException when the start key is null and this SortedMap does
 * 				not support null keys
 */
public SortedMap tailMap(Object startKey);
}
