package java.util;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 * SortedSet is a Set which iterates its elements in sorted order.
 *
 * @author		OTI
 * @version		initial
 */

public interface SortedSet extends Set {
/**
 * Answers the Comparator used to compare elements in this SortedSet.
 *
 * @author		OTI
 * @version		initial
 *
 * @return	a Comparator or null if the natural order is used
 */
public Comparator comparator();
/**
 * Answer the first sorted element in this SortedSet.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the first sorted element
 *
 * @exception	NoSuchElementException when this SortedSet is empty
 */
public Object first();
/**
 * Answers a SortedSet of the specified portion of this SortedSet
 * which contains elements less than the end element. The returned SortedSet is
 * backed by this SortedSet so changes to one are reflected by the other.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		end		the end element
 * @return		a subset where the elements are less than <code>end</code>
 *
 * @exception	ClassCastException when the class of the end element is inappropriate
 *				for this SubSet
 * @exception	NullPointerException when the end element is null and this SortedSet does
 * 				not support null elements
 */
public SortedSet headSet(Object end);
/**
 * Answer the last sorted element in this SortedSet.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		the last sorted element
 *
 * @exception	NoSuchElementException when this SortedSet is empty
 */
public Object last();
/**
 * Answers a SortedSet of the specified portion of this SortedSet which
 * contains elements greater or equal to the start element but less than the
 * end element. The returned SortedSet is backed by this SortedMap so changes
 * to one are reflected by the other.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		start	the start element
 * @param		end		the end element
 * @return		a subset where the elements are greater or equal to <code>start</code>
 *				and less than <code>end</code>
 *
 * @exception	ClassCastException when the class of the start or end element is inappropriate
 *				for this SubSet
 * @exception	NullPointerException when the start or end element is null and this SortedSet does
 * 				not support null elements
 * @exception	IllegalArgumentException when the start element is greater than the end element
 */
public SortedSet subSet(Object start, Object end);
/**
 * Answers a SortedSet of the specified portion of this SortedSet
 * which contains elements greater or equal to the start element. The
 * returned SortedSet is backed by this SortedSet so changes to one
 * are reflected by the other.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		start		the start element
 * @return		a subset where the elements are greater or equal to <code>start</code>
 *
 * @exception	ClassCastException when the class of the start element is inappropriate
 *				for this SubSet
 * @exception	NullPointerException when the start element is null and this SortedSet does
 * 				not support null elements
 */
public SortedSet tailSet(Object start);
}
