package java.util.jar;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.util.zip.ZipEntry;
import java.security.cert.*;

public class JarEntry extends java.util.zip.ZipEntry {
	private Attributes attributes;
	JarFile parentJar;
	Certificate certificates[];

/**
 * Create a new JarEntry named name
 *
 * @author		OTI
 * @version		initial
 *
 * @param		name 	The name of the new JarEntry
 */
public JarEntry(String name) {
	super(name);
}
/**
 * Create a new JarEntry using the values obtained from
 * entry.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		entry The ZipEntry to obtain values from.
 */
public JarEntry(ZipEntry entry) {
	super(entry);
}

/**
 * Returns the Attributes object associated with this
 * entry or null if none exists.
 *
 * @author		OTI
 * @version		initial
 *
 * @return 	java.util.jar.Attributes	Attributes for this entry
 * @exception	java.io.IOException	If an error occurs obtaining the Attributes
 */
public Attributes getAttributes() throws java.io.IOException{
	if (attributes != null || parentJar == null)
		return attributes;
	Manifest manifest = parentJar.getManifest();
	if (manifest == null) return null;
	return attributes = manifest.getAttributes(getName());
}
/**
 * Returns an array of Certificate Objects associated with this entry
 * or null if none exist.
 *
 * @author		OTI
 * @version		initial
 *
 * @return 	java.security.cert.Certificate[]	Certificates for this entry
 */
public Certificate[] getCertificates() {
	return certificates;
}
/**
 *
 */
void setAttributes(Attributes attrib) {
	attributes = attrib;
}
/**
 * Create a new JarEntry using the values obtained from
 * je.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		je The JarEntry to obtain values from
 */
public JarEntry(JarEntry je) {
	super((ZipEntry)je);
	parentJar = je.parentJar;
	attributes = je.attributes;
	certificates = je.certificates;
}

}
