package java.util.jar;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.io.IOException;
import java.io.OutputStream;
import java.io.ByteArrayOutputStream;
import java.util.zip.*;

public class JarInputStream extends ZipInputStream {

	private Manifest manifest;
	private boolean eos = false;
	private JarEntry mEntry;
	private JarEntry jarEntry;
	private boolean isMeta;
	private JarVerifier verifier;
	private OutputStream verStream;

/**
 * Constructs a new JarInputStream from stream
 *
 * @author		OTI
 * @version		initial
 *
 */
public JarInputStream(java.io.InputStream stream, boolean verify) throws java.io.IOException{
	super(stream);
	if (verify)
		verifier = new JarVerifier("JarInputStream");
	if ((mEntry = getNextJarEntry()) == null) return;
	String name = mEntry.getName().toUpperCase();
	if (name.equals(JarFile.META_DIR)) {
		mEntry = null; // modifies behavior of getNextJarEntry()
		closeEntry();
		mEntry = getNextJarEntry();
		name = mEntry.getName().toUpperCase();
	}
	if (name.equals(JarFile.MANIFEST_NAME)) {
		mEntry = null;
		manifest = new Manifest(this, verify);
		closeEntry();
		if (verify)
			verifier.setManifest(manifest);
	} else {
		Attributes temp = new Attributes(3);
		temp.map.put("hidden", null);
		mEntry.setAttributes(temp);
	}
}
public JarInputStream(java.io.InputStream stream) throws java.io.IOException{
	this(stream, true);
	}
/**
 * Returns the Manifest object associated with this JarInputStream or null
 * if no manifest entry exists.
 *
 * @author		OTI
 * @version		initial
 *
 * @return java.util.jar.Manifest
 */
public Manifest getManifest() {
	return manifest;
}
/**
 * Returns the next JarEntry contained in this stream or null if no
 * more entries are present.
 *
 * @author		OTI
 * @version		initial
 *
 * @return 		java.util.jar.JarEntry
 * @exception 	java.io.IOException If an error occurs while reading the entry
 */
public JarEntry getNextJarEntry() throws java.io.IOException{
	return (JarEntry)getNextEntry();
}
public int read (byte[] buf, int off, int len) throws java.io.IOException{
	if (mEntry != null) return -1;
	int r = super.read(buf, off, len);
	if (verStream != null && !eos) {
		if (r == -1) {
			eos = true;
			if (isMeta) {
				verifier.addMetaEntry(jarEntry.getName(), ((ByteArrayOutputStream)verStream).toByteArray());
				verifier.readCertificates();
			} else
				verifier.verifySignatures((JarVerifier.VerifierEntry)verStream, jarEntry);
		} else verStream.write(buf, off, r);
	}
	return r;
}
/**
 * Returns the next ZipEntry contained in this stream or null if no
 * more entries are present.
 *
 * @author		OTI
 * @version		initial
 *
 * @return 		java.util.zip.ZipEntry
 * @exception 	java.io.IOException If an error occurs while reading the entry
 */
public ZipEntry getNextEntry () throws IOException{
	eos = false;
	if (mEntry != null) {
		jarEntry = mEntry;
		mEntry = null;
		jarEntry.setAttributes(null);
		return jarEntry;
	} else jarEntry = (JarEntry)super.getNextEntry();
	if (jarEntry == null) return null;
	if (verifier != null) {
		if (isMeta = jarEntry.getName().toUpperCase().startsWith(JarFile.META_DIR))
			verStream = new ByteArrayOutputStream();
		else
			verStream = verifier.initEntry(jarEntry.getName());
	}
	return jarEntry;
}
protected ZipEntry createZipEntry (String name) {
	JarEntry entry = new JarEntry(name);
	if (manifest != null)
		entry.setAttributes(manifest.getAttributes(name));
	return entry;
}
}
