package java.util.jar;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

import java.util.zip.*;
import java.io.OutputStream;
import java.io.IOException;

/**
 * The JarOutputStream is used to output data in JarFile format.
 *
 * @author		OTI
 * @version		initial
 *
 */

public class JarOutputStream extends ZipOutputStream {
	private Manifest manifest;

/**
 * Contructs a new JarOuputStream using os as the
 * underlying stream. Manifest information for the JarFile
 * to be written is obtained from the parameter Manifest, mf.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		os The OutputStream to write to
 * @param		mf The Manifest to output for this Jar.
 * @exception	IOException If an error occurs creating the JarOutputStream
 */
public JarOutputStream(OutputStream os, Manifest mf) throws IOException{
	super(os);
	if (mf == null) throw new NullPointerException();
	manifest = mf;
	ZipEntry ze = new ZipEntry(JarFile.MANIFEST_NAME);
	putNextEntry(ze);
	manifest.write(this);
	closeEntry();
}
/**
 * Contructs a new JarOuputStream using os as the
 * underlying stream.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		os The OutputStream to write to
 * @exception	IOException If an error occurs creating the JarOutputStream
 */
public JarOutputStream(OutputStream os) throws IOException{
	super(os);
}
/**
 * Writes the specified entry to the underlying stream. The previous
 * entry is closed if it is still open.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		ze The ZipEntry to write
 * @exception	IOException If an error occurs writing the entry
 */
public void putNextEntry (ZipEntry ze) throws java.io.IOException{
	super.putNextEntry(ze);
}

}
