package java.util.zip;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

/**
 * The Adler32 class is used to compute the Adler32 Checksum
 * from a set of data.
 *
 * @author		OTI
 * @version		initial
 *
 */

public class Adler32 implements java.util.zip.Checksum {

	private long adler = 1;

/**
 * Returns the Adler32 checksum for all input received
 *
 * @author		OTI
 * @version		initial
 *
 * @return		The checksum for this instance
 */
public long getValue(){
		return adler;
}
/**
 * Reset this instance to its initial checksum
 *
 * @author		OTI
 * @version		initial
 *
 */
public void reset(){
	adler = 1;
}
/**
 * Update this Adler32 checksum using val.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		i	byte to update checksum with
 */
public void update(int i){
	adler = updateByteImpl(i, adler);
}
/**
 * Update this Adler32 checksum using the contents of buf.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		buf	bytes to update checksum with
 */
public void update(byte[] buf){
	 update(buf, 0, buf.length);
}
/**
 * Update this Adler32 checksum with the contents
 * of buf, starting from offset and using nbytes
 * of data.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		buf	bufffer to obtain dat from
 * @param		off	offset i  buf to conpy from
 * @param		nbytes	number of bytes from buf to use
 */
public void update(byte[] buf, int off, int nbytes){
	// avoid int overflow, check null buf
	if (off <= buf.length && nbytes >= 0 && off >= 0 && buf.length - off >= nbytes)
		adler = updateImpl(buf, off, nbytes, adler);
	else throw new ArrayIndexOutOfBoundsException();
}
private native long updateImpl(byte[] buf, int off, int nbytes, long adler);
private native long updateByteImpl(int val, long adler);
}

