package java.util.zip;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

 /**
 * The CRC32 class is used to compute a CRC32 Checksum
 * from a set of data.
 *
 * @author		OTI
 * @version		initial
 *
 */

public class CRC32 implements java.util.zip.Checksum {

	private long crc = 0L;
	long tbytes = 0L;

/**
 * Returns the CRC32 Checksum for all input received.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		The checksum for this instance
 */
public long getValue(){
	return crc;
}
/**
 * Returns the CRC32 checksum to it initial state.
 *
 * @author		OTI
 * @version		initial
 *
 */
public void reset(){
	tbytes = crc = 0;

}
/**
 * Updates this Checksum with value val
 *
 * @author		OTI
 * @version		initial
 *
 */
public void update(int val){
	crc = updateByteImpl((byte)val, crc);
}
/**
 * Updates this Checksum with the bytes contained
 * in buffer buf.
 *
 * @author		OTI
 * @version		initial
 *
 * @param	buf	Buffer to update Checksum
 */
public void update(byte[] buf){
	update(buf, 0, buf.length);
}
/**
 * Updates this Checksum with nbytes of data from buffer
 * buf, starting at offset off.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		buf	Buffer to update Checksum
 * @param	    off	Offest in buf to obtain data from
 * @param		nbytes	Number of bytes to read from buf
 */
public void update(byte[] buf, int off, int nbytes){
	// avoid int overflow, check null buf
	if (off <= buf.length && nbytes >= 0 && off >= 0 && buf.length - off >= nbytes) {
		tbytes += nbytes;
		crc = updateImpl(buf, off, nbytes, crc);
	} else throw new ArrayIndexOutOfBoundsException();
}
private native long updateImpl(byte[] buf, int off, int nbytes, long crc);
private native long updateByteImpl(byte val, long crc);
}
