package java.util.zip;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2004  All Rights Reserved
 */

 import java.io.OutputStream;
import java.io.IOException;

/**
 * The CheckedOutputStream class is used to maintain
 * a running Checksum of all data written to a stream.
 *
 * @author 		OTI
 * @version 	initial
 *
 */
public class CheckedOutputStream extends java.io.FilterOutputStream {

	private Checksum check;
/**
 * Constructs a new CheckedOutputStream on OutputStream os.
 * The Checksum will be calculated using the algorithm
 * implemented by csum.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		os		OutputStream to calculate checksum from
 * @param		cs		Type of Checksum to calculate
 */
public CheckedOutputStream(OutputStream os, Checksum cs){
	super(os);
	check = cs;
}
/**
 * Returns the Checksum calculated on the stream thus far.
 *
 * @author		OTI
 * @version		initial
 *
 * @return		A java.util.zip.Checksum
 */
public Checksum getChecksum() {
	return check;
}
/**
 * Writes byte value val to the underlying stream.
 * The Checksum is updated with val.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		val Value of the byte to write out
 *
 * @throws IOException	if an IO error has occured
 */
public void write (int val) throws IOException{
	out.write(val);
	check.update(val);
}
/**
 * Writes nbytes of data from buf starting at offset off
 * to the underlying stream.
 * The Checksum is updated with the bytes written.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		buf 	data to write out
 * @param		off		the start offset of the data
 * @param		nbytes	number of bytes to write out
 *
 * @throws IOException	if an IO error has occured
 */
public void write (byte[] buf, int off, int nbytes) throws IOException{
	out.write(buf, off, nbytes);
	check.update(buf, off, nbytes);
}

}
