package java.util.zip;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2002  All Rights Reserved
 */

/**
 * The GZIPOutputStream class is used to write data to a stream
 * in the GZIP storage format.
 *
 * @author		OTI
 * @version		initial
 *
 */
import java.io.OutputStream;
import java.io.IOException;
public class GZIPOutputStream extends DeflaterOutputStream {

	protected CRC32 crc = new CRC32();
/**
 * Construct a new GZIPOutputStream to write data in GZIP format
 * to the underlying stream.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		os  	OutputStream to write to
 */
public GZIPOutputStream(OutputStream os) throws IOException {
	this(os, BUF_SIZE);
}
/**
 * Construct a new GZIPOutputStream to write data in GZIP format
 * to the underlying stream. Set the internal compression buffer
 * to sise size.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		os  	OutputStream to write to
 * @param		size	Internal buffer size
 */
public GZIPOutputStream(OutputStream os, int size) throws IOException{
	super(os, new Deflater(Deflater.DEFAULT_COMPRESSION, true), size);
	writeShort(GZIPInputStream.GZIP_MAGIC);
	out.write(Deflater.DEFLATED);
	out.write(0); // flags
	writeLong(0); // mod time
	out.write(0); // extra flags
	out.write(0); // operating system
}
/**
 * Indicates to the stream that all data has been written out,
 * and any GZIP terminal data can now be output.
 *
 * @author		OTI
 * @version		initial
 *
 */
public void finish () throws IOException {
	super.finish();
	writeLong(crc.getValue());
	writeLong(crc.tbytes);

}
/**
 * Close this stream and any underlying streams.
 *
 * @author		OTI
 * @version		initial
 *
 */
public void close() throws IOException {
	if (!done) finish();
	super.close();
}
/**
 * Write up to nbytes of data from buf, starting at offset off,
 * to the underlying stream in GZIP format.
 *
 * @author		OTI
 * @version		initial
 *
 */
public void write (byte[] buf, int off, int nbytes) throws IOException{
	super.write(buf, off, nbytes);
	crc.update(buf, off, nbytes);
}

private int writeShort(int i)throws java.io.IOException{
	out.write(i & 0xFF);
	out.write((i >> 8) & 0xFF);
	return i;
}

private long writeLong(long i)  throws java.io.IOException{
	//Write out the long value as an unsigned int
	out.write((int)(i & 0xFF));
	out.write((int)(i >> 8)& 0xFF);
	out.write((int)(i >> 16)& 0xFF);
	out.write((int)(i >> 24)& 0xFF);
	return i;
}
}

