package javax.microedition.xlet;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2005  All Rights Reserved
 */

/**
 * This interface is used by an Xlet Application Manager to control the lifecycle of an Xlet.
 */
public interface Xlet {

	/**
	 * This method is called by the Xlet Application Manager to notify the receiver that it is being
	 * initialized.  The Xlet should not aquire shared resources, but should prepare to activate
	 * itself in a reasonable amount of time.  When this method returns, the Xlet will enter the
	 * Paused state.
	 *
	 * @param context  XletContext for this Xlet
	 *
	 * @throws XletStateChangeException the Xlet can not initialize
	 */
	public void initXlet(XletContext context) throws XletStateChangeException;

	/**
	 * This method is called by the Xlet Application Manager to notify the receiver that it is being started.
	 * When this method returns, the Xlet will enter the Active state.
	 *
	 * @throws XletStateChangeException the Xlet can not start
	 */
	public void startXlet() throws XletStateChangeException;

	/**
	 * This method is called by the Xlet Application Manager to notify the receiver that it is being paused.
	 * This Xlet should stop providing service and become quiescent.  Resource usage should be minimized, within
	 * the constraint that the Xlet should be ready to activate quickly.  When this method returns, the Xlet will
	 * enter the Paused state.
	 */
	public void pauseXlet();

	/**
	 * This method is called by the Xlet Application Manager to notify the receiver that it is being destroyed.
	 * The receiver needs to release all resources and save any necessary preferences or state.  If the
	 * unconditional flag is false, the Xlet can request that it not enter the Destroyed state by throwing an
	 * XletStateChangeException.  Otherwise, when this method returns, the Xlet will terminated and enter the
	 * Destroyed state,
	 *
	 * @param unconditional  <code>true</code>, requires that the Xlet will enter the Destroyed state.
	 * 		<code>false</code>, allows the Xlet to remain in its current state by throwing an XletStateChangeException.
	 *
	 * @throws XletStateChangeException Used to signal that the Xlet does not wish to enter the Destroyed state.
	 * 		This exception is ignored if the unconditional flag is true.
	 */
	public void destroyXlet(boolean unconditional)
		throws XletStateChangeException;

}
