package javax.microedition.xlet;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2006  All Rights Reserved
 */

import java.awt.Container;

public interface XletContext {

	/** The property key used for the Xlet arguments.  The arguments are returned as
	 * an array of Strings. If there are no arguments, a zero length array will be returned.
	 */
	public static final String ARGS = "javax.microedition.xlet.args"; //$NON-NLS-1$

	/**
	 * Retrieves the property <code>key</code> from this XletContext.
	 *
	 * @return Object the value found for <code>key</code>, null if none was found.
	 * @throws NullPointerException  if <code>key</code> is null.
	 * @throws IllegalArgumentException  if <code>key</code> is an empty string.
	 */
	public Object getXletProperty(String key);

	/**
	 * This method is called by an active Xlet to notify its Xlet Application Manager that it has
	 * entered the Paused state.  Prior to calling this method, the Xlet needs to have
	 * become quiescent; the Xlet Application Manager will not call Xlet.pauseXlet().
	 */
	public void notifyPaused();

	/**
	 * This method is called by a paused Xlet to request that its Xlet Application Manager move it to
	 * Active state.  If the request is satisfied, the Xlet Application Manager will call Xlet.startXlet(),
	 * on a different thread.  This method does not guarentee that Xlet startXlet() will be called.
	 */
	public void resumeRequest();

	/**
	 * This method is called by an Xlet to notify its Xlet Application Manager that it has
	 * entered the Destroyed state.  Prior to calling this method, the Xlet needs to have
	 * performed its cleanup; the Xlet Application Manager will not call Xlet.destroyXlet().
	 */
	public void notifyDestroyed();

	/**
	 * Returns the AWT container for the Xlet to use.  The container will be an instance of java.awt.Frame or a
	 * java.awt.Container within a Frame.  The container will be invisible, with an arbitrary position and size.
	 *
	 * @return java.awt.Container  AWT container for the Xlet to use.
	 * @throws UnavailableContainerException  if the container is unavailable
	 */
	public Container getContainer() throws UnavailableContainerException;

	public ClassLoader getClassLoader();

}
