package javax.microedition.xlet.ixc;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2005, 2006  All Rights Reserved
 */

import java.security.Permission;
import java.security.PermissionCollection;
import java.util.StringTokenizer;

import com.ibm.oti.xlet.ixc.IxcPermissionCollection;
import com.ibm.oti.xlet.ixc.IxcUtil;

public final class IxcPermission extends Permission {

	private static final long serialVersionUID = 3256442495323026484L;
	private String actions;

	public IxcPermission(String name, String actions) {
		super(name);
		_setActions(actions);
	}

	private void _setActions(String actions) {
		if (actions == null) throw new IllegalArgumentException("The 'actions' should be not null");
		if (actions.length() == 0) throw new IllegalArgumentException("The 'actions' should be not empty");
		String temp = actions.toLowerCase().trim();
		StringBuffer result = new StringBuffer("");
		if (temp.indexOf("bind") >= 0) {
			result.append("bind");
		}
		if (temp.indexOf("lookup") >= 0) {
			if (result.length() > 0) {
				result.append(',');
			}
			result.append("lookup");
		}
		this.actions = result.toString();
	}

	public boolean implies(Permission permission) {
		if (!(permission instanceof IxcPermission)) return false;
		if (!actionsContain(permission.getActions())) return false;
		if (!IxcUtil.pathNameImplies(this, (IxcPermission)permission)) return false;
		return true;
	}
	private boolean actionsContain(String otherActions) {
		StringTokenizer tok = new StringTokenizer(otherActions, ",");
		String actions = getActions();
		while (tok.hasMoreTokens()) {
			if (actions.indexOf(tok.nextToken()) == -1) return false;
		}
		return true;
	}

	public boolean equals(Object object) {
		if (object == this)
			return true;
		if (!(object instanceof IxcPermission))
			return false;
		IxcPermission permission = (IxcPermission) object;
		return (
			getName().equals(permission.getName()) &&
			getActions().equals(permission.getActions()));
	}

	public int hashCode() {
		int hash = 7;
		hash = 31 * hash + getName().hashCode();
		hash = 31 * hash + getActions().hashCode();
		return hash;
	}

	public String getActions() {
		return actions;
	}

	public PermissionCollection newPermissionCollection() {
		return new IxcPermissionCollection();
	}

}
