package javax.microedition.xlet.ixc;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2003, 2006  All Rights Reserved
 */

import java.rmi.*;
import java.rmi.registry.Registry;
import java.util.Hashtable;

import javax.microedition.xlet.XletContext;

/**
 * This class is used to provide a remote object registry for inter-Xlet communication (IXC).
 *
 * @author      IBM
 * @version		initial
 */
public abstract class IxcRegistry implements Registry {

	private static Hashtable contextToRegistry = new Hashtable();

	protected IxcRegistry() {
		super();
	}

	/**
	 * Answers the inter-Xlet communication registry for the requesting Xlet.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		context XletContext
	 *					context of the requesting Xlet.
	 * @return		IxcRegistry
	 *					Xlet registry of remote object references.
	 * @exception	NullPointerException
	 *					if context is null.
	 */
	public static synchronized IxcRegistry getRegistry(XletContext context) {
		if (context == null)
			throw new NullPointerException("Context is null"); //$NON-NLS-1$
		if (context.getClassLoader() == null)
			throw new NullPointerException("Context ClassLoader is null"); //$NON-NLS-1$
		//		ClassLoader cl_nonboot = VM.getNonBootstrapClassLoader();
		//		ClassLoader cl_caller = VM.callerClassLoader();
		//		ClassLoader cl2 = context.getClass().getClassLoader();
		//		ClassLoader[] cls = VM.getStackClassLoaders(20);

		IxcRegistry registry = (IxcRegistry) contextToRegistry.get(context);
		if (registry == null) {
			registry = new com.ibm.oti.xlet.ixc.IxcRegistryImpl(context);
			contextToRegistry.put(context, registry);
		}
		return registry;
	}

	/**
	 * Answers a reference to a remote object specified by its name.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		name String
	 *					name of the remote object.
	 * @return		Remote
	 *					reference to the remote object.
	 * @exception	NotBoundException
	 *					if the name was not found.
	 * @exception	StubException
	 *					if a remote object stub can not be generated.
	 */
	public abstract Remote lookup(String name)
		throws StubException, NotBoundException;

	/**
	 * Binds a name to a remote object.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		name String
	 *					name of the remote object.
	 * @param		remote Remote
	 *					reference to the remote object.
	 *
	 * @exception	AlreadyBoundException
	 *					if a remote object with this name was already bound in the registry.
	 * @exception	StubException
	 *					if a remote object stub can not be generated.
	 */
	public abstract void bind(String name, Remote remote)
		throws StubException, AlreadyBoundException;

	/**
	 * Unbinds a name from a remote object.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		name String
	 *					name of the remote object.
	 *
	 * @exception	NotBoundException
	 *					if the name was not found.
	 * @exception	AccessException
	 *					if the name was not originally bound by the calling Xlet.
	 */
	public abstract void unbind(String name)
		throws NotBoundException, AccessException;

	public abstract void unbindAll();

	/**
	 * Binds a name to a different remote object, replacing any existing binding for this name.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param		name String
	 *					name of the remote object.
	 * @param		remote Remote
	 *					reference to the remote object.
	 * @exception	StubException
	 *					if a remote object stub can not be generated.
	 * @exception	AccessException
	 *					if the name was not originally bound by the calling Xlet.
	 */
	public abstract void rebind(String name, Remote remote)
		throws StubException, AccessException;

	/**
	 * Answers an array of Strings with all the names of the remote objects
	 * bound in the registry.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @return		String[]
	 *					a list of all the names of the remote objects bound in the registry.
	 */
	public abstract String[] list();
}
