package org.ubicollab.core.sm.model;

import java.util.Date;

import org.ubicollab.core.lm.model.ISensorReading;

public class Space {
	//Unique identification of Space
	private long Id;
	
	//Human readable name of space
	private String Name;
	
	//Owner of this space
	private String Owner;

	//Description of space
	private String Description;
	
	//Id of parent space
	private long ParentSpaceId;
	
	//Date time retrieved on
	private Date Created;
	
	//SensorReadings associated with this space
	private ISensorReading[] SensorReadings;
	
	//Default constructor
	public Space(){		
	}
	
	//Overloaded constructor
	public Space(long Id, String Name, String Description, String Owner, long ParentSpaceId,  
			Date Created, ISensorReading[] Readings){
		this.Id = Id;
		this.Name = Name;
		this.Owner = Owner;
		this.Description = Description;
		this.ParentSpaceId = ParentSpaceId;
		this.Created = Created;
		this.SensorReadings = Readings;
	}
	
	//Returns Space details in XML format
	public String getXML(){
		String xml = "<Space>";
		xml += "<Id>" + this.Id + "</Id>";
		xml += "<Name>" + this.Name + "</Name>";
		xml += "<Description>" + this.Description + "</Description>";
		xml += "<ParentSpaceId>" + this.ParentSpaceId + "</ParentSpaceId>";
		
		if( SensorReadings != null ){
			xml += "<Sensors>";
			for(int i=0; i< SensorReadings.length; i++){
				xml += "<Sensor>";
				xml += "<UniqueId>" + SensorReadings[i].getUniqueID() + "</UniqueId>";
				xml += "<Name>" + SensorReadings[i].getHumanReadableName() + "</Name>";
				xml += "<Reading>" + SensorReadings[i].getSignalStrength() + "</Reading>";
				xml += "<SensorType>" + SensorReadings[i].getSensorType() + "</SensorType>";
				xml += "</Sensor>";
			}
			
			xml += "</Sensors>";
		}
			
		xml += "</Space>";
		
		return "<?xml version=\"1.0\" encoding=\"utf-8\" ?>" + xml;
	}
	
	//Returns Space Unique Identifier
	public long getId(){
		return this.Id;
	}
	
	//Set Space Unique Identifier
	public void setId(long Id){
		this.Id = Id;
	}
	
	//Returns Space Name
	public String getName(){
		return this.Name;
	}
	
	//Sets Space Name
	public void setName(String Name){
		this.Name = Name;
	}
	
	//Returns Owner of Space
	public String getOwner(){ 
		return this.Owner;
	}
	
	//Set Owner of Space
	public void setOwner(String Owner){
		this.Owner = Owner;
	}
	
	//Returns Unique Identifier of Parent Space
	public long getParentSpaceId(){
		return this.ParentSpaceId;
	}
	
	//Set Unique Identifier of Parent Space
	public void setParentSpaceId(long ParentSpaceId){
		this.ParentSpaceId = ParentSpaceId;		
	}
	
	//Return Space Description
	public String getDescription(){
		return this.Description;		
	}
	
	//Sets Space Description
	public void setDescription(String Description){
		this.Description = Description;
	}
	
	//Returns Created Date
	public Date getDateCreated(){
		return this.Created;
	}
	
	//Sets Created Date
	public void setDateRetrieved(Date Created){
		this.Created = Created;
	}
	
	//Returns associated sensor readings
	public ISensorReading[] getSensorReadings(){
		return this.SensorReadings;
	}
	
	//Associate sensor reading to space
	public void setSensorReadings(ISensorReading[] readings){
		this.SensorReadings = readings;
	}
}
