package org.ubicollab.core.sm.model;

import java.net.URL;
import java.sql.ResultSet;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;


import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.ubicollab.core.lm.model.Coordinate;
import org.ubicollab.core.lm.model.IObservableSensor;
import org.ubicollab.core.lm.model.ISensor;
import org.ubicollab.core.lm.model.ISensorReading;
import org.ubicollab.core.lm.model.SensorReading;
import org.ubicollab.core.lm.model.SensorType;
import org.ubicollab.core.spacedb.DBHelper;

public class SpaceManagerImpl implements ISpaceManager {
	private DBHelper _dbHelper = null;
	private BundleContext _context = null;
	
	public SpaceManagerImpl(){
		_dbHelper = Activator.dbHelper;
		_context = Activator.context;
	}
	
	public long CreateSpace(String Name, String Description, String Purpose, 
			boolean Shareable, long ParentSpaceId, ISensorReading[] SensorReadings) {
		
		long newSpaceId = 1;
		
		try {						
			_dbHelper.Open();			
			
			String strSql = "SELECT MAX(Id) FROM Space";
			Object o = _dbHelper.ExecuteScalar(strSql);
			
			if( o != null ){
				newSpaceId += Integer.parseInt(o.toString());
			}
			
			String DATE_FORMAT = "yyyy-mm-dd hh:mm:ss";
		    SimpleDateFormat sdf =
		          new SimpleDateFormat(DATE_FORMAT);
		    Calendar c1 = Calendar.getInstance(); // today		    
			
			strSql = "INSERT INTO Space(Id, Name, Owner, Description, IsShareable, DateCreated, ParentSpaceId) VALUES(" +
				newSpaceId + ", '" + Name + "', '', '" + Description + "', " + Shareable + 
				", '" + sdf.format(c1.getTime()) + "', -1);";
			
			int count = _dbHelper.ExecuteUpdate(strSql);					
			
			if( count > 0 ){
				if( SensorReadings != null ){
					for(int i=0; i<SensorReadings.length; i++){									
						int sensorId = GetSensorId(SensorReadings[i].getUniqueID());
						
						if( sensorId == -1 ){
							sensorId = AddNewSensor(SensorReadings[i].getUniqueID(), 
									SensorReadings[i].getHumanReadableName(), 
									SensorReadings[i].getSensorType());
						}
						
						AddSensorReading(newSpaceId, sensorId, SensorReadings[i].getSignalStrength(), 
								SensorReadings[i].getCoordinates());
					}				
				}
			}
						
			_dbHelper.Close();
		} catch (Exception e) {
			System.out.println("An error has occured in CreateSpace method: " + e.getMessage());
		}
		
		return newSpaceId;
	}

	private int AddSensorReading(long SpaceId, int SensorId, int Reading, Coordinate coordinate) throws Exception{
		int maxId = 1;
		String strSql = "SELECT MAX(ReadingId) FROM SensorReading";		
		Object o = _dbHelper.ExecuteScalar(strSql);
		if( o != null ){
			maxId += Integer.parseInt(o.toString());
		}
		
		String cord = "";
		if( coordinate != null )
			cord = coordinate.toString();
		
		strSql = "INSERT INTO SensorReading(ReadingId, SpaceId, SensorId, Reading, Coordinate) VALUES(" + maxId + 
			", " + SpaceId + ", " + SensorId + ", " + Reading + ", '" + cord + "');";
		
		int iRows = _dbHelper.ExecuteUpdate(strSql);
		
		if( iRows > 0 )
			return maxId;
		
		return -1;
	}
	
	private int AddNewSensor(String UniqueId, String HumanReadableName, String SensorType) throws Exception {
		int maxId = 1;
		
		String strSql = "SELECT MAX(SensorId) FROM Sensor";
		Object o = _dbHelper.ExecuteScalar(strSql);
		
		if( o != null ){
			maxId += Integer.parseInt(o.toString());
		}
		
		strSql = "INSERT INTO Sensor(SensorId, SensorType, UniqueIdentifier, HumanReadableName) VALUES(" + maxId +
			", '" + SensorType + "', '" + UniqueId + "', '" + HumanReadableName + "');";
		
		int iRows = _dbHelper.ExecuteUpdate(strSql);		
		
		if( iRows > 0 )
			return maxId;
		
		return -1;
	}
	
	private int GetSensorId(String UniqueID) throws Exception{		
		int sensorId = -1;
		
		Object o = _dbHelper.ExecuteScalar("SELECT SensorId FROM Sensor WHERE UniqueIdentifier = '" + 
				UniqueID + "'");
		
		if( o != null ){
			sensorId = Integer.parseInt(o.toString());
		}
		
		return sensorId;
	}
	
	public Space GetSpace(ISensorReading[] SensorReadings) {
		Space objSpace = null;
		try {
			if( SensorReadings != null ){				
				if( SensorReadings[0].getSensorType() == SensorType.WiFi ){
					String strSql =
						"SELECT sr.SpaceId as SpaceID, sr.Reading as SensorReading, s.UniqueIdentifier as UniqueId, s.HumanReadableName as Name " +
				        "FROM SensorReading sr " +
				        "INNER JOIN Space sp ON sr.SpaceId = sp.Id " +
				        "INNER JOIN Sensor s ON sr.SensorId = s.SensorId WHERE ";
					
					String strFilter = "";
					for(int i=0; i<SensorReadings.length; i++ ){
						if( SensorReadings[i].getSensorType() == SensorType.WiFi )
							strFilter += " s.UniqueIdentifier = '" + SensorReadings[i].getUniqueID() + "' OR";
					}
					
					strFilter = strFilter.substring(0, strFilter.length() - 2);	
					strFilter = strFilter + " AND SensorType = '" + SensorType.WiFi + "'";
					
					strSql += strFilter;
					
					_dbHelper.Open();
					
					ResultSet rs = _dbHelper.ExecuteResultSet(strSql); 
					int prevLocationId = -1;
					int signalIndex = 0;
					SensorReadingData fpData = new SensorReadingData();		
					ArrayList fpArray = new ArrayList();	
					
					while(rs.next()){
						int currentLocId = rs.getInt("SpaceId");
						if( prevLocationId != currentLocId ){
							if( prevLocationId != -1 ){
								fpArray.add(fpData);							
							}
							
							fpData = new SensorReadingData();
							fpData.SpaceId = currentLocId;
							signalIndex = 0;
							prevLocationId = currentLocId;
						}					
						
						fpData.readings[signalIndex] = new SensorReading(
								rs.getString("UniqueId"), 
								rs.getString("Name"), 
								rs.getInt("SensorReading"), null, SensorType.WiFi);
						
						signalIndex++;
					}
					
					fpArray.add(fpData);
					
					SensorReadingData foundReading = MatchFingerPrint(SensorReadings, fpArray);
					
					if( foundReading != null ){
						objSpace = GetSpace(foundReading.SpaceId);
					}
					
					_dbHelper.Close();
				} else if( SensorReadings[0].getSensorType() == SensorType.GPS ){
					
				} else if( SensorReadings[0].getSensorType() == SensorType.RFID ){
					
				}
			}
		} catch (Exception e){
			System.out.println(e.getMessage());
		}
		return objSpace;
	}

	private class SensorReadingData{
		public long SpaceId = -1;
		public ISensorReading[] readings = new ISensorReading[3];
	}
		
	private SensorReadingData MatchFingerPrint(ISensorReading[] observedAPs, ArrayList fpArray){
		double closestMatch = 0;
		int closestMatchIndex = -1;
		
		for(int i=0; i<fpArray.size(); i++){
			SensorReadingData objFP = (SensorReadingData) fpArray.get(i);
			ISensorReading[] aps = objFP.readings;
			
			double distance = EuclideanDistance(observedAPs, aps);
			if( distance < closestMatch || i == 0 ){
				closestMatch = distance;
				closestMatchIndex = i;
			}
		}
		
		return (SensorReadingData) fpArray.get(closestMatchIndex);
	}
	
	private double EuclideanDistance(ISensorReading[] observedAPs, ISensorReading[] savedAPs){
		double distance = 0.0;
		
		for(int i=0; i<observedAPs.length; i++){
			boolean bFoundBssId = false;
			
			for(int j=0; j<savedAPs.length; j++){
				if( savedAPs[j] != null){					
					if( savedAPs[j].getUniqueID().equals(observedAPs[i].getUniqueID()) ){
						distance += Math.pow(
								observedAPs[i].getSignalStrength() - observedAPs[i].getSignalStrength(), 2);
						bFoundBssId = true;
					}
				}
			}
			
			if( bFoundBssId == false ){
				distance += Math.pow(65, 2);
			}
		}
		
		return Math.sqrt(distance);
	}
	
	public Space GetCurrentSpace() {
		return null;
	}

	public Space GetSpace(long SpaceId) {
		Space sp = null;
		try {
			String strSql = "SELECT * FROM Space WHERE Id = " + SpaceId;
			_dbHelper.Open();
			
			ResultSet rs = _dbHelper.ExecuteResultSet(strSql);
			
			if( rs.next() ){
				sp = new Space(rs.getLong("Id"), rs.getString("Name"), rs.getString("Description"), rs.getString("Owner"),
						rs.getLong("ParentSpaceId"), rs.getDate("DateCreated"), null);				
			}
			
			if( sp != null ){
				ArrayList readings = new ArrayList();
				strSql = "SELECT s.*, sr.Reading FROM Sensor s INNER JOIN SensorReading sr ON s.SensorId = sr.SensorId WHERE SpaceId = 1";
				rs = _dbHelper.ExecuteResultSet(strSql);
				
				while( rs.next() ){
					ISensorReading sr = new SensorReading(rs.getString("UniqueIdentifier"), 
							rs.getString("HumanReadableName"), rs.getInt("Reading"), new Coordinate(), 
							rs.getString("SensorType"));
					
					readings.add(sr);
				}
				
				ISensorReading[] sensorReadings = new ISensorReading[readings.size()];
				for(int i=0; i<readings.size(); i++){
					sensorReadings[i] = (SensorReading) readings.get(i);
				}
				
				sp.setSensorReadings(sensorReadings);
			}
			
			_dbHelper.Close();
		} catch (Exception e){
			System.out.println("GetSpace(long SpaceId): " + e.getMessage());
		}
		
		return sp;
	}

	public Space SetSpace() {
		Space spSet = null;
		try {			
			ServiceReference[] sRefs = _context.getServiceReferences(
					ISensor.class.getName(), null);
			
			if( sRefs != null ){
				for( int i=0; i<sRefs.length; i++){
					ISensor sensor = (ISensor) _context.getService(sRefs[i]);
					if( sensor != null && sensor instanceof IObservableSensor ){
						ISensorReading[] readings = sensor.Read();
						
						spSet = GetSpace(readings);
						if( spSet != null )
							break;								
					}
				}
			}
			
			if( spSet != null ){
				SetSpace(spSet.getId());
			}
		} catch (Exception e){
			System.out.println(e.getStackTrace());
		}
		
		return spSet;
	}

	public Space SetSpace(long SpaceId) {
		String DATE_FORMAT = "yyyy-mm-dd hh:mm:ss";
	    SimpleDateFormat sdf =
	          new SimpleDateFormat(DATE_FORMAT);
	    Calendar c1 = Calendar.getInstance();
	    
		String strSql = "UPDATE CurrentSpace SET SpaceId = " + SpaceId + ", UpdatedDate = '" + 
			sdf.format(c1.getTime()) + "'";
		try {
			_dbHelper.Open();
			int iCount = _dbHelper.ExecuteUpdate(strSql);
			if( iCount <= 0 ){
				strSql = "INSERT INTO CurrentSpace(SpaceId, UpdatedDate) VALUES(" + SpaceId + ", '" +
				sdf.format(c1.getTime()) + "');";
				
				iCount = _dbHelper.ExecuteUpdate(strSql);
				
				if( iCount > 0 ){
					return GetSpace(SpaceId);
				}
			}
		} catch (Exception e){
			System.out.println(e.getStackTrace());
		}
		
		return null;
	}	
	
	public Space[] GetUserSpace(){
		Space[] userSpaces = null;
		ArrayList list = new ArrayList();
		
		try {
			_dbHelper.Open();
		
			String strSql = "SELECT * FROM Space";
			ResultSet rs = _dbHelper.ExecuteResultSet(strSql);
			while( rs.next() ){
				Space sp = new Space(rs.getLong("Id"), rs.getString("Name"), rs.getString("Description"),
						rs.getString("Owner"), rs.getLong("ParentSpaceId"), new Date(), null);
				System.out.println( sp.getName() );
				list.add(sp);
			}
		
			userSpaces = new Space[list.size()];
			for( int i=0; i<list.size(); i++){
				userSpaces[i] = (Space) list.get(i);
			}
			
			_dbHelper.Close();
		} catch (Exception e){
			System.out.println(e.getMessage());
		}
		
		return userSpaces;
	}

	public Space DownloadSpace(String Uri) {
		String xmlContent = ReadUrl(Uri);
		
		Space sp = null;
		try { 
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
			
			InputSource inputSource = new InputSource();
			inputSource.setCharacterStream(new StringReader(xmlContent));
			
			Document doc = docBuilder.parse(inputSource);
			
			doc.getDocumentElement ().normalize ();
			System.out.println ("Root element of the doc is " + doc.getDocumentElement().getNodeName());


			NodeList listOfSpace = doc.getElementsByTagName("Space");
			int totalSpace = listOfSpace.getLength();
			
			sp = new Space();
			
			System.out.println("Total no of spaces : " + totalSpace);			
			
			for(int s=0; s<listOfSpace.getLength() ; s++){
				Node spaceNode = listOfSpace.item(s);
				if(spaceNode.getNodeType() == Node.ELEMENT_NODE){
					Element spaceElement = (Element)spaceNode;
					
					NodeList spaceIdList = spaceElement.getElementsByTagName("Id");
					Element spaceIdElement = (Element)spaceIdList.item(0);

					NodeList txtSpaceIdList = spaceIdElement.getChildNodes();
					System.out.println("Space Id : " + ((Node)txtSpaceIdList.item(0)).getNodeValue().trim());
					sp.setId(Long.parseLong(((Node)txtSpaceIdList.item(0)).getNodeValue().trim()));
					
					NodeList spaceNameNodeList = spaceElement.getElementsByTagName("Name");
					Element spaceNameElement = (Element)spaceNameNodeList.item(0);

					NodeList txtSpaceNameList = spaceNameElement.getChildNodes();
					System.out.println("Space Name : " + ((Node)txtSpaceNameList.item(0)).getNodeValue().trim());
					sp.setName(((Node)txtSpaceNameList.item(0)).getNodeValue().trim());
					
					NodeList spaceDescList = spaceElement.getElementsByTagName("Description");
					Element spaceDescElement = (Element)spaceDescList.item(0);

					NodeList txtSpaceDescList = spaceDescElement.getChildNodes();
					System.out.println("Description : " + ((Node)txtSpaceDescList.item(0)).getNodeValue().trim());
					sp.setDescription(((Node)txtSpaceDescList.item(0)).getNodeValue().trim());
					
					NodeList parentSpaceList = spaceElement.getElementsByTagName("ParentSpaceId");
					Element parentSpaceElement = (Element)parentSpaceList.item(0);

					NodeList txtParentSpaceIdList = parentSpaceElement.getChildNodes();
					System.out.println("ParentSpaceId : " + ((Node)txtParentSpaceIdList.item(0)).getNodeValue().trim());
					sp.setParentSpaceId(Long.parseLong(((Node)txtParentSpaceIdList.item(0)).getNodeValue().trim()));
					
					NodeList listOfSensors = doc.getElementsByTagName("Sensor");
					int sensorCount = listOfSensors.getLength();
					System.out.println(sensorCount);
					
					SensorReading[] readings = new SensorReading[sensorCount];
					
					for(int j=0; j<listOfSensors.getLength(); j++){
						Node sensorNode = listOfSensors.item(j);
						if( sensorNode.getNodeType() == Node.ELEMENT_NODE ){							
							String UniqueId = "", Name = "", SensorType = "";
							Coordinate c = null;
							int Signal = -1;
							
							Element sensorElement = (Element)sensorNode;
							
							NodeList sensorIdList = sensorElement.getElementsByTagName("UniqueId");
							Element sesorIdElement = (Element)sensorIdList.item(0);

							NodeList txtSensorIdList = sesorIdElement.getChildNodes();
							System.out.println("Sensor Id : " + ((Node)txtSensorIdList.item(0)).getNodeValue().trim());
							UniqueId = ((Node)txtSensorIdList.item(0)).getNodeValue().trim();
							
							NodeList sensorNameList = sensorElement.getElementsByTagName("Name");
							Element sesorNameElement = (Element)sensorNameList.item(0);

							NodeList txtSensorNameList = sesorNameElement.getChildNodes();
							System.out.println("Sensor Name : " + ((Node)txtSensorNameList.item(0)).getNodeValue().trim());
							Name = ((Node)txtSensorNameList.item(0)).getNodeValue().trim();
							
							NodeList sensorSignalList = sensorElement.getElementsByTagName("Reading");
							Element sesorSignalElement = (Element)sensorSignalList.item(0);

							NodeList txtSensorSignalList = sesorSignalElement.getChildNodes();
							System.out.println("Sensor Signal : " + ((Node)txtSensorSignalList.item(0)).getNodeValue().trim());
							Signal = Integer.parseInt(((Node)txtSensorSignalList.item(0)).getNodeValue().trim());
							
							NodeList sensorTypeList = sensorElement.getElementsByTagName("SensorType");
							Element sensorTypeElement = (Element)sensorTypeList.item(0);

							NodeList txtSensorTypeList = sensorTypeElement.getChildNodes();
							System.out.println("Sensor Signal : " + ((Node)txtSensorTypeList.item(0)).getNodeValue().trim());
							SensorType = ((Node)txtSensorTypeList.item(0)).getNodeValue().trim();
							
							readings[j] = new SensorReading(UniqueId, Name, Signal, null, SensorType);
						}
					}
					
					sp.setSensorReadings(readings);
				}
			}
			
		} catch (Exception e){
			System.out.println(e.getMessage());
		}
		
		return sp;
	}
	
	private String ReadUrl(String Uri){
		String returnXml = "";
		
		try {
			URL yahoo = new URL(Uri);
			BufferedReader in = new BufferedReader(new InputStreamReader(yahoo.openStream()));
			
			String inputLine;

			while ((inputLine = in.readLine()) != null)
				returnXml += inputLine;

			in.close();
			
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return returnXml;
	}
}
