package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.awt.*;
import java.lang.reflect.InvocationTargetException;

/**
 *
 */
public class InvocationEvent extends AWTEvent implements ActiveEvent {

	// The time when the receiver was created
	// For now there is no constructor to set this field...
	long when = System.currentTimeMillis();
	/**
	 *
	 */
	public static final int INVOCATION_FIRST = 1200;

	/**
	 *
	 */
	public static final int INVOCATION_DEFAULT = 1200;

	/**
	 *
	 */
	public static final int INVOCATION_LAST = 1200;

	/**
	 *
	 */
	protected Runnable runnable;

	/**
	 *
	 */
	protected Object notifier;

	/**
	 *
	 */
	protected boolean catchExceptions;

	Exception exception;

	static final long serialVersionUID = 436056344909459450l;

	/**
	 *
	 */
	public InvocationEvent(Object source, Runnable runnable) {
		this(source,INVOCATION_DEFAULT,runnable,null,false);
	}

	/**
	 *
	 */
	public InvocationEvent(Object source, Runnable runnable, Object notifier, boolean catchExceptions) {
		this(source,INVOCATION_DEFAULT,runnable,notifier,catchExceptions);
	}

	/**
	 *
	 */
	protected InvocationEvent(Object source, int id, Runnable runnable, Object notifier, boolean catchExceptions) {
		super(source,id);

		this.runnable        = runnable;
		this.notifier        = notifier;
		this.catchExceptions = catchExceptions;
	}

	/**
	 *
	 */
	public void dispatch() {
		Object tempNotifier = (notifier == null) ? new Object() : notifier;

		synchronized(tempNotifier) {
			try {
				exception= null;
				Exception tempException= null;

				try {
					EventQueue.invokeAndWait(runnable);
				} catch (InvocationTargetException e) {
					tempException= e;
				} catch (InterruptedException e) {
					tempException= e;
				}

				if (tempException != null) {
					if (tempException instanceof InvocationTargetException) {
						Throwable t= ((InvocationTargetException) tempException).getTargetException();
						if (t instanceof Exception) {
							tempException= (Exception) t;
						}
					}

					if (catchExceptions) {
						exception= tempException;
					} else {
						if (tempException instanceof RuntimeException) throw (RuntimeException) tempException;
						throw new RuntimeException(tempException.toString());
					}
				}
			} finally {
				tempNotifier.notifyAll();
			}
		}
	}

	/**
	 *
	 */
	public Exception getException() {
		return exception;
	}

	/**
	 *
	 */
	public String paramString() {
		StringBuffer buffer = new StringBuffer(super.paramString());

		switch (getID()) {
			case INVOCATION_DEFAULT:
				buffer.append("INVOCATION_DEFAULT"); //$NON-NLS-1$
				break;
			default:
				buffer.append("unknown"); //$NON-NLS-1$
		}

		return buffer.toString();
	}
	/**
	 * Answer the time that the event represented by
	 * the receiver occured.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @return	a long	the time of the event
	 */
	public long getWhen() {
		return when;
	}

}
