package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2004, 2005  All Rights Reserved
 */

import java.awt.Component;

public class MouseWheelEvent extends MouseEvent {

	private static final long serialVersionUID = 6459879390515399677L;

	public static final int WHEEL_BLOCK_SCROLL = 1;
	public static final int WHEEL_UNIT_SCROLL = 0;

	int scrollType;
	int scrollAmount;
	int wheelRotation;

	public MouseWheelEvent(Component source, int id, long when, int modifiers, int x, int y, int clickCount, boolean popupTrigger, int scrollType, int scrollAmount, int wheelRotation) {
		super(source, id, when, modifiers, x, y, clickCount, popupTrigger);
		this.scrollType = scrollType;
		this.scrollAmount = scrollAmount;
		this.wheelRotation = wheelRotation;
	}

	public int getScrollType() {
		return scrollType;
	}

	public int getScrollAmount() {
		return scrollAmount;
	}

	public int getWheelRotation() {
		return wheelRotation;
	}

	public int getUnitsToScroll() {
		return getWheelRotation() * getScrollAmount();
	}

	public String paramString() {
		StringBuffer buffer = new StringBuffer(super.paramString());

		buffer.append(",scrollType=");
		switch (scrollType) {
			case WHEEL_BLOCK_SCROLL:
				buffer.append("WHEEL_BLOCK_SCROLL");
				break;
			case WHEEL_UNIT_SCROLL:
				buffer.append("WHEEL_UNIT_SCROLL");
				break;
			default:
				buffer.append("unknown");
		}

		buffer.append(",scrollAmount=");
		buffer.append(scrollAmount);

		buffer.append(",wheelRotation=");
		buffer.append(wheelRotation);

		return buffer.toString();
	}

}
