package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.awt.Component;
import java.awt.Rectangle;

/**
 * This class represents events which model the drawing of
 * a <code>Component</code>.
 * This event is not propogated to a listener as there are no
 * listeners for paint and update events.  To get the event
 * subclasses of <code>Component</code> should re-implement
 * <code>paint</code> and/or <code>update</code> and drawn
 * themselves.
 *
 * @author      IBM
 * @version		initial
 *
 */
public class PaintEvent extends ComponentEvent {

	static final long serialVersionUID = 1267492026433337593L;

	// The rectangle that needs to be redrawn in the source of the receiver
	Rectangle updateRect;

	// Paint event ID constants
	public static final int PAINT = 800;
	public static final int PAINT_FIRST = 800;
	public static final int PAINT_LAST = 801;
	public static final int UPDATE = 801;

	/**
	 * Constructs a new <code>PaintEvent</code> using the
	 * specified id and damage rectangle.  The receiver will
	 * be an event that occured in the specified source object.
	 *
	 * @author      IBM
	 * @version 	initial
	 *
	 * @param	source		the source object of the receiver
	 * @param	id			the id of the receiver
	 * @param	damageRect	the rectangle to draw
	 *
	 */
	public PaintEvent(Component source, int id, Rectangle damageRect) {
		super(source, id);

		this.updateRect = damageRect;
	}

	/**
	 * Answer the rectangle that represents the area in the receiver's
	 * source that has been damaged and needs to be redrawn.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @return	a <code>Rectangle</code>	the damaged area
	 */
	public Rectangle getUpdateRect() {
		return updateRect;
	}

	/**
	 * Answer a string with the relevant information describing the state
	 * of the receiver.  This will not be <code>null</code>.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @return	a <code>String</code> 	the param string.
	 */
	public String paramString() {
		StringBuffer buffer = new StringBuffer(super.paramString());
		switch (getID()) {
			case PAINT :
				buffer.append("PAINT"); //$NON-NLS-1$
				break;
			case UPDATE :
				buffer.append("UPDATE"); //$NON-NLS-1$
				break;
			default :
				buffer.append("unknown"); //$NON-NLS-1$
		}
		buffer.append(",updateRect="); //$NON-NLS-1$
		buffer.append(getUpdateRect());
		return buffer.toString();
	}

	/**
	 * Set the rectangle that represents the area in the receiver's
	 * source that has been damaged and needs to be redrawn.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @param	damageRect	the damaged area
	 */
	public void setUpdateRect(Rectangle damageRect) {
		this.updateRect = damageRect;
	}
}
