package java.awt.event;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2005  All Rights Reserved
 */

import java.awt.AWTEvent;

/**
 * This class represents events which model text editing that occurs
 * on a <code>Component</code>.
 * <p>
 * This event is propogated to all <code>TextListeners</code> that
 * are registered on the <code>Component</code> using the
 * <code>addTextListener</code> method, text events occurring
 * on that component will be delivered to the text listener via
 * the sending of the <code>textValueChanged</code> method.
 * <p>
 * @author		IBM
 * @version		initial
 *
 */
public class TextEvent extends AWTEvent {

	static final long serialVersionUID = 6269902291250941179L;

	/**
	 * The first number in the range of ids used for text events.
	 */
	public static final int TEXT_FIRST = 900;

	/**
	 * The last number in the range of ids used for text events.
	 */
	public static final int TEXT_LAST = 900;

	/**
	 * This event id indicates that object's text changed.
	 */
	public static final int TEXT_VALUE_CHANGED = 900;

	/**
	 * Constructs a new <code>TextEvent</code> using the
	 * specified id.  The receiver will be
	 * an event that occured in the specified source object.
	 *
	 * @author	IBM
	 * @version	initial
	 *
	 * @param	source			the source object of the receiver
	 * @param	id				the id of the receiver
	 *
	 */
	public TextEvent(Object source, int id) {
		super(source, id);
	}

	/**
	 * Answer a string with the relevant information describing the state
	 * of the receiver.  This will not be <code>null</code>.
	 *
	 * @author      IBM
	 * @version		initial
	 *
	 * @return	a <code>String</code> 	the param string.
	 */
	public String paramString() {
		StringBuffer buffer = new StringBuffer(super.paramString());
		switch (getID()) {
			case TEXT_VALUE_CHANGED:
				buffer.append("TEXT_VALUE_CHANGED"); //$NON-NLS-1$
				break;
			default:
				buffer.append("unknown"); //$NON-NLS-1$
		}
		return buffer.toString();
	}

}
