package java.util;

/*
 * Licensed Materials - Property of IBM,
* (c) Copyright IBM Corp. 2006  All Rights Reserved
 */

final class TimeZones {

	private static final int ONE_HOUR = 3600000;

public static TimeZone[] getTimeZones() {
	return new TimeZone[] {

	// Pacific/Apia
	new SimpleTimeZone(-11 * ONE_HOUR, "MIT"),
	new SimpleTimeZone(-10 * ONE_HOUR, "HST"),
	// America/Anchorage
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-9 * ONE_HOUR, "AST",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-9 * ONE_HOUR, "AST",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167642000000L,	// 2007
	}),
	// America/Los_Angeles
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-8 * ONE_HOUR, "PST",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-8 * ONE_HOUR, "PST",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167638400000L,	// 2007
	}),
	// America/Denver
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-7 * ONE_HOUR, "MST",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-7 * ONE_HOUR, "MST",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167634800000L,	// 2007
	}),
	// America/Phoenix
	new SimpleTimeZone(-7 * ONE_HOUR, "PNT"),
	// America/Chicago
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "CST",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "CST",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167631200000L,	// 2007
	}),
	// America/New_York
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "EST",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "EST",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	// America/Indiana/Indianapolis
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "IET"),
		new SimpleTimeZone(-5 * ONE_HOUR, "IET",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "IET",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},	// 2006
			{2007, 0, 0, 0},
	}, new long[] {
			1136091600000L,	// 2006
			1167627600000L,	// 2007
	}),
	// America/Caracas
	new SimpleTimeZone(-4 * ONE_HOUR, "PRT"),
	// America/St_Johns
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-3 * ONE_HOUR - 1800000, "CNT",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-3 * ONE_HOUR - 1800000, "CNT",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167622200000L,	// 2007
	}),
	// America/Argentina/Buenos_Aires
	new SimpleTimeZone(-3 * ONE_HOUR, "AGT"),
	// America/Sao_Paulo
	new SimpleTimeZone(-3 * ONE_HOUR, "BET",
		Calendar.OCTOBER, 15, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.FEBRUARY, 15, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	// Etc/UTC
	new SimpleTimeZone(0 * ONE_HOUR, "UTC"),
	new SimpleTimeZone(0 * ONE_HOUR, "WET",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "CET",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	// Europe/Paris
	new SimpleTimeZone(1 * ONE_HOUR, "ECT",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "MET",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	// Africa/Cairo
	new SimpleTimeZone(2 * ONE_HOUR, "ART",
		Calendar.APRIL, -1, Calendar.FRIDAY, 0 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.SEPTEMBER, -1, Calendar.THURSDAY, 23 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	// Africa/Johannesburg
	new SimpleTimeZone(2 * ONE_HOUR, "CAT"),
	new SimpleTimeZone(2 * ONE_HOUR, "EET",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Asia/Riyadh
	new SimpleTimeZone(3 * ONE_HOUR, "EAT"),
	// Asia/Yerevan
	new SimpleTimeZone(4 * ONE_HOUR, "NET",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	// Asia/Karachi
	new SimpleTimeZone(5 * ONE_HOUR, "PLT"),
	// Asia/Calcutta
	new SimpleTimeZone(5 * ONE_HOUR + 1800000, "IST"),
	// Asia/Dhaka
	new SimpleTimeZone(6 * ONE_HOUR, "BST"),
	// Asia/Bangkok
	new SimpleTimeZone(7 * ONE_HOUR, "VST"),
	// Asia/Shanghai
	new SimpleTimeZone(8 * ONE_HOUR, "CTT"),
	// Asia/Shanghai
	new SimpleTimeZone(8 * ONE_HOUR, "PRC"),
	// Asia/Tokyo
	new SimpleTimeZone(9 * ONE_HOUR, "JST"),
	// Asia/Seoul
	new SimpleTimeZone(9 * ONE_HOUR, "ROK"),
	// Australia/Darwin
	new SimpleTimeZone(9 * ONE_HOUR + 1800000, "ACT"),
	// Australia/Sydney
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(10 * ONE_HOUR, "AET",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "AET",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "AET",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136034000000L,	// 2006
			1167570000000L,	// 2007
	}),
	// Pacific/Guadalcanal
	new SimpleTimeZone(11 * ONE_HOUR, "SST"),
	// Pacific/Auckland
	new SimpleTimeZone(12 * ONE_HOUR, "NST",
		Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.MARCH, 15, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-6 * ONE_HOUR, "America/Costa_Rica"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-4 * ONE_HOUR, "America/Halifax",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-4 * ONE_HOUR, "America/Halifax",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167624000000L,	// 2007
	}),
	new SimpleTimeZone(-2 * ONE_HOUR, "Atlantic/South_Georgia"),
	new SimpleTimeZone(0 * ONE_HOUR, "Europe/London",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Algiers"),
	new SimpleTimeZone(2 * ONE_HOUR, "Africa/Cairo",
		Calendar.APRIL, -1, Calendar.FRIDAY, 0 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.SEPTEMBER, -1, Calendar.THURSDAY, 23 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Africa/Harare"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 9, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 16, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 5, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 27, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 12, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 2, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 23, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 8, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 28, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 20, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 9, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 24, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 16, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 6, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 27, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 12, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 2, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 24, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 6, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 28, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 20, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 10, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 24, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 16, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 6, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 21, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 12, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 2, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 17, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 7, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 28, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 13, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Jerusalem"),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
			{2008, 0, 0, 0},
			{2009, 0, 0, 0},
			{2010, 0, 0, 0},
			{2011, 0, 0, 0},
			{2012, 0, 0, 0},
			{2013, 0, 0, 0},
			{2014, 0, 0, 0},
			{2015, 0, 0, 0},
			{2016, 0, 0, 0},
			{2017, 0, 0, 0},
			{2018, 0, 0, 0},
			{2019, 0, 0, 0},
			{2020, 0, 0, 0},
			{2021, 0, 0, 0},
			{2022, 0, 0, 0},
			{2023, 0, 0, 0},
			{2024, 0, 0, 0},
			{2025, 0, 0, 0},
			{2026, 0, 0, 0},
			{2027, 0, 0, 0},
			{2028, 0, 0, 0},
			{2029, 0, 0, 0},
			{2030, 0, 0, 0},
			{2031, 0, 0, 0},
			{2032, 0, 0, 0},
			{2033, 0, 0, 0},
			{2034, 0, 0, 0},
			{2035, 0, 0, 0},
			{2036, 0, 0, 0},
			{2037, 0, 0, 0},
			{2038, 0, 0, 0},
	}, new long[] {
			1136066400000L,	// 2006
			1167602400000L,	// 2007
			1199138400000L,	// 2008
			1230760800000L,	// 2009
			1262296800000L,	// 2010
			1293832800000L,	// 2011
			1325368800000L,	// 2012
			1356991200000L,	// 2013
			1388527200000L,	// 2014
			1420063200000L,	// 2015
			1451599200000L,	// 2016
			1483221600000L,	// 2017
			1514757600000L,	// 2018
			1546293600000L,	// 2019
			1577829600000L,	// 2020
			1609452000000L,	// 2021
			1640988000000L,	// 2022
			1672524000000L,	// 2023
			1704060000000L,	// 2024
			1735682400000L,	// 2025
			1767218400000L,	// 2026
			1798754400000L,	// 2027
			1830290400000L,	// 2028
			1861912800000L,	// 2029
			1893448800000L,	// 2030
			1924984800000L,	// 2031
			1956520800000L,	// 2032
			1988143200000L,	// 2033
			2019679200000L,	// 2034
			2051215200000L,	// 2035
			2082751200000L,	// 2036
			2114373600000L,	// 2037
			2145909600000L,	// 2038
	}),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Bucharest",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(3 * ONE_HOUR, "Europe/Moscow",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(3 * ONE_HOUR + 1800000, "Asia/Tehran",
			Calendar.MARCH, 22, 0, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 22, 0, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(3 * ONE_HOUR + 1800000, "Asia/Tehran"),
	}, new int[][] {
			{2006, 0, 0, 0},
	}, new long[] {
			1136061000000L,	// 2006
	}),
	new SimpleTimeZone(4 * ONE_HOUR + 1800000, "Asia/Kabul"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/Adelaide",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/Adelaide",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/Adelaide",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136035800000L,	// 2006
			1167571800000L,	// 2007
	}),
	new SimpleTimeZone(10 * ONE_HOUR, "Australia/Brisbane"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Hobart",
			Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Hobart",
			Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Hobart",
			Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136034000000L,	// 2006
			1167570000000L,	// 2007
	}),

	new SimpleTimeZone(-12 * ONE_HOUR, "Etc/GMT+12"),
	new SimpleTimeZone(-11 * ONE_HOUR, "Etc/GMT+11"),
	new SimpleTimeZone(-11 * ONE_HOUR, "Pacific/Apia"),
	new SimpleTimeZone(-11 * ONE_HOUR, "Pacific/Midway"),
	new SimpleTimeZone(-11 * ONE_HOUR, "Pacific/Niue"),
	new SimpleTimeZone(-11 * ONE_HOUR, "Pacific/Pago_Pago"),
	// Pacific/Pago_Pago
	new SimpleTimeZone(-11 * ONE_HOUR, "Pacific/Samoa"),
	// Pacific/Pago_Pago
	new SimpleTimeZone(-11 * ONE_HOUR, "US/Samoa"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-10 * ONE_HOUR, "America/Adak",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-10 * ONE_HOUR, "America/Adak",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167645600000L,	// 2007
	}),
	// America/Adak
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-10 * ONE_HOUR, "America/Atka",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-10 * ONE_HOUR, "America/Atka",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167645600000L,	// 2007
	}),
	new SimpleTimeZone(-10 * ONE_HOUR, "Etc/GMT+10"),
	new SimpleTimeZone(-10 * ONE_HOUR, "Pacific/Fakaofo"),
	new SimpleTimeZone(-10 * ONE_HOUR, "Pacific/Honolulu"),
	new SimpleTimeZone(-10 * ONE_HOUR, "Pacific/Johnston"),
	new SimpleTimeZone(-10 * ONE_HOUR, "Pacific/Rarotonga"),
	new SimpleTimeZone(-10 * ONE_HOUR, "Pacific/Tahiti"),
	// America/Adak
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-10 * ONE_HOUR, "US/Aleutian",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-10 * ONE_HOUR, "US/Aleutian",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167645600000L,	// 2007
	}),
	// Pacific/Honolulu
	new SimpleTimeZone(-10 * ONE_HOUR, "US/Hawaii"),
	new SimpleTimeZone(-9 * ONE_HOUR - 1800000, "Pacific/Marquesas"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-9 * ONE_HOUR, "America/Anchorage",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-9 * ONE_HOUR, "America/Anchorage",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167642000000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-9 * ONE_HOUR, "America/Juneau",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-9 * ONE_HOUR, "America/Juneau",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167642000000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-9 * ONE_HOUR, "America/Nome",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-9 * ONE_HOUR, "America/Nome",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167642000000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-9 * ONE_HOUR, "America/Yakutat",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-9 * ONE_HOUR, "America/Yakutat",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167642000000L,	// 2007
	}),
	new SimpleTimeZone(-9 * ONE_HOUR, "Etc/GMT+9"),
	new SimpleTimeZone(-9 * ONE_HOUR, "Pacific/Gambier"),
	// America/Anchorage
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-9 * ONE_HOUR, "US/Alaska",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-9 * ONE_HOUR, "US/Alaska",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167642000000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-8 * ONE_HOUR, "America/Dawson",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-8 * ONE_HOUR, "America/Dawson",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167638400000L,	// 2007
	}),
	// America/Tijuana
	new SimpleTimeZone(-8 * ONE_HOUR, "America/Ensenada",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-8 * ONE_HOUR, "America/Los_Angeles",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-8 * ONE_HOUR, "America/Los_Angeles",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167638400000L,	// 2007
	}),
	new SimpleTimeZone(-8 * ONE_HOUR, "America/Tijuana",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-8 * ONE_HOUR, "America/Vancouver",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-8 * ONE_HOUR, "America/Vancouver",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167638400000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-8 * ONE_HOUR, "America/Whitehorse",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-8 * ONE_HOUR, "America/Whitehorse",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167638400000L,	// 2007
	}),
	// America/Vancouver
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-8 * ONE_HOUR, "Canada/Pacific",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-8 * ONE_HOUR, "Canada/Pacific",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167638400000L,	// 2007
	}),
	// America/Whitehorse
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-8 * ONE_HOUR, "Canada/Yukon",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-8 * ONE_HOUR, "Canada/Yukon",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167638400000L,	// 2007
	}),
	new SimpleTimeZone(-8 * ONE_HOUR, "Etc/GMT+8"),
	// America/Tijuana
	new SimpleTimeZone(-8 * ONE_HOUR, "Mexico/BajaNorte",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-8 * ONE_HOUR, "PST8PDT",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-8 * ONE_HOUR, "PST8PDT",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167638400000L,	// 2007
	}),
	new SimpleTimeZone(-8 * ONE_HOUR, "Pacific/Pitcairn"),
	// America/Los_Angeles
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-8 * ONE_HOUR, "US/Pacific",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-8 * ONE_HOUR, "US/Pacific",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167638400000L,	// 2007
	}),
	// America/Los_Angeles
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-8 * ONE_HOUR, "US/Pacific-New",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-8 * ONE_HOUR, "US/Pacific-New",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167638400000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Boise",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Boise",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167634800000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Cambridge_Bay",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Cambridge_Bay",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167634800000L,	// 2007
	}),
	new SimpleTimeZone(-7 * ONE_HOUR, "America/Chihuahua",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-7 * ONE_HOUR, "America/Dawson_Creek"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Denver",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Denver",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167634800000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Edmonton",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Edmonton",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167634800000L,	// 2007
	}),
	new SimpleTimeZone(-7 * ONE_HOUR, "America/Hermosillo"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Inuvik",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Inuvik",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167634800000L,	// 2007
	}),
	new SimpleTimeZone(-7 * ONE_HOUR, "America/Mazatlan",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-7 * ONE_HOUR, "America/Phoenix"),
	// America/Denver
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Shiprock",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Shiprock",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167634800000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Yellowknife",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-7 * ONE_HOUR, "America/Yellowknife",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167634800000L,	// 2007
	}),
	// America/Edmonton
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-7 * ONE_HOUR, "Canada/Mountain",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-7 * ONE_HOUR, "Canada/Mountain",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167634800000L,	// 2007
	}),
	new SimpleTimeZone(-7 * ONE_HOUR, "Etc/GMT+7"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-7 * ONE_HOUR, "MST7MDT",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-7 * ONE_HOUR, "MST7MDT",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167634800000L,	// 2007
	}),
	// America/Mazatlan
	new SimpleTimeZone(-7 * ONE_HOUR, "Mexico/BajaSur",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	// America/Denver
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-7 * ONE_HOUR, "Navajo",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-7 * ONE_HOUR, "Navajo",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167634800000L,	// 2007
	}),
	// America/Phoenix
	new SimpleTimeZone(-7 * ONE_HOUR, "US/Arizona"),
	// America/Denver
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-7 * ONE_HOUR, "US/Mountain",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-7 * ONE_HOUR, "US/Mountain",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167634800000L,	// 2007
	}),
	new SimpleTimeZone(-6 * ONE_HOUR, "America/Belize"),
	new SimpleTimeZone(-6 * ONE_HOUR, "America/Cancun",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Chicago",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Chicago",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167631200000L,	// 2007
	}),
	new SimpleTimeZone(-6 * ONE_HOUR, "America/El_Salvador"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Guatemala"),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Guatemala",
			Calendar.APRIL, 30, 0, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 1, 0, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Guatemala"),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136095200000L,	// 2006
			1167631200000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indiana/Knox"),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Indiana/Knox",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Indiana/Knox",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 3, 2, 7200000},	// Sun Apr 2, 2006 02:00
			{2007, 0, 0, 0},
	}, new long[] {
			1143961200000L,	// Sun Apr 2, 2006 02:00
			1167631200000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indiana/Petersburg"),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Indiana/Petersburg",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Indiana/Petersburg",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 3, 2, 7200000},	// Sun Apr 2, 2006 02:00
			{2007, 0, 0, 0},
	}, new long[] {
			1143961200000L,	// Sun Apr 2, 2006 02:00
			1167631200000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indiana/Vincennes"),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Indiana/Vincennes",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Indiana/Vincennes",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 3, 2, 7200000},	// Sun Apr 2, 2006 02:00
			{2007, 0, 0, 0},
	}, new long[] {
			1143961200000L,	// Sun Apr 2, 2006 02:00
			1167631200000L,	// 2007
	}),
	// America/Indiana/Knox
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Knox_IN"),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Knox_IN",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Knox_IN",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 3, 2, 7200000},	// Sun Apr 2, 2006 02:00
			{2007, 0, 0, 0},
	}, new long[] {
			1143961200000L,	// Sun Apr 2, 2006 02:00
			1167631200000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Managua",
			Calendar.APRIL, 10, 0, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 1, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Managua",
			Calendar.APRIL, 30, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 1, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Managua"),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136095200000L,	// 2006
			1167631200000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Menominee",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Menominee",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167631200000L,	// 2007
	}),
	new SimpleTimeZone(-6 * ONE_HOUR, "America/Merida",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-6 * ONE_HOUR, "America/Mexico_City",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-6 * ONE_HOUR, "America/Monterrey",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "America/North_Dakota/Center",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/North_Dakota/Center",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167631200000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "America/North_Dakota/New_Salem",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/North_Dakota/New_Salem",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167631200000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Rainy_River",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Rainy_River",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167631200000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Rankin_Inlet",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Rankin_Inlet",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167631200000L,	// 2007
	}),
	new SimpleTimeZone(-6 * ONE_HOUR, "America/Regina"),
	new SimpleTimeZone(-6 * ONE_HOUR, "America/Swift_Current"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Tegucigalpa"),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Tegucigalpa",
			Calendar.MAY, 1, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.AUGUST, 1, -Calendar.MONDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Tegucigalpa"),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2010, 0, 0, 0},
	}, new long[] {
			1136095200000L,	// 2006
			1262325600000L,	// 2010
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Winnipeg",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Winnipeg",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "America/Winnipeg",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},	// 2006
			{2007, 0, 0, 0},
	}, new long[] {
			1136095200000L,	// 2006
			1167631200000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "CST6CDT",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "CST6CDT",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167631200000L,	// 2007
	}),
	// America/Winnipeg
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "Canada/Central",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "Canada/Central",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "Canada/Central",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},	// 2006
			{2007, 0, 0, 0},
	}, new long[] {
			1136095200000L,	// 2006
			1167631200000L,	// 2007
	}),
	// America/Regina
	new SimpleTimeZone(-6 * ONE_HOUR, "Canada/East-Saskatchewan"),
	// America/Regina
	new SimpleTimeZone(-6 * ONE_HOUR, "Canada/Saskatchewan"),
	// Pacific/Easter
	new SimpleTimeZone(-6 * ONE_HOUR, "Chile/EasterIsland",
		Calendar.OCTOBER, 9, -Calendar.SUNDAY, 4 * ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.MARCH, 9, -Calendar.SUNDAY, 3 * ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-6 * ONE_HOUR, "Etc/GMT+6"),
	// America/Mexico_City
	new SimpleTimeZone(-6 * ONE_HOUR, "Mexico/General",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-6 * ONE_HOUR, "Pacific/Easter",
		Calendar.OCTOBER, 9, -Calendar.SUNDAY, 4 * ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.MARCH, 9, -Calendar.SUNDAY, 3 * ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-6 * ONE_HOUR, "Pacific/Galapagos"),
	// America/Chicago
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-6 * ONE_HOUR, "US/Central",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "US/Central",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167631200000L,	// 2007
	}),
	// America/Indiana/Knox
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "US/Indiana-Starke"),
		new SimpleTimeZone(-6 * ONE_HOUR, "US/Indiana-Starke",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-6 * ONE_HOUR, "US/Indiana-Starke",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 3, 2, 7200000},	// Sun Apr 2, 2006 02:00
			{2007, 0, 0, 0},
	}, new long[] {
			1143961200000L,	// Sun Apr 2, 2006 02:00
			1167631200000L,	// 2007
	}),
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Atikokan"),
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Bogota"),
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Cayman"),
	// America/Atikokan
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Coral_Harbour"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Detroit",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Detroit",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Eirunepe"),
	// America/Indiana/Indianapolis
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Fort_Wayne"),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Fort_Wayne",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Fort_Wayne",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},	// 2006
			{2007, 0, 0, 0},
	}, new long[] {
			1136091600000L,	// 2006
			1167627600000L,	// 2007
	}),
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Grand_Turk",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Guayaquil"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Havana",
			Calendar.JANUARY, 1, 0, 0 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.DECEMBER, 31, 0, 86399999, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Havana",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indiana/Indianapolis"),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indiana/Indianapolis",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indiana/Indianapolis",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},	// 2006
			{2007, 0, 0, 0},
	}, new long[] {
			1136091600000L,	// 2006
			1167627600000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indiana/Marengo"),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indiana/Marengo",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indiana/Marengo",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},	// 2006
			{2007, 0, 0, 0},
	}, new long[] {
			1136091600000L,	// 2006
			1167627600000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indiana/Vevay"),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indiana/Vevay",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indiana/Vevay",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},	// 2006
			{2007, 0, 0, 0},
	}, new long[] {
			1136091600000L,	// 2006
			1167627600000L,	// 2007
	}),
	// America/Indiana/Indianapolis
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indianapolis"),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indianapolis",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Indianapolis",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},	// 2006
			{2007, 0, 0, 0},
	}, new long[] {
			1136091600000L,	// 2006
			1167627600000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Iqaluit",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Iqaluit",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Jamaica"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Kentucky/Louisville",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Kentucky/Louisville",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Kentucky/Monticello",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Kentucky/Monticello",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Lima"),
	// America/Kentucky/Louisville
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Louisville",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Louisville",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Montreal",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Montreal",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Nassau",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/New_York",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/New_York",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Nipigon",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Nipigon",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Panama"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Pangnirtung",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Pangnirtung",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Port-au-Prince",
		Calendar.APRIL, 1, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	// America/Rio_Branco
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Porto_Acre"),
	new SimpleTimeZone(-5 * ONE_HOUR, "America/Rio_Branco"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Thunder_Bay",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Thunder_Bay",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Toronto",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "America/Toronto",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	// America/Rio_Branco
	new SimpleTimeZone(-5 * ONE_HOUR, "Brazil/Acre"),
	// America/Toronto
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "Canada/Eastern",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "Canada/Eastern",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	// America/Havana
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "Cuba",
			Calendar.JANUARY, 1, 0, 0 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.DECEMBER, 31, 0, 86399999, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "Cuba",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "EST5EDT",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "EST5EDT",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new SimpleTimeZone(-5 * ONE_HOUR, "Etc/GMT+5"),
	// America/Jamaica
	new SimpleTimeZone(-5 * ONE_HOUR, "Jamaica"),
	// America/Indiana/Indianapolis
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "US/East-Indiana"),
		new SimpleTimeZone(-5 * ONE_HOUR, "US/East-Indiana",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "US/East-Indiana",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},	// 2006
			{2007, 0, 0, 0},
	}, new long[] {
			1136091600000L,	// 2006
			1167627600000L,	// 2007
	}),
	// America/New_York
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "US/Eastern",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "US/Eastern",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	// America/Detroit
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-5 * ONE_HOUR, "US/Michigan",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-5 * ONE_HOUR, "US/Michigan",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167627600000L,	// 2007
	}),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Anguilla"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Antigua"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Aruba"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Asuncion",
		Calendar.OCTOBER, 15, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.MARCH, 8, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Barbados"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Blanc-Sablon"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Boa_Vista"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Campo_Grande",
		Calendar.OCTOBER, 15, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.FEBRUARY, 15, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Caracas"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Cuiaba",
		Calendar.OCTOBER, 15, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.FEBRUARY, 15, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Curacao"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Dominica"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-4 * ONE_HOUR, "America/Glace_Bay",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-4 * ONE_HOUR, "America/Glace_Bay",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167624000000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-4 * ONE_HOUR, "America/Goose_Bay",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-4 * ONE_HOUR, "America/Goose_Bay",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167624000000L,	// 2007
	}),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Grenada"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Guadeloupe"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Guyana"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/La_Paz"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Manaus"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Martinique"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-4 * ONE_HOUR, "America/Moncton",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-4 * ONE_HOUR, "America/Moncton",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},	// 2007
	}, new long[] {
			1167624000000L,	// 2007
	}),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Montserrat"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Port_of_Spain"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Porto_Velho"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Puerto_Rico"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Santiago",
		Calendar.OCTOBER, 9, -Calendar.SUNDAY, 4 * ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.MARCH, 9, -Calendar.SUNDAY, 3 * ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Santo_Domingo"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/St_Kitts"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/St_Lucia"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/St_Thomas"),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/St_Vincent"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-4 * ONE_HOUR, "America/Thule",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-4 * ONE_HOUR, "America/Thule",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167624000000L,	// 2007
	}),
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Tortola"),
	// America/St_Thomas
	new SimpleTimeZone(-4 * ONE_HOUR, "America/Virgin"),
	new SimpleTimeZone(-4 * ONE_HOUR, "Antarctica/Palmer",
		Calendar.OCTOBER, 9, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.MARCH, 9, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-4 * ONE_HOUR, "Atlantic/Bermuda",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-4 * ONE_HOUR, "Atlantic/Bermuda",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167624000000L,	// 2007
	}),
	new SimpleTimeZone(-4 * ONE_HOUR, "Atlantic/Stanley",
		Calendar.SEPTEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.APRIL, 15, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	// America/Manaus
	new SimpleTimeZone(-4 * ONE_HOUR, "Brazil/West"),
	// America/Halifax
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-4 * ONE_HOUR, "Canada/Atlantic",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-4 * ONE_HOUR, "Canada/Atlantic",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167624000000L,	// 2007
	}),
	// America/Santiago
	new SimpleTimeZone(-4 * ONE_HOUR, "Chile/Continental",
		Calendar.OCTOBER, 9, -Calendar.SUNDAY, 4 * ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.MARCH, 9, -Calendar.SUNDAY, 3 * ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-4 * ONE_HOUR, "Etc/GMT+4"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-3 * ONE_HOUR - 1800000, "America/St_Johns",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-3 * ONE_HOUR - 1800000, "America/St_Johns",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167622200000L,	// 2007
	}),
	// America/St_Johns
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-3 * ONE_HOUR - 1800000, "Canada/Newfoundland",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-3 * ONE_HOUR - 1800000, "Canada/Newfoundland",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 60000, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167622200000L,	// 2007
	}),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Araguaina"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Argentina/Buenos_Aires"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Argentina/Catamarca"),
	// America/Argentina/Catamarca
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Argentina/ComodRivadavia"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Argentina/Cordoba"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Argentina/Jujuy"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Argentina/La_Rioja"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Argentina/Mendoza"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Argentina/Rio_Gallegos"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Argentina/San_Juan"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Argentina/Tucuman"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Argentina/Ushuaia"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Bahia"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Belem"),
	// America/Argentina/Buenos_Aires
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Buenos_Aires"),
	// America/Argentina/Catamarca
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Catamarca"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Cayenne"),
	// America/Argentina/Cordoba
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Cordoba"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Fortaleza"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Godthab",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// America/Argentina/Jujuy
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Jujuy"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Maceio"),
	// America/Argentina/Mendoza
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Mendoza"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-3 * ONE_HOUR, "America/Miquelon",
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-3 * ONE_HOUR, "America/Miquelon",
			Calendar.MARCH, 8, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.NOVEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2007, 0, 0, 0},
	}, new long[] {
			1167620400000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(-3 * ONE_HOUR, "America/Montevideo",
			Calendar.OCTOBER, 9, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.MARCH, 27, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(-3 * ONE_HOUR, "America/Montevideo"),
	}, new int[][] {
			{2006, 0, 0, 0},
	}, new long[] {
			1136084400000L,	// 2006
	}),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Paramaribo"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Recife"),
	// America/Argentina/Cordoba
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Rosario"),
	new SimpleTimeZone(-3 * ONE_HOUR, "America/Sao_Paulo",
		Calendar.OCTOBER, 15, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.FEBRUARY, 15, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-3 * ONE_HOUR, "Antarctica/Rothera"),
	// America/Sao_Paulo
	new SimpleTimeZone(-3 * ONE_HOUR, "Brazil/East",
		Calendar.OCTOBER, 15, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.FEBRUARY, 15, -Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-3 * ONE_HOUR, "Etc/GMT+3"),
	new SimpleTimeZone(-2 * ONE_HOUR, "America/Noronha"),
	// America/Noronha
	new SimpleTimeZone(-2 * ONE_HOUR, "Brazil/DeNoronha"),
	new SimpleTimeZone(-2 * ONE_HOUR, "Etc/GMT+2"),
	new SimpleTimeZone(-1 * ONE_HOUR, "America/Scoresbysund",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-1 * ONE_HOUR, "Atlantic/Azores",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(-1 * ONE_HOUR, "Atlantic/Cape_Verde"),
	new SimpleTimeZone(-1 * ONE_HOUR, "Etc/GMT+1"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Abidjan"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Accra"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Bamako"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Banjul"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Bissau"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Casablanca"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Conakry"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Dakar"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/El_Aaiun"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Freetown"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Lome"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Monrovia"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Nouakchott"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Ouagadougou"),
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Sao_Tome"),
	// Africa/Bamako
	new SimpleTimeZone(0 * ONE_HOUR, "Africa/Timbuktu"),
	new SimpleTimeZone(0 * ONE_HOUR, "America/Danmarkshavn"),
	new SimpleTimeZone(0 * ONE_HOUR, "Atlantic/Canary",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(0 * ONE_HOUR, "Atlantic/Faeroe",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(0 * ONE_HOUR, "Atlantic/Madeira",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(0 * ONE_HOUR, "Atlantic/Reykjavik"),
	new SimpleTimeZone(0 * ONE_HOUR, "Atlantic/St_Helena"),
	// Europe/Dublin
	new SimpleTimeZone(0 * ONE_HOUR, "Eire",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(0 * ONE_HOUR, "Etc/GMT"),
	// Etc/GMT
	new SimpleTimeZone(0 * ONE_HOUR, "Etc/GMT+0"),
	// Etc/GMT
	new SimpleTimeZone(0 * ONE_HOUR, "Etc/GMT-0"),
	// Etc/GMT
	new SimpleTimeZone(0 * ONE_HOUR, "Etc/GMT0"),
	// Etc/GMT
	new SimpleTimeZone(0 * ONE_HOUR, "Etc/Greenwich"),
	new SimpleTimeZone(0 * ONE_HOUR, "Etc/UCT"),
	new SimpleTimeZone(0 * ONE_HOUR, "Etc/UTC"),
	// Etc/UTC
	new SimpleTimeZone(0 * ONE_HOUR, "Etc/Universal"),
	// Etc/UTC
	new SimpleTimeZone(0 * ONE_HOUR, "Etc/Zulu"),
	// Europe/London
	new SimpleTimeZone(0 * ONE_HOUR, "Europe/Belfast",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(0 * ONE_HOUR, "Europe/Dublin",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/London
	new SimpleTimeZone(0 * ONE_HOUR, "Europe/Guernsey",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/London
	new SimpleTimeZone(0 * ONE_HOUR, "Europe/Isle_of_Man",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/London
	new SimpleTimeZone(0 * ONE_HOUR, "Europe/Jersey",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(0 * ONE_HOUR, "Europe/Lisbon",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/London
	new SimpleTimeZone(0 * ONE_HOUR, "GB",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/London
	new SimpleTimeZone(0 * ONE_HOUR, "GB-Eire",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Etc/GMT
	new SimpleTimeZone(0 * ONE_HOUR, "GMT0"),
	// Etc/GMT
	new SimpleTimeZone(0 * ONE_HOUR, "Greenwich"),
	// Atlantic/Reykjavik
	new SimpleTimeZone(0 * ONE_HOUR, "Iceland"),
	// Europe/Lisbon
	new SimpleTimeZone(0 * ONE_HOUR, "Portugal",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Etc/UCT
	new SimpleTimeZone(0 * ONE_HOUR, "UCT"),
	// Etc/UTC
	new SimpleTimeZone(0 * ONE_HOUR, "Universal"),
	// Etc/UTC
	new SimpleTimeZone(0 * ONE_HOUR, "Zulu"),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Bangui"),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Brazzaville"),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Ceuta",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Douala"),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Kinshasa"),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Lagos"),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Libreville"),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Luanda"),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Malabo"),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Ndjamena"),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Niamey"),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Porto-Novo"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(1 * ONE_HOUR, "Africa/Tunis",
			Calendar.MAY, 1, 0, 0 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.SEPTEMBER, 30, 0, ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(1 * ONE_HOUR, "Africa/Tunis",
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
	}, new long[] {
			1136070000000L,	// 2006
	}),
	new SimpleTimeZone(1 * ONE_HOUR, "Africa/Windhoek",
		Calendar.SEPTEMBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	// Europe/Oslo
	new SimpleTimeZone(1 * ONE_HOUR, "Arctic/Longyearbyen",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/Oslo
	new SimpleTimeZone(1 * ONE_HOUR, "Atlantic/Jan_Mayen",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Etc/GMT-1"),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Amsterdam",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Andorra",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Belgrade",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Berlin",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/Prague
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Bratislava",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Brussels",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Budapest",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Copenhagen",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Gibraltar",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/Belgrade
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Ljubljana",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Luxembourg",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Madrid",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Malta",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Monaco",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Oslo",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Paris",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Prague",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Rome",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/Rome
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/San_Marino",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/Belgrade
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Sarajevo",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/Belgrade
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Skopje",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Stockholm",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Tirane",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Vaduz",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/Rome
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Vatican",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Vienna",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Warsaw",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/Belgrade
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Zagreb",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(1 * ONE_HOUR, "Europe/Zurich",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/Warsaw
	new SimpleTimeZone(1 * ONE_HOUR, "Poland",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Africa/Blantyre"),
	new SimpleTimeZone(2 * ONE_HOUR, "Africa/Bujumbura"),
	new SimpleTimeZone(2 * ONE_HOUR, "Africa/Gaborone"),
	new SimpleTimeZone(2 * ONE_HOUR, "Africa/Johannesburg"),
	new SimpleTimeZone(2 * ONE_HOUR, "Africa/Kigali"),
	new SimpleTimeZone(2 * ONE_HOUR, "Africa/Lubumbashi"),
	new SimpleTimeZone(2 * ONE_HOUR, "Africa/Lusaka"),
	new SimpleTimeZone(2 * ONE_HOUR, "Africa/Maputo"),
	new SimpleTimeZone(2 * ONE_HOUR, "Africa/Maseru"),
	new SimpleTimeZone(2 * ONE_HOUR, "Africa/Mbabane"),
	new SimpleTimeZone(2 * ONE_HOUR, "Africa/Tripoli"),
	new SimpleTimeZone(2 * ONE_HOUR, "Asia/Amman",
		Calendar.MARCH, -1, Calendar.THURSDAY, 0 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.SEPTEMBER, -1, Calendar.FRIDAY, 0 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Asia/Beirut",
		Calendar.MARCH, -1, Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Asia/Damascus",
		Calendar.APRIL, 1, 0, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, 1, 0, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Gaza",
			Calendar.APRIL, 15, -Calendar.FRIDAY, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 4, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Gaza",
			Calendar.APRIL, 1, 0, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 4, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
	}, new long[] {
			1136066400000L,	// 2006
	}),
	// Europe/Istanbul
	new SimpleTimeZone(2 * ONE_HOUR, "Asia/Istanbul",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Asia/Nicosia",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Asia/Jerusalem
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 9, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 16, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 5, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 27, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 12, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 2, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 23, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 8, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 28, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 20, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 9, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 24, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 16, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 6, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 27, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 12, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 2, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 24, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 6, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 28, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 20, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 10, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 24, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 16, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 6, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 21, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 12, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 2, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 17, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 7, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 28, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 13, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Asia/Tel_Aviv"),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
			{2008, 0, 0, 0},
			{2009, 0, 0, 0},
			{2010, 0, 0, 0},
			{2011, 0, 0, 0},
			{2012, 0, 0, 0},
			{2013, 0, 0, 0},
			{2014, 0, 0, 0},
			{2015, 0, 0, 0},
			{2016, 0, 0, 0},
			{2017, 0, 0, 0},
			{2018, 0, 0, 0},
			{2019, 0, 0, 0},
			{2020, 0, 0, 0},
			{2021, 0, 0, 0},
			{2022, 0, 0, 0},
			{2023, 0, 0, 0},
			{2024, 0, 0, 0},
			{2025, 0, 0, 0},
			{2026, 0, 0, 0},
			{2027, 0, 0, 0},
			{2028, 0, 0, 0},
			{2029, 0, 0, 0},
			{2030, 0, 0, 0},
			{2031, 0, 0, 0},
			{2032, 0, 0, 0},
			{2033, 0, 0, 0},
			{2034, 0, 0, 0},
			{2035, 0, 0, 0},
			{2036, 0, 0, 0},
			{2037, 0, 0, 0},
			{2038, 0, 0, 0},
	}, new long[] {
			1136066400000L,	// 2006
			1167602400000L,	// 2007
			1199138400000L,	// 2008
			1230760800000L,	// 2009
			1262296800000L,	// 2010
			1293832800000L,	// 2011
			1325368800000L,	// 2012
			1356991200000L,	// 2013
			1388527200000L,	// 2014
			1420063200000L,	// 2015
			1451599200000L,	// 2016
			1483221600000L,	// 2017
			1514757600000L,	// 2018
			1546293600000L,	// 2019
			1577829600000L,	// 2020
			1609452000000L,	// 2021
			1640988000000L,	// 2022
			1672524000000L,	// 2023
			1704060000000L,	// 2024
			1735682400000L,	// 2025
			1767218400000L,	// 2026
			1798754400000L,	// 2027
			1830290400000L,	// 2028
			1861912800000L,	// 2029
			1893448800000L,	// 2030
			1924984800000L,	// 2031
			1956520800000L,	// 2032
			1988143200000L,	// 2033
			2019679200000L,	// 2034
			2051215200000L,	// 2035
			2082751200000L,	// 2036
			2114373600000L,	// 2037
			2145909600000L,	// 2038
	}),
	// Africa/Cairo
	new SimpleTimeZone(2 * ONE_HOUR, "Egypt",
		Calendar.APRIL, -1, Calendar.FRIDAY, 0 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.SEPTEMBER, -1, Calendar.THURSDAY, 23 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Etc/GMT-2"),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Athens",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Chisinau",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Helsinki",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Istanbul",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Kaliningrad",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Kiev",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/Helsinki
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Mariehamn",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Minsk",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	// Asia/Nicosia
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Nicosia",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Riga",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Simferopol",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Sofia",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Tallinn",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Europe/Chisinau
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Tiraspol",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Uzhgorod",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Vilnius",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(2 * ONE_HOUR, "Europe/Zaporozhye",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	// Asia/Jerusalem
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 9, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 16, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 5, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 27, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 12, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 2, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 23, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 8, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 28, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 20, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 9, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 24, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 16, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 6, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 27, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 12, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 2, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 24, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 6, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 28, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 20, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 10, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 24, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 16, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 6, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 21, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 12, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.APRIL, 1, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 2, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 17, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.OCTOBER, 7, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 28, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel",
			Calendar.MARCH, 26, -Calendar.FRIDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 13, 0, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(2 * ONE_HOUR, "Israel"),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
			{2008, 0, 0, 0},
			{2009, 0, 0, 0},
			{2010, 0, 0, 0},
			{2011, 0, 0, 0},
			{2012, 0, 0, 0},
			{2013, 0, 0, 0},
			{2014, 0, 0, 0},
			{2015, 0, 0, 0},
			{2016, 0, 0, 0},
			{2017, 0, 0, 0},
			{2018, 0, 0, 0},
			{2019, 0, 0, 0},
			{2020, 0, 0, 0},
			{2021, 0, 0, 0},
			{2022, 0, 0, 0},
			{2023, 0, 0, 0},
			{2024, 0, 0, 0},
			{2025, 0, 0, 0},
			{2026, 0, 0, 0},
			{2027, 0, 0, 0},
			{2028, 0, 0, 0},
			{2029, 0, 0, 0},
			{2030, 0, 0, 0},
			{2031, 0, 0, 0},
			{2032, 0, 0, 0},
			{2033, 0, 0, 0},
			{2034, 0, 0, 0},
			{2035, 0, 0, 0},
			{2036, 0, 0, 0},
			{2037, 0, 0, 0},
			{2038, 0, 0, 0},
	}, new long[] {
			1136066400000L,	// 2006
			1167602400000L,	// 2007
			1199138400000L,	// 2008
			1230760800000L,	// 2009
			1262296800000L,	// 2010
			1293832800000L,	// 2011
			1325368800000L,	// 2012
			1356991200000L,	// 2013
			1388527200000L,	// 2014
			1420063200000L,	// 2015
			1451599200000L,	// 2016
			1483221600000L,	// 2017
			1514757600000L,	// 2018
			1546293600000L,	// 2019
			1577829600000L,	// 2020
			1609452000000L,	// 2021
			1640988000000L,	// 2022
			1672524000000L,	// 2023
			1704060000000L,	// 2024
			1735682400000L,	// 2025
			1767218400000L,	// 2026
			1798754400000L,	// 2027
			1830290400000L,	// 2028
			1861912800000L,	// 2029
			1893448800000L,	// 2030
			1924984800000L,	// 2031
			1956520800000L,	// 2032
			1988143200000L,	// 2033
			2019679200000L,	// 2034
			2051215200000L,	// 2035
			2082751200000L,	// 2036
			2114373600000L,	// 2037
			2145909600000L,	// 2038
	}),
	// Africa/Tripoli
	new SimpleTimeZone(2 * ONE_HOUR, "Libya"),
	// Europe/Istanbul
	new SimpleTimeZone(2 * ONE_HOUR, "Turkey",
		Calendar.MARCH, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, ONE_HOUR, SimpleTimeZone.UTC_TIME,
		ONE_HOUR),
	new SimpleTimeZone(3 * ONE_HOUR, "Africa/Addis_Ababa"),
	new SimpleTimeZone(3 * ONE_HOUR, "Africa/Asmera"),
	new SimpleTimeZone(3 * ONE_HOUR, "Africa/Dar_es_Salaam"),
	new SimpleTimeZone(3 * ONE_HOUR, "Africa/Djibouti"),
	new SimpleTimeZone(3 * ONE_HOUR, "Africa/Kampala"),
	new SimpleTimeZone(3 * ONE_HOUR, "Africa/Khartoum"),
	new SimpleTimeZone(3 * ONE_HOUR, "Africa/Mogadishu"),
	new SimpleTimeZone(3 * ONE_HOUR, "Africa/Nairobi"),
	new SimpleTimeZone(3 * ONE_HOUR, "Antarctica/Syowa"),
	new SimpleTimeZone(3 * ONE_HOUR, "Asia/Aden"),
	new SimpleTimeZone(3 * ONE_HOUR, "Asia/Baghdad",
		Calendar.APRIL, 1, 0, 3 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, 1, 0, 3 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(3 * ONE_HOUR, "Asia/Bahrain"),
	new SimpleTimeZone(3 * ONE_HOUR, "Asia/Kuwait"),
	new SimpleTimeZone(3 * ONE_HOUR, "Asia/Qatar"),
	new SimpleTimeZone(3 * ONE_HOUR, "Asia/Riyadh"),
	new SimpleTimeZone(3 * ONE_HOUR, "Etc/GMT-3"),
	new SimpleTimeZone(3 * ONE_HOUR, "Europe/Volgograd",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(3 * ONE_HOUR, "Indian/Antananarivo"),
	new SimpleTimeZone(3 * ONE_HOUR, "Indian/Comoro"),
	new SimpleTimeZone(3 * ONE_HOUR, "Indian/Mayotte"),
	// Europe/Moscow
	new SimpleTimeZone(3 * ONE_HOUR, "W-SU",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	// Asia/Tehran
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(3 * ONE_HOUR + 1800000, "Iran",
			Calendar.MARCH, 22, 0, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.SEPTEMBER, 22, 0, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(3 * ONE_HOUR + 1800000, "Iran"),
	}, new int[][] {
			{2006, 0, 0, 0},
	}, new long[] {
			1136061000000L,	// 2006
	}),
	new SimpleTimeZone(4 * ONE_HOUR, "Asia/Baku",
		Calendar.MARCH, -1, Calendar.SUNDAY, 4 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 5 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(4 * ONE_HOUR, "Asia/Dubai"),
	new SimpleTimeZone(4 * ONE_HOUR, "Asia/Muscat"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(3 * ONE_HOUR, "Asia/Tbilisi",
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(4 * ONE_HOUR, "Asia/Tbilisi"),
	}, new int[][] {
			{2005, 2, 27, 7200000},	// Sun Mar 27, 2005 01:00
	}, new long[] {
			1111874400000L,	// Sun Mar 27, 2005 01:00
	}),
	new SimpleTimeZone(4 * ONE_HOUR, "Asia/Yerevan",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(4 * ONE_HOUR, "Etc/GMT-4"),
	new SimpleTimeZone(4 * ONE_HOUR, "Europe/Samara",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(4 * ONE_HOUR, "Indian/Mahe"),
	new SimpleTimeZone(4 * ONE_HOUR, "Indian/Mauritius"),
	new SimpleTimeZone(4 * ONE_HOUR, "Indian/Reunion"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(4 * ONE_HOUR, "Asia/Aqtau",
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(5 * ONE_HOUR, "Asia/Aqtau"),
	}, new int[][] {
			{2005, 2, 15, 0},	// Tue Mar 15, 2005 00:00
	}, new long[] {
			1110830400000L,	// Tue Mar 15, 2005 00:00
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(5 * ONE_HOUR, "Asia/Aqtobe",
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(5 * ONE_HOUR, "Asia/Aqtobe"),
	}, new int[][] {
			{2005, 2, 15, 0},	// Tue Mar 15, 2005 00:00
	}, new long[] {
			1110826800000L,	// Tue Mar 15, 2005 00:00
	}),
	new SimpleTimeZone(5 * ONE_HOUR, "Asia/Ashgabat"),
	// Asia/Ashgabat
	new SimpleTimeZone(5 * ONE_HOUR, "Asia/Ashkhabad"),
	new SimpleTimeZone(5 * ONE_HOUR, "Asia/Dushanbe"),
	new SimpleTimeZone(5 * ONE_HOUR, "Asia/Karachi"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(4 * ONE_HOUR, "Asia/Oral",
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(5 * ONE_HOUR, "Asia/Oral"),
	}, new int[][] {
			{2005, 2, 15, 0},	// Tue Mar 15, 2005 00:00
	}, new long[] {
			1110830400000L,	// Tue Mar 15, 2005 00:00
	}),
	new SimpleTimeZone(5 * ONE_HOUR, "Asia/Samarkand"),
	new SimpleTimeZone(5 * ONE_HOUR, "Asia/Tashkent"),
	new SimpleTimeZone(5 * ONE_HOUR, "Asia/Yekaterinburg",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(5 * ONE_HOUR, "Etc/GMT-5"),
	new SimpleTimeZone(5 * ONE_HOUR, "Indian/Kerguelen"),
	new SimpleTimeZone(5 * ONE_HOUR, "Indian/Maldives"),
	new SimpleTimeZone(5 * ONE_HOUR + 1800000, "Asia/Calcutta"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(6 * ONE_HOUR, "Asia/Colombo"),
		new SimpleTimeZone(5 * ONE_HOUR + 1800000, "Asia/Colombo"),
	}, new int[][] {
			{2006, 3, 15, 1800000},	// Sat Apr 15, 2006 00:30
	}, new long[] {
			1145039400000L,	// Sat Apr 15, 2006 00:30
	}),
	new SimpleTimeZone(20700000, "Asia/Katmandu"),
	new SimpleTimeZone(6 * ONE_HOUR, "Antarctica/Mawson"),
	new SimpleTimeZone(6 * ONE_HOUR, "Antarctica/Vostok"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(6 * ONE_HOUR, "Asia/Almaty",
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(6 * ONE_HOUR, "Asia/Almaty"),
	}, new int[][] {
			{2005, 2, 15, 0},	// Tue Mar 15, 2005 00:00
	}, new long[] {
			1110823200000L,	// Tue Mar 15, 2005 00:00
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(5 * ONE_HOUR, "Asia/Bishkek",
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR + 1800000, SimpleTimeZone.WALL_TIME,
			Calendar.AUGUST, 12, 0, 0 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			ONE_HOUR),
		new SimpleTimeZone(6 * ONE_HOUR, "Asia/Bishkek"),
	}, new int[][] {
			{2005, 7, 12, 0},	// Fri Aug 12, 2005 00:00
	}, new long[] {
			1123786800000L,	// Fri Aug 12, 2005 00:00
	}),
	// Asia/Dhaka
	new SimpleTimeZone(6 * ONE_HOUR, "Asia/Dacca"),
	new SimpleTimeZone(6 * ONE_HOUR, "Asia/Dhaka"),
	new SimpleTimeZone(6 * ONE_HOUR, "Asia/Novosibirsk",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(6 * ONE_HOUR, "Asia/Omsk",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(6 * ONE_HOUR, "Asia/Qyzylorda",
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(6 * ONE_HOUR, "Asia/Qyzylorda"),
	}, new int[][] {
			{2005, 2, 15, 0},	// Tue Mar 15, 2005 00:00
	}, new long[] {
			1110823200000L,	// Tue Mar 15, 2005 00:00
	}),
	// Asia/Thimphu
	new SimpleTimeZone(6 * ONE_HOUR, "Asia/Thimbu"),
	new SimpleTimeZone(6 * ONE_HOUR, "Asia/Thimphu"),
	new SimpleTimeZone(6 * ONE_HOUR, "Etc/GMT-6"),
	new SimpleTimeZone(6 * ONE_HOUR, "Indian/Chagos"),
	new SimpleTimeZone(6 * ONE_HOUR + 1800000, "Asia/Rangoon"),
	new SimpleTimeZone(6 * ONE_HOUR + 1800000, "Indian/Cocos"),
	new SimpleTimeZone(7 * ONE_HOUR, "Antarctica/Davis"),
	new SimpleTimeZone(7 * ONE_HOUR, "Asia/Bangkok"),
	new SimpleTimeZone(7 * ONE_HOUR, "Asia/Hovd",
		Calendar.MARCH, -1, Calendar.SATURDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.SEPTEMBER, -1, Calendar.SATURDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(7 * ONE_HOUR, "Asia/Jakarta"),
	new SimpleTimeZone(7 * ONE_HOUR, "Asia/Krasnoyarsk",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(7 * ONE_HOUR, "Asia/Phnom_Penh"),
	new SimpleTimeZone(7 * ONE_HOUR, "Asia/Pontianak"),
	new SimpleTimeZone(7 * ONE_HOUR, "Asia/Saigon"),
	new SimpleTimeZone(7 * ONE_HOUR, "Asia/Vientiane"),
	new SimpleTimeZone(7 * ONE_HOUR, "Etc/GMT-7"),
	new SimpleTimeZone(7 * ONE_HOUR, "Indian/Christmas"),
	new SimpleTimeZone(8 * ONE_HOUR, "Antarctica/Casey"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Brunei"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Chongqing"),
	// Asia/Chongqing
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Chungking"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Harbin"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Hong_Kong"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Irkutsk",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Kashgar"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Kuala_Lumpur"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Kuching"),
	// Asia/Macau
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Macao"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Macau"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Makassar"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Manila"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Shanghai"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Singapore"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Taipei"),
	// Asia/Makassar
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Ujung_Pandang"),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Ulaanbaatar",
		Calendar.MARCH, -1, Calendar.SATURDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.SEPTEMBER, -1, Calendar.SATURDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	// Asia/Ulaanbaatar
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Ulan_Bator",
		Calendar.MARCH, -1, Calendar.SATURDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.SEPTEMBER, -1, Calendar.SATURDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(8 * ONE_HOUR, "Asia/Urumqi"),
	new SimpleTimeZone(8 * ONE_HOUR, "Australia/Perth"),
	// Australia/Perth
	new SimpleTimeZone(8 * ONE_HOUR, "Australia/West"),
	new SimpleTimeZone(8 * ONE_HOUR, "Etc/GMT-8"),
	// Asia/Hong_Kong
	new SimpleTimeZone(8 * ONE_HOUR, "Hongkong"),
	// Asia/Singapore
	new SimpleTimeZone(8 * ONE_HOUR, "Singapore"),
	new SimpleTimeZone(9 * ONE_HOUR, "Asia/Choibalsan",
		Calendar.MARCH, -1, Calendar.SATURDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		Calendar.SEPTEMBER, -1, Calendar.SATURDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
		ONE_HOUR),
	new SimpleTimeZone(9 * ONE_HOUR, "Asia/Dili"),
	new SimpleTimeZone(9 * ONE_HOUR, "Asia/Jayapura"),
	new SimpleTimeZone(9 * ONE_HOUR, "Asia/Pyongyang"),
	new SimpleTimeZone(9 * ONE_HOUR, "Asia/Seoul"),
	new SimpleTimeZone(9 * ONE_HOUR, "Asia/Tokyo"),
	new SimpleTimeZone(9 * ONE_HOUR, "Asia/Yakutsk",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(9 * ONE_HOUR, "Etc/GMT-9"),
	// Asia/Tokyo
	new SimpleTimeZone(9 * ONE_HOUR, "Japan"),
	new SimpleTimeZone(9 * ONE_HOUR, "Pacific/Palau"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/Broken_Hill",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/Broken_Hill",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/Broken_Hill",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136035800000L,	// 2006
			1167571800000L,	// 2007
	}),
	new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/Darwin"),
	// Australia/Darwin
	new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/North"),
	// Australia/Adelaide
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/South",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/South",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/South",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136035800000L,	// 2006
			1167571800000L,	// 2007
	}),
	// Australia/Broken_Hill
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/Yancowinna",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/Yancowinna",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(9 * ONE_HOUR + 1800000, "Australia/Yancowinna",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136035800000L,	// 2006
			1167571800000L,	// 2007
	}),
	new SimpleTimeZone(10 * ONE_HOUR, "Antarctica/DumontDUrville"),
	new SimpleTimeZone(10 * ONE_HOUR, "Asia/Sakhalin",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(10 * ONE_HOUR, "Asia/Vladivostok",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	// Australia/Sydney
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/ACT",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/ACT",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/ACT",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136034000000L,	// 2006
			1167570000000L,	// 2007
	}),
	// Australia/Sydney
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Canberra",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Canberra",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Canberra",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136034000000L,	// 2006
			1167570000000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Currie",
			Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Currie",
			Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Currie",
			Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136034000000L,	// 2006
			1167570000000L,	// 2007
	}),
	new SimpleTimeZone(10 * ONE_HOUR, "Australia/Lindeman"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Melbourne",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Melbourne",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Melbourne",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136034000000L,	// 2006
			1167570000000L,	// 2007
	}),
	// Australia/Sydney
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/NSW",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/NSW",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/NSW",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136034000000L,	// 2006
			1167570000000L,	// 2007
	}),
	// Australia/Brisbane
	new SimpleTimeZone(10 * ONE_HOUR, "Australia/Queensland"),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Sydney",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Sydney",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Sydney",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136034000000L,	// 2006
			1167570000000L,	// 2007
	}),
	// Australia/Hobart
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Tasmania",
			Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Tasmania",
			Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Tasmania",
			Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136034000000L,	// 2006
			1167570000000L,	// 2007
	}),
	// Australia/Melbourne
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Victoria",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Victoria",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
		new SimpleTimeZone(10 * ONE_HOUR, "Australia/Victoria",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
			ONE_HOUR),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136034000000L,	// 2006
			1167570000000L,	// 2007
	}),
	new SimpleTimeZone(10 * ONE_HOUR, "Etc/GMT-10"),
	new SimpleTimeZone(10 * ONE_HOUR, "Pacific/Guam"),
	new SimpleTimeZone(10 * ONE_HOUR, "Pacific/Port_Moresby"),
	new SimpleTimeZone(10 * ONE_HOUR, "Pacific/Saipan"),
	new SimpleTimeZone(10 * ONE_HOUR, "Pacific/Truk"),
	// Pacific/Truk
	new SimpleTimeZone(10 * ONE_HOUR, "Pacific/Yap"),
	// Australia/Lord_Howe
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(10 * ONE_HOUR + 1800000, "Australia/LHI",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			0 * ONE_HOUR + 1800000),
		new SimpleTimeZone(10 * ONE_HOUR + 1800000, "Australia/LHI",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			0 * ONE_HOUR + 1800000),
		new SimpleTimeZone(10 * ONE_HOUR + 1800000, "Australia/LHI",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			0 * ONE_HOUR + 1800000),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136034000000L,	// 2006
			1167570000000L,	// 2007
	}),
	new TimeZoneTable(new SimpleTimeZone[] {
		new SimpleTimeZone(10 * ONE_HOUR + 1800000, "Australia/Lord_Howe",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			0 * ONE_HOUR + 1800000),
		new SimpleTimeZone(10 * ONE_HOUR + 1800000, "Australia/Lord_Howe",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.APRIL, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			0 * ONE_HOUR + 1800000),
		new SimpleTimeZone(10 * ONE_HOUR + 1800000, "Australia/Lord_Howe",
			Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.WALL_TIME,
			0 * ONE_HOUR + 1800000),
	}, new int[][] {
			{2006, 0, 0, 0},
			{2007, 0, 0, 0},
	}, new long[] {
			1136034000000L,	// 2006
			1167570000000L,	// 2007
	}),
	new SimpleTimeZone(11 * ONE_HOUR, "Asia/Magadan",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(11 * ONE_HOUR, "Etc/GMT-11"),
	new SimpleTimeZone(11 * ONE_HOUR, "Pacific/Efate"),
	new SimpleTimeZone(11 * ONE_HOUR, "Pacific/Guadalcanal"),
	new SimpleTimeZone(11 * ONE_HOUR, "Pacific/Kosrae"),
	new SimpleTimeZone(11 * ONE_HOUR, "Pacific/Noumea"),
	new SimpleTimeZone(11 * ONE_HOUR, "Pacific/Ponape"),
	new SimpleTimeZone(11 * ONE_HOUR + 1800000, "Pacific/Norfolk"),
	new SimpleTimeZone(12 * ONE_HOUR, "Antarctica/McMurdo",
		Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.MARCH, 15, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	// Antarctica/McMurdo
	new SimpleTimeZone(12 * ONE_HOUR, "Antarctica/South_Pole",
		Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.MARCH, 15, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(12 * ONE_HOUR, "Asia/Anadyr",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(12 * ONE_HOUR, "Asia/Kamchatka",
		Calendar.MARCH, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(12 * ONE_HOUR, "Etc/GMT-12"),
	// Pacific/Kwajalein
	new SimpleTimeZone(12 * ONE_HOUR, "Kwajalein"),
	// Pacific/Auckland
	new SimpleTimeZone(12 * ONE_HOUR, "NZ",
		Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.MARCH, 15, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(12 * ONE_HOUR, "Pacific/Auckland",
		Calendar.OCTOBER, 1, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		Calendar.MARCH, 15, -Calendar.SUNDAY, 2 * ONE_HOUR, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(12 * ONE_HOUR, "Pacific/Fiji"),
	new SimpleTimeZone(12 * ONE_HOUR, "Pacific/Funafuti"),
	new SimpleTimeZone(12 * ONE_HOUR, "Pacific/Kwajalein"),
	new SimpleTimeZone(12 * ONE_HOUR, "Pacific/Majuro"),
	new SimpleTimeZone(12 * ONE_HOUR, "Pacific/Nauru"),
	new SimpleTimeZone(12 * ONE_HOUR, "Pacific/Tarawa"),
	new SimpleTimeZone(12 * ONE_HOUR, "Pacific/Wake"),
	new SimpleTimeZone(12 * ONE_HOUR, "Pacific/Wallis"),
	// Pacific/Chatham
	new SimpleTimeZone(45900000, "NZ-CHAT",
		Calendar.OCTOBER, 1, -Calendar.SUNDAY, 9900000, SimpleTimeZone.STANDARD_TIME,
		Calendar.MARCH, 15, -Calendar.SUNDAY, 9900000, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(45900000, "Pacific/Chatham",
		Calendar.OCTOBER, 1, -Calendar.SUNDAY, 9900000, SimpleTimeZone.STANDARD_TIME,
		Calendar.MARCH, 15, -Calendar.SUNDAY, 9900000, SimpleTimeZone.STANDARD_TIME,
		ONE_HOUR),
	new SimpleTimeZone(13 * ONE_HOUR, "Etc/GMT-13"),
	new SimpleTimeZone(13 * ONE_HOUR, "Pacific/Enderbury"),
	new SimpleTimeZone(13 * ONE_HOUR, "Pacific/Tongatapu"),
	new SimpleTimeZone(14 * ONE_HOUR, "Etc/GMT-14"),
	new SimpleTimeZone(14 * ONE_HOUR, "Pacific/Kiritimati"),
	};
}
}
